/**
*** (c) Copyright 2010-2018 Kalray SA.

***
*** This program is free software; you can redistribute it and/or
*** modify it under the terms of the GNU General Public License
*** as published by the Free Software Foundation; either version
*** 2 of the License, or (at your option) any later version.
***
*** This program is distributed in the hope that it will be useful,
*** but WITHOUT ANY WARRANTY; without even the implied warranty of
*** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*** General Public License for more details.
***
*** You should have received a copy of the GNU General Public License
*** along with this program; if not, write to the Free Software
*** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
***
**/

#ifndef OPCODE_KVX_H
#define OPCODE_KVX_H

#define KVX_REGFILE_FIRST_GPR 0
#define KVX_REGFILE_LAST_GPR 1
#define KVX_REGFILE_DEC_GPR 2
#define KVX_REGFILE_FIRST_PGR 3
#define KVX_REGFILE_LAST_PGR 4
#define KVX_REGFILE_DEC_PGR 5
#define KVX_REGFILE_FIRST_QGR 6
#define KVX_REGFILE_LAST_QGR 7
#define KVX_REGFILE_DEC_QGR 8
#define KVX_REGFILE_FIRST_SFR 9
#define KVX_REGFILE_LAST_SFR 10
#define KVX_REGFILE_DEC_SFR 11
#define KVX_REGFILE_FIRST_XBR 12
#define KVX_REGFILE_LAST_XBR 13
#define KVX_REGFILE_DEC_XBR 14
#define KVX_REGFILE_FIRST_XCR 15
#define KVX_REGFILE_LAST_XCR 16
#define KVX_REGFILE_DEC_XCR 17
#define KVX_REGFILE_FIRST_XMR 18
#define KVX_REGFILE_LAST_XMR 19
#define KVX_REGFILE_DEC_XMR 20
#define KVX_REGFILE_FIRST_XVR 21
#define KVX_REGFILE_LAST_XVR 22
#define KVX_REGFILE_DEC_XVR 23
#define KVX_REGFILE_FIRST_XWR 24
#define KVX_REGFILE_LAST_XWR 25
#define KVX_REGFILE_DEC_XWR 26
#define KVX_REGFILE_REGISTERS 27
#define KVX_REGFILE_DEC_REGISTERS 28


typedef struct {
  int id;
  const char *name;
} kvx_Register;

extern int kvx_kv3_v1_regfiles[];
extern kvx_Register kvx_kv3_v1_registers[];

extern int kvx_kv3_v1_dec_registers[];

extern int kvx_kv3_v2_regfiles[];
extern kvx_Register kvx_kv3_v2_registers[];

extern int kvx_kv3_v2_dec_registers[];

#define KVXMAXSYLLABLES 3
#define KVXMAXOPERANDS 7
#define KVXMAXBUNDLEISSUE 6
#define KVXMAXBUNDLEWORDS 8
#define KVXNUMCORES 2
#define KVXNUMBUNDLINGS 19

#define kv3_IMMX_1_fld(x) (int)(((unsigned int)(x) >> 27) & 0xf)
#define kv3_IMMX_2_fld(x) (int)(((unsigned int)(x) >> 27) & 0xf)
#define kv3_IMMX_3_fld(x) (int)(((unsigned int)(x) >> 27) & 0xf)
#define kv3_IMMX_4_fld(x) (int)(((unsigned int)(x) >> 27) & 0xf)
#define kv3_IMMX_5_fld(x) (int)(((unsigned int)(x) >> 27) & 0xf)
#define kv3_IMMX_6_fld(x) (int)(((unsigned int)(x) >> 27) & 0xf)
#define kv3_IMMX_7_fld(x) (int)(((unsigned int)(x) >> 27) & 0xf)
#define kv3_ENCODE_0_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_ENCODE_1_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_ENCODE_2_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_ENCODE_3_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_ENCODE_4_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_ENCODE_5_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_ENCODE_6_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_ENCODE_7_fld(x) (int)((unsigned int)(x) & 0x7fffffff)
#define kv3_P_0_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_P_1_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_P_2_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_P_3_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_P_4_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_P_5_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_P_6_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_P_7_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_DOUBLE_fld(x) (int)((unsigned int)(x) & 0xffffffffffffffff)
#define kv3_WORD_fld(x) (int)((unsigned int)(x) & 0xffffffff)
#define kv3_HALF_fld(x) (int)((unsigned int)(x) & 0xffff)
#define kv3_BYTE_fld(x) (int)((unsigned int)(x) & 0xff)
#define kv3_bcucode1_fld(x) (int)(((unsigned int)(x) >> 27) & 0x3)
#define kv3_bcucode2_fld(x) (int)(((unsigned int)(x) >> 23) & 0xf)
#define kv3_bcucode3_fld(x) (int)(((unsigned int)(x) >> 18) & 0x1f)
#define kv3_bcucode4_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3)
#define kv3_bcucode5_fld(x) (int)(((unsigned int)(x) >> 12) & 0x3f)
#define kv3_bcucode6_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3f)
#define kv3_bcucode7_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_bcucode8_fld(x) (int)(((unsigned int)(x) >> 15) & 0x7)
#define kv3_branchcond_fld(x) (int)(((unsigned int)(x) >> 23) & 0xf)
#define kv3_byteshift_fld(x) (int)((unsigned int)(x) & 0x3f)
#define kv3_cachelev_fld(x) (int)(((unsigned int)(x) >> 24) & 0x3)
#define kv3_column_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3)
#define kv3_comparison_fld(x) (int)(((unsigned int)(x) >> 24) & 0xf)
#define kv3_doscale_fld(x) (int)(((unsigned int)(x) >> 12) & 0x1)
#define kv3_extend27_fld(x) (int)((unsigned int)(x) & 0x7ffffff)
#define kv3_extend6_fld(x) (int)((unsigned int)(x) & 0x3f)
#define kv3_exubit0_fld(x) (int)((unsigned int)(x) & 0x1)
#define kv3_exubit1_fld(x) (int)(((unsigned int)(x) >> 1) & 0x1)
#define kv3_exubit11_fld(x) (int)(((unsigned int)(x) >> 11) & 0x1)
#define kv3_exubit12_fld(x) (int)(((unsigned int)(x) >> 12) & 0x1)
#define kv3_exubit18_fld(x) (int)(((unsigned int)(x) >> 18) & 0x1)
#define kv3_exubit27_fld(x) (int)(((unsigned int)(x) >> 27) & 0x1)
#define kv3_exubit28_fld(x) (int)(((unsigned int)(x) >> 28) & 0x1)
#define kv3_exubit6_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1)
#define kv3_exubit7_fld(x) (int)(((unsigned int)(x) >> 7) & 0x1)
#define kv3_exucode2_fld(x) (int)(((unsigned int)(x) >> 24) & 0xf)
#define kv3_exucode4_fld(x) (int)(((unsigned int)(x) >> 16) & 0x3)
#define kv3_exucode5_fld(x) (int)(((unsigned int)(x) >> 12) & 0xf)
#define kv3_exucode6_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3f)
#define kv3_exucode7_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3)
#define kv3_exunum2_fld(x) (int)(((unsigned int)(x) >> 27) & 0x3)
#define kv3_exunum3_fld(x) (int)(((unsigned int)(x) >> 27) & 0x3)
#define kv3_floatcomp_fld(x) (int)(((unsigned int)(x) >> 24) & 0x7)
#define kv3_loadcode_fld(x) (int)(((unsigned int)(x) >> 26) & 0x7)
#define kv3_lower10_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3ff)
#define kv3_lower5_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1f)
#define kv3_lsubit18_fld(x) (int)(((unsigned int)(x) >> 18) & 0x1)
#define kv3_lsubit19_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1)
#define kv3_lsubit25_fld(x) (int)(((unsigned int)(x) >> 25) & 0x1)
#define kv3_lsucode0_fld(x) (int)(((unsigned int)(x) >> 24) & 0x1f)
#define kv3_lsucode1_fld(x) (int)(((unsigned int)(x) >> 26) & 0x7)
#define kv3_lsucode2_fld(x) (int)(((unsigned int)(x) >> 20) & 0xf)
#define kv3_lsucode3_fld(x) (int)(((unsigned int)(x) >> 24) & 0x7)
#define kv3_lsucode4_fld(x) (int)(((unsigned int)(x) >> 16) & 0x3)
#define kv3_lsucode5_fld(x) (int)(((unsigned int)(x) >> 13) & 0x7)
#define kv3_lsucond_fld(x) (int)(((unsigned int)(x) >> 12) & 0xf)
#define kv3_offset27_fld(x) (int)((unsigned int)(x) & 0x7ffffff)
#define kv3_parallel_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_parallel2_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_parallel3_fld(x) (int)(((unsigned int)(x) >> 31) & 0x1)
#define kv3_pcrel17_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ffff)
#define kv3_pcrel27_fld(x) (int)((unsigned int)(x) & 0x7ffffff)
#define kv3_pow2scale_fld(x) (int)((unsigned int)(x) & 0x3f)
#define kv3_rectify_fld(x) (int)(((unsigned int)(x) >> 7) & 0x1)
#define kv3_registerA_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerA0_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerA1_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerA2_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerA3_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerAh_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerAl_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerAp_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerAq_fld(x) (int)(((unsigned int)(x) >> 20) & 0xf)
#define kv3_registerAt_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerAx_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerAy_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerAz_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerB_fld(x) (int)(((unsigned int)(x) >> 12) & 0x3f)
#define kv3_registerBe_fld(x) (int)(((unsigned int)(x) >> 13) & 0x1f)
#define kv3_registerBo_fld(x) (int)(((unsigned int)(x) >> 13) & 0x1f)
#define kv3_registerBp_fld(x) (int)(((unsigned int)(x) >> 13) & 0x1f)
#define kv3_registerBq_fld(x) (int)(((unsigned int)(x) >> 14) & 0xf)
#define kv3_registerC_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3f)
#define kv3_registerCe_fld(x) (int)(((unsigned int)(x) >> 7) & 0x1f)
#define kv3_registerCo_fld(x) (int)(((unsigned int)(x) >> 7) & 0x1f)
#define kv3_registerD_fld(x) (int)((unsigned int)(x) & 0x3f)
#define kv3_registerE_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerM_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerN_fld(x) (int)(((unsigned int)(x) >> 20) & 0xf)
#define kv3_registerO_fld(x) (int)(((unsigned int)(x) >> 7) & 0x1f)
#define kv3_registerP_fld(x) (int)(((unsigned int)(x) >> 1) & 0x1f)
#define kv3_registerR_fld(x) (int)(((unsigned int)(x) >> 2) & 0xf)
#define kv3_registerT_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerU_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerV_fld(x) (int)(((unsigned int)(x) >> 20) & 0xf)
#define kv3_registerW_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3f)
#define kv3_registerY_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3f)
#define kv3_registerZ_fld(x) (int)((unsigned int)(x) & 0x3f)
#define kv3_rounding_fld(x) (int)(((unsigned int)(x) >> 12) & 0x7)
#define kv3_rounding2_fld(x) (int)(((unsigned int)(x) >> 8) & 0x7)
#define kv3_roundint_fld(x) (int)(((unsigned int)(x) >> 8) & 0x7)
#define kv3_saturate_fld(x) (int)(((unsigned int)(x) >> 7) & 0x1)
#define kv3_scalarcond_fld(x) (int)(((unsigned int)(x) >> 24) & 0xf)
#define kv3_signed10_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3ff)
#define kv3_signed16_fld(x) (int)((unsigned int)(x) & 0xffff)
#define kv3_silent_fld(x) (int)(((unsigned int)(x) >> 15) & 0x1)
#define kv3_silent2_fld(x) (int)(((unsigned int)(x) >> 11) & 0x1)
#define kv3_simdcond_fld(x) (int)(((unsigned int)(x) >> 24) & 0x7)
#define kv3_speculate_fld(x) (int)(((unsigned int)(x) >> 24) & 0x1)
#define kv3_splat32_fld(x) (int)(((unsigned int)(x) >> 11) & 0x1)
#define kv3_startbit_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3f)
#define kv3_steerimx_fld(x) (int)(((unsigned int)(x) >> 29) & 0x3)
#define kv3_steerimx3_fld(x) (int)(((unsigned int)(x) >> 29) & 0x3)
#define kv3_steering_fld(x) (int)(((unsigned int)(x) >> 29) & 0x3)
#define kv3_stopbit2_fld(x) (int)(((unsigned int)(x) >> 24) & 0x3)
#define kv3_stopbit4_fld(x) (int)(((unsigned int)(x) >> 12) & 0xf)
#define kv3_sysnumber_fld(x) (int)((unsigned int)(x) & 0xfff)
#define kv3_systemAlone_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_systemRA_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_systemS2_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_systemS3_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_systemS4_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_systemT2_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_systemT3_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_systemT4_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1ff)
#define kv3_tcacode1_fld(x) (int)(((unsigned int)(x) >> 24) & 0x7)
#define kv3_tcacode2_fld(x) (int)(((unsigned int)(x) >> 14) & 0xf)
#define kv3_tcacode3_fld(x) (int)(((unsigned int)(x) >> 18) & 0x3)
#define kv3_tcacode5_fld(x) (int)(((unsigned int)(x) >> 12) & 0x3)
#define kv3_unsigned6_fld(x) (int)(((unsigned int)(x) >> 6) & 0x3f)
#define kv3_unused16_fld(x) (int)((unsigned int)(x) & 0xffff)
#define kv3_unused6_fld(x) (int)((unsigned int)(x) & 0x3f)
#define kv3_upper27_fld(x) (int)((unsigned int)(x) & 0x7ffffff)
#define kv3_variant_fld(x) (int)(((unsigned int)(x) >> 24) & 0x3)
#define kv3_wricode1_fld(x) (int)(((unsigned int)(x) >> 26) & 0x3)
#define kv3_xrounding_fld(x) (int)(((unsigned int)(x) >> 8) & 0x7)
#define kv3_boolcas_fld(x) (int)(((unsigned int)(x) >> 25) & 0x1)
#define kv3_brknumber_fld(x) (int)((unsigned int)(x) & 0x3)
#define kv3_cascode_fld(x) (int)(((unsigned int)(x) >> 24) & 0x1)
#define kv3_channel_fld(x) (int)(((unsigned int)(x) >> 24) & 0x1)
#define kv3_conjugate_fld(x) (int)(((unsigned int)(x) >> 24) & 0x1)
#define kv3_exucode3_fld(x) (int)(((unsigned int)(x) >> 25) & 0x7)
#define kv3_multransl_fld(x) (int)(((unsigned int)(x) >> 6) & 0x1)
#define kv3_multransm_fld(x) (int)(((unsigned int)(x) >> 18) & 0x1)
#define kv3_registerAe_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerAo_fld(x) (int)(((unsigned int)(x) >> 19) & 0x1f)
#define kv3_registerDp_fld(x) (int)(((unsigned int)(x) >> 1) & 0x1f)
#define kv3_registerQ_fld(x) (int)(((unsigned int)(x) >> 8) & 0xf)
#define kv3_tcacode4_fld(x) (int)(((unsigned int)(x) >> 8) & 0xf)
#define kv3_unused4_fld(x) (int)(((unsigned int)(x) >> 2) & 0xf)

#define kv3_registerN_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerN_0_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerN_1_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerN_2_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerN_3_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerBe_opd(w) (((unsigned int)(w) >> 13) & 0x1f)
#define kv3_registerCo_opd(w) (((unsigned int)(w) >> 7) & 0x1f)
#define kv3_byteshift_opd(w) ((unsigned int)(w) & 0x3f)
#define kv3_registerZ_opd(w) ((unsigned int)(w) & 0x3f)
#define kv3_registerBo_opd(w) (((unsigned int)(w) >> 13) & 0x1f)
#define kv3_registerCe_opd(w) (((unsigned int)(w) >> 7) & 0x1f)
#define kv3_registerA_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerA_0_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerA_1_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerA_2_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerA_3_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerAp_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerAp_0_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerAp_1_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerBp_opd(w) (((unsigned int)(w) >> 13) & 0x1f)
#define kv3_registerBp_0_opd(w) (((unsigned int)(w) >> 13) & 0x1f)
#define kv3_registerBp_1_opd(w) (((unsigned int)(w) >> 13) & 0x1f)
#define kv3_registerC_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_registerC_0_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_registerC_1_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_registerC_2_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_registerC_3_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_registerD_opd(w) ((unsigned int)(w) & 0x3f)
#define kv3_registerD_0_opd(w) ((unsigned int)(w) & 0x3f)
#define kv3_registerD_1_opd(w) ((unsigned int)(w) & 0x3f)
#define kv3_registerD_2_opd(w) ((unsigned int)(w) & 0x3f)
#define kv3_registerD_3_opd(w) ((unsigned int)(w) & 0x3f)
#define kv3_registerA0_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerA1_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerA2_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerA3_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerAq_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerAq_0_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerAq_1_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerAq_2_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerAq_3_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerBq_opd(w) (((unsigned int)(w) >> 14) & 0xf)
#define kv3_registerBq_0_opd(w) (((unsigned int)(w) >> 14) & 0xf)
#define kv3_registerBq_1_opd(w) (((unsigned int)(w) >> 14) & 0xf)
#define kv3_registerBq_2_opd(w) (((unsigned int)(w) >> 14) & 0xf)
#define kv3_registerBq_3_opd(w) (((unsigned int)(w) >> 14) & 0xf)
#define kv3_registerAl_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_roundint_opd(w) (((unsigned int)(w) >> 8) & 0x7)
#define kv3_saturate_opd(w) (((unsigned int)(w) >> 7) & 0x1)
#define kv3_registerAh_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerAx_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerAy_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerAz_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerAt_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerB_opd(w) (((unsigned int)(w) >> 12) & 0x3f)
#define kv3_registerB_0_opd(w) (((unsigned int)(w) >> 12) & 0x3f)
#define kv3_registerB_1_opd(w) (((unsigned int)(w) >> 12) & 0x3f)
#define kv3_registerB_2_opd(w) (((unsigned int)(w) >> 12) & 0x3f)
#define kv3_registerB_3_opd(w) (((unsigned int)(w) >> 12) & 0x3f)
#define kv3_xrounding_opd(w) (((unsigned int)(w) >> 8) & 0x7)
#define kv3_silent2_opd(w) (((unsigned int)(w) >> 11) & 0x1)
#define kv3_rectify_opd(w) (((unsigned int)(w) >> 7) & 0x1)
#define kv3_branchcond_opd(w) (((unsigned int)(w) >> 23) & 0xf)
#define kv3_pcrel17_opd(w) (((unsigned int)(w) >> 6) & 0x1ffff)
#define kv3_systemT2_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_systemAlone_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_systemT4_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_systemT3_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_systemRA_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_systemS2_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_systemS3_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_systemS4_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_sysnumber_opd(w) ((unsigned int)(w) & 0xfff)
#define kv3_pcrel27_opd(w) ((unsigned int)(w) & 0x7ffffff)
#define kv3_registerW_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_signed10_opd(w) (((unsigned int)(w) >> 6) & 0x3ff)
#define kv3_variant_opd(w) (((unsigned int)(w) >> 24) & 0x3)
#define kv3_upper27_lower10_opd(w0,w1) ((((unsigned int)(w1) & 0x7ffffff) << 10) | (((unsigned int)(w0) >> 6) & 0x3ff))
#define kv3_extend27_upper27_lower10_opd(w0,w1,w2) ((((((unsigned int)(w2) & 0x7ffffff) << 27) | ((unsigned int)(w1) & 0x7ffffff)) << 10) | (((unsigned int)(w0) >> 6) & 0x3ff))
#define kv3_registerM_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerM_0_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerM_1_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_speculate_opd(w) (((unsigned int)(w) >> 24) & 0x1)
#define kv3_column_opd(w) (((unsigned int)(w) >> 18) & 0x3)
#define kv3_registerT_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerU_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerU_0_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerU_1_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerV_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerV_0_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerV_1_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerV_2_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerV_3_opd(w) (((unsigned int)(w) >> 20) & 0xf)
#define kv3_registerE_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerE_0_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerE_1_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerE_2_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerE_3_opd(w) (((unsigned int)(w) >> 18) & 0x3f)
#define kv3_registerR_opd(w) (((unsigned int)(w) >> 2) & 0xf)
#define kv3_registerR_0_opd(w) (((unsigned int)(w) >> 2) & 0xf)
#define kv3_registerR_1_opd(w) (((unsigned int)(w) >> 2) & 0xf)
#define kv3_registerR_2_opd(w) (((unsigned int)(w) >> 2) & 0xf)
#define kv3_registerR_3_opd(w) (((unsigned int)(w) >> 2) & 0xf)
#define kv3_registerY_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_lsucond_opd(w) (((unsigned int)(w) >> 12) & 0xf)
#define kv3_offset27_opd(w) ((unsigned int)(w) & 0x7ffffff)
#define kv3_extend27_offset27_opd(w0,w1) ((((unsigned int)(w1) & 0x7ffffff) << 27) | ((unsigned int)(w0) & 0x7ffffff))
#define kv3_doscale_opd(w) (((unsigned int)(w) >> 12) & 0x1)
#define kv3_signed16_opd(w) ((unsigned int)(w) & 0xffff)
#define kv3_extend6_upper27_lower10_opd(w0,w1,w2) ((((((unsigned int)(w2) & 0x3f) << 27) | ((unsigned int)(w1) & 0x7ffffff)) << 10) | (((unsigned int)(w0) >> 6) & 0x3ff))
#define kv3_comparison_opd(w) (((unsigned int)(w) >> 24) & 0xf)
#define kv3_scalarcond_opd(w) (((unsigned int)(w) >> 24) & 0xf)
#define kv3_stopbit2_stopbit4_opd(w0,w1) (((((unsigned int)(w1) >> 24) & 0x3) << 4) | (((unsigned int)(w0) >> 12) & 0xf))
#define kv3_startbit_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_upper27_lower5_opd(w0,w1) ((((unsigned int)(w1) & 0x7ffffff) << 5) | (((unsigned int)(w0) >> 6) & 0x1f))
#define kv3_splat32_opd(w) (((unsigned int)(w) >> 11) & 0x1)
#define kv3_simdcond_opd(w) (((unsigned int)(w) >> 24) & 0x7)
#define kv3_unsigned6_opd(w) (((unsigned int)(w) >> 6) & 0x3f)
#define kv3_floatcomp_opd(w) (((unsigned int)(w) >> 24) & 0x7)
#define kv3_registerP_opd(w) (((unsigned int)(w) >> 1) & 0x1f)
#define kv3_registerP_0_opd(w) (((unsigned int)(w) >> 1) & 0x1f)
#define kv3_registerP_1_opd(w) (((unsigned int)(w) >> 1) & 0x1f)
#define kv3_rounding2_opd(w) (((unsigned int)(w) >> 8) & 0x7)
#define kv3_rounding_opd(w) (((unsigned int)(w) >> 12) & 0x7)
#define kv3_silent_opd(w) (((unsigned int)(w) >> 15) & 0x1)
#define kv3_registerO_opd(w) (((unsigned int)(w) >> 7) & 0x1f)
#define kv3_registerO_0_opd(w) (((unsigned int)(w) >> 7) & 0x1f)
#define kv3_registerO_1_opd(w) (((unsigned int)(w) >> 7) & 0x1f)
#define kv3_brknumber_opd(w) ((unsigned int)(w) & 0x3)
#define kv3_registerDp_opd(w) (((unsigned int)(w) >> 1) & 0x1f)
#define kv3_registerDp_0_opd(w) (((unsigned int)(w) >> 1) & 0x1f)
#define kv3_registerDp_1_opd(w) (((unsigned int)(w) >> 1) & 0x1f)
#define kv3_registerAe_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define kv3_registerAo_opd(w) (((unsigned int)(w) >> 19) & 0x1f)
#define v2_systemT2_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define v2_systemAlone_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define v2_systemT4_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define v2_systemT3_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define v2_systemS2_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define v2_systemS3_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define v2_systemS4_opd(w) (((unsigned int)(w) >> 6) & 0x1ff)
#define kv3_cachelev_opd(w) (((unsigned int)(w) >> 24) & 0x3)
#define kv3_boolcas_opd(w) (((unsigned int)(w) >> 25) & 0x1)
#define kv3_channel_opd(w) (((unsigned int)(w) >> 24) & 0x1)
#define kv3_conjugate_opd(w) (((unsigned int)(w) >> 24) & 0x1)
#define kv3_multransm_multransl_opd(w0,w1) (((((unsigned int)(w1) >> 18) & 0x1) << 1) | (((unsigned int)(w0) >> 6) & 0x1))
#define kv3_registerQ_opd(w) (((unsigned int)(w) >> 8) & 0xf)
#define kv3_registerQ_0_opd(w) (((unsigned int)(w) >> 8) & 0xf)
#define kv3_registerQ_1_opd(w) (((unsigned int)(w) >> 8) & 0xf)
#define kv3_registerQ_2_opd(w) (((unsigned int)(w) >> 8) & 0xf)
#define kv3_registerQ_3_opd(w) (((unsigned int)(w) >> 8) & 0xf)

#define is_v1_AWAIT_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fa00000)
#define is_v1_SLEEP_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fa40000)
#define is_v1_STOP_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fa80000)
#define is_v1_BARRIER_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fac0000)
#define is_v1_TLBREAD_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f800000)
#define is_v1_TLBPROBE_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f840000)
#define is_v1_TLBWRITE_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f880000)
#define is_v1_TLBDINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f8c0000)
#define is_v1_TLBIINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f900000)
#define is_v1_ERROP_simple_insn(x) (int)(((unsigned int)(x) & 0x7fffffc0) == 0x00000000)
#define is_v1_RET_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fd00000)
#define is_v1_RFE_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fd40000)
#define v1_IGOTO_registerZ_simple_insn(registerZ) (unsigned int)(0x0fd80000 | ((int)(registerZ) & 0x3f))
#define is_v1_IGOTO_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fd80000)
#define v1_ICALL_registerZ_simple_insn(registerZ) (unsigned int)(0x0fdc0000 | ((int)(registerZ) & 0x3f))
#define is_v1_ICALL_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fdc0000)
#define v1_SCALL_registerZ_simple_insn(registerZ) (unsigned int)(0x0fe40000 | ((int)(registerZ) & 0x3f))
#define is_v1_SCALL_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fe40000)
#define v1_SCALL_sysnumber_simple_insn(sysnumber) (unsigned int)(0x0fe00000 | ((int)(sysnumber) & 0xfff))
#define is_v1_SCALL_sysnumber_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fe00000)
#define v1_LOOPDO_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0f000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define is_v1_LOOPDO_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0f000000)
#define v1_WFXL_systemT2_registerZ_simple_insn(systemT2,registerZ) (unsigned int)(0x0fb80000 | (((int)(systemT2) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_WFXL_systemT2_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb80000)
#define v1_WFXL_systemAlone_registerZ_simple_insn(systemAlone,registerZ) (unsigned int)(0x0fb80000 | (((int)(systemAlone) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_WFXL_systemAlone_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb80000)
#define v1_WFXL_systemT4_registerZ_simple_insn(systemT4,registerZ) (unsigned int)(0x0fb80000 | (((int)(systemT4) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_WFXL_systemT4_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb80000)
#define v1_WFXM_systemT2_registerZ_simple_insn(systemT2,registerZ) (unsigned int)(0x0fbc0000 | (((int)(systemT2) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_WFXM_systemT2_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fbc0000)
#define v1_WFXM_systemAlone_registerZ_simple_insn(systemAlone,registerZ) (unsigned int)(0x0fbc0000 | (((int)(systemAlone) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_WFXM_systemAlone_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fbc0000)
#define v1_WFXM_systemT4_registerZ_simple_insn(systemT4,registerZ) (unsigned int)(0x0fbc0000 | (((int)(systemT4) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_WFXM_systemT4_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fbc0000)
#define v1_GET_registerZ_systemS2_simple_insn(registerZ,systemS2) (unsigned int)(0x0fc40000 | ((int)(registerZ) & 0x3f) | (((int)(systemS2) & 0x1ff) << 6))
#define is_v1_GET_registerZ_systemS2_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc40000)
#define v1_GET_registerZ_systemS3_simple_insn(registerZ,systemS3) (unsigned int)(0x0fc40000 | ((int)(registerZ) & 0x3f) | (((int)(systemS3) & 0x1ff) << 6))
#define is_v1_GET_registerZ_systemS3_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc40000)
#define v1_IGET_registerZ_simple_insn(registerZ) (unsigned int)(0x0fcc0000 | ((int)(registerZ) & 0x3f))
#define is_v1_IGET_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fcc0000)
#define v1_SET_systemT3_registerZ_simple_insn(systemT3,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemT3) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_SET_systemT3_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v1_SET_systemAlone_registerZ_simple_insn(systemAlone,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemAlone) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_SET_systemAlone_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v1_SET_systemRA_registerZ_simple_insn(systemRA,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemRA) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_SET_systemRA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v1_SET_systemT4_registerZ_simple_insn(systemT4,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemT4) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_SET_systemT4_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v1_RSWAP_registerZ_systemS4_simple_insn(registerZ,systemS4) (unsigned int)(0x0fc80000 | ((int)(registerZ) & 0x3f) | (((int)(systemS4) & 0x1ff) << 6))
#define is_v1_RSWAP_registerZ_systemS4_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc80000)
#define v1_RSWAP_registerZ_systemAlone_simple_insn(registerZ,systemAlone) (unsigned int)(0x0fc80000 | ((int)(registerZ) & 0x3f) | (((int)(systemAlone) & 0x1ff) << 6))
#define is_v1_RSWAP_registerZ_systemAlone_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc80000)
#define v1_RSWAP_registerZ_systemS3_simple_insn(registerZ,systemS3) (unsigned int)(0x0fc80000 | ((int)(registerZ) & 0x3f) | (((int)(systemS3) & 0x1ff) << 6))
#define is_v1_RSWAP_registerZ_systemS3_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc80000)
#define v1_WAITIT_registerZ_simple_insn(registerZ) (unsigned int)(0x0fb00000 | ((int)(registerZ) & 0x3f))
#define is_v1_WAITIT_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb00000)
#define v1_SYNCGROUP_registerZ_simple_insn(registerZ) (unsigned int)(0x0fb40000 | ((int)(registerZ) & 0x3f))
#define is_v1_SYNCGROUP_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb40000)
#define v1_GOTO_pcrel27_simple_insn(pcrel27) (unsigned int)(0x10000000 | (((int)(pcrel27) >> 2) & 0x7ffffff))
#define is_v1_GOTO_pcrel27_simple_insn(x) (int)(((unsigned int)(x) & 0x78000000) == 0x10000000)
#define v1_CALL_pcrel27_simple_insn(pcrel27) (unsigned int)(0x18000000 | (((int)(pcrel27) >> 2) & 0x7ffffff))
#define is_v1_CALL_pcrel27_simple_insn(x) (int)(((unsigned int)(x) & 0x78000000) == 0x18000000)
#define v1_CB_deqz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x08800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_dgez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x09800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_dgtz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0a800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_dlez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0a000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_dltz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x09000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_dnez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x08000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_even_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0b800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_odd_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0b000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_weqz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0c800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_wgez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0d800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_wgtz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0e800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_wlez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0e000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_wltz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0d000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v1_CB_wnez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0c000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define is_v1_CB_deqz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x08800000)
#define is_v1_CB_dgez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x09800000)
#define is_v1_CB_dgtz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0a800000)
#define is_v1_CB_dlez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0a000000)
#define is_v1_CB_dltz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x09000000)
#define is_v1_CB_dnez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x08000000)
#define is_v1_CB_even_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0b800000)
#define is_v1_CB_odd_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0b000000)
#define is_v1_CB_weqz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0c800000)
#define is_v1_CB_wgez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0d800000)
#define is_v1_CB_wgtz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0e800000)
#define is_v1_CB_wlez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0e000000)
#define is_v1_CB_wltz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0d000000)
#define is_v1_CB_wnez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0c000000)
#define v1_LBZ__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x20000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x21000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x22000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x23000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LBZ__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x20000000)
#define is_v1_LBZ_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x21000000)
#define is_v1_LBZ_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x22000000)
#define is_v1_LBZ_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x23000000)
#define v1_LBZ__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa0000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LBZ_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa1000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LBZ_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa2000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LBZ_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa3000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LBZ__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0000000)
#define is_v1_LBZ_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1000000)
#define is_v1_LBZ_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa2000000)
#define is_v1_LBZ_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa3000000)
#define v1_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa1000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0000000)
#define is_v1_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1000000)
#define is_v1_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa2000000)
#define is_v1_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa3000000)
#define v1_LBZ_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LBZ_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21021000)
#define is_v1_LBZ_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21023000)
#define is_v1_LBZ_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21025000)
#define is_v1_LBZ_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21024000)
#define is_v1_LBZ_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21022000)
#define is_v1_LBZ_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21020000)
#define is_v1_LBZ_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21027000)
#define is_v1_LBZ_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21026000)
#define is_v1_LBZ_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21029000)
#define is_v1_LBZ_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102b000)
#define is_v1_LBZ_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102d000)
#define is_v1_LBZ_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102c000)
#define is_v1_LBZ_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102a000)
#define is_v1_LBZ_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21028000)
#define is_v1_LBZ_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23021000)
#define is_v1_LBZ_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23023000)
#define is_v1_LBZ_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23025000)
#define is_v1_LBZ_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23024000)
#define is_v1_LBZ_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23022000)
#define is_v1_LBZ_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23020000)
#define is_v1_LBZ_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23027000)
#define is_v1_LBZ_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23026000)
#define is_v1_LBZ_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23029000)
#define is_v1_LBZ_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302b000)
#define is_v1_LBZ_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302d000)
#define is_v1_LBZ_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302c000)
#define is_v1_LBZ_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302a000)
#define is_v1_LBZ_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23028000)
#define is_v1_LBZ_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22021000)
#define is_v1_LBZ_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22023000)
#define is_v1_LBZ_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22025000)
#define is_v1_LBZ_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22024000)
#define is_v1_LBZ_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22022000)
#define is_v1_LBZ_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22020000)
#define is_v1_LBZ_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22027000)
#define is_v1_LBZ_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22026000)
#define is_v1_LBZ_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22029000)
#define is_v1_LBZ_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202b000)
#define is_v1_LBZ_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202d000)
#define is_v1_LBZ_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202c000)
#define is_v1_LBZ_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202a000)
#define is_v1_LBZ_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22028000)
#define is_v1_LBZ__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20021000)
#define is_v1_LBZ__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20023000)
#define is_v1_LBZ__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20025000)
#define is_v1_LBZ__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20024000)
#define is_v1_LBZ__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20022000)
#define is_v1_LBZ__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20020000)
#define is_v1_LBZ__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20027000)
#define is_v1_LBZ__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20026000)
#define is_v1_LBZ__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20029000)
#define is_v1_LBZ__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002b000)
#define is_v1_LBZ__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002d000)
#define is_v1_LBZ__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002c000)
#define is_v1_LBZ__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002a000)
#define is_v1_LBZ__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20028000)
#define v1_LBZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LBZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1021000)
#define is_v1_LBZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1023000)
#define is_v1_LBZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1025000)
#define is_v1_LBZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1024000)
#define is_v1_LBZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1022000)
#define is_v1_LBZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1020000)
#define is_v1_LBZ_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1027000)
#define is_v1_LBZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1026000)
#define is_v1_LBZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1029000)
#define is_v1_LBZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102b000)
#define is_v1_LBZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102d000)
#define is_v1_LBZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102c000)
#define is_v1_LBZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102a000)
#define is_v1_LBZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1028000)
#define is_v1_LBZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3021000)
#define is_v1_LBZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3023000)
#define is_v1_LBZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3025000)
#define is_v1_LBZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3024000)
#define is_v1_LBZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3022000)
#define is_v1_LBZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3020000)
#define is_v1_LBZ_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3027000)
#define is_v1_LBZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3026000)
#define is_v1_LBZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3029000)
#define is_v1_LBZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302b000)
#define is_v1_LBZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302d000)
#define is_v1_LBZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302c000)
#define is_v1_LBZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302a000)
#define is_v1_LBZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3028000)
#define is_v1_LBZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2021000)
#define is_v1_LBZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2023000)
#define is_v1_LBZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2025000)
#define is_v1_LBZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2024000)
#define is_v1_LBZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2022000)
#define is_v1_LBZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2020000)
#define is_v1_LBZ_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2027000)
#define is_v1_LBZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2026000)
#define is_v1_LBZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2029000)
#define is_v1_LBZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202b000)
#define is_v1_LBZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202d000)
#define is_v1_LBZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202c000)
#define is_v1_LBZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202a000)
#define is_v1_LBZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2028000)
#define is_v1_LBZ__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0021000)
#define is_v1_LBZ__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0023000)
#define is_v1_LBZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0025000)
#define is_v1_LBZ__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0024000)
#define is_v1_LBZ__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0022000)
#define is_v1_LBZ__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0020000)
#define is_v1_LBZ__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0027000)
#define is_v1_LBZ__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0026000)
#define is_v1_LBZ__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0029000)
#define is_v1_LBZ__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002b000)
#define is_v1_LBZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002d000)
#define is_v1_LBZ__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002c000)
#define is_v1_LBZ__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002a000)
#define is_v1_LBZ__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0028000)
#define v1_LBZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LBZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1021000)
#define is_v1_LBZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1023000)
#define is_v1_LBZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1025000)
#define is_v1_LBZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1024000)
#define is_v1_LBZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1022000)
#define is_v1_LBZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1020000)
#define is_v1_LBZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1027000)
#define is_v1_LBZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1026000)
#define is_v1_LBZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1029000)
#define is_v1_LBZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102b000)
#define is_v1_LBZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102d000)
#define is_v1_LBZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102c000)
#define is_v1_LBZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102a000)
#define is_v1_LBZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1028000)
#define is_v1_LBZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3021000)
#define is_v1_LBZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3023000)
#define is_v1_LBZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3025000)
#define is_v1_LBZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3024000)
#define is_v1_LBZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3022000)
#define is_v1_LBZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3020000)
#define is_v1_LBZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3027000)
#define is_v1_LBZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3026000)
#define is_v1_LBZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3029000)
#define is_v1_LBZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302b000)
#define is_v1_LBZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302d000)
#define is_v1_LBZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302c000)
#define is_v1_LBZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302a000)
#define is_v1_LBZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3028000)
#define is_v1_LBZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2021000)
#define is_v1_LBZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2023000)
#define is_v1_LBZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2025000)
#define is_v1_LBZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2024000)
#define is_v1_LBZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2022000)
#define is_v1_LBZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2020000)
#define is_v1_LBZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2027000)
#define is_v1_LBZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2026000)
#define is_v1_LBZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2029000)
#define is_v1_LBZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202b000)
#define is_v1_LBZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202d000)
#define is_v1_LBZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202c000)
#define is_v1_LBZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202a000)
#define is_v1_LBZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2028000)
#define is_v1_LBZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0021000)
#define is_v1_LBZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0023000)
#define is_v1_LBZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0025000)
#define is_v1_LBZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0024000)
#define is_v1_LBZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0022000)
#define is_v1_LBZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0020000)
#define is_v1_LBZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0027000)
#define is_v1_LBZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0026000)
#define is_v1_LBZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0029000)
#define is_v1_LBZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002b000)
#define is_v1_LBZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002d000)
#define is_v1_LBZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002c000)
#define is_v1_LBZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002a000)
#define is_v1_LBZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0028000)
#define v1_LBZ_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2102e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2102f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2302e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2302f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2202e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2202f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2002e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBZ__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2002f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LBZ_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102e000)
#define is_v1_LBZ_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102f000)
#define is_v1_LBZ_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302e000)
#define is_v1_LBZ_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302f000)
#define is_v1_LBZ_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202e000)
#define is_v1_LBZ_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202f000)
#define is_v1_LBZ___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002e000)
#define is_v1_LBZ__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002f000)
#define v1_LBS__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x24000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x25000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x26000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x27000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LBS__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x24000000)
#define is_v1_LBS_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x25000000)
#define is_v1_LBS_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x26000000)
#define is_v1_LBS_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x27000000)
#define v1_LBS__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa4000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LBS_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa5000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LBS_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa6000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LBS_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa7000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LBS__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4000000)
#define is_v1_LBS_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5000000)
#define is_v1_LBS_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6000000)
#define is_v1_LBS_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7000000)
#define v1_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa4000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa5000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa6000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa7000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4000000)
#define is_v1_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5000000)
#define is_v1_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6000000)
#define is_v1_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7000000)
#define v1_LBS_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LBS_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25021000)
#define is_v1_LBS_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25023000)
#define is_v1_LBS_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25025000)
#define is_v1_LBS_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25024000)
#define is_v1_LBS_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25022000)
#define is_v1_LBS_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25020000)
#define is_v1_LBS_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25027000)
#define is_v1_LBS_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25026000)
#define is_v1_LBS_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25029000)
#define is_v1_LBS_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502b000)
#define is_v1_LBS_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502d000)
#define is_v1_LBS_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502c000)
#define is_v1_LBS_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502a000)
#define is_v1_LBS_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25028000)
#define is_v1_LBS_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27021000)
#define is_v1_LBS_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27023000)
#define is_v1_LBS_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27025000)
#define is_v1_LBS_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27024000)
#define is_v1_LBS_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27022000)
#define is_v1_LBS_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27020000)
#define is_v1_LBS_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27027000)
#define is_v1_LBS_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27026000)
#define is_v1_LBS_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27029000)
#define is_v1_LBS_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702b000)
#define is_v1_LBS_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702d000)
#define is_v1_LBS_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702c000)
#define is_v1_LBS_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702a000)
#define is_v1_LBS_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27028000)
#define is_v1_LBS_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26021000)
#define is_v1_LBS_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26023000)
#define is_v1_LBS_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26025000)
#define is_v1_LBS_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26024000)
#define is_v1_LBS_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26022000)
#define is_v1_LBS_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26020000)
#define is_v1_LBS_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26027000)
#define is_v1_LBS_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26026000)
#define is_v1_LBS_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26029000)
#define is_v1_LBS_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602b000)
#define is_v1_LBS_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602d000)
#define is_v1_LBS_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602c000)
#define is_v1_LBS_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602a000)
#define is_v1_LBS_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26028000)
#define is_v1_LBS__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24021000)
#define is_v1_LBS__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24023000)
#define is_v1_LBS__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24025000)
#define is_v1_LBS__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24024000)
#define is_v1_LBS__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24022000)
#define is_v1_LBS__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24020000)
#define is_v1_LBS__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24027000)
#define is_v1_LBS__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24026000)
#define is_v1_LBS__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24029000)
#define is_v1_LBS__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402b000)
#define is_v1_LBS__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402d000)
#define is_v1_LBS__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402c000)
#define is_v1_LBS__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402a000)
#define is_v1_LBS__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24028000)
#define v1_LBS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LBS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5021000)
#define is_v1_LBS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5023000)
#define is_v1_LBS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5025000)
#define is_v1_LBS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5024000)
#define is_v1_LBS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5022000)
#define is_v1_LBS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5020000)
#define is_v1_LBS_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5027000)
#define is_v1_LBS_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5026000)
#define is_v1_LBS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5029000)
#define is_v1_LBS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502b000)
#define is_v1_LBS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502d000)
#define is_v1_LBS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502c000)
#define is_v1_LBS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502a000)
#define is_v1_LBS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5028000)
#define is_v1_LBS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7021000)
#define is_v1_LBS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7023000)
#define is_v1_LBS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7025000)
#define is_v1_LBS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7024000)
#define is_v1_LBS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7022000)
#define is_v1_LBS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7020000)
#define is_v1_LBS_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7027000)
#define is_v1_LBS_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7026000)
#define is_v1_LBS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7029000)
#define is_v1_LBS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702b000)
#define is_v1_LBS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702d000)
#define is_v1_LBS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702c000)
#define is_v1_LBS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702a000)
#define is_v1_LBS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7028000)
#define is_v1_LBS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6021000)
#define is_v1_LBS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6023000)
#define is_v1_LBS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6025000)
#define is_v1_LBS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6024000)
#define is_v1_LBS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6022000)
#define is_v1_LBS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6020000)
#define is_v1_LBS_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6027000)
#define is_v1_LBS_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6026000)
#define is_v1_LBS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6029000)
#define is_v1_LBS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602b000)
#define is_v1_LBS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602d000)
#define is_v1_LBS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602c000)
#define is_v1_LBS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602a000)
#define is_v1_LBS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6028000)
#define is_v1_LBS__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4021000)
#define is_v1_LBS__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4023000)
#define is_v1_LBS__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4025000)
#define is_v1_LBS__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4024000)
#define is_v1_LBS__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4022000)
#define is_v1_LBS__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4020000)
#define is_v1_LBS__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4027000)
#define is_v1_LBS__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4026000)
#define is_v1_LBS__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4029000)
#define is_v1_LBS__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402b000)
#define is_v1_LBS__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402d000)
#define is_v1_LBS__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402c000)
#define is_v1_LBS__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402a000)
#define is_v1_LBS__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4028000)
#define v1_LBS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LBS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LBS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LBS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5021000)
#define is_v1_LBS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5023000)
#define is_v1_LBS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5025000)
#define is_v1_LBS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5024000)
#define is_v1_LBS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5022000)
#define is_v1_LBS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5020000)
#define is_v1_LBS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5027000)
#define is_v1_LBS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5026000)
#define is_v1_LBS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5029000)
#define is_v1_LBS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502b000)
#define is_v1_LBS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502d000)
#define is_v1_LBS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502c000)
#define is_v1_LBS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502a000)
#define is_v1_LBS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5028000)
#define is_v1_LBS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7021000)
#define is_v1_LBS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7023000)
#define is_v1_LBS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7025000)
#define is_v1_LBS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7024000)
#define is_v1_LBS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7022000)
#define is_v1_LBS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7020000)
#define is_v1_LBS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7027000)
#define is_v1_LBS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7026000)
#define is_v1_LBS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7029000)
#define is_v1_LBS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702b000)
#define is_v1_LBS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702d000)
#define is_v1_LBS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702c000)
#define is_v1_LBS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702a000)
#define is_v1_LBS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7028000)
#define is_v1_LBS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6021000)
#define is_v1_LBS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6023000)
#define is_v1_LBS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6025000)
#define is_v1_LBS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6024000)
#define is_v1_LBS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6022000)
#define is_v1_LBS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6020000)
#define is_v1_LBS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6027000)
#define is_v1_LBS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6026000)
#define is_v1_LBS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6029000)
#define is_v1_LBS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602b000)
#define is_v1_LBS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602d000)
#define is_v1_LBS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602c000)
#define is_v1_LBS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602a000)
#define is_v1_LBS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6028000)
#define is_v1_LBS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4021000)
#define is_v1_LBS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4023000)
#define is_v1_LBS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4025000)
#define is_v1_LBS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4024000)
#define is_v1_LBS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4022000)
#define is_v1_LBS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4020000)
#define is_v1_LBS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4027000)
#define is_v1_LBS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4026000)
#define is_v1_LBS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4029000)
#define is_v1_LBS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402b000)
#define is_v1_LBS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402d000)
#define is_v1_LBS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402c000)
#define is_v1_LBS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402a000)
#define is_v1_LBS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4028000)
#define v1_LBS_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2502e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2502f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2702e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2702f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2602e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2602f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2402e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LBS__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2402f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LBS_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502e000)
#define is_v1_LBS_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502f000)
#define is_v1_LBS_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702e000)
#define is_v1_LBS_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702f000)
#define is_v1_LBS_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602e000)
#define is_v1_LBS_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602f000)
#define is_v1_LBS___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402e000)
#define is_v1_LBS__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402f000)
#define v1_LHZ__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x28000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x29000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2a000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2b000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LHZ__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x28000000)
#define is_v1_LHZ_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x29000000)
#define is_v1_LHZ_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2a000000)
#define is_v1_LHZ_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2b000000)
#define v1_LHZ__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa8000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LHZ_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa9000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LHZ_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xaa000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LHZ_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xab000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LHZ__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa8000000)
#define is_v1_LHZ_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9000000)
#define is_v1_LHZ_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa000000)
#define is_v1_LHZ_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab000000)
#define v1_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa8000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa9000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xaa000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xab000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa8000000)
#define is_v1_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9000000)
#define is_v1_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa000000)
#define is_v1_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab000000)
#define v1_LHZ_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LHZ_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29021000)
#define is_v1_LHZ_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29023000)
#define is_v1_LHZ_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29025000)
#define is_v1_LHZ_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29024000)
#define is_v1_LHZ_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29022000)
#define is_v1_LHZ_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29020000)
#define is_v1_LHZ_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29027000)
#define is_v1_LHZ_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29026000)
#define is_v1_LHZ_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29029000)
#define is_v1_LHZ_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902b000)
#define is_v1_LHZ_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902d000)
#define is_v1_LHZ_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902c000)
#define is_v1_LHZ_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902a000)
#define is_v1_LHZ_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29028000)
#define is_v1_LHZ_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b021000)
#define is_v1_LHZ_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b023000)
#define is_v1_LHZ_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b025000)
#define is_v1_LHZ_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b024000)
#define is_v1_LHZ_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b022000)
#define is_v1_LHZ_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b020000)
#define is_v1_LHZ_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b027000)
#define is_v1_LHZ_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b026000)
#define is_v1_LHZ_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b029000)
#define is_v1_LHZ_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02b000)
#define is_v1_LHZ_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02d000)
#define is_v1_LHZ_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02c000)
#define is_v1_LHZ_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02a000)
#define is_v1_LHZ_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b028000)
#define is_v1_LHZ_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a021000)
#define is_v1_LHZ_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a023000)
#define is_v1_LHZ_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a025000)
#define is_v1_LHZ_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a024000)
#define is_v1_LHZ_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a022000)
#define is_v1_LHZ_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a020000)
#define is_v1_LHZ_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a027000)
#define is_v1_LHZ_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a026000)
#define is_v1_LHZ_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a029000)
#define is_v1_LHZ_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02b000)
#define is_v1_LHZ_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02d000)
#define is_v1_LHZ_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02c000)
#define is_v1_LHZ_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02a000)
#define is_v1_LHZ_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a028000)
#define is_v1_LHZ__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28021000)
#define is_v1_LHZ__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28023000)
#define is_v1_LHZ__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28025000)
#define is_v1_LHZ__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28024000)
#define is_v1_LHZ__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28022000)
#define is_v1_LHZ__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28020000)
#define is_v1_LHZ__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28027000)
#define is_v1_LHZ__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28026000)
#define is_v1_LHZ__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28029000)
#define is_v1_LHZ__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802b000)
#define is_v1_LHZ__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802d000)
#define is_v1_LHZ__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802c000)
#define is_v1_LHZ__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802a000)
#define is_v1_LHZ__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28028000)
#define v1_LHZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LHZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9021000)
#define is_v1_LHZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9023000)
#define is_v1_LHZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9025000)
#define is_v1_LHZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9024000)
#define is_v1_LHZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9022000)
#define is_v1_LHZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9020000)
#define is_v1_LHZ_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9027000)
#define is_v1_LHZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9026000)
#define is_v1_LHZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9029000)
#define is_v1_LHZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902b000)
#define is_v1_LHZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902d000)
#define is_v1_LHZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902c000)
#define is_v1_LHZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902a000)
#define is_v1_LHZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9028000)
#define is_v1_LHZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab021000)
#define is_v1_LHZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab023000)
#define is_v1_LHZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab025000)
#define is_v1_LHZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab024000)
#define is_v1_LHZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab022000)
#define is_v1_LHZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab020000)
#define is_v1_LHZ_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab027000)
#define is_v1_LHZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab026000)
#define is_v1_LHZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab029000)
#define is_v1_LHZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02b000)
#define is_v1_LHZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02d000)
#define is_v1_LHZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02c000)
#define is_v1_LHZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02a000)
#define is_v1_LHZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab028000)
#define is_v1_LHZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa021000)
#define is_v1_LHZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa023000)
#define is_v1_LHZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa025000)
#define is_v1_LHZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa024000)
#define is_v1_LHZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa022000)
#define is_v1_LHZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa020000)
#define is_v1_LHZ_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa027000)
#define is_v1_LHZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa026000)
#define is_v1_LHZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa029000)
#define is_v1_LHZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02b000)
#define is_v1_LHZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02d000)
#define is_v1_LHZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02c000)
#define is_v1_LHZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02a000)
#define is_v1_LHZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa028000)
#define is_v1_LHZ__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8021000)
#define is_v1_LHZ__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8023000)
#define is_v1_LHZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8025000)
#define is_v1_LHZ__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8024000)
#define is_v1_LHZ__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8022000)
#define is_v1_LHZ__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8020000)
#define is_v1_LHZ__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8027000)
#define is_v1_LHZ__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8026000)
#define is_v1_LHZ__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8029000)
#define is_v1_LHZ__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802b000)
#define is_v1_LHZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802d000)
#define is_v1_LHZ__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802c000)
#define is_v1_LHZ__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802a000)
#define is_v1_LHZ__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8028000)
#define v1_LHZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LHZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9021000)
#define is_v1_LHZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9023000)
#define is_v1_LHZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9025000)
#define is_v1_LHZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9024000)
#define is_v1_LHZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9022000)
#define is_v1_LHZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9020000)
#define is_v1_LHZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9027000)
#define is_v1_LHZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9026000)
#define is_v1_LHZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9029000)
#define is_v1_LHZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902b000)
#define is_v1_LHZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902d000)
#define is_v1_LHZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902c000)
#define is_v1_LHZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902a000)
#define is_v1_LHZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9028000)
#define is_v1_LHZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab021000)
#define is_v1_LHZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab023000)
#define is_v1_LHZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab025000)
#define is_v1_LHZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab024000)
#define is_v1_LHZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab022000)
#define is_v1_LHZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab020000)
#define is_v1_LHZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab027000)
#define is_v1_LHZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab026000)
#define is_v1_LHZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab029000)
#define is_v1_LHZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02b000)
#define is_v1_LHZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02d000)
#define is_v1_LHZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02c000)
#define is_v1_LHZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02a000)
#define is_v1_LHZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab028000)
#define is_v1_LHZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa021000)
#define is_v1_LHZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa023000)
#define is_v1_LHZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa025000)
#define is_v1_LHZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa024000)
#define is_v1_LHZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa022000)
#define is_v1_LHZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa020000)
#define is_v1_LHZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa027000)
#define is_v1_LHZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa026000)
#define is_v1_LHZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa029000)
#define is_v1_LHZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02b000)
#define is_v1_LHZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02d000)
#define is_v1_LHZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02c000)
#define is_v1_LHZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02a000)
#define is_v1_LHZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa028000)
#define is_v1_LHZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8021000)
#define is_v1_LHZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8023000)
#define is_v1_LHZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8025000)
#define is_v1_LHZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8024000)
#define is_v1_LHZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8022000)
#define is_v1_LHZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8020000)
#define is_v1_LHZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8027000)
#define is_v1_LHZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8026000)
#define is_v1_LHZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8029000)
#define is_v1_LHZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802b000)
#define is_v1_LHZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802d000)
#define is_v1_LHZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802c000)
#define is_v1_LHZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802a000)
#define is_v1_LHZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8028000)
#define v1_LHZ_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2902e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2902f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2802e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHZ__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2802f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LHZ_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902e000)
#define is_v1_LHZ_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902f000)
#define is_v1_LHZ_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02e000)
#define is_v1_LHZ_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02f000)
#define is_v1_LHZ_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02e000)
#define is_v1_LHZ_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02f000)
#define is_v1_LHZ___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802e000)
#define is_v1_LHZ__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802f000)
#define v1_LHS__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2c000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2d000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2e000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2f000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LHS__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2c000000)
#define is_v1_LHS_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2d000000)
#define is_v1_LHS_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2e000000)
#define is_v1_LHS_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2f000000)
#define v1_LHS__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xac000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LHS_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xad000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LHS_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xae000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LHS_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xaf000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LHS__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xac000000)
#define is_v1_LHS_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xad000000)
#define is_v1_LHS_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae000000)
#define is_v1_LHS_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf000000)
#define v1_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xac000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xad000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xae000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xaf000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xac000000)
#define is_v1_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xad000000)
#define is_v1_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae000000)
#define is_v1_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf000000)
#define v1_LHS_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LHS_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d021000)
#define is_v1_LHS_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d023000)
#define is_v1_LHS_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d025000)
#define is_v1_LHS_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d024000)
#define is_v1_LHS_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d022000)
#define is_v1_LHS_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d020000)
#define is_v1_LHS_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d027000)
#define is_v1_LHS_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d026000)
#define is_v1_LHS_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d029000)
#define is_v1_LHS_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02b000)
#define is_v1_LHS_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02d000)
#define is_v1_LHS_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02c000)
#define is_v1_LHS_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02a000)
#define is_v1_LHS_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d028000)
#define is_v1_LHS_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f021000)
#define is_v1_LHS_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f023000)
#define is_v1_LHS_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f025000)
#define is_v1_LHS_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f024000)
#define is_v1_LHS_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f022000)
#define is_v1_LHS_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f020000)
#define is_v1_LHS_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f027000)
#define is_v1_LHS_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f026000)
#define is_v1_LHS_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f029000)
#define is_v1_LHS_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02b000)
#define is_v1_LHS_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02d000)
#define is_v1_LHS_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02c000)
#define is_v1_LHS_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02a000)
#define is_v1_LHS_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f028000)
#define is_v1_LHS_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e021000)
#define is_v1_LHS_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e023000)
#define is_v1_LHS_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e025000)
#define is_v1_LHS_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e024000)
#define is_v1_LHS_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e022000)
#define is_v1_LHS_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e020000)
#define is_v1_LHS_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e027000)
#define is_v1_LHS_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e026000)
#define is_v1_LHS_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e029000)
#define is_v1_LHS_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02b000)
#define is_v1_LHS_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02d000)
#define is_v1_LHS_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02c000)
#define is_v1_LHS_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02a000)
#define is_v1_LHS_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e028000)
#define is_v1_LHS__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c021000)
#define is_v1_LHS__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c023000)
#define is_v1_LHS__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c025000)
#define is_v1_LHS__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c024000)
#define is_v1_LHS__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c022000)
#define is_v1_LHS__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c020000)
#define is_v1_LHS__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c027000)
#define is_v1_LHS__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c026000)
#define is_v1_LHS__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c029000)
#define is_v1_LHS__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02b000)
#define is_v1_LHS__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02d000)
#define is_v1_LHS__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02c000)
#define is_v1_LHS__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02a000)
#define is_v1_LHS__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c028000)
#define v1_LHS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LHS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad021000)
#define is_v1_LHS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad023000)
#define is_v1_LHS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad025000)
#define is_v1_LHS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad024000)
#define is_v1_LHS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad022000)
#define is_v1_LHS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad020000)
#define is_v1_LHS_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad027000)
#define is_v1_LHS_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad026000)
#define is_v1_LHS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad029000)
#define is_v1_LHS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02b000)
#define is_v1_LHS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02d000)
#define is_v1_LHS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02c000)
#define is_v1_LHS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02a000)
#define is_v1_LHS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad028000)
#define is_v1_LHS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf021000)
#define is_v1_LHS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf023000)
#define is_v1_LHS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf025000)
#define is_v1_LHS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf024000)
#define is_v1_LHS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf022000)
#define is_v1_LHS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf020000)
#define is_v1_LHS_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf027000)
#define is_v1_LHS_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf026000)
#define is_v1_LHS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf029000)
#define is_v1_LHS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02b000)
#define is_v1_LHS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02d000)
#define is_v1_LHS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02c000)
#define is_v1_LHS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02a000)
#define is_v1_LHS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf028000)
#define is_v1_LHS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae021000)
#define is_v1_LHS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae023000)
#define is_v1_LHS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae025000)
#define is_v1_LHS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae024000)
#define is_v1_LHS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae022000)
#define is_v1_LHS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae020000)
#define is_v1_LHS_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae027000)
#define is_v1_LHS_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae026000)
#define is_v1_LHS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae029000)
#define is_v1_LHS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02b000)
#define is_v1_LHS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02d000)
#define is_v1_LHS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02c000)
#define is_v1_LHS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02a000)
#define is_v1_LHS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae028000)
#define is_v1_LHS__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac021000)
#define is_v1_LHS__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac023000)
#define is_v1_LHS__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac025000)
#define is_v1_LHS__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac024000)
#define is_v1_LHS__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac022000)
#define is_v1_LHS__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac020000)
#define is_v1_LHS__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac027000)
#define is_v1_LHS__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac026000)
#define is_v1_LHS__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac029000)
#define is_v1_LHS__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02b000)
#define is_v1_LHS__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02d000)
#define is_v1_LHS__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02c000)
#define is_v1_LHS__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02a000)
#define is_v1_LHS__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac028000)
#define v1_LHS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LHS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LHS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LHS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad021000)
#define is_v1_LHS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad023000)
#define is_v1_LHS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad025000)
#define is_v1_LHS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad024000)
#define is_v1_LHS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad022000)
#define is_v1_LHS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad020000)
#define is_v1_LHS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad027000)
#define is_v1_LHS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad026000)
#define is_v1_LHS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad029000)
#define is_v1_LHS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02b000)
#define is_v1_LHS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02d000)
#define is_v1_LHS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02c000)
#define is_v1_LHS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02a000)
#define is_v1_LHS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad028000)
#define is_v1_LHS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf021000)
#define is_v1_LHS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf023000)
#define is_v1_LHS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf025000)
#define is_v1_LHS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf024000)
#define is_v1_LHS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf022000)
#define is_v1_LHS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf020000)
#define is_v1_LHS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf027000)
#define is_v1_LHS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf026000)
#define is_v1_LHS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf029000)
#define is_v1_LHS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02b000)
#define is_v1_LHS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02d000)
#define is_v1_LHS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02c000)
#define is_v1_LHS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02a000)
#define is_v1_LHS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf028000)
#define is_v1_LHS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae021000)
#define is_v1_LHS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae023000)
#define is_v1_LHS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae025000)
#define is_v1_LHS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae024000)
#define is_v1_LHS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae022000)
#define is_v1_LHS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae020000)
#define is_v1_LHS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae027000)
#define is_v1_LHS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae026000)
#define is_v1_LHS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae029000)
#define is_v1_LHS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02b000)
#define is_v1_LHS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02d000)
#define is_v1_LHS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02c000)
#define is_v1_LHS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02a000)
#define is_v1_LHS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae028000)
#define is_v1_LHS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac021000)
#define is_v1_LHS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac023000)
#define is_v1_LHS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac025000)
#define is_v1_LHS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac024000)
#define is_v1_LHS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac022000)
#define is_v1_LHS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac020000)
#define is_v1_LHS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac027000)
#define is_v1_LHS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac026000)
#define is_v1_LHS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac029000)
#define is_v1_LHS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02b000)
#define is_v1_LHS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02d000)
#define is_v1_LHS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02c000)
#define is_v1_LHS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02a000)
#define is_v1_LHS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac028000)
#define v1_LHS_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2d02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2d02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2f02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2f02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2e02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2e02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2c02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LHS__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2c02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LHS_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02e000)
#define is_v1_LHS_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02f000)
#define is_v1_LHS_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02e000)
#define is_v1_LHS_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02f000)
#define is_v1_LHS_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02e000)
#define is_v1_LHS_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02f000)
#define is_v1_LHS___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02e000)
#define is_v1_LHS__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02f000)
#define v1_LWZ__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x30000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x31000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x32000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x33000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LWZ__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x30000000)
#define is_v1_LWZ_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x31000000)
#define is_v1_LWZ_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x32000000)
#define is_v1_LWZ_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x33000000)
#define v1_LWZ__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb0000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LWZ_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb1000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LWZ_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb2000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LWZ_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb3000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LWZ__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb0000000)
#define is_v1_LWZ_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb1000000)
#define is_v1_LWZ_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb2000000)
#define is_v1_LWZ_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb3000000)
#define v1_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb1000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb2000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb3000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb0000000)
#define is_v1_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb1000000)
#define is_v1_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb2000000)
#define is_v1_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb3000000)
#define v1_LWZ_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LWZ_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31021000)
#define is_v1_LWZ_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31023000)
#define is_v1_LWZ_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31025000)
#define is_v1_LWZ_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31024000)
#define is_v1_LWZ_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31022000)
#define is_v1_LWZ_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31020000)
#define is_v1_LWZ_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31027000)
#define is_v1_LWZ_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31026000)
#define is_v1_LWZ_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31029000)
#define is_v1_LWZ_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102b000)
#define is_v1_LWZ_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102d000)
#define is_v1_LWZ_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102c000)
#define is_v1_LWZ_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102a000)
#define is_v1_LWZ_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31028000)
#define is_v1_LWZ_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33021000)
#define is_v1_LWZ_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33023000)
#define is_v1_LWZ_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33025000)
#define is_v1_LWZ_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33024000)
#define is_v1_LWZ_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33022000)
#define is_v1_LWZ_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33020000)
#define is_v1_LWZ_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33027000)
#define is_v1_LWZ_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33026000)
#define is_v1_LWZ_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33029000)
#define is_v1_LWZ_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302b000)
#define is_v1_LWZ_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302d000)
#define is_v1_LWZ_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302c000)
#define is_v1_LWZ_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302a000)
#define is_v1_LWZ_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33028000)
#define is_v1_LWZ_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32021000)
#define is_v1_LWZ_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32023000)
#define is_v1_LWZ_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32025000)
#define is_v1_LWZ_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32024000)
#define is_v1_LWZ_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32022000)
#define is_v1_LWZ_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32020000)
#define is_v1_LWZ_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32027000)
#define is_v1_LWZ_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32026000)
#define is_v1_LWZ_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32029000)
#define is_v1_LWZ_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202b000)
#define is_v1_LWZ_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202d000)
#define is_v1_LWZ_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202c000)
#define is_v1_LWZ_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202a000)
#define is_v1_LWZ_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32028000)
#define is_v1_LWZ__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30021000)
#define is_v1_LWZ__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30023000)
#define is_v1_LWZ__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30025000)
#define is_v1_LWZ__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30024000)
#define is_v1_LWZ__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30022000)
#define is_v1_LWZ__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30020000)
#define is_v1_LWZ__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30027000)
#define is_v1_LWZ__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30026000)
#define is_v1_LWZ__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30029000)
#define is_v1_LWZ__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002b000)
#define is_v1_LWZ__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002d000)
#define is_v1_LWZ__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002c000)
#define is_v1_LWZ__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002a000)
#define is_v1_LWZ__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30028000)
#define v1_LWZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LWZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1021000)
#define is_v1_LWZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1023000)
#define is_v1_LWZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1025000)
#define is_v1_LWZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1024000)
#define is_v1_LWZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1022000)
#define is_v1_LWZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1020000)
#define is_v1_LWZ_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1027000)
#define is_v1_LWZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1026000)
#define is_v1_LWZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1029000)
#define is_v1_LWZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102b000)
#define is_v1_LWZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102d000)
#define is_v1_LWZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102c000)
#define is_v1_LWZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102a000)
#define is_v1_LWZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1028000)
#define is_v1_LWZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3021000)
#define is_v1_LWZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3023000)
#define is_v1_LWZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3025000)
#define is_v1_LWZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3024000)
#define is_v1_LWZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3022000)
#define is_v1_LWZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3020000)
#define is_v1_LWZ_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3027000)
#define is_v1_LWZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3026000)
#define is_v1_LWZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3029000)
#define is_v1_LWZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302b000)
#define is_v1_LWZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302d000)
#define is_v1_LWZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302c000)
#define is_v1_LWZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302a000)
#define is_v1_LWZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3028000)
#define is_v1_LWZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2021000)
#define is_v1_LWZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2023000)
#define is_v1_LWZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2025000)
#define is_v1_LWZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2024000)
#define is_v1_LWZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2022000)
#define is_v1_LWZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2020000)
#define is_v1_LWZ_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2027000)
#define is_v1_LWZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2026000)
#define is_v1_LWZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2029000)
#define is_v1_LWZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202b000)
#define is_v1_LWZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202d000)
#define is_v1_LWZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202c000)
#define is_v1_LWZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202a000)
#define is_v1_LWZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2028000)
#define is_v1_LWZ__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0021000)
#define is_v1_LWZ__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0023000)
#define is_v1_LWZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0025000)
#define is_v1_LWZ__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0024000)
#define is_v1_LWZ__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0022000)
#define is_v1_LWZ__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0020000)
#define is_v1_LWZ__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0027000)
#define is_v1_LWZ__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0026000)
#define is_v1_LWZ__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0029000)
#define is_v1_LWZ__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002b000)
#define is_v1_LWZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002d000)
#define is_v1_LWZ__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002c000)
#define is_v1_LWZ__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002a000)
#define is_v1_LWZ__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0028000)
#define v1_LWZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LWZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1021000)
#define is_v1_LWZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1023000)
#define is_v1_LWZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1025000)
#define is_v1_LWZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1024000)
#define is_v1_LWZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1022000)
#define is_v1_LWZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1020000)
#define is_v1_LWZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1027000)
#define is_v1_LWZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1026000)
#define is_v1_LWZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1029000)
#define is_v1_LWZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102b000)
#define is_v1_LWZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102d000)
#define is_v1_LWZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102c000)
#define is_v1_LWZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102a000)
#define is_v1_LWZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1028000)
#define is_v1_LWZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3021000)
#define is_v1_LWZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3023000)
#define is_v1_LWZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3025000)
#define is_v1_LWZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3024000)
#define is_v1_LWZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3022000)
#define is_v1_LWZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3020000)
#define is_v1_LWZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3027000)
#define is_v1_LWZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3026000)
#define is_v1_LWZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3029000)
#define is_v1_LWZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302b000)
#define is_v1_LWZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302d000)
#define is_v1_LWZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302c000)
#define is_v1_LWZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302a000)
#define is_v1_LWZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3028000)
#define is_v1_LWZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2021000)
#define is_v1_LWZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2023000)
#define is_v1_LWZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2025000)
#define is_v1_LWZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2024000)
#define is_v1_LWZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2022000)
#define is_v1_LWZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2020000)
#define is_v1_LWZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2027000)
#define is_v1_LWZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2026000)
#define is_v1_LWZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2029000)
#define is_v1_LWZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202b000)
#define is_v1_LWZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202d000)
#define is_v1_LWZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202c000)
#define is_v1_LWZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202a000)
#define is_v1_LWZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2028000)
#define is_v1_LWZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0021000)
#define is_v1_LWZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0023000)
#define is_v1_LWZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0025000)
#define is_v1_LWZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0024000)
#define is_v1_LWZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0022000)
#define is_v1_LWZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0020000)
#define is_v1_LWZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0027000)
#define is_v1_LWZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0026000)
#define is_v1_LWZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0029000)
#define is_v1_LWZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002b000)
#define is_v1_LWZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002d000)
#define is_v1_LWZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002c000)
#define is_v1_LWZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002a000)
#define is_v1_LWZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0028000)
#define v1_LWZ_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3102e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3102f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3302e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3302f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3202e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3202f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3002e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWZ__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3002f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LWZ_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102e000)
#define is_v1_LWZ_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102f000)
#define is_v1_LWZ_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302e000)
#define is_v1_LWZ_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302f000)
#define is_v1_LWZ_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202e000)
#define is_v1_LWZ_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202f000)
#define is_v1_LWZ___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002e000)
#define is_v1_LWZ__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002f000)
#define v1_LWS__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x34000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x35000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x36000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x37000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LWS__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x34000000)
#define is_v1_LWS_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x35000000)
#define is_v1_LWS_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x36000000)
#define is_v1_LWS_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x37000000)
#define v1_LWS__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb4000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LWS_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb5000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LWS_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb6000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LWS_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb7000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LWS__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb4000000)
#define is_v1_LWS_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb5000000)
#define is_v1_LWS_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb6000000)
#define is_v1_LWS_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb7000000)
#define v1_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb4000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb5000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb6000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb7000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb4000000)
#define is_v1_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb5000000)
#define is_v1_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb6000000)
#define is_v1_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb7000000)
#define v1_LWS_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LWS_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35021000)
#define is_v1_LWS_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35023000)
#define is_v1_LWS_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35025000)
#define is_v1_LWS_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35024000)
#define is_v1_LWS_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35022000)
#define is_v1_LWS_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35020000)
#define is_v1_LWS_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35027000)
#define is_v1_LWS_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35026000)
#define is_v1_LWS_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35029000)
#define is_v1_LWS_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502b000)
#define is_v1_LWS_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502d000)
#define is_v1_LWS_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502c000)
#define is_v1_LWS_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502a000)
#define is_v1_LWS_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35028000)
#define is_v1_LWS_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37021000)
#define is_v1_LWS_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37023000)
#define is_v1_LWS_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37025000)
#define is_v1_LWS_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37024000)
#define is_v1_LWS_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37022000)
#define is_v1_LWS_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37020000)
#define is_v1_LWS_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37027000)
#define is_v1_LWS_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37026000)
#define is_v1_LWS_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37029000)
#define is_v1_LWS_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702b000)
#define is_v1_LWS_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702d000)
#define is_v1_LWS_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702c000)
#define is_v1_LWS_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702a000)
#define is_v1_LWS_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37028000)
#define is_v1_LWS_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36021000)
#define is_v1_LWS_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36023000)
#define is_v1_LWS_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36025000)
#define is_v1_LWS_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36024000)
#define is_v1_LWS_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36022000)
#define is_v1_LWS_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36020000)
#define is_v1_LWS_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36027000)
#define is_v1_LWS_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36026000)
#define is_v1_LWS_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36029000)
#define is_v1_LWS_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602b000)
#define is_v1_LWS_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602d000)
#define is_v1_LWS_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602c000)
#define is_v1_LWS_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602a000)
#define is_v1_LWS_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36028000)
#define is_v1_LWS__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34021000)
#define is_v1_LWS__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34023000)
#define is_v1_LWS__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34025000)
#define is_v1_LWS__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34024000)
#define is_v1_LWS__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34022000)
#define is_v1_LWS__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34020000)
#define is_v1_LWS__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34027000)
#define is_v1_LWS__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34026000)
#define is_v1_LWS__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34029000)
#define is_v1_LWS__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402b000)
#define is_v1_LWS__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402d000)
#define is_v1_LWS__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402c000)
#define is_v1_LWS__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402a000)
#define is_v1_LWS__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34028000)
#define v1_LWS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LWS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5021000)
#define is_v1_LWS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5023000)
#define is_v1_LWS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5025000)
#define is_v1_LWS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5024000)
#define is_v1_LWS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5022000)
#define is_v1_LWS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5020000)
#define is_v1_LWS_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5027000)
#define is_v1_LWS_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5026000)
#define is_v1_LWS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5029000)
#define is_v1_LWS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502b000)
#define is_v1_LWS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502d000)
#define is_v1_LWS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502c000)
#define is_v1_LWS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502a000)
#define is_v1_LWS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5028000)
#define is_v1_LWS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7021000)
#define is_v1_LWS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7023000)
#define is_v1_LWS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7025000)
#define is_v1_LWS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7024000)
#define is_v1_LWS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7022000)
#define is_v1_LWS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7020000)
#define is_v1_LWS_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7027000)
#define is_v1_LWS_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7026000)
#define is_v1_LWS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7029000)
#define is_v1_LWS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702b000)
#define is_v1_LWS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702d000)
#define is_v1_LWS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702c000)
#define is_v1_LWS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702a000)
#define is_v1_LWS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7028000)
#define is_v1_LWS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6021000)
#define is_v1_LWS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6023000)
#define is_v1_LWS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6025000)
#define is_v1_LWS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6024000)
#define is_v1_LWS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6022000)
#define is_v1_LWS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6020000)
#define is_v1_LWS_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6027000)
#define is_v1_LWS_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6026000)
#define is_v1_LWS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6029000)
#define is_v1_LWS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602b000)
#define is_v1_LWS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602d000)
#define is_v1_LWS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602c000)
#define is_v1_LWS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602a000)
#define is_v1_LWS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6028000)
#define is_v1_LWS__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4021000)
#define is_v1_LWS__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4023000)
#define is_v1_LWS__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4025000)
#define is_v1_LWS__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4024000)
#define is_v1_LWS__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4022000)
#define is_v1_LWS__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4020000)
#define is_v1_LWS__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4027000)
#define is_v1_LWS__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4026000)
#define is_v1_LWS__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4029000)
#define is_v1_LWS__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402b000)
#define is_v1_LWS__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402d000)
#define is_v1_LWS__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402c000)
#define is_v1_LWS__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402a000)
#define is_v1_LWS__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4028000)
#define v1_LWS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LWS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LWS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LWS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5021000)
#define is_v1_LWS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5023000)
#define is_v1_LWS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5025000)
#define is_v1_LWS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5024000)
#define is_v1_LWS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5022000)
#define is_v1_LWS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5020000)
#define is_v1_LWS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5027000)
#define is_v1_LWS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5026000)
#define is_v1_LWS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5029000)
#define is_v1_LWS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502b000)
#define is_v1_LWS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502d000)
#define is_v1_LWS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502c000)
#define is_v1_LWS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502a000)
#define is_v1_LWS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5028000)
#define is_v1_LWS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7021000)
#define is_v1_LWS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7023000)
#define is_v1_LWS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7025000)
#define is_v1_LWS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7024000)
#define is_v1_LWS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7022000)
#define is_v1_LWS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7020000)
#define is_v1_LWS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7027000)
#define is_v1_LWS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7026000)
#define is_v1_LWS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7029000)
#define is_v1_LWS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702b000)
#define is_v1_LWS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702d000)
#define is_v1_LWS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702c000)
#define is_v1_LWS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702a000)
#define is_v1_LWS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7028000)
#define is_v1_LWS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6021000)
#define is_v1_LWS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6023000)
#define is_v1_LWS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6025000)
#define is_v1_LWS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6024000)
#define is_v1_LWS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6022000)
#define is_v1_LWS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6020000)
#define is_v1_LWS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6027000)
#define is_v1_LWS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6026000)
#define is_v1_LWS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6029000)
#define is_v1_LWS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602b000)
#define is_v1_LWS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602d000)
#define is_v1_LWS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602c000)
#define is_v1_LWS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602a000)
#define is_v1_LWS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6028000)
#define is_v1_LWS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4021000)
#define is_v1_LWS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4023000)
#define is_v1_LWS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4025000)
#define is_v1_LWS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4024000)
#define is_v1_LWS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4022000)
#define is_v1_LWS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4020000)
#define is_v1_LWS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4027000)
#define is_v1_LWS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4026000)
#define is_v1_LWS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4029000)
#define is_v1_LWS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402b000)
#define is_v1_LWS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402d000)
#define is_v1_LWS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402c000)
#define is_v1_LWS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402a000)
#define is_v1_LWS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4028000)
#define v1_LWS_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3502e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3502f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3702e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3702f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3602e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3602f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3402e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LWS__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3402f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LWS_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502e000)
#define is_v1_LWS_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502f000)
#define is_v1_LWS_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702e000)
#define is_v1_LWS_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702f000)
#define is_v1_LWS_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602e000)
#define is_v1_LWS_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602f000)
#define is_v1_LWS___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402e000)
#define is_v1_LWS__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402f000)
#define v1_LD__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x38000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x39000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x3a000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x3b000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LD__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x38000000)
#define is_v1_LD_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x39000000)
#define is_v1_LD_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x3a000000)
#define is_v1_LD_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x3b000000)
#define v1_LD__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb8000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LD_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb9000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LD_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xba000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LD_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xbb000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LD__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb8000000)
#define is_v1_LD_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb9000000)
#define is_v1_LD_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xba000000)
#define is_v1_LD_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xbb000000)
#define v1_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb8000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb9000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xba000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xbb000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb8000000)
#define is_v1_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb9000000)
#define is_v1_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xba000000)
#define is_v1_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xbb000000)
#define v1_LD_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LD__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LD_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39021000)
#define is_v1_LD_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39023000)
#define is_v1_LD_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39025000)
#define is_v1_LD_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39024000)
#define is_v1_LD_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39022000)
#define is_v1_LD_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39020000)
#define is_v1_LD_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39027000)
#define is_v1_LD_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39026000)
#define is_v1_LD_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39029000)
#define is_v1_LD_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902b000)
#define is_v1_LD_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902d000)
#define is_v1_LD_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902c000)
#define is_v1_LD_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902a000)
#define is_v1_LD_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39028000)
#define is_v1_LD_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b021000)
#define is_v1_LD_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b023000)
#define is_v1_LD_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b025000)
#define is_v1_LD_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b024000)
#define is_v1_LD_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b022000)
#define is_v1_LD_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b020000)
#define is_v1_LD_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b027000)
#define is_v1_LD_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b026000)
#define is_v1_LD_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b029000)
#define is_v1_LD_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02b000)
#define is_v1_LD_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02d000)
#define is_v1_LD_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02c000)
#define is_v1_LD_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02a000)
#define is_v1_LD_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b028000)
#define is_v1_LD_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a021000)
#define is_v1_LD_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a023000)
#define is_v1_LD_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a025000)
#define is_v1_LD_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a024000)
#define is_v1_LD_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a022000)
#define is_v1_LD_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a020000)
#define is_v1_LD_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a027000)
#define is_v1_LD_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a026000)
#define is_v1_LD_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a029000)
#define is_v1_LD_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02b000)
#define is_v1_LD_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02d000)
#define is_v1_LD_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02c000)
#define is_v1_LD_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02a000)
#define is_v1_LD_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a028000)
#define is_v1_LD__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38021000)
#define is_v1_LD__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38023000)
#define is_v1_LD__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38025000)
#define is_v1_LD__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38024000)
#define is_v1_LD__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38022000)
#define is_v1_LD__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38020000)
#define is_v1_LD__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38027000)
#define is_v1_LD__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38026000)
#define is_v1_LD__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38029000)
#define is_v1_LD__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802b000)
#define is_v1_LD__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802d000)
#define is_v1_LD__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802c000)
#define is_v1_LD__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802a000)
#define is_v1_LD__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38028000)
#define v1_LD_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LD_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9021000)
#define is_v1_LD_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9023000)
#define is_v1_LD_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9025000)
#define is_v1_LD_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9024000)
#define is_v1_LD_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9022000)
#define is_v1_LD_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9020000)
#define is_v1_LD_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9027000)
#define is_v1_LD_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9026000)
#define is_v1_LD_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9029000)
#define is_v1_LD_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902b000)
#define is_v1_LD_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902d000)
#define is_v1_LD_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902c000)
#define is_v1_LD_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902a000)
#define is_v1_LD_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9028000)
#define is_v1_LD_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb021000)
#define is_v1_LD_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb023000)
#define is_v1_LD_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb025000)
#define is_v1_LD_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb024000)
#define is_v1_LD_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb022000)
#define is_v1_LD_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb020000)
#define is_v1_LD_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb027000)
#define is_v1_LD_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb026000)
#define is_v1_LD_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb029000)
#define is_v1_LD_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02b000)
#define is_v1_LD_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02d000)
#define is_v1_LD_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02c000)
#define is_v1_LD_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02a000)
#define is_v1_LD_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb028000)
#define is_v1_LD_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba021000)
#define is_v1_LD_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba023000)
#define is_v1_LD_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba025000)
#define is_v1_LD_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba024000)
#define is_v1_LD_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba022000)
#define is_v1_LD_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba020000)
#define is_v1_LD_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba027000)
#define is_v1_LD_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba026000)
#define is_v1_LD_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba029000)
#define is_v1_LD_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02b000)
#define is_v1_LD_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02d000)
#define is_v1_LD_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02c000)
#define is_v1_LD_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02a000)
#define is_v1_LD_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba028000)
#define is_v1_LD__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8021000)
#define is_v1_LD__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8023000)
#define is_v1_LD__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8025000)
#define is_v1_LD__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8024000)
#define is_v1_LD__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8022000)
#define is_v1_LD__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8020000)
#define is_v1_LD__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8027000)
#define is_v1_LD__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8026000)
#define is_v1_LD__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8029000)
#define is_v1_LD__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802b000)
#define is_v1_LD__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802d000)
#define is_v1_LD__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802c000)
#define is_v1_LD__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802a000)
#define is_v1_LD__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8028000)
#define v1_LD_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LD__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LD__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LD_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9021000)
#define is_v1_LD_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9023000)
#define is_v1_LD_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9025000)
#define is_v1_LD_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9024000)
#define is_v1_LD_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9022000)
#define is_v1_LD_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9020000)
#define is_v1_LD_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9027000)
#define is_v1_LD_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9026000)
#define is_v1_LD_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9029000)
#define is_v1_LD_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902b000)
#define is_v1_LD_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902d000)
#define is_v1_LD_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902c000)
#define is_v1_LD_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902a000)
#define is_v1_LD_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9028000)
#define is_v1_LD_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb021000)
#define is_v1_LD_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb023000)
#define is_v1_LD_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb025000)
#define is_v1_LD_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb024000)
#define is_v1_LD_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb022000)
#define is_v1_LD_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb020000)
#define is_v1_LD_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb027000)
#define is_v1_LD_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb026000)
#define is_v1_LD_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb029000)
#define is_v1_LD_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02b000)
#define is_v1_LD_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02d000)
#define is_v1_LD_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02c000)
#define is_v1_LD_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02a000)
#define is_v1_LD_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb028000)
#define is_v1_LD_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba021000)
#define is_v1_LD_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba023000)
#define is_v1_LD_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba025000)
#define is_v1_LD_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba024000)
#define is_v1_LD_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba022000)
#define is_v1_LD_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba020000)
#define is_v1_LD_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba027000)
#define is_v1_LD_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba026000)
#define is_v1_LD_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba029000)
#define is_v1_LD_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02b000)
#define is_v1_LD_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02d000)
#define is_v1_LD_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02c000)
#define is_v1_LD_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02a000)
#define is_v1_LD_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba028000)
#define is_v1_LD__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8021000)
#define is_v1_LD__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8023000)
#define is_v1_LD__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8025000)
#define is_v1_LD__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8024000)
#define is_v1_LD__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8022000)
#define is_v1_LD__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8020000)
#define is_v1_LD__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8027000)
#define is_v1_LD__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8026000)
#define is_v1_LD__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8029000)
#define is_v1_LD__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802b000)
#define is_v1_LD__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802d000)
#define is_v1_LD__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802c000)
#define is_v1_LD__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802a000)
#define is_v1_LD__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8028000)
#define v1_LD_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3902e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3902f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3b02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3b02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3a02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3a02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3802e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LD__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3802f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LD_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902e000)
#define is_v1_LD_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902f000)
#define is_v1_LD_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02e000)
#define is_v1_LD_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02f000)
#define is_v1_LD_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02e000)
#define is_v1_LD_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02f000)
#define is_v1_LD___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802e000)
#define is_v1_LD__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802f000)
#define v1_LQ__registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3c000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3d000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3e000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3f000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LQ__registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3c000000)
#define is_v1_LQ_s_registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3d000000)
#define is_v1_LQ_u_registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3e000000)
#define is_v1_LQ_us_registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3f000000)
#define v1_LQ__registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbc000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ__registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LQ_s_registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbd000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LQ_u_registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbe000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LQ_us_registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbf000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LQ__registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbc000000)
#define is_v1_LQ_s_registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbd000000)
#define is_v1_LQ_u_registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbe000000)
#define is_v1_LQ_us_registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbf000000)
#define v1_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbc000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbd000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbe000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbf000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbc000000)
#define is_v1_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbd000000)
#define is_v1_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbe000000)
#define is_v1_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbf000000)
#define v1_LQ_s_deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define is_v1_LQ_s_deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d021000)
#define is_v1_LQ_s_dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d023000)
#define is_v1_LQ_s_dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d025000)
#define is_v1_LQ_s_dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d024000)
#define is_v1_LQ_s_dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d022000)
#define is_v1_LQ_s_dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d020000)
#define is_v1_LQ_s_even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d027000)
#define is_v1_LQ_s_odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d026000)
#define is_v1_LQ_s_weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d029000)
#define is_v1_LQ_s_wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02b000)
#define is_v1_LQ_s_wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02d000)
#define is_v1_LQ_s_wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02c000)
#define is_v1_LQ_s_wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02a000)
#define is_v1_LQ_s_wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d028000)
#define is_v1_LQ_us_deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f021000)
#define is_v1_LQ_us_dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f023000)
#define is_v1_LQ_us_dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f025000)
#define is_v1_LQ_us_dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f024000)
#define is_v1_LQ_us_dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f022000)
#define is_v1_LQ_us_dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f020000)
#define is_v1_LQ_us_even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f027000)
#define is_v1_LQ_us_odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f026000)
#define is_v1_LQ_us_weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f029000)
#define is_v1_LQ_us_wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02b000)
#define is_v1_LQ_us_wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02d000)
#define is_v1_LQ_us_wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02c000)
#define is_v1_LQ_us_wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02a000)
#define is_v1_LQ_us_wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f028000)
#define is_v1_LQ_u_deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e021000)
#define is_v1_LQ_u_dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e023000)
#define is_v1_LQ_u_dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e025000)
#define is_v1_LQ_u_dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e024000)
#define is_v1_LQ_u_dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e022000)
#define is_v1_LQ_u_dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e020000)
#define is_v1_LQ_u_even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e027000)
#define is_v1_LQ_u_odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e026000)
#define is_v1_LQ_u_weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e029000)
#define is_v1_LQ_u_wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02b000)
#define is_v1_LQ_u_wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02d000)
#define is_v1_LQ_u_wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02c000)
#define is_v1_LQ_u_wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02a000)
#define is_v1_LQ_u_wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e028000)
#define is_v1_LQ__deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c021000)
#define is_v1_LQ__dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c023000)
#define is_v1_LQ__dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c025000)
#define is_v1_LQ__dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c024000)
#define is_v1_LQ__dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c022000)
#define is_v1_LQ__dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c020000)
#define is_v1_LQ__even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c027000)
#define is_v1_LQ__odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c026000)
#define is_v1_LQ__weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c029000)
#define is_v1_LQ__wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02b000)
#define is_v1_LQ__wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02d000)
#define is_v1_LQ__wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02c000)
#define is_v1_LQ__wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02a000)
#define is_v1_LQ__wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c028000)
#define v1_LQ_s_deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LQ_s_deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd021000)
#define is_v1_LQ_s_dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd023000)
#define is_v1_LQ_s_dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd025000)
#define is_v1_LQ_s_dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd024000)
#define is_v1_LQ_s_dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd022000)
#define is_v1_LQ_s_dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd020000)
#define is_v1_LQ_s_even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd027000)
#define is_v1_LQ_s_odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd026000)
#define is_v1_LQ_s_weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd029000)
#define is_v1_LQ_s_wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02b000)
#define is_v1_LQ_s_wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02d000)
#define is_v1_LQ_s_wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02c000)
#define is_v1_LQ_s_wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02a000)
#define is_v1_LQ_s_wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd028000)
#define is_v1_LQ_us_deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf021000)
#define is_v1_LQ_us_dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf023000)
#define is_v1_LQ_us_dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf025000)
#define is_v1_LQ_us_dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf024000)
#define is_v1_LQ_us_dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf022000)
#define is_v1_LQ_us_dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf020000)
#define is_v1_LQ_us_even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf027000)
#define is_v1_LQ_us_odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf026000)
#define is_v1_LQ_us_weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf029000)
#define is_v1_LQ_us_wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02b000)
#define is_v1_LQ_us_wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02d000)
#define is_v1_LQ_us_wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02c000)
#define is_v1_LQ_us_wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02a000)
#define is_v1_LQ_us_wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf028000)
#define is_v1_LQ_u_deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe021000)
#define is_v1_LQ_u_dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe023000)
#define is_v1_LQ_u_dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe025000)
#define is_v1_LQ_u_dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe024000)
#define is_v1_LQ_u_dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe022000)
#define is_v1_LQ_u_dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe020000)
#define is_v1_LQ_u_even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe027000)
#define is_v1_LQ_u_odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe026000)
#define is_v1_LQ_u_weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe029000)
#define is_v1_LQ_u_wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02b000)
#define is_v1_LQ_u_wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02d000)
#define is_v1_LQ_u_wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02c000)
#define is_v1_LQ_u_wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02a000)
#define is_v1_LQ_u_wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe028000)
#define is_v1_LQ__deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc021000)
#define is_v1_LQ__dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc023000)
#define is_v1_LQ__dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc025000)
#define is_v1_LQ__dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc024000)
#define is_v1_LQ__dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc022000)
#define is_v1_LQ__dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc020000)
#define is_v1_LQ__even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc027000)
#define is_v1_LQ__odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc026000)
#define is_v1_LQ__weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc029000)
#define is_v1_LQ__wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02b000)
#define is_v1_LQ__wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02d000)
#define is_v1_LQ__wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02c000)
#define is_v1_LQ__wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02a000)
#define is_v1_LQ__wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc028000)
#define v1_LQ_s_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_s_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_us_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ_u_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LQ__wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LQ__wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LQ_s_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd021000)
#define is_v1_LQ_s_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd023000)
#define is_v1_LQ_s_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd025000)
#define is_v1_LQ_s_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd024000)
#define is_v1_LQ_s_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd022000)
#define is_v1_LQ_s_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd020000)
#define is_v1_LQ_s_even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd027000)
#define is_v1_LQ_s_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd026000)
#define is_v1_LQ_s_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd029000)
#define is_v1_LQ_s_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02b000)
#define is_v1_LQ_s_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02d000)
#define is_v1_LQ_s_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02c000)
#define is_v1_LQ_s_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02a000)
#define is_v1_LQ_s_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd028000)
#define is_v1_LQ_us_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf021000)
#define is_v1_LQ_us_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf023000)
#define is_v1_LQ_us_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf025000)
#define is_v1_LQ_us_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf024000)
#define is_v1_LQ_us_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf022000)
#define is_v1_LQ_us_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf020000)
#define is_v1_LQ_us_even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf027000)
#define is_v1_LQ_us_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf026000)
#define is_v1_LQ_us_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf029000)
#define is_v1_LQ_us_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02b000)
#define is_v1_LQ_us_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02d000)
#define is_v1_LQ_us_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02c000)
#define is_v1_LQ_us_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02a000)
#define is_v1_LQ_us_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf028000)
#define is_v1_LQ_u_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe021000)
#define is_v1_LQ_u_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe023000)
#define is_v1_LQ_u_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe025000)
#define is_v1_LQ_u_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe024000)
#define is_v1_LQ_u_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe022000)
#define is_v1_LQ_u_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe020000)
#define is_v1_LQ_u_even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe027000)
#define is_v1_LQ_u_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe026000)
#define is_v1_LQ_u_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe029000)
#define is_v1_LQ_u_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02b000)
#define is_v1_LQ_u_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02d000)
#define is_v1_LQ_u_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02c000)
#define is_v1_LQ_u_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02a000)
#define is_v1_LQ_u_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe028000)
#define is_v1_LQ__deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc021000)
#define is_v1_LQ__dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc023000)
#define is_v1_LQ__dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc025000)
#define is_v1_LQ__dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc024000)
#define is_v1_LQ__dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc022000)
#define is_v1_LQ__dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc020000)
#define is_v1_LQ__even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc027000)
#define is_v1_LQ__odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc026000)
#define is_v1_LQ__weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc029000)
#define is_v1_LQ__wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02b000)
#define is_v1_LQ__wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02d000)
#define is_v1_LQ__wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02c000)
#define is_v1_LQ__wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02a000)
#define is_v1_LQ__wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc028000)
#define v1_LQ_s__registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3d02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_s_xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3d02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us__registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3f02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_us_xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3f02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u__registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3e02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ_u_xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3e02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ___registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3c02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LQ__xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3c02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LQ_s__registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02e000)
#define is_v1_LQ_s_xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02f000)
#define is_v1_LQ_us__registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02e000)
#define is_v1_LQ_us_xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02f000)
#define is_v1_LQ_u__registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02e000)
#define is_v1_LQ_u_xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02f000)
#define is_v1_LQ___registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02e000)
#define is_v1_LQ__xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02f000)
#define v1_LO__registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3c040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3d040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3e040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3f040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LO__registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3c040000)
#define is_v1_LO_s_registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3d040000)
#define is_v1_LO_u_registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3e040000)
#define is_v1_LO_us_registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3f040000)
#define v1_LO__registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbc040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO__registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LO_s_registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbd040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LO_u_registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbe040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LO_us_registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbf040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LO__registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbc040000)
#define is_v1_LO_s_registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbd040000)
#define is_v1_LO_u_registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbe040000)
#define is_v1_LO_us_registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbf040000)
#define v1_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbc040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbd040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbe040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbf040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbc040000)
#define is_v1_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbd040000)
#define is_v1_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbe040000)
#define is_v1_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbf040000)
#define v1_LO_s_deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LO__wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define is_v1_LO_s_deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d061000)
#define is_v1_LO_s_dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d063000)
#define is_v1_LO_s_dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d065000)
#define is_v1_LO_s_dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d064000)
#define is_v1_LO_s_dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d062000)
#define is_v1_LO_s_dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d060000)
#define is_v1_LO_s_even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d067000)
#define is_v1_LO_s_odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d066000)
#define is_v1_LO_s_weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d069000)
#define is_v1_LO_s_wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06b000)
#define is_v1_LO_s_wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06d000)
#define is_v1_LO_s_wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06c000)
#define is_v1_LO_s_wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06a000)
#define is_v1_LO_s_wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d068000)
#define is_v1_LO_us_deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f061000)
#define is_v1_LO_us_dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f063000)
#define is_v1_LO_us_dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f065000)
#define is_v1_LO_us_dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f064000)
#define is_v1_LO_us_dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f062000)
#define is_v1_LO_us_dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f060000)
#define is_v1_LO_us_even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f067000)
#define is_v1_LO_us_odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f066000)
#define is_v1_LO_us_weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f069000)
#define is_v1_LO_us_wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06b000)
#define is_v1_LO_us_wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06d000)
#define is_v1_LO_us_wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06c000)
#define is_v1_LO_us_wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06a000)
#define is_v1_LO_us_wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f068000)
#define is_v1_LO_u_deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e061000)
#define is_v1_LO_u_dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e063000)
#define is_v1_LO_u_dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e065000)
#define is_v1_LO_u_dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e064000)
#define is_v1_LO_u_dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e062000)
#define is_v1_LO_u_dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e060000)
#define is_v1_LO_u_even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e067000)
#define is_v1_LO_u_odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e066000)
#define is_v1_LO_u_weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e069000)
#define is_v1_LO_u_wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06b000)
#define is_v1_LO_u_wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06d000)
#define is_v1_LO_u_wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06c000)
#define is_v1_LO_u_wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06a000)
#define is_v1_LO_u_wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e068000)
#define is_v1_LO__deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c061000)
#define is_v1_LO__dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c063000)
#define is_v1_LO__dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c065000)
#define is_v1_LO__dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c064000)
#define is_v1_LO__dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c062000)
#define is_v1_LO__dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c060000)
#define is_v1_LO__even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c067000)
#define is_v1_LO__odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c066000)
#define is_v1_LO__weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c069000)
#define is_v1_LO__wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06b000)
#define is_v1_LO__wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06d000)
#define is_v1_LO__wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06c000)
#define is_v1_LO__wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06a000)
#define is_v1_LO__wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c068000)
#define v1_LO_s_deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LO_s_deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd061000)
#define is_v1_LO_s_dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd063000)
#define is_v1_LO_s_dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd065000)
#define is_v1_LO_s_dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd064000)
#define is_v1_LO_s_dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd062000)
#define is_v1_LO_s_dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd060000)
#define is_v1_LO_s_even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd067000)
#define is_v1_LO_s_odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd066000)
#define is_v1_LO_s_weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd069000)
#define is_v1_LO_s_wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06b000)
#define is_v1_LO_s_wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06d000)
#define is_v1_LO_s_wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06c000)
#define is_v1_LO_s_wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06a000)
#define is_v1_LO_s_wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd068000)
#define is_v1_LO_us_deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf061000)
#define is_v1_LO_us_dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf063000)
#define is_v1_LO_us_dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf065000)
#define is_v1_LO_us_dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf064000)
#define is_v1_LO_us_dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf062000)
#define is_v1_LO_us_dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf060000)
#define is_v1_LO_us_even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf067000)
#define is_v1_LO_us_odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf066000)
#define is_v1_LO_us_weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf069000)
#define is_v1_LO_us_wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06b000)
#define is_v1_LO_us_wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06d000)
#define is_v1_LO_us_wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06c000)
#define is_v1_LO_us_wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06a000)
#define is_v1_LO_us_wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf068000)
#define is_v1_LO_u_deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe061000)
#define is_v1_LO_u_dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe063000)
#define is_v1_LO_u_dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe065000)
#define is_v1_LO_u_dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe064000)
#define is_v1_LO_u_dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe062000)
#define is_v1_LO_u_dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe060000)
#define is_v1_LO_u_even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe067000)
#define is_v1_LO_u_odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe066000)
#define is_v1_LO_u_weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe069000)
#define is_v1_LO_u_wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06b000)
#define is_v1_LO_u_wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06d000)
#define is_v1_LO_u_wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06c000)
#define is_v1_LO_u_wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06a000)
#define is_v1_LO_u_wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe068000)
#define is_v1_LO__deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc061000)
#define is_v1_LO__dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc063000)
#define is_v1_LO__dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc065000)
#define is_v1_LO__dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc064000)
#define is_v1_LO__dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc062000)
#define is_v1_LO__dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc060000)
#define is_v1_LO__even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc067000)
#define is_v1_LO__odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc066000)
#define is_v1_LO__weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc069000)
#define is_v1_LO__wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06b000)
#define is_v1_LO__wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06d000)
#define is_v1_LO__wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06c000)
#define is_v1_LO__wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06a000)
#define is_v1_LO__wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc068000)
#define v1_LO_s_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_s_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_us_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO_u_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LO__wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LO__wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LO_s_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd061000)
#define is_v1_LO_s_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd063000)
#define is_v1_LO_s_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd065000)
#define is_v1_LO_s_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd064000)
#define is_v1_LO_s_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd062000)
#define is_v1_LO_s_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd060000)
#define is_v1_LO_s_even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd067000)
#define is_v1_LO_s_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd066000)
#define is_v1_LO_s_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd069000)
#define is_v1_LO_s_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06b000)
#define is_v1_LO_s_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06d000)
#define is_v1_LO_s_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06c000)
#define is_v1_LO_s_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06a000)
#define is_v1_LO_s_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd068000)
#define is_v1_LO_us_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf061000)
#define is_v1_LO_us_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf063000)
#define is_v1_LO_us_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf065000)
#define is_v1_LO_us_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf064000)
#define is_v1_LO_us_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf062000)
#define is_v1_LO_us_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf060000)
#define is_v1_LO_us_even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf067000)
#define is_v1_LO_us_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf066000)
#define is_v1_LO_us_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf069000)
#define is_v1_LO_us_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06b000)
#define is_v1_LO_us_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06d000)
#define is_v1_LO_us_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06c000)
#define is_v1_LO_us_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06a000)
#define is_v1_LO_us_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf068000)
#define is_v1_LO_u_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe061000)
#define is_v1_LO_u_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe063000)
#define is_v1_LO_u_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe065000)
#define is_v1_LO_u_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe064000)
#define is_v1_LO_u_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe062000)
#define is_v1_LO_u_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe060000)
#define is_v1_LO_u_even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe067000)
#define is_v1_LO_u_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe066000)
#define is_v1_LO_u_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe069000)
#define is_v1_LO_u_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06b000)
#define is_v1_LO_u_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06d000)
#define is_v1_LO_u_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06c000)
#define is_v1_LO_u_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06a000)
#define is_v1_LO_u_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe068000)
#define is_v1_LO__deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc061000)
#define is_v1_LO__dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc063000)
#define is_v1_LO__dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc065000)
#define is_v1_LO__dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc064000)
#define is_v1_LO__dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc062000)
#define is_v1_LO__dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc060000)
#define is_v1_LO__even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc067000)
#define is_v1_LO__odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc066000)
#define is_v1_LO__weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc069000)
#define is_v1_LO__wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06b000)
#define is_v1_LO__wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06d000)
#define is_v1_LO__wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06c000)
#define is_v1_LO__wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06a000)
#define is_v1_LO__wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc068000)
#define v1_LO_s__registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3d06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_s_xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3d06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_us__registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3f06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_us_xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3f06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_u__registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3e06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO_u_xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3e06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO___registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3c06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LO__xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3c06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LO_s__registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06e000)
#define is_v1_LO_s_xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06f000)
#define is_v1_LO_us__registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06e000)
#define is_v1_LO_us_xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06f000)
#define is_v1_LO_u__registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06e000)
#define is_v1_LO_u_xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06f000)
#define is_v1_LO___registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06e000)
#define is_v1_LO__xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06f000)
#define v1_LV__registerA_signed10_registerZ_simple_insn(registerA,signed10,registerZ) (unsigned int)(0x20010000 | (((int)(registerA) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_registerA_signed10_registerZ_simple_insn(registerA,signed10,registerZ) (unsigned int)(0x21010000 | (((int)(registerA) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LV__registerA_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x20010000)
#define is_v1_LV_s_registerA_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x21010000)
#define v1_LV__registerA_upper27_lower10_registerZ_double_insn_0(registerA,upper27_lower10,registerZ) (unsigned int)(0xa0010000 | (((int)(registerA) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV__registerA_upper27_lower10_registerZ_double_insn_1(registerA,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_s_registerA_upper27_lower10_registerZ_double_insn_0(registerA,upper27_lower10,registerZ) (unsigned int)(0xa1010000 | (((int)(registerA) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_registerA_upper27_lower10_registerZ_double_insn_1(registerA,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LV__registerA_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0010000)
#define is_v1_LV_s_registerA_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1010000)
#define v1_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn_0(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(0xa0010000 | (((int)(registerA) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn_1(registerA,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn_2(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn_0(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(0xa1010000 | (((int)(registerA) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn_1(registerA,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn_2(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0010000)
#define is_v1_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1010000)
#define v1_LV_s_deqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_even_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_odd_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_weqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__deqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__dgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__dgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__dlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__dltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__dnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__even_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__odd_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__weqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__wgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__wgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__wlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__wltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_LV__wnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_LV_s_deqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21031000)
#define is_v1_LV_s_dgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21033000)
#define is_v1_LV_s_dgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21035000)
#define is_v1_LV_s_dlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21034000)
#define is_v1_LV_s_dltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21032000)
#define is_v1_LV_s_dnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21030000)
#define is_v1_LV_s_even_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21037000)
#define is_v1_LV_s_odd_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21036000)
#define is_v1_LV_s_weqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21039000)
#define is_v1_LV_s_wgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103b000)
#define is_v1_LV_s_wgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103d000)
#define is_v1_LV_s_wlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103c000)
#define is_v1_LV_s_wltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103a000)
#define is_v1_LV_s_wnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21038000)
#define is_v1_LV__deqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20031000)
#define is_v1_LV__dgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20033000)
#define is_v1_LV__dgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20035000)
#define is_v1_LV__dlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20034000)
#define is_v1_LV__dltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20032000)
#define is_v1_LV__dnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20030000)
#define is_v1_LV__even_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20037000)
#define is_v1_LV__odd_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20036000)
#define is_v1_LV__weqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20039000)
#define is_v1_LV__wgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003b000)
#define is_v1_LV__wgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003d000)
#define is_v1_LV__wlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003c000)
#define is_v1_LV__wltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003a000)
#define is_v1_LV__wnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20038000)
#define v1_LV_s_deqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_even_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_odd_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_weqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__deqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__even_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__odd_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__weqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LV_s_deqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1031000)
#define is_v1_LV_s_dgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1033000)
#define is_v1_LV_s_dgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1035000)
#define is_v1_LV_s_dlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1034000)
#define is_v1_LV_s_dltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1032000)
#define is_v1_LV_s_dnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1030000)
#define is_v1_LV_s_even_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1037000)
#define is_v1_LV_s_odd_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1036000)
#define is_v1_LV_s_weqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1039000)
#define is_v1_LV_s_wgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103b000)
#define is_v1_LV_s_wgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103d000)
#define is_v1_LV_s_wlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103c000)
#define is_v1_LV_s_wltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103a000)
#define is_v1_LV_s_wnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1038000)
#define is_v1_LV__deqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0031000)
#define is_v1_LV__dgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0033000)
#define is_v1_LV__dgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0035000)
#define is_v1_LV__dlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0034000)
#define is_v1_LV__dltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0032000)
#define is_v1_LV__dnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0030000)
#define is_v1_LV__even_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0037000)
#define is_v1_LV__odd_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0036000)
#define is_v1_LV__weqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0039000)
#define is_v1_LV__wgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003b000)
#define is_v1_LV__wgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003d000)
#define is_v1_LV__wlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003c000)
#define is_v1_LV__wltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003a000)
#define is_v1_LV__wnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0038000)
#define v1_LV_s_deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_even_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_even_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_s_wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__even_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__even_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV__wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV__wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LV_s_deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1031000)
#define is_v1_LV_s_dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1033000)
#define is_v1_LV_s_dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1035000)
#define is_v1_LV_s_dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1034000)
#define is_v1_LV_s_dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1032000)
#define is_v1_LV_s_dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1030000)
#define is_v1_LV_s_even_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1037000)
#define is_v1_LV_s_odd_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1036000)
#define is_v1_LV_s_weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1039000)
#define is_v1_LV_s_wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103b000)
#define is_v1_LV_s_wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103d000)
#define is_v1_LV_s_wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103c000)
#define is_v1_LV_s_wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103a000)
#define is_v1_LV_s_wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1038000)
#define is_v1_LV__deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0031000)
#define is_v1_LV__dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0033000)
#define is_v1_LV__dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0035000)
#define is_v1_LV__dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0034000)
#define is_v1_LV__dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0032000)
#define is_v1_LV__dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0030000)
#define is_v1_LV__even_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0037000)
#define is_v1_LV__odd_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0036000)
#define is_v1_LV__weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0039000)
#define is_v1_LV__wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003b000)
#define is_v1_LV__wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003d000)
#define is_v1_LV__wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003c000)
#define is_v1_LV__wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003a000)
#define is_v1_LV__wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0038000)
#define v1_LV_s__registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2103e000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_s_xs_registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2103f000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV___registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2003e000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV__xs_registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2003f000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LV_s__registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103e000)
#define is_v1_LV_s_xs_registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103f000)
#define is_v1_LV___registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003e000)
#define is_v1_LV__xs_registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003f000)
#define v1_LV_c0__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x22010000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x23010000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x22050000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x23050000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x22090000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x23090000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x220d0000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x230d0000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LV_c0__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x22010000)
#define is_v1_LV_c0_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x23010000)
#define is_v1_LV_c1__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x22050000)
#define is_v1_LV_c1_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x23050000)
#define is_v1_LV_c2__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x22090000)
#define is_v1_LV_c2_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x23090000)
#define is_v1_LV_c3__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x220d0000)
#define is_v1_LV_c3_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x230d0000)
#define v1_LV_c0__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa2010000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_c0_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa3010000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_c1__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa2050000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_c1_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa3050000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_c2__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa2090000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_c2_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa3090000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_c3__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa20d0000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_LV_c3_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa30d0000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_LV_c0__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2010000)
#define is_v1_LV_c0_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3010000)
#define is_v1_LV_c1__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2050000)
#define is_v1_LV_c1_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3050000)
#define is_v1_LV_c2__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2090000)
#define is_v1_LV_c2_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3090000)
#define is_v1_LV_c3__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa20d0000)
#define is_v1_LV_c3_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa30d0000)
#define v1_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2010000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3010000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2050000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3050000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2090000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3090000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa20d0000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa30d0000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2010000)
#define is_v1_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3010000)
#define is_v1_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2050000)
#define is_v1_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3050000)
#define is_v1_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2090000)
#define is_v1_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3090000)
#define is_v1_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa20d0000)
#define is_v1_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa30d0000)
#define v1_LV_c0_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define is_v1_LV_c0_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23031000)
#define is_v1_LV_c0_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23033000)
#define is_v1_LV_c0_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23035000)
#define is_v1_LV_c0_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23034000)
#define is_v1_LV_c0_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23032000)
#define is_v1_LV_c0_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23030000)
#define is_v1_LV_c0_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23037000)
#define is_v1_LV_c0_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23036000)
#define is_v1_LV_c0_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23039000)
#define is_v1_LV_c0_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303b000)
#define is_v1_LV_c0_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303d000)
#define is_v1_LV_c0_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303c000)
#define is_v1_LV_c0_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303a000)
#define is_v1_LV_c0_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23038000)
#define is_v1_LV_c0__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22031000)
#define is_v1_LV_c0__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22033000)
#define is_v1_LV_c0__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22035000)
#define is_v1_LV_c0__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22034000)
#define is_v1_LV_c0__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22032000)
#define is_v1_LV_c0__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22030000)
#define is_v1_LV_c0__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22037000)
#define is_v1_LV_c0__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22036000)
#define is_v1_LV_c0__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22039000)
#define is_v1_LV_c0__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203b000)
#define is_v1_LV_c0__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203d000)
#define is_v1_LV_c0__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203c000)
#define is_v1_LV_c0__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203a000)
#define is_v1_LV_c0__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22038000)
#define is_v1_LV_c1_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23071000)
#define is_v1_LV_c1_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23073000)
#define is_v1_LV_c1_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23075000)
#define is_v1_LV_c1_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23074000)
#define is_v1_LV_c1_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23072000)
#define is_v1_LV_c1_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23070000)
#define is_v1_LV_c1_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23077000)
#define is_v1_LV_c1_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23076000)
#define is_v1_LV_c1_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23079000)
#define is_v1_LV_c1_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307b000)
#define is_v1_LV_c1_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307d000)
#define is_v1_LV_c1_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307c000)
#define is_v1_LV_c1_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307a000)
#define is_v1_LV_c1_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23078000)
#define is_v1_LV_c1__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22071000)
#define is_v1_LV_c1__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22073000)
#define is_v1_LV_c1__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22075000)
#define is_v1_LV_c1__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22074000)
#define is_v1_LV_c1__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22072000)
#define is_v1_LV_c1__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22070000)
#define is_v1_LV_c1__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22077000)
#define is_v1_LV_c1__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22076000)
#define is_v1_LV_c1__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22079000)
#define is_v1_LV_c1__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207b000)
#define is_v1_LV_c1__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207d000)
#define is_v1_LV_c1__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207c000)
#define is_v1_LV_c1__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207a000)
#define is_v1_LV_c1__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22078000)
#define is_v1_LV_c2_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b1000)
#define is_v1_LV_c2_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b3000)
#define is_v1_LV_c2_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b5000)
#define is_v1_LV_c2_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b4000)
#define is_v1_LV_c2_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b2000)
#define is_v1_LV_c2_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b0000)
#define is_v1_LV_c2_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b7000)
#define is_v1_LV_c2_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b6000)
#define is_v1_LV_c2_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b9000)
#define is_v1_LV_c2_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bb000)
#define is_v1_LV_c2_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bd000)
#define is_v1_LV_c2_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bc000)
#define is_v1_LV_c2_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230ba000)
#define is_v1_LV_c2_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b8000)
#define is_v1_LV_c2__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b1000)
#define is_v1_LV_c2__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b3000)
#define is_v1_LV_c2__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b5000)
#define is_v1_LV_c2__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b4000)
#define is_v1_LV_c2__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b2000)
#define is_v1_LV_c2__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b0000)
#define is_v1_LV_c2__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b7000)
#define is_v1_LV_c2__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b6000)
#define is_v1_LV_c2__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b9000)
#define is_v1_LV_c2__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bb000)
#define is_v1_LV_c2__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bd000)
#define is_v1_LV_c2__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bc000)
#define is_v1_LV_c2__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220ba000)
#define is_v1_LV_c2__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b8000)
#define is_v1_LV_c3_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f1000)
#define is_v1_LV_c3_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f3000)
#define is_v1_LV_c3_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f5000)
#define is_v1_LV_c3_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f4000)
#define is_v1_LV_c3_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f2000)
#define is_v1_LV_c3_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f0000)
#define is_v1_LV_c3_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f7000)
#define is_v1_LV_c3_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f6000)
#define is_v1_LV_c3_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f9000)
#define is_v1_LV_c3_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fb000)
#define is_v1_LV_c3_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fd000)
#define is_v1_LV_c3_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fc000)
#define is_v1_LV_c3_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fa000)
#define is_v1_LV_c3_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f8000)
#define is_v1_LV_c3__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f1000)
#define is_v1_LV_c3__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f3000)
#define is_v1_LV_c3__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f5000)
#define is_v1_LV_c3__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f4000)
#define is_v1_LV_c3__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f2000)
#define is_v1_LV_c3__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f0000)
#define is_v1_LV_c3__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f7000)
#define is_v1_LV_c3__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f6000)
#define is_v1_LV_c3__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f9000)
#define is_v1_LV_c3__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fb000)
#define is_v1_LV_c3__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fd000)
#define is_v1_LV_c3__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fc000)
#define is_v1_LV_c3__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fa000)
#define is_v1_LV_c3__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f8000)
#define v1_LV_c0_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_LV_c0_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3031000)
#define is_v1_LV_c0_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3033000)
#define is_v1_LV_c0_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3035000)
#define is_v1_LV_c0_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3034000)
#define is_v1_LV_c0_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3032000)
#define is_v1_LV_c0_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3030000)
#define is_v1_LV_c0_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3037000)
#define is_v1_LV_c0_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3036000)
#define is_v1_LV_c0_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3039000)
#define is_v1_LV_c0_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303b000)
#define is_v1_LV_c0_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303d000)
#define is_v1_LV_c0_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303c000)
#define is_v1_LV_c0_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303a000)
#define is_v1_LV_c0_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3038000)
#define is_v1_LV_c0__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2031000)
#define is_v1_LV_c0__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2033000)
#define is_v1_LV_c0__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2035000)
#define is_v1_LV_c0__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2034000)
#define is_v1_LV_c0__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2032000)
#define is_v1_LV_c0__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2030000)
#define is_v1_LV_c0__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2037000)
#define is_v1_LV_c0__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2036000)
#define is_v1_LV_c0__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2039000)
#define is_v1_LV_c0__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203b000)
#define is_v1_LV_c0__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203d000)
#define is_v1_LV_c0__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203c000)
#define is_v1_LV_c0__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203a000)
#define is_v1_LV_c0__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2038000)
#define is_v1_LV_c1_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3071000)
#define is_v1_LV_c1_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3073000)
#define is_v1_LV_c1_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3075000)
#define is_v1_LV_c1_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3074000)
#define is_v1_LV_c1_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3072000)
#define is_v1_LV_c1_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3070000)
#define is_v1_LV_c1_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3077000)
#define is_v1_LV_c1_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3076000)
#define is_v1_LV_c1_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3079000)
#define is_v1_LV_c1_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307b000)
#define is_v1_LV_c1_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307d000)
#define is_v1_LV_c1_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307c000)
#define is_v1_LV_c1_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307a000)
#define is_v1_LV_c1_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3078000)
#define is_v1_LV_c1__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2071000)
#define is_v1_LV_c1__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2073000)
#define is_v1_LV_c1__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2075000)
#define is_v1_LV_c1__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2074000)
#define is_v1_LV_c1__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2072000)
#define is_v1_LV_c1__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2070000)
#define is_v1_LV_c1__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2077000)
#define is_v1_LV_c1__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2076000)
#define is_v1_LV_c1__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2079000)
#define is_v1_LV_c1__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207b000)
#define is_v1_LV_c1__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207d000)
#define is_v1_LV_c1__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207c000)
#define is_v1_LV_c1__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207a000)
#define is_v1_LV_c1__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2078000)
#define is_v1_LV_c2_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b1000)
#define is_v1_LV_c2_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b3000)
#define is_v1_LV_c2_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b5000)
#define is_v1_LV_c2_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b4000)
#define is_v1_LV_c2_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b2000)
#define is_v1_LV_c2_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b0000)
#define is_v1_LV_c2_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b7000)
#define is_v1_LV_c2_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b6000)
#define is_v1_LV_c2_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b9000)
#define is_v1_LV_c2_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bb000)
#define is_v1_LV_c2_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bd000)
#define is_v1_LV_c2_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bc000)
#define is_v1_LV_c2_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30ba000)
#define is_v1_LV_c2_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b8000)
#define is_v1_LV_c2__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b1000)
#define is_v1_LV_c2__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b3000)
#define is_v1_LV_c2__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b5000)
#define is_v1_LV_c2__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b4000)
#define is_v1_LV_c2__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b2000)
#define is_v1_LV_c2__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b0000)
#define is_v1_LV_c2__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b7000)
#define is_v1_LV_c2__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b6000)
#define is_v1_LV_c2__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b9000)
#define is_v1_LV_c2__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bb000)
#define is_v1_LV_c2__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bd000)
#define is_v1_LV_c2__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bc000)
#define is_v1_LV_c2__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20ba000)
#define is_v1_LV_c2__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b8000)
#define is_v1_LV_c3_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f1000)
#define is_v1_LV_c3_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f3000)
#define is_v1_LV_c3_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f5000)
#define is_v1_LV_c3_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f4000)
#define is_v1_LV_c3_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f2000)
#define is_v1_LV_c3_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f0000)
#define is_v1_LV_c3_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f7000)
#define is_v1_LV_c3_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f6000)
#define is_v1_LV_c3_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f9000)
#define is_v1_LV_c3_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fb000)
#define is_v1_LV_c3_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fd000)
#define is_v1_LV_c3_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fc000)
#define is_v1_LV_c3_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fa000)
#define is_v1_LV_c3_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f8000)
#define is_v1_LV_c3__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f1000)
#define is_v1_LV_c3__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f3000)
#define is_v1_LV_c3__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f5000)
#define is_v1_LV_c3__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f4000)
#define is_v1_LV_c3__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f2000)
#define is_v1_LV_c3__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f0000)
#define is_v1_LV_c3__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f7000)
#define is_v1_LV_c3__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f6000)
#define is_v1_LV_c3__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f9000)
#define is_v1_LV_c3__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fb000)
#define is_v1_LV_c3__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fd000)
#define is_v1_LV_c3__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fc000)
#define is_v1_LV_c3__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fa000)
#define is_v1_LV_c3__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f8000)
#define v1_LV_c0_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c0__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c1__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c2__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_LV_c3__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_LV_c0_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3031000)
#define is_v1_LV_c0_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3033000)
#define is_v1_LV_c0_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3035000)
#define is_v1_LV_c0_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3034000)
#define is_v1_LV_c0_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3032000)
#define is_v1_LV_c0_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3030000)
#define is_v1_LV_c0_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3037000)
#define is_v1_LV_c0_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3036000)
#define is_v1_LV_c0_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3039000)
#define is_v1_LV_c0_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303b000)
#define is_v1_LV_c0_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303d000)
#define is_v1_LV_c0_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303c000)
#define is_v1_LV_c0_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303a000)
#define is_v1_LV_c0_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3038000)
#define is_v1_LV_c0__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2031000)
#define is_v1_LV_c0__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2033000)
#define is_v1_LV_c0__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2035000)
#define is_v1_LV_c0__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2034000)
#define is_v1_LV_c0__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2032000)
#define is_v1_LV_c0__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2030000)
#define is_v1_LV_c0__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2037000)
#define is_v1_LV_c0__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2036000)
#define is_v1_LV_c0__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2039000)
#define is_v1_LV_c0__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203b000)
#define is_v1_LV_c0__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203d000)
#define is_v1_LV_c0__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203c000)
#define is_v1_LV_c0__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203a000)
#define is_v1_LV_c0__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2038000)
#define is_v1_LV_c1_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3071000)
#define is_v1_LV_c1_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3073000)
#define is_v1_LV_c1_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3075000)
#define is_v1_LV_c1_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3074000)
#define is_v1_LV_c1_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3072000)
#define is_v1_LV_c1_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3070000)
#define is_v1_LV_c1_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3077000)
#define is_v1_LV_c1_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3076000)
#define is_v1_LV_c1_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3079000)
#define is_v1_LV_c1_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307b000)
#define is_v1_LV_c1_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307d000)
#define is_v1_LV_c1_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307c000)
#define is_v1_LV_c1_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307a000)
#define is_v1_LV_c1_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3078000)
#define is_v1_LV_c1__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2071000)
#define is_v1_LV_c1__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2073000)
#define is_v1_LV_c1__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2075000)
#define is_v1_LV_c1__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2074000)
#define is_v1_LV_c1__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2072000)
#define is_v1_LV_c1__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2070000)
#define is_v1_LV_c1__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2077000)
#define is_v1_LV_c1__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2076000)
#define is_v1_LV_c1__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2079000)
#define is_v1_LV_c1__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207b000)
#define is_v1_LV_c1__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207d000)
#define is_v1_LV_c1__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207c000)
#define is_v1_LV_c1__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207a000)
#define is_v1_LV_c1__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2078000)
#define is_v1_LV_c2_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b1000)
#define is_v1_LV_c2_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b3000)
#define is_v1_LV_c2_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b5000)
#define is_v1_LV_c2_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b4000)
#define is_v1_LV_c2_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b2000)
#define is_v1_LV_c2_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b0000)
#define is_v1_LV_c2_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b7000)
#define is_v1_LV_c2_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b6000)
#define is_v1_LV_c2_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b9000)
#define is_v1_LV_c2_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bb000)
#define is_v1_LV_c2_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bd000)
#define is_v1_LV_c2_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bc000)
#define is_v1_LV_c2_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30ba000)
#define is_v1_LV_c2_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b8000)
#define is_v1_LV_c2__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b1000)
#define is_v1_LV_c2__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b3000)
#define is_v1_LV_c2__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b5000)
#define is_v1_LV_c2__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b4000)
#define is_v1_LV_c2__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b2000)
#define is_v1_LV_c2__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b0000)
#define is_v1_LV_c2__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b7000)
#define is_v1_LV_c2__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b6000)
#define is_v1_LV_c2__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b9000)
#define is_v1_LV_c2__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bb000)
#define is_v1_LV_c2__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bd000)
#define is_v1_LV_c2__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bc000)
#define is_v1_LV_c2__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20ba000)
#define is_v1_LV_c2__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b8000)
#define is_v1_LV_c3_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f1000)
#define is_v1_LV_c3_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f3000)
#define is_v1_LV_c3_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f5000)
#define is_v1_LV_c3_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f4000)
#define is_v1_LV_c3_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f2000)
#define is_v1_LV_c3_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f0000)
#define is_v1_LV_c3_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f7000)
#define is_v1_LV_c3_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f6000)
#define is_v1_LV_c3_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f9000)
#define is_v1_LV_c3_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fb000)
#define is_v1_LV_c3_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fd000)
#define is_v1_LV_c3_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fc000)
#define is_v1_LV_c3_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fa000)
#define is_v1_LV_c3_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f8000)
#define is_v1_LV_c3__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f1000)
#define is_v1_LV_c3__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f3000)
#define is_v1_LV_c3__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f5000)
#define is_v1_LV_c3__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f4000)
#define is_v1_LV_c3__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f2000)
#define is_v1_LV_c3__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f0000)
#define is_v1_LV_c3__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f7000)
#define is_v1_LV_c3__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f6000)
#define is_v1_LV_c3__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f9000)
#define is_v1_LV_c3__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fb000)
#define is_v1_LV_c3__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fd000)
#define is_v1_LV_c3__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fc000)
#define is_v1_LV_c3__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fa000)
#define is_v1_LV_c3__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f8000)
#define v1_LV_c0_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2303e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2303f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2203e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c0__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2203f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2307e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2307f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2207e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c1__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2207f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230be000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230bf000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220be000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c2__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220bf000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230fe000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230ff000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220fe000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_LV_c3__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220ff000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_LV_c0_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303e000)
#define is_v1_LV_c0_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303f000)
#define is_v1_LV_c0___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203e000)
#define is_v1_LV_c0__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203f000)
#define is_v1_LV_c1_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307e000)
#define is_v1_LV_c1_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307f000)
#define is_v1_LV_c1___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207e000)
#define is_v1_LV_c1__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207f000)
#define is_v1_LV_c2_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230be000)
#define is_v1_LV_c2_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bf000)
#define is_v1_LV_c2___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220be000)
#define is_v1_LV_c2__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bf000)
#define is_v1_LV_c3_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fe000)
#define is_v1_LV_c3_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230ff000)
#define is_v1_LV_c3___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fe000)
#define is_v1_LV_c3__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220ff000)
#define v1_SB_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x24010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SB_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x24010000)
#define v1_SB_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa4010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SB_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4010000)
#define v1_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa4010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4010000)
#define v1_SB__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SB__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403e000)
#define is_v1_SB_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403f000)
#define v1_SB_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SB_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24031000)
#define is_v1_SB_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24033000)
#define is_v1_SB_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24035000)
#define is_v1_SB_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24034000)
#define is_v1_SB_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24032000)
#define is_v1_SB_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24030000)
#define is_v1_SB_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24037000)
#define is_v1_SB_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24036000)
#define is_v1_SB_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24039000)
#define is_v1_SB_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403b000)
#define is_v1_SB_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403d000)
#define is_v1_SB_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403c000)
#define is_v1_SB_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403a000)
#define is_v1_SB_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24038000)
#define v1_SB_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SB_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4031000)
#define is_v1_SB_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4033000)
#define is_v1_SB_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4035000)
#define is_v1_SB_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4034000)
#define is_v1_SB_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4032000)
#define is_v1_SB_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4030000)
#define is_v1_SB_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4037000)
#define is_v1_SB_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4036000)
#define is_v1_SB_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4039000)
#define is_v1_SB_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403b000)
#define is_v1_SB_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403d000)
#define is_v1_SB_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403c000)
#define is_v1_SB_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403a000)
#define is_v1_SB_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4038000)
#define v1_SB_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SB_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SB_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_SB_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4031000)
#define is_v1_SB_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4033000)
#define is_v1_SB_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4035000)
#define is_v1_SB_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4034000)
#define is_v1_SB_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4032000)
#define is_v1_SB_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4030000)
#define is_v1_SB_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4037000)
#define is_v1_SB_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4036000)
#define is_v1_SB_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4039000)
#define is_v1_SB_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403b000)
#define is_v1_SB_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403d000)
#define is_v1_SB_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403c000)
#define is_v1_SB_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403a000)
#define is_v1_SB_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4038000)
#define v1_SH_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x25010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SH_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x25010000)
#define v1_SH_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa5010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SH_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5010000)
#define v1_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa5010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5010000)
#define v1_SH__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SH__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503e000)
#define is_v1_SH_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503f000)
#define v1_SH_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SH_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25031000)
#define is_v1_SH_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25033000)
#define is_v1_SH_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25035000)
#define is_v1_SH_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25034000)
#define is_v1_SH_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25032000)
#define is_v1_SH_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25030000)
#define is_v1_SH_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25037000)
#define is_v1_SH_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25036000)
#define is_v1_SH_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25039000)
#define is_v1_SH_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503b000)
#define is_v1_SH_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503d000)
#define is_v1_SH_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503c000)
#define is_v1_SH_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503a000)
#define is_v1_SH_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25038000)
#define v1_SH_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SH_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5031000)
#define is_v1_SH_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5033000)
#define is_v1_SH_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5035000)
#define is_v1_SH_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5034000)
#define is_v1_SH_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5032000)
#define is_v1_SH_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5030000)
#define is_v1_SH_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5037000)
#define is_v1_SH_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5036000)
#define is_v1_SH_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5039000)
#define is_v1_SH_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503b000)
#define is_v1_SH_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503d000)
#define is_v1_SH_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503c000)
#define is_v1_SH_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503a000)
#define is_v1_SH_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5038000)
#define v1_SH_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SH_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SH_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_SH_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5031000)
#define is_v1_SH_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5033000)
#define is_v1_SH_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5035000)
#define is_v1_SH_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5034000)
#define is_v1_SH_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5032000)
#define is_v1_SH_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5030000)
#define is_v1_SH_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5037000)
#define is_v1_SH_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5036000)
#define is_v1_SH_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5039000)
#define is_v1_SH_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503b000)
#define is_v1_SH_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503d000)
#define is_v1_SH_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503c000)
#define is_v1_SH_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503a000)
#define is_v1_SH_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5038000)
#define v1_SW_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x26010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SW_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x26010000)
#define v1_SW_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa6010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SW_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6010000)
#define v1_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa6010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6010000)
#define v1_SW__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SW__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603e000)
#define is_v1_SW_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603f000)
#define v1_SW_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SW_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26031000)
#define is_v1_SW_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26033000)
#define is_v1_SW_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26035000)
#define is_v1_SW_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26034000)
#define is_v1_SW_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26032000)
#define is_v1_SW_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26030000)
#define is_v1_SW_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26037000)
#define is_v1_SW_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26036000)
#define is_v1_SW_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26039000)
#define is_v1_SW_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603b000)
#define is_v1_SW_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603d000)
#define is_v1_SW_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603c000)
#define is_v1_SW_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603a000)
#define is_v1_SW_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26038000)
#define v1_SW_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SW_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6031000)
#define is_v1_SW_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6033000)
#define is_v1_SW_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6035000)
#define is_v1_SW_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6034000)
#define is_v1_SW_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6032000)
#define is_v1_SW_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6030000)
#define is_v1_SW_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6037000)
#define is_v1_SW_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6036000)
#define is_v1_SW_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6039000)
#define is_v1_SW_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603b000)
#define is_v1_SW_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603d000)
#define is_v1_SW_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603c000)
#define is_v1_SW_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603a000)
#define is_v1_SW_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6038000)
#define v1_SW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_SW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6031000)
#define is_v1_SW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6033000)
#define is_v1_SW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6035000)
#define is_v1_SW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6034000)
#define is_v1_SW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6032000)
#define is_v1_SW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6030000)
#define is_v1_SW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6037000)
#define is_v1_SW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6036000)
#define is_v1_SW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6039000)
#define is_v1_SW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603b000)
#define is_v1_SW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603d000)
#define is_v1_SW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603c000)
#define is_v1_SW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603a000)
#define is_v1_SW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6038000)
#define v1_SD_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x27010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SD_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x27010000)
#define v1_SD_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa7010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SD_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7010000)
#define v1_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa7010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7010000)
#define v1_SD__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SD__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703e000)
#define is_v1_SD_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703f000)
#define v1_SD_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SD_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27031000)
#define is_v1_SD_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27033000)
#define is_v1_SD_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27035000)
#define is_v1_SD_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27034000)
#define is_v1_SD_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27032000)
#define is_v1_SD_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27030000)
#define is_v1_SD_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27037000)
#define is_v1_SD_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27036000)
#define is_v1_SD_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27039000)
#define is_v1_SD_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703b000)
#define is_v1_SD_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703d000)
#define is_v1_SD_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703c000)
#define is_v1_SD_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703a000)
#define is_v1_SD_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27038000)
#define v1_SD_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_SD_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7031000)
#define is_v1_SD_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7033000)
#define is_v1_SD_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7035000)
#define is_v1_SD_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7034000)
#define is_v1_SD_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7032000)
#define is_v1_SD_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7030000)
#define is_v1_SD_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7037000)
#define is_v1_SD_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7036000)
#define is_v1_SD_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7039000)
#define is_v1_SD_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703b000)
#define is_v1_SD_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703d000)
#define is_v1_SD_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703c000)
#define is_v1_SD_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703a000)
#define is_v1_SD_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7038000)
#define v1_SD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_SD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_SD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7031000)
#define is_v1_SD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7033000)
#define is_v1_SD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7035000)
#define is_v1_SD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7034000)
#define is_v1_SD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7032000)
#define is_v1_SD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7030000)
#define is_v1_SD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7037000)
#define is_v1_SD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7036000)
#define is_v1_SD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7039000)
#define is_v1_SD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703b000)
#define is_v1_SD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703d000)
#define is_v1_SD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703c000)
#define is_v1_SD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703a000)
#define is_v1_SD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7038000)
#define v1_SQ_signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x28010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_SQ_signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x28010000)
#define v1_SQ_upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xa8010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SQ_upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xa8010000)
#define v1_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn_0(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(0xa8010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn_1(extend27_upper27_lower10,registerZ,registerU) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn_2(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xa8010000)
#define v1_SQ__registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_SQ__registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803e000)
#define is_v1_SQ_xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803f000)
#define v1_SQ_deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_SQ_deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28031000)
#define is_v1_SQ_dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28033000)
#define is_v1_SQ_dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28035000)
#define is_v1_SQ_dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28034000)
#define is_v1_SQ_dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28032000)
#define is_v1_SQ_dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28030000)
#define is_v1_SQ_even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28037000)
#define is_v1_SQ_odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28036000)
#define is_v1_SQ_weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28039000)
#define is_v1_SQ_wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803b000)
#define is_v1_SQ_wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803d000)
#define is_v1_SQ_wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803c000)
#define is_v1_SQ_wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803a000)
#define is_v1_SQ_wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28038000)
#define v1_SQ_deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_SQ_deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8031000)
#define is_v1_SQ_dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8033000)
#define is_v1_SQ_dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8035000)
#define is_v1_SQ_dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8034000)
#define is_v1_SQ_dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8032000)
#define is_v1_SQ_dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8030000)
#define is_v1_SQ_even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8037000)
#define is_v1_SQ_odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8036000)
#define is_v1_SQ_weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8039000)
#define is_v1_SQ_wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803b000)
#define is_v1_SQ_wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803d000)
#define is_v1_SQ_wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803c000)
#define is_v1_SQ_wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803a000)
#define is_v1_SQ_wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8038000)
#define v1_SQ_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SQ_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_SQ_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_SQ_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8031000)
#define is_v1_SQ_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8033000)
#define is_v1_SQ_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8035000)
#define is_v1_SQ_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8034000)
#define is_v1_SQ_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8032000)
#define is_v1_SQ_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8030000)
#define is_v1_SQ_even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8037000)
#define is_v1_SQ_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8036000)
#define is_v1_SQ_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8039000)
#define is_v1_SQ_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803b000)
#define is_v1_SQ_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803d000)
#define is_v1_SQ_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803c000)
#define is_v1_SQ_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803a000)
#define is_v1_SQ_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8038000)
#define v1_SO_signed10_registerZ_registerV_simple_insn(signed10,registerZ,registerV) (unsigned int)(0x28050000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v1_SO_signed10_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x28050000)
#define v1_SO_upper27_lower10_registerZ_registerV_double_insn_0(upper27_lower10,registerZ,registerV) (unsigned int)(0xa8050000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_upper27_lower10_registerZ_registerV_double_insn_1(upper27_lower10,registerZ,registerV) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SO_upper27_lower10_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa8050000)
#define v1_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn_0(extend27_upper27_lower10,registerZ,registerV) (unsigned int)(0xa8050000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn_1(extend27_upper27_lower10,registerZ,registerV) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn_2(extend27_upper27_lower10,registerZ,registerV) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa8050000)
#define v1_SO__registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_xs_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v1_SO__registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807e000)
#define is_v1_SO_xs_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807f000)
#define v1_SO_deqz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28071000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dgez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28073000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dgtz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28075000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dlez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28074000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dltz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28072000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dnez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28070000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_even_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28077000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_odd_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28076000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_weqz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28079000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wgez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wgtz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wlez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wltz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wnez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28078000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v1_SO_deqz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28071000)
#define is_v1_SO_dgez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28073000)
#define is_v1_SO_dgtz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28075000)
#define is_v1_SO_dlez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28074000)
#define is_v1_SO_dltz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28072000)
#define is_v1_SO_dnez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28070000)
#define is_v1_SO_even_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28077000)
#define is_v1_SO_odd_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28076000)
#define is_v1_SO_weqz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28079000)
#define is_v1_SO_wgez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807b000)
#define is_v1_SO_wgtz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807d000)
#define is_v1_SO_wlez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807c000)
#define is_v1_SO_wltz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807a000)
#define is_v1_SO_wnez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28078000)
#define v1_SO_deqz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8071000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dgez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8073000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dgtz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8075000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dlez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8074000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dltz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8072000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dnez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8070000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_even_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8077000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_odd_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8076000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_weqz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8079000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wgez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wgtz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wlez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wltz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wnez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8078000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v1_SO_deqz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8071000)
#define is_v1_SO_dgez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8073000)
#define is_v1_SO_dgtz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8075000)
#define is_v1_SO_dlez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8074000)
#define is_v1_SO_dltz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8072000)
#define is_v1_SO_dnez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8070000)
#define is_v1_SO_even_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8077000)
#define is_v1_SO_odd_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8076000)
#define is_v1_SO_weqz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8079000)
#define is_v1_SO_wgez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807b000)
#define is_v1_SO_wgtz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807d000)
#define is_v1_SO_wlez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807c000)
#define is_v1_SO_wltz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807a000)
#define is_v1_SO_wnez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8078000)
#define v1_SO_deqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_deqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_dgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_dgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_dlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_dltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_dnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_dnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_even_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_even_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_odd_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_odd_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_weqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_weqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_wgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_wgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_wlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_wltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SO_wnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v1_SO_wnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_SO_deqz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8071000)
#define is_v1_SO_dgez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8073000)
#define is_v1_SO_dgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8075000)
#define is_v1_SO_dlez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8074000)
#define is_v1_SO_dltz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8072000)
#define is_v1_SO_dnez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8070000)
#define is_v1_SO_even_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8077000)
#define is_v1_SO_odd_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8076000)
#define is_v1_SO_weqz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8079000)
#define is_v1_SO_wgez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807b000)
#define is_v1_SO_wgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807d000)
#define is_v1_SO_wlez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807c000)
#define is_v1_SO_wltz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807a000)
#define is_v1_SO_wnez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8078000)
#define v1_SV_signed10_registerZ_registerE_simple_insn(signed10,registerZ,registerE) (unsigned int)(0x29010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v1_SV_signed10_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x29010000)
#define v1_SV_upper27_lower10_registerZ_registerE_double_insn_0(upper27_lower10,registerZ,registerE) (unsigned int)(0xa9010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_upper27_lower10_registerZ_registerE_double_insn_1(upper27_lower10,registerZ,registerE) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SV_upper27_lower10_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9010000)
#define v1_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn_0(extend27_upper27_lower10,registerZ,registerE) (unsigned int)(0xa9010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn_1(extend27_upper27_lower10,registerZ,registerE) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn_2(extend27_upper27_lower10,registerZ,registerE) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9010000)
#define v1_SV__registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_xs_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v1_SV__registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903e000)
#define is_v1_SV_xs_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903f000)
#define v1_SV_deqz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dgez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dgtz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dlez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dltz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dnez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_even_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_odd_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_weqz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wgez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wgtz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wlez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wltz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wnez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v1_SV_deqz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29031000)
#define is_v1_SV_dgez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29033000)
#define is_v1_SV_dgtz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29035000)
#define is_v1_SV_dlez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29034000)
#define is_v1_SV_dltz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29032000)
#define is_v1_SV_dnez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29030000)
#define is_v1_SV_even_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29037000)
#define is_v1_SV_odd_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29036000)
#define is_v1_SV_weqz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29039000)
#define is_v1_SV_wgez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903b000)
#define is_v1_SV_wgtz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903d000)
#define is_v1_SV_wlez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903c000)
#define is_v1_SV_wltz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903a000)
#define is_v1_SV_wnez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29038000)
#define v1_SV_deqz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dgez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dgtz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dlez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dltz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dnez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_even_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_odd_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_weqz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wgez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wgtz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wlez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wltz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wnez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v1_SV_deqz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9031000)
#define is_v1_SV_dgez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9033000)
#define is_v1_SV_dgtz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9035000)
#define is_v1_SV_dlez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9034000)
#define is_v1_SV_dltz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9032000)
#define is_v1_SV_dnez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9030000)
#define is_v1_SV_even_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9037000)
#define is_v1_SV_odd_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9036000)
#define is_v1_SV_weqz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9039000)
#define is_v1_SV_wgez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903b000)
#define is_v1_SV_wgtz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903d000)
#define is_v1_SV_wlez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903c000)
#define is_v1_SV_wltz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903a000)
#define is_v1_SV_wnez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9038000)
#define v1_SV_deqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_deqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_dgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_dgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_dlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_dltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_dnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_dnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_even_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_even_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_odd_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_odd_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_weqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_weqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_wgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_wgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_wlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_wltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_SV_wnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v1_SV_wnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_SV_deqz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9031000)
#define is_v1_SV_dgez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9033000)
#define is_v1_SV_dgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9035000)
#define is_v1_SV_dlez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9034000)
#define is_v1_SV_dltz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9032000)
#define is_v1_SV_dnez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9030000)
#define is_v1_SV_even_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9037000)
#define is_v1_SV_odd_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9036000)
#define is_v1_SV_weqz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9039000)
#define is_v1_SV_wgez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903b000)
#define is_v1_SV_wgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903d000)
#define is_v1_SV_wlez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903c000)
#define is_v1_SV_wltz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903a000)
#define is_v1_SV_wnez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9038000)
#define v1_ACSWAPW_signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x3e010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPW_signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3e010000)
#define v1_ACSWAPW_upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xbe010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ACSWAPW_upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbe010000)
#define v1_ACSWAPW_extend27_upper27_lower10_registerZ_registerU_triple_insn_0(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(0xbe010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_extend27_upper27_lower10_registerZ_registerU_triple_insn_1(extend27_upper27_lower10,registerZ,registerU) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ACSWAPW_extend27_upper27_lower10_registerZ_registerU_triple_insn_2(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ACSWAPW_extend27_upper27_lower10_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbe010000)
#define v1_ACSWAPW__registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPW__registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03e000)
#define is_v1_ACSWAPW_xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03f000)
#define v1_ACSWAPW_deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPW_deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e031000)
#define is_v1_ACSWAPW_dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e033000)
#define is_v1_ACSWAPW_dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e035000)
#define is_v1_ACSWAPW_dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e034000)
#define is_v1_ACSWAPW_dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e032000)
#define is_v1_ACSWAPW_dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e030000)
#define is_v1_ACSWAPW_even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e037000)
#define is_v1_ACSWAPW_odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e036000)
#define is_v1_ACSWAPW_weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e039000)
#define is_v1_ACSWAPW_wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03b000)
#define is_v1_ACSWAPW_wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03d000)
#define is_v1_ACSWAPW_wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03c000)
#define is_v1_ACSWAPW_wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03a000)
#define is_v1_ACSWAPW_wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e038000)
#define v1_ACSWAPW_deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPW_deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe031000)
#define is_v1_ACSWAPW_dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe033000)
#define is_v1_ACSWAPW_dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe035000)
#define is_v1_ACSWAPW_dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe034000)
#define is_v1_ACSWAPW_dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe032000)
#define is_v1_ACSWAPW_dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe030000)
#define is_v1_ACSWAPW_even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe037000)
#define is_v1_ACSWAPW_odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe036000)
#define is_v1_ACSWAPW_weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe039000)
#define is_v1_ACSWAPW_wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03b000)
#define is_v1_ACSWAPW_wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03d000)
#define is_v1_ACSWAPW_wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03c000)
#define is_v1_ACSWAPW_wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03a000)
#define is_v1_ACSWAPW_wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe038000)
#define v1_ACSWAPW_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPW_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPW_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_ACSWAPW_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe031000)
#define is_v1_ACSWAPW_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe033000)
#define is_v1_ACSWAPW_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe035000)
#define is_v1_ACSWAPW_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe034000)
#define is_v1_ACSWAPW_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe032000)
#define is_v1_ACSWAPW_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe030000)
#define is_v1_ACSWAPW_even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe037000)
#define is_v1_ACSWAPW_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe036000)
#define is_v1_ACSWAPW_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe039000)
#define is_v1_ACSWAPW_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03b000)
#define is_v1_ACSWAPW_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03d000)
#define is_v1_ACSWAPW_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03c000)
#define is_v1_ACSWAPW_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03a000)
#define is_v1_ACSWAPW_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe038000)
#define v1_ACSWAPD_signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x3f010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPD_signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3f010000)
#define v1_ACSWAPD_upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xbf010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ACSWAPD_upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbf010000)
#define v1_ACSWAPD_extend27_upper27_lower10_registerZ_registerU_triple_insn_0(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(0xbf010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_extend27_upper27_lower10_registerZ_registerU_triple_insn_1(extend27_upper27_lower10,registerZ,registerU) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ACSWAPD_extend27_upper27_lower10_registerZ_registerU_triple_insn_2(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ACSWAPD_extend27_upper27_lower10_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbf010000)
#define v1_ACSWAPD__registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPD__registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03e000)
#define is_v1_ACSWAPD_xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03f000)
#define v1_ACSWAPD_deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPD_deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f031000)
#define is_v1_ACSWAPD_dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f033000)
#define is_v1_ACSWAPD_dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f035000)
#define is_v1_ACSWAPD_dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f034000)
#define is_v1_ACSWAPD_dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f032000)
#define is_v1_ACSWAPD_dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f030000)
#define is_v1_ACSWAPD_even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f037000)
#define is_v1_ACSWAPD_odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f036000)
#define is_v1_ACSWAPD_weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f039000)
#define is_v1_ACSWAPD_wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03b000)
#define is_v1_ACSWAPD_wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03d000)
#define is_v1_ACSWAPD_wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03c000)
#define is_v1_ACSWAPD_wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03a000)
#define is_v1_ACSWAPD_wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f038000)
#define v1_ACSWAPD_deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v1_ACSWAPD_deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf031000)
#define is_v1_ACSWAPD_dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf033000)
#define is_v1_ACSWAPD_dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf035000)
#define is_v1_ACSWAPD_dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf034000)
#define is_v1_ACSWAPD_dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf032000)
#define is_v1_ACSWAPD_dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf030000)
#define is_v1_ACSWAPD_even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf037000)
#define is_v1_ACSWAPD_odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf036000)
#define is_v1_ACSWAPD_weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf039000)
#define is_v1_ACSWAPD_wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03b000)
#define is_v1_ACSWAPD_wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03d000)
#define is_v1_ACSWAPD_wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03c000)
#define is_v1_ACSWAPD_wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03a000)
#define is_v1_ACSWAPD_wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf038000)
#define v1_ACSWAPD_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ACSWAPD_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v1_ACSWAPD_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_ACSWAPD_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf031000)
#define is_v1_ACSWAPD_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf033000)
#define is_v1_ACSWAPD_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf035000)
#define is_v1_ACSWAPD_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf034000)
#define is_v1_ACSWAPD_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf032000)
#define is_v1_ACSWAPD_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf030000)
#define is_v1_ACSWAPD_even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf037000)
#define is_v1_ACSWAPD_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf036000)
#define is_v1_ACSWAPD_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf039000)
#define is_v1_ACSWAPD_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03b000)
#define is_v1_ACSWAPD_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03d000)
#define is_v1_ACSWAPD_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03c000)
#define is_v1_ACSWAPD_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03a000)
#define is_v1_ACSWAPD_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf038000)
#define v1_ALCLRW_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2a010000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRW_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2a010000)
#define v1_ALCLRW_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xaa010000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ALCLRW_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa010000)
#define v1_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xaa010000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa010000)
#define v1_ALCLRW__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a03e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a03f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRW__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03e000)
#define is_v1_ALCLRW_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03f000)
#define v1_ALCLRW_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRW_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a031000)
#define is_v1_ALCLRW_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a033000)
#define is_v1_ALCLRW_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a035000)
#define is_v1_ALCLRW_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a034000)
#define is_v1_ALCLRW_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a032000)
#define is_v1_ALCLRW_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a030000)
#define is_v1_ALCLRW_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a037000)
#define is_v1_ALCLRW_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a036000)
#define is_v1_ALCLRW_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a039000)
#define is_v1_ALCLRW_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03b000)
#define is_v1_ALCLRW_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03d000)
#define is_v1_ALCLRW_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03c000)
#define is_v1_ALCLRW_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03a000)
#define is_v1_ALCLRW_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a038000)
#define v1_ALCLRW_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRW_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa031000)
#define is_v1_ALCLRW_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa033000)
#define is_v1_ALCLRW_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa035000)
#define is_v1_ALCLRW_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa034000)
#define is_v1_ALCLRW_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa032000)
#define is_v1_ALCLRW_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa030000)
#define is_v1_ALCLRW_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa037000)
#define is_v1_ALCLRW_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa036000)
#define is_v1_ALCLRW_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa039000)
#define is_v1_ALCLRW_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03b000)
#define is_v1_ALCLRW_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03d000)
#define is_v1_ALCLRW_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03c000)
#define is_v1_ALCLRW_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03a000)
#define is_v1_ALCLRW_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa038000)
#define v1_ALCLRW_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRW_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRW_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_ALCLRW_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa031000)
#define is_v1_ALCLRW_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa033000)
#define is_v1_ALCLRW_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa035000)
#define is_v1_ALCLRW_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa034000)
#define is_v1_ALCLRW_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa032000)
#define is_v1_ALCLRW_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa030000)
#define is_v1_ALCLRW_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa037000)
#define is_v1_ALCLRW_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa036000)
#define is_v1_ALCLRW_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa039000)
#define is_v1_ALCLRW_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03b000)
#define is_v1_ALCLRW_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03d000)
#define is_v1_ALCLRW_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03c000)
#define is_v1_ALCLRW_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03a000)
#define is_v1_ALCLRW_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa038000)
#define v1_ALCLRD_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2b010000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRD_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2b010000)
#define v1_ALCLRD_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xab010000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ALCLRD_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab010000)
#define v1_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xab010000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab010000)
#define v1_ALCLRD__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b03e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b03f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRD__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03e000)
#define is_v1_ALCLRD_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03f000)
#define v1_ALCLRD_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRD_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b031000)
#define is_v1_ALCLRD_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b033000)
#define is_v1_ALCLRD_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b035000)
#define is_v1_ALCLRD_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b034000)
#define is_v1_ALCLRD_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b032000)
#define is_v1_ALCLRD_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b030000)
#define is_v1_ALCLRD_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b037000)
#define is_v1_ALCLRD_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b036000)
#define is_v1_ALCLRD_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b039000)
#define is_v1_ALCLRD_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03b000)
#define is_v1_ALCLRD_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03d000)
#define is_v1_ALCLRD_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03c000)
#define is_v1_ALCLRD_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03a000)
#define is_v1_ALCLRD_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b038000)
#define v1_ALCLRD_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_ALCLRD_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab031000)
#define is_v1_ALCLRD_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab033000)
#define is_v1_ALCLRD_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab035000)
#define is_v1_ALCLRD_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab034000)
#define is_v1_ALCLRD_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab032000)
#define is_v1_ALCLRD_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab030000)
#define is_v1_ALCLRD_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab037000)
#define is_v1_ALCLRD_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab036000)
#define is_v1_ALCLRD_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab039000)
#define is_v1_ALCLRD_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03b000)
#define is_v1_ALCLRD_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03d000)
#define is_v1_ALCLRD_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03c000)
#define is_v1_ALCLRD_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03a000)
#define is_v1_ALCLRD_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab038000)
#define v1_ALCLRD_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALCLRD_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_ALCLRD_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_ALCLRD_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab031000)
#define is_v1_ALCLRD_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab033000)
#define is_v1_ALCLRD_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab035000)
#define is_v1_ALCLRD_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab034000)
#define is_v1_ALCLRD_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab032000)
#define is_v1_ALCLRD_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab030000)
#define is_v1_ALCLRD_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab037000)
#define is_v1_ALCLRD_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab036000)
#define is_v1_ALCLRD_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab039000)
#define is_v1_ALCLRD_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03b000)
#define is_v1_ALCLRD_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03d000)
#define is_v1_ALCLRD_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03c000)
#define is_v1_ALCLRD_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03a000)
#define is_v1_ALCLRD_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab038000)
#define v1_ALADDW_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x2e010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDW_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2e010000)
#define v1_ALADDW_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xae010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ALADDW_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae010000)
#define v1_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xae010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae010000)
#define v1_ALADDW__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDW__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03e000)
#define is_v1_ALADDW_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03f000)
#define v1_ALADDW_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDW_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e031000)
#define is_v1_ALADDW_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e033000)
#define is_v1_ALADDW_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e035000)
#define is_v1_ALADDW_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e034000)
#define is_v1_ALADDW_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e032000)
#define is_v1_ALADDW_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e030000)
#define is_v1_ALADDW_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e037000)
#define is_v1_ALADDW_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e036000)
#define is_v1_ALADDW_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e039000)
#define is_v1_ALADDW_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03b000)
#define is_v1_ALADDW_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03d000)
#define is_v1_ALADDW_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03c000)
#define is_v1_ALADDW_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03a000)
#define is_v1_ALADDW_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e038000)
#define v1_ALADDW_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDW_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae031000)
#define is_v1_ALADDW_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae033000)
#define is_v1_ALADDW_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae035000)
#define is_v1_ALADDW_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae034000)
#define is_v1_ALADDW_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae032000)
#define is_v1_ALADDW_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae030000)
#define is_v1_ALADDW_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae037000)
#define is_v1_ALADDW_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae036000)
#define is_v1_ALADDW_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae039000)
#define is_v1_ALADDW_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03b000)
#define is_v1_ALADDW_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03d000)
#define is_v1_ALADDW_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03c000)
#define is_v1_ALADDW_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03a000)
#define is_v1_ALADDW_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae038000)
#define v1_ALADDW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_ALADDW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae031000)
#define is_v1_ALADDW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae033000)
#define is_v1_ALADDW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae035000)
#define is_v1_ALADDW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae034000)
#define is_v1_ALADDW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae032000)
#define is_v1_ALADDW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae030000)
#define is_v1_ALADDW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae037000)
#define is_v1_ALADDW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae036000)
#define is_v1_ALADDW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae039000)
#define is_v1_ALADDW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03b000)
#define is_v1_ALADDW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03d000)
#define is_v1_ALADDW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03c000)
#define is_v1_ALADDW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03a000)
#define is_v1_ALADDW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae038000)
#define v1_ALADDD_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x2f010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDD_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2f010000)
#define v1_ALADDD_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xaf010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ALADDD_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf010000)
#define v1_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xaf010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf010000)
#define v1_ALADDD__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDD__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03e000)
#define is_v1_ALADDD_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03f000)
#define v1_ALADDD_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDD_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f031000)
#define is_v1_ALADDD_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f033000)
#define is_v1_ALADDD_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f035000)
#define is_v1_ALADDD_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f034000)
#define is_v1_ALADDD_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f032000)
#define is_v1_ALADDD_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f030000)
#define is_v1_ALADDD_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f037000)
#define is_v1_ALADDD_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f036000)
#define is_v1_ALADDD_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f039000)
#define is_v1_ALADDD_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03b000)
#define is_v1_ALADDD_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03d000)
#define is_v1_ALADDD_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03c000)
#define is_v1_ALADDD_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03a000)
#define is_v1_ALADDD_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f038000)
#define v1_ALADDD_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v1_ALADDD_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf031000)
#define is_v1_ALADDD_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf033000)
#define is_v1_ALADDD_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf035000)
#define is_v1_ALADDD_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf034000)
#define is_v1_ALADDD_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf032000)
#define is_v1_ALADDD_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf030000)
#define is_v1_ALADDD_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf037000)
#define is_v1_ALADDD_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf036000)
#define is_v1_ALADDD_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf039000)
#define is_v1_ALADDD_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03b000)
#define is_v1_ALADDD_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03d000)
#define is_v1_ALADDD_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03c000)
#define is_v1_ALADDD_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03a000)
#define is_v1_ALADDD_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf038000)
#define v1_ALADDD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_ALADDD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v1_ALADDD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_ALADDD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf031000)
#define is_v1_ALADDD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf033000)
#define is_v1_ALADDD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf035000)
#define is_v1_ALADDD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf034000)
#define is_v1_ALADDD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf032000)
#define is_v1_ALADDD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf030000)
#define is_v1_ALADDD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf037000)
#define is_v1_ALADDD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf036000)
#define is_v1_ALADDD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf039000)
#define is_v1_ALADDD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03b000)
#define is_v1_ALADDD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03d000)
#define is_v1_ALADDD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03c000)
#define is_v1_ALADDD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03a000)
#define is_v1_ALADDD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf038000)
#define v1_DTOUCHL_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3c0c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DTOUCHL_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cff0000) == 0x3c0c0000)
#define v1_DTOUCHL_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbc0c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_DTOUCHL_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcff0000) == 0xbc0c0000)
#define v1_DTOUCHL_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbc0c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_DTOUCHL_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_DTOUCHL_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcff0000) == 0xbc0c0000)
#define v1_DTOUCHL_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DTOUCHL_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0ee000)
#define v1_DTOUCHL_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DTOUCHL_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e1000)
#define is_v1_DTOUCHL_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e3000)
#define is_v1_DTOUCHL_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e5000)
#define is_v1_DTOUCHL_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e4000)
#define is_v1_DTOUCHL_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e2000)
#define is_v1_DTOUCHL_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e0000)
#define is_v1_DTOUCHL_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e7000)
#define is_v1_DTOUCHL_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e6000)
#define is_v1_DTOUCHL_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e9000)
#define is_v1_DTOUCHL_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0eb000)
#define is_v1_DTOUCHL_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0ed000)
#define is_v1_DTOUCHL_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0ec000)
#define is_v1_DTOUCHL_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0ea000)
#define is_v1_DTOUCHL_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c0e8000)
#define v1_DTOUCHL_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_DTOUCHL_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e1000)
#define is_v1_DTOUCHL_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e3000)
#define is_v1_DTOUCHL_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e5000)
#define is_v1_DTOUCHL_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e4000)
#define is_v1_DTOUCHL_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e2000)
#define is_v1_DTOUCHL_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e0000)
#define is_v1_DTOUCHL_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e7000)
#define is_v1_DTOUCHL_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e6000)
#define is_v1_DTOUCHL_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e9000)
#define is_v1_DTOUCHL_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0eb000)
#define is_v1_DTOUCHL_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0ed000)
#define is_v1_DTOUCHL_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0ec000)
#define is_v1_DTOUCHL_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0ea000)
#define is_v1_DTOUCHL_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e8000)
#define v1_DTOUCHL_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DTOUCHL_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DTOUCHL_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_DTOUCHL_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e1000)
#define is_v1_DTOUCHL_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e3000)
#define is_v1_DTOUCHL_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e5000)
#define is_v1_DTOUCHL_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e4000)
#define is_v1_DTOUCHL_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e2000)
#define is_v1_DTOUCHL_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e0000)
#define is_v1_DTOUCHL_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e7000)
#define is_v1_DTOUCHL_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e6000)
#define is_v1_DTOUCHL_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e9000)
#define is_v1_DTOUCHL_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0eb000)
#define is_v1_DTOUCHL_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0ed000)
#define is_v1_DTOUCHL_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0ec000)
#define is_v1_DTOUCHL_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0ea000)
#define is_v1_DTOUCHL_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc0e8000)
#define v1_DINVALL_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3c1c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DINVALL_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cff0000) == 0x3c1c0000)
#define v1_DINVALL_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbc1c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_DINVALL_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcff0000) == 0xbc1c0000)
#define v1_DINVALL_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbc1c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_DINVALL_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_DINVALL_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcff0000) == 0xbc1c0000)
#define v1_DINVALL_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DINVALL_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1ee000)
#define v1_DINVALL_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DINVALL_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e1000)
#define is_v1_DINVALL_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e3000)
#define is_v1_DINVALL_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e5000)
#define is_v1_DINVALL_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e4000)
#define is_v1_DINVALL_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e2000)
#define is_v1_DINVALL_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e0000)
#define is_v1_DINVALL_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e7000)
#define is_v1_DINVALL_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e6000)
#define is_v1_DINVALL_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e9000)
#define is_v1_DINVALL_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1eb000)
#define is_v1_DINVALL_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1ed000)
#define is_v1_DINVALL_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1ec000)
#define is_v1_DINVALL_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1ea000)
#define is_v1_DINVALL_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c1e8000)
#define v1_DINVALL_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_DINVALL_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e1000)
#define is_v1_DINVALL_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e3000)
#define is_v1_DINVALL_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e5000)
#define is_v1_DINVALL_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e4000)
#define is_v1_DINVALL_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e2000)
#define is_v1_DINVALL_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e0000)
#define is_v1_DINVALL_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e7000)
#define is_v1_DINVALL_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e6000)
#define is_v1_DINVALL_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e9000)
#define is_v1_DINVALL_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1eb000)
#define is_v1_DINVALL_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1ed000)
#define is_v1_DINVALL_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1ec000)
#define is_v1_DINVALL_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1ea000)
#define is_v1_DINVALL_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e8000)
#define v1_DINVALL_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DINVALL_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DINVALL_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_DINVALL_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e1000)
#define is_v1_DINVALL_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e3000)
#define is_v1_DINVALL_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e5000)
#define is_v1_DINVALL_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e4000)
#define is_v1_DINVALL_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e2000)
#define is_v1_DINVALL_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e0000)
#define is_v1_DINVALL_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e7000)
#define is_v1_DINVALL_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e6000)
#define is_v1_DINVALL_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e9000)
#define is_v1_DINVALL_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1eb000)
#define is_v1_DINVALL_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1ed000)
#define is_v1_DINVALL_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1ec000)
#define is_v1_DINVALL_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1ea000)
#define is_v1_DINVALL_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc1e8000)
#define v1_DZEROL_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x280d0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DZEROL_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x280d0000)
#define v1_DZEROL_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xa80d0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_DZEROL_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xa80d0000)
#define v1_DZEROL_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xa80d0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_DZEROL_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_DZEROL_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xa80d0000)
#define v1_DZEROL_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280fe000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DZEROL_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280fe000)
#define v1_DZEROL_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280fb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280fd000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280fc000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280fa000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x280f8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_DZEROL_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f1000)
#define is_v1_DZEROL_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f3000)
#define is_v1_DZEROL_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f5000)
#define is_v1_DZEROL_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f4000)
#define is_v1_DZEROL_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f2000)
#define is_v1_DZEROL_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f0000)
#define is_v1_DZEROL_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f7000)
#define is_v1_DZEROL_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f6000)
#define is_v1_DZEROL_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f9000)
#define is_v1_DZEROL_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280fb000)
#define is_v1_DZEROL_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280fd000)
#define is_v1_DZEROL_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280fc000)
#define is_v1_DZEROL_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280fa000)
#define is_v1_DZEROL_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x280f8000)
#define v1_DZEROL_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80fb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80fd000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80fc000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80fa000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xa80f8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_DZEROL_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f1000)
#define is_v1_DZEROL_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f3000)
#define is_v1_DZEROL_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f5000)
#define is_v1_DZEROL_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f4000)
#define is_v1_DZEROL_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f2000)
#define is_v1_DZEROL_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f0000)
#define is_v1_DZEROL_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f7000)
#define is_v1_DZEROL_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f6000)
#define is_v1_DZEROL_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f9000)
#define is_v1_DZEROL_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fb000)
#define is_v1_DZEROL_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fd000)
#define is_v1_DZEROL_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fc000)
#define is_v1_DZEROL_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fa000)
#define is_v1_DZEROL_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f8000)
#define v1_DZEROL_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_DZEROL_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xa80f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_DZEROL_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_DZEROL_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f1000)
#define is_v1_DZEROL_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f3000)
#define is_v1_DZEROL_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f5000)
#define is_v1_DZEROL_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f4000)
#define is_v1_DZEROL_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f2000)
#define is_v1_DZEROL_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f0000)
#define is_v1_DZEROL_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f7000)
#define is_v1_DZEROL_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f6000)
#define is_v1_DZEROL_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f9000)
#define is_v1_DZEROL_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fb000)
#define is_v1_DZEROL_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fd000)
#define is_v1_DZEROL_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fc000)
#define is_v1_DZEROL_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80fa000)
#define is_v1_DZEROL_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xa80f8000)
#define v1_IINVALS_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3c5c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_IINVALS_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cff0000) == 0x3c5c0000)
#define v1_IINVALS_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbc5c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_IINVALS_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcff0000) == 0xbc5c0000)
#define v1_IINVALS_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbc5c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_IINVALS_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_IINVALS_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcff0000) == 0xbc5c0000)
#define v1_IINVALS_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_IINVALS_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5ee000)
#define v1_IINVALS_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v1_IINVALS_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e1000)
#define is_v1_IINVALS_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e3000)
#define is_v1_IINVALS_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e5000)
#define is_v1_IINVALS_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e4000)
#define is_v1_IINVALS_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e2000)
#define is_v1_IINVALS_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e0000)
#define is_v1_IINVALS_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e7000)
#define is_v1_IINVALS_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e6000)
#define is_v1_IINVALS_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e9000)
#define is_v1_IINVALS_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5eb000)
#define is_v1_IINVALS_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5ed000)
#define is_v1_IINVALS_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5ec000)
#define is_v1_IINVALS_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5ea000)
#define is_v1_IINVALS_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7cfff000) == 0x3c5e8000)
#define v1_IINVALS_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v1_IINVALS_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e1000)
#define is_v1_IINVALS_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e3000)
#define is_v1_IINVALS_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e5000)
#define is_v1_IINVALS_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e4000)
#define is_v1_IINVALS_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e2000)
#define is_v1_IINVALS_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e0000)
#define is_v1_IINVALS_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e7000)
#define is_v1_IINVALS_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e6000)
#define is_v1_IINVALS_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e9000)
#define is_v1_IINVALS_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5eb000)
#define is_v1_IINVALS_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5ed000)
#define is_v1_IINVALS_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5ec000)
#define is_v1_IINVALS_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5ea000)
#define is_v1_IINVALS_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e8000)
#define v1_IINVALS_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v1_IINVALS_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v1_IINVALS_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v1_IINVALS_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e1000)
#define is_v1_IINVALS_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e3000)
#define is_v1_IINVALS_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e5000)
#define is_v1_IINVALS_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e4000)
#define is_v1_IINVALS_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e2000)
#define is_v1_IINVALS_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e0000)
#define is_v1_IINVALS_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e7000)
#define is_v1_IINVALS_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e6000)
#define is_v1_IINVALS_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e9000)
#define is_v1_IINVALS_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5eb000)
#define is_v1_IINVALS_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5ed000)
#define is_v1_IINVALS_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5ec000)
#define is_v1_IINVALS_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5ea000)
#define is_v1_IINVALS_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfcfff000) == 0xbc5e8000)
#define is_v1_DINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3f8d0000)
#define is_v1_IINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3f9d0000)
#define is_v1_FENCE_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3fcd0000)
#define is_v1_NOP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f03f000)
#define v1_STSUD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_STSUD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70010000)
#define v1_ADDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71010000)
#define v1_ADDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x61000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ADDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x61000000)
#define v1_ADDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ADDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ADDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1000000)
#define v1_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1000000)
#define v1_SBFD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75010000)
#define v1_SBFD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x65000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_SBFD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x65000000)
#define v1_SBFD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_SBFD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SBFD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5000000)
#define v1_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5000000)
#define v1_ABDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ABDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74010000)
#define v1_ABDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x64000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ABDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x64000000)
#define v1_ABDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ABDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ABDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4000000)
#define v1_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4000000)
#define v1_MIND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MIND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72010000)
#define v1_MIND_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x62000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MIND_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x62000000)
#define v1_MIND_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MIND_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MIND_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2000000)
#define v1_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2000000)
#define v1_MINUD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MINUD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76010000)
#define v1_MINUD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x66000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MINUD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x66000000)
#define v1_MINUD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MINUD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MINUD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6000000)
#define v1_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6000000)
#define v1_MAXD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73010000)
#define v1_MAXD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x63000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MAXD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x63000000)
#define v1_MAXD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe3000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MAXD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MAXD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3000000)
#define v1_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe3000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3000000)
#define v1_MAXUD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXUD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77010000)
#define v1_MAXUD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x67000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MAXUD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x67000000)
#define v1_MAXUD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MAXUD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MAXUD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7000000)
#define v1_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7000000)
#define v1_ANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78010000)
#define v1_ANDD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ANDD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ANDD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8010800)
#define v1_ANDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x68000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ANDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x68000000)
#define v1_ANDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe8000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ANDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ANDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8000000)
#define v1_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe8000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8000000)
#define v1_NANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_NANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79010000)
#define v1_NANDD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_NANDD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_NANDD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9010800)
#define v1_NANDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x69000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_NANDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x69000000)
#define v1_NANDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe9000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_NANDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_NANDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9000000)
#define v1_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe9000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9000000)
#define v1_ORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a010000)
#define v1_ORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa010800)
#define v1_ORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6a000000)
#define v1_ORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xea000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea000000)
#define v1_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xea000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea000000)
#define v1_NORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_NORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b010000)
#define v1_NORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_NORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_NORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb010800)
#define v1_NORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6b000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_NORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6b000000)
#define v1_NORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xeb000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_NORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_NORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb000000)
#define v1_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xeb000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb000000)
#define v1_XORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_XORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c010000)
#define v1_XORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_XORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_XORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc010800)
#define v1_XORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6c000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_XORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6c000000)
#define v1_XORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xec000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_XORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_XORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec000000)
#define v1_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xec000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec000000)
#define v1_NXORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_NXORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d010000)
#define v1_NXORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_NXORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_NXORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd010800)
#define v1_NXORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6d000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_NXORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6d000000)
#define v1_NXORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xed000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_NXORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_NXORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed000000)
#define v1_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xed000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed000000)
#define v1_ANDND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ANDND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e010000)
#define v1_ANDND_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ANDND_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ANDND_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe010800)
#define v1_ANDND_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6e000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ANDND_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6e000000)
#define v1_ANDND_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xee000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ANDND_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ANDND_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee000000)
#define v1_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xee000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee000000)
#define v1_ORND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ORND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f010000)
#define v1_ORND_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ORND_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ORND_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff010800)
#define v1_ORND_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6f000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ORND_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6f000000)
#define v1_ORND_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xef000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ORND_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ORND_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef000000)
#define v1_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xef000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef000000)
#define v1_SBMM8_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBMM8_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e020000)
#define v1_SBMM8_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe020800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBMM8_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBMM8_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe020800)
#define v1_SBMM8_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6e020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_SBMM8_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6e020000)
#define v1_SBMM8_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xee020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_SBMM8_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SBMM8_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee020000)
#define v1_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xee020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee020000)
#define v1_SBMMT8_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBMMT8_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f020000)
#define v1_SBMMT8_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff020800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBMMT8_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBMMT8_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff020800)
#define v1_SBMMT8_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6f020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_SBMMT8_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6f020000)
#define v1_SBMMT8_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xef020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_SBMMT8_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SBMMT8_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef020000)
#define v1_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xef020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef020000)
#define v1_ADDSD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6e010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ADDSD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6e010000)
#define v1_ADDSD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xee010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ADDSD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ADDSD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee010000)
#define v1_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xee010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee010000)
#define v1_ADDSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01a000)
#define v1_SBFSD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6f010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_SBFSD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6f010000)
#define v1_SBFSD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xef010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_SBFSD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SBFSD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef010000)
#define v1_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xef010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef010000)
#define v1_SBFSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01a000)
#define v1_ADDX2D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX2D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70014000)
#define v1_ADDX2D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX2D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX2D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0014000)
#define is_v1_ADDX2D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0014800)
#define v1_ADDX4D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX4D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72014000)
#define v1_ADDX4D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX4D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX4D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2014000)
#define is_v1_ADDX4D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2014800)
#define v1_ADDX8D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX8D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74014000)
#define v1_ADDX8D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX8D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX8D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4014000)
#define is_v1_ADDX8D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4014800)
#define v1_ADDX16D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX16D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76014000)
#define v1_ADDX16D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX16D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX16D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6014000)
#define is_v1_ADDX16D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6014800)
#define v1_SBFX2D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX2D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71014000)
#define v1_SBFX2D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX2D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX2D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1014000)
#define is_v1_SBFX2D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1014800)
#define v1_SBFX4D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX4D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73014000)
#define v1_SBFX4D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX4D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX4D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3014000)
#define is_v1_SBFX4D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3014800)
#define v1_SBFX8D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX8D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75014000)
#define v1_SBFX8D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX8D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX8D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5014000)
#define is_v1_SBFX8D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5014800)
#define v1_SBFX16D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX16D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77014000)
#define v1_SBFX16D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX16D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX16D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7014000)
#define is_v1_SBFX16D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7014800)
#define v1_ADDWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78014000)
#define v1_ADDWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8014000)
#define is_v1_ADDWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8014800)
#define v1_SBFWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79014000)
#define v1_SBFWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9014000)
#define is_v1_SBFWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9014800)
#define v1_ADDUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a014000)
#define v1_ADDUWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDUWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDUWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDUWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDUWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa014000)
#define is_v1_ADDUWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa014800)
#define v1_SBFUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b014000)
#define v1_SBFUWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFUWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFUWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFUWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFUWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb014000)
#define is_v1_SBFUWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb014800)
#define v1_LANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c014000)
#define v1_LANDD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LANDD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LANDD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LANDD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LANDD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc014000)
#define is_v1_LANDD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc014800)
#define v1_LNANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d014000)
#define v1_LNANDD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNANDD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LNANDD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNANDD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNANDD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd014000)
#define is_v1_LNANDD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd014800)
#define v1_LORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e014000)
#define v1_LORD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LORD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LORD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LORD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LORD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe014000)
#define is_v1_LORD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe014800)
#define v1_LNORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f014000)
#define v1_LNORD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNORD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LNORD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNORD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNORD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff014000)
#define is_v1_LNORD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff014800)
#define v1_STSUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_STSUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70011000)
#define v1_ADDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x71000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ADDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x71000000)
#define v1_ADDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ADDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ADDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf1000000)
#define v1_ADDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71011000)
#define v1_SBFW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x75000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_SBFW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x75000000)
#define v1_SBFW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_SBFW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_SBFW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf5000000)
#define v1_SBFW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75011000)
#define v1_ABDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x74000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ABDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x74000000)
#define v1_ABDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ABDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ABDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf4000000)
#define v1_ABDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ABDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74011000)
#define v1_MINW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x72000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MINW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x72000000)
#define v1_MINW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MINW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MINW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf2000000)
#define v1_MINW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MINW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72011000)
#define v1_MINUW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x76000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MINUW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x76000000)
#define v1_MINUW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MINUW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MINUW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf6000000)
#define v1_MINUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MINUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76011000)
#define v1_MAXW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x73000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MAXW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x73000000)
#define v1_MAXW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf3000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MAXW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MAXW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf3000000)
#define v1_MAXW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73011000)
#define v1_MAXUW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x77000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MAXUW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x77000000)
#define v1_MAXUW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MAXUW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MAXUW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf7000000)
#define v1_MAXUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77011000)
#define v1_ANDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x78000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ANDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x78000000)
#define v1_ANDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf8000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ANDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ANDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf8000000)
#define v1_ANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78011000)
#define v1_NANDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x79000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_NANDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x79000000)
#define v1_NANDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf9000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_NANDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_NANDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf9000000)
#define v1_NANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_NANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79011000)
#define v1_ORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7a000000)
#define v1_ORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfa000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfa000000)
#define v1_ORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a011000)
#define v1_NORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7b000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_NORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7b000000)
#define v1_NORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfb000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_NORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_NORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfb000000)
#define v1_NORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_NORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b011000)
#define v1_XORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7c000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_XORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7c000000)
#define v1_XORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfc000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_XORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_XORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfc000000)
#define v1_XORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_XORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c011000)
#define v1_NXORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7d000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_NXORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7d000000)
#define v1_NXORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfd000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_NXORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_NXORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfd000000)
#define v1_NXORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_NXORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d011000)
#define v1_ANDNW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7e000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ANDNW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7e000000)
#define v1_ANDNW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfe000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ANDNW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ANDNW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfe000000)
#define v1_ANDNW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ANDNW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e011000)
#define v1_ORNW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7f000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_ORNW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7f000000)
#define v1_ORNW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xff000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_ORNW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_ORNW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xff000000)
#define v1_ORNW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ORNW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f011000)
#define v1_ADDSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01b000)
#define v1_ADDSW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDSW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDSW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01b000)
#define v1_SBFSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01b000)
#define v1_SBFSW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFSW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFSW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01b000)
#define v1_ADDX2W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX2W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70015000)
#define v1_ADDX2W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX2W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0015000)
#define v1_ADDX4W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX4W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72015000)
#define v1_ADDX4W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX4W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2015000)
#define v1_ADDX8W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX8W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74015000)
#define v1_ADDX8W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX8W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4015000)
#define v1_ADDX16W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX16W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76015000)
#define v1_ADDX16W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX16W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6015000)
#define v1_SBFX2W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX2W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71015000)
#define v1_SBFX2W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX2W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1015000)
#define v1_SBFX4W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX4W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73015000)
#define v1_SBFX4W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX4W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3015000)
#define v1_SBFX8W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX8W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75015000)
#define v1_SBFX8W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX8W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5015000)
#define v1_SBFX16W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX16W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77015000)
#define v1_SBFX16W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX16W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7015000)
#define v1_AVGW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78015000)
#define v1_AVGW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8015000)
#define v1_AVGUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79015000)
#define v1_AVGUW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGUW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGUW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9015000)
#define v1_AVGRW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGRW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a015000)
#define v1_AVGRW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGRW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa015000)
#define v1_AVGRUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGRUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b015000)
#define v1_AVGRUW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRUW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGRUW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb015000)
#define v1_LANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c015000)
#define v1_LANDW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LANDW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LANDW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc015000)
#define v1_LNANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d015000)
#define v1_LNANDW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNANDW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNANDW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd015000)
#define v1_LORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e015000)
#define v1_LORW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LORW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LORW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe015000)
#define v1_LNORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f015000)
#define v1_LNORW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNORW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNORW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff015000)
#define v1_ADDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71012000)
#define v1_ADDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1012000)
#define is_v1_ADDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1012800)
#define v1_SBFWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75012000)
#define v1_SBFWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5012000)
#define is_v1_SBFWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5012800)
#define v1_ABDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ABDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74012000)
#define v1_ABDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ABDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ABDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ABDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ABDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4012000)
#define is_v1_ABDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4012800)
#define v1_MINWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MINWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72012000)
#define v1_MINWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MINWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MINWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2012000)
#define is_v1_MINWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2012800)
#define v1_MINUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MINUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76012000)
#define v1_MINUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MINUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MINUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6012000)
#define is_v1_MINUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6012800)
#define v1_MAXWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73012000)
#define v1_MAXWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MAXWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MAXWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3012000)
#define is_v1_MAXWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3012800)
#define v1_MAXUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77012000)
#define v1_MAXUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MAXUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MAXUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7012000)
#define is_v1_MAXUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7012800)
#define v1_ADDCWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDCWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c012000)
#define v1_ADDCWC__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDCWC__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDCWC___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDCWC___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDCWC__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc012000)
#define is_v1_ADDCWC___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc012800)
#define v1_SBFCWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFCWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d012000)
#define v1_SBFCWC__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFCWC__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFCWC___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFCWC___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFCWC__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd012000)
#define is_v1_SBFCWC___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd012800)
#define v1_ADDSWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDSWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01e000)
#define v1_ADDSWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDSWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDSWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01e800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDSWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDSWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01e000)
#define is_v1_ADDSWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01e800)
#define v1_SBFSWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFSWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01e000)
#define v1_SBFSWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFSWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFSWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01e800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFSWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFSWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01e000)
#define is_v1_SBFSWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01e800)
#define v1_ADDX2WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX2WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70016000)
#define v1_ADDX2WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX2WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX2WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0016000)
#define is_v1_ADDX2WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0016800)
#define v1_ADDX4WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX4WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72016000)
#define v1_ADDX4WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX4WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX4WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2016000)
#define is_v1_ADDX4WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2016800)
#define v1_ADDX8WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX8WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74016000)
#define v1_ADDX8WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX8WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX8WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4016000)
#define is_v1_ADDX8WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4016800)
#define v1_ADDX16WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX16WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76016000)
#define v1_ADDX16WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX16WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX16WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6016000)
#define is_v1_ADDX16WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6016800)
#define v1_SBFX2WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX2WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71016000)
#define v1_SBFX2WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX2WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX2WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1016000)
#define is_v1_SBFX2WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1016800)
#define v1_SBFX4WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX4WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73016000)
#define v1_SBFX4WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX4WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX4WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3016000)
#define is_v1_SBFX4WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3016800)
#define v1_SBFX8WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX8WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75016000)
#define v1_SBFX8WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX8WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX8WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5016000)
#define is_v1_SBFX8WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5016800)
#define v1_SBFX16WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX16WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77016000)
#define v1_SBFX16WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX16WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX16WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7016000)
#define is_v1_SBFX16WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7016800)
#define v1_AVGWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78016000)
#define v1_AVGWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8016000)
#define is_v1_AVGWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8016800)
#define v1_AVGUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79016000)
#define v1_AVGUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9016000)
#define is_v1_AVGUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9016800)
#define v1_AVGRWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGRWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a016000)
#define v1_AVGRWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGRWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGRWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa016000)
#define is_v1_AVGRWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa016800)
#define v1_AVGRUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGRUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b016000)
#define v1_AVGRUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGRUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGRUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb016000)
#define is_v1_AVGRUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb016800)
#define v1_LANDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LANDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c016000)
#define v1_LANDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LANDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LANDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LANDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LANDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc016000)
#define is_v1_LANDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc016800)
#define v1_LNANDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNANDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d016000)
#define v1_LNANDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNANDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LNANDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNANDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNANDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd016000)
#define is_v1_LNANDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd016800)
#define v1_LORWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LORWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e016000)
#define v1_LORWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LORWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LORWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LORWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LORWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe016000)
#define is_v1_LORWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe016800)
#define v1_LNORWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNORWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f016000)
#define v1_LNORWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNORWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LNORWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNORWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNORWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff016000)
#define is_v1_LNORWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff016800)
#define v1_ADDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71013000)
#define v1_ADDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1013000)
#define is_v1_ADDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1013800)
#define v1_SBFHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75013000)
#define v1_SBFHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5013000)
#define is_v1_SBFHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5013800)
#define v1_ABDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ABDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74013000)
#define v1_ABDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ABDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ABDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ABDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ABDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4013000)
#define is_v1_ABDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4013800)
#define v1_MINHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MINHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72013000)
#define v1_MINHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MINHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MINHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2013000)
#define is_v1_MINHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2013800)
#define v1_MINUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MINUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76013000)
#define v1_MINUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MINUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MINUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MINUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6013000)
#define is_v1_MINUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6013800)
#define v1_MAXHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73013000)
#define v1_MAXHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MAXHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MAXHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3013000)
#define is_v1_MAXHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3013800)
#define v1_MAXUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MAXUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77013000)
#define v1_MAXUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_MAXUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MAXUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MAXUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7013000)
#define is_v1_MAXUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7013800)
#define v1_ADDCHCP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDCHCP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c013000)
#define v1_ADDCHCP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDCHCP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDCHCP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDCHCP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDCHCP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc013000)
#define is_v1_ADDCHCP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc013800)
#define v1_SBFCHCP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFCHCP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d013000)
#define v1_SBFCHCP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFCHCP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFCHCP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFCHCP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFCHCP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd013000)
#define is_v1_SBFCHCP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd013800)
#define v1_ADDSHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDSHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01f000)
#define v1_ADDSHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDSHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDSHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01f800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDSHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDSHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01f000)
#define is_v1_ADDSHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01f800)
#define v1_SBFSHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFSHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01f000)
#define v1_SBFSHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFSHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFSHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01f800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFSHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFSHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01f000)
#define is_v1_SBFSHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01f800)
#define v1_ADDX2HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX2HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70017000)
#define v1_ADDX2HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX2HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX2HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0017000)
#define is_v1_ADDX2HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0017800)
#define v1_ADDX4HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX4HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72017000)
#define v1_ADDX4HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX4HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX4HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2017000)
#define is_v1_ADDX4HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2017800)
#define v1_ADDX8HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX8HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74017000)
#define v1_ADDX8HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX8HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX8HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4017000)
#define is_v1_ADDX8HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4017800)
#define v1_ADDX16HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX16HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76017000)
#define v1_ADDX16HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_ADDX16HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX16HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6017000)
#define is_v1_ADDX16HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6017800)
#define v1_SBFX2HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX2HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71017000)
#define v1_SBFX2HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX2HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX2HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1017000)
#define is_v1_SBFX2HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1017800)
#define v1_SBFX4HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX4HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73017000)
#define v1_SBFX4HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX4HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX4HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3017000)
#define is_v1_SBFX4HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3017800)
#define v1_SBFX8HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX8HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75017000)
#define v1_SBFX8HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX8HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX8HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5017000)
#define is_v1_SBFX8HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5017800)
#define v1_SBFX16HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX16HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77017000)
#define v1_SBFX16HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_SBFX16HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX16HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7017000)
#define is_v1_SBFX16HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7017800)
#define v1_AVGHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78017000)
#define v1_AVGHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8017000)
#define is_v1_AVGHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8017800)
#define v1_AVGUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79017000)
#define v1_AVGUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9017000)
#define is_v1_AVGUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9017800)
#define v1_AVGRHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGRHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a017000)
#define v1_AVGRHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGRHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGRHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa017000)
#define is_v1_AVGRHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa017800)
#define v1_AVGRUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_AVGRUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b017000)
#define v1_AVGRUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_AVGRUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_AVGRUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_AVGRUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb017000)
#define is_v1_AVGRUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb017800)
#define v1_LANDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LANDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c017000)
#define v1_LANDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LANDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LANDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LANDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LANDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc017000)
#define is_v1_LANDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc017800)
#define v1_LNANDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNANDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d017000)
#define v1_LNANDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNANDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LNANDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNANDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNANDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd017000)
#define is_v1_LNANDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd017800)
#define v1_LORHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LORHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e017000)
#define v1_LORHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LORHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LORHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LORHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LORHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe017000)
#define is_v1_LORHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe017800)
#define v1_LNORHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_LNORHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f017000)
#define v1_LNORHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNORHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_LNORHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_LNORHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_LNORHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff017000)
#define is_v1_LNORHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff017800)
#define v1_ADDX2WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX2WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70018000)
#define v1_ADDX2WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX2WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0018000)
#define v1_ADDX4WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX4WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72018000)
#define v1_ADDX4WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX4WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2018000)
#define v1_ADDX8WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX8WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74018000)
#define v1_ADDX8WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX8WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4018000)
#define v1_ADDX16WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX16WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76018000)
#define v1_ADDX16WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX16WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6018000)
#define v1_SBFX2WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX2WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71018000)
#define v1_SBFX2WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX2WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1018000)
#define v1_SBFX4WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX4WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73018000)
#define v1_SBFX4WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX4WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3018000)
#define v1_SBFX8WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX8WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75018000)
#define v1_SBFX8WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX8WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5018000)
#define v1_SBFX16WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX16WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77018000)
#define v1_SBFX16WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX16WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7018000)
#define v1_ADDX2UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX2UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78018000)
#define v1_ADDX2UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX2UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX2UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8018000)
#define v1_ADDX4UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX4UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a018000)
#define v1_ADDX4UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX4UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX4UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa018000)
#define v1_ADDX8UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX8UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c018000)
#define v1_ADDX8UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX8UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX8UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc018000)
#define v1_ADDX16UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDX16UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e018000)
#define v1_ADDX16UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDX16UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDX16UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe018000)
#define v1_SBFX2UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX2UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79018000)
#define v1_SBFX2UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX2UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX2UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9018000)
#define v1_SBFX4UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX4UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b018000)
#define v1_SBFX4UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX4UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX4UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb018000)
#define v1_SBFX8UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX8UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d018000)
#define v1_SBFX8UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX8UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX8UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd018000)
#define v1_SBFX16UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFX16UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f018000)
#define v1_SBFX16UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFX16UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFX16UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff018000)
#define v1_ADDCD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDCD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01c000)
#define v1_ADDCD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDCD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDCD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01c000)
#define v1_SBFCD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFCD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01c000)
#define v1_SBFCD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFCD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFCD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01c000)
#define v1_ADDCD_I_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ADDCD_I_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01d000)
#define v1_ADDCD_I_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_ADDCD_I_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_ADDCD_I_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01d000)
#define v1_SBFCD_I_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SBFCD_I_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01d000)
#define v1_SBFCD_I_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_SBFCD_I_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_SBFCD_I_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01d000)
#define v1_FNEGD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FNEGD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70032000)
#define v1_FABSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FABSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71032000)
#define v1_FSRECD__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FSRECD_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70034800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FSRECD__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70034000)
#define is_v1_FSRECD_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70034800)
#define v1_FSRSRD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FSRSRD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78032000)
#define v1_FWIDENLWD__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FWIDENLWD_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FWIDENLWD__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x78033000)
#define is_v1_FWIDENLWD_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x78033800)
#define v1_FWIDENMWD__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x79033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FWIDENMWD_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x79033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FWIDENMWD__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x79033000)
#define is_v1_FWIDENMWD_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x79033800)
#define v1_FNARROWDW_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWDW__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FNARROWDW_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036200)
#define is_v1_FNARROWDW_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036a00)
#define is_v1_FNARROWDW_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036400)
#define is_v1_FNARROWDW_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036c00)
#define is_v1_FNARROWDW_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036500)
#define is_v1_FNARROWDW_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036d00)
#define is_v1_FNARROWDW_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036000)
#define is_v1_FNARROWDW_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036800)
#define is_v1_FNARROWDW_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036600)
#define is_v1_FNARROWDW_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036e00)
#define is_v1_FNARROWDW_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036100)
#define is_v1_FNARROWDW_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036900)
#define is_v1_FNARROWDW_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036300)
#define is_v1_FNARROWDW_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036b00)
#define is_v1_FNARROWDW___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036700)
#define is_v1_FNARROWDW__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036f00)
#define v1_FNEGW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FNEGW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72032000)
#define v1_FABSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FABSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73032000)
#define v1_FRECW_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRECW__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FRECW_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036200)
#define is_v1_FRECW_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036a00)
#define is_v1_FRECW_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036400)
#define is_v1_FRECW_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036c00)
#define is_v1_FRECW_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036500)
#define is_v1_FRECW_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036d00)
#define is_v1_FRECW_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036000)
#define is_v1_FRECW_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036800)
#define is_v1_FRECW_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036600)
#define is_v1_FRECW_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036e00)
#define is_v1_FRECW_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036100)
#define is_v1_FRECW_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036900)
#define is_v1_FRECW_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036300)
#define is_v1_FRECW_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036b00)
#define is_v1_FRECW___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036700)
#define is_v1_FRECW__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036f00)
#define v1_FRSRW_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FRSRW__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FRSRW_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036200)
#define is_v1_FRSRW_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036a00)
#define is_v1_FRSRW_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036400)
#define is_v1_FRSRW_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036c00)
#define is_v1_FRSRW_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036500)
#define is_v1_FRSRW_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036d00)
#define is_v1_FRSRW_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036000)
#define is_v1_FRSRW_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036800)
#define is_v1_FRSRW_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036600)
#define is_v1_FRSRW_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036e00)
#define is_v1_FRSRW_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036100)
#define is_v1_FRSRW_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036900)
#define is_v1_FRSRW_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036300)
#define is_v1_FRSRW_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036b00)
#define is_v1_FRSRW___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036700)
#define is_v1_FRSRW__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036f00)
#define v1_FSRECW__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FSRECW_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72034800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FSRECW__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72034000)
#define is_v1_FSRECW_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72034800)
#define v1_FSRSRW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FSRSRW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a032000)
#define v1_FWIDENLHW__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FWIDENLHW_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FWIDENLHW__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7a033000)
#define is_v1_FWIDENLHW_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7a033800)
#define v1_FWIDENMHW__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7b033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FWIDENMHW_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7b033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FWIDENMHW__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7b033000)
#define is_v1_FWIDENMHW_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7b033800)
#define v1_FNARROWWH_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FNARROWWH__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FNARROWWH_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036200)
#define is_v1_FNARROWWH_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036a00)
#define is_v1_FNARROWWH_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036400)
#define is_v1_FNARROWWH_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036c00)
#define is_v1_FNARROWWH_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036500)
#define is_v1_FNARROWWH_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036d00)
#define is_v1_FNARROWWH_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036000)
#define is_v1_FNARROWWH_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036800)
#define is_v1_FNARROWWH_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036600)
#define is_v1_FNARROWWH_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036e00)
#define is_v1_FNARROWWH_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036100)
#define is_v1_FNARROWWH_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036900)
#define is_v1_FNARROWWH_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036300)
#define is_v1_FNARROWWH_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036b00)
#define is_v1_FNARROWWH___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036700)
#define is_v1_FNARROWWH__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036f00)
#define v1_FNEGWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FNEGWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74032000)
#define v1_FABSWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x75032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FABSWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75032000)
#define v1_FSRECWP__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FSRECWP_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74034800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FSRECWP__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74034000)
#define is_v1_FSRECWP_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74034800)
#define v1_FSRSRWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FSRSRWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c032000)
#define v1_FWIDENLHWP__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FWIDENLHWP_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FWIDENLHWP__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7c033000)
#define is_v1_FWIDENLHWP_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7c033800)
#define v1_FWIDENMHWP__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7d033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v1_FWIDENMHWP_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7d033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FWIDENMHWP__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7d033000)
#define is_v1_FWIDENMHWP_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7d033800)
#define v1_FNARROWDWP_rd__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036200 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rd_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036a00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rna__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036400 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rna_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036c00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rnz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036500 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rnz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036d00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rn__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rn_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_ro__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036600 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_ro_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036e00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_ru__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036100 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_ru_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036900 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036300 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP_rz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036b00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP___registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036700 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWDWP__s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036f00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FNARROWDWP_rd__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036200)
#define is_v1_FNARROWDWP_rd_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036a00)
#define is_v1_FNARROWDWP_rna__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036400)
#define is_v1_FNARROWDWP_rna_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036c00)
#define is_v1_FNARROWDWP_rnz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036500)
#define is_v1_FNARROWDWP_rnz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036d00)
#define is_v1_FNARROWDWP_rn__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036000)
#define is_v1_FNARROWDWP_rn_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036800)
#define is_v1_FNARROWDWP_ro__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036600)
#define is_v1_FNARROWDWP_ro_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036e00)
#define is_v1_FNARROWDWP_ru__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036100)
#define is_v1_FNARROWDWP_ru_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036900)
#define is_v1_FNARROWDWP_rz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036300)
#define is_v1_FNARROWDWP_rz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036b00)
#define is_v1_FNARROWDWP___registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036700)
#define is_v1_FNARROWDWP__s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036f00)
#define v1_FNEGHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x76032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FNEGHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76032000)
#define v1_FABSHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x77032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_FABSHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77032000)
#define v1_FNARROWWHQ_rd__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036200 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rd_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036a00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rna__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036400 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rna_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036c00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rnz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036500 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rnz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036d00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rn__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rn_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_ro__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036600 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_ro_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036e00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_ru__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036100 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_ru_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036900 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036300 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ_rz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036b00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ___registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036700 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FNARROWWHQ__s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036f00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FNARROWWHQ_rd__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036200)
#define is_v1_FNARROWWHQ_rd_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036a00)
#define is_v1_FNARROWWHQ_rna__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036400)
#define is_v1_FNARROWWHQ_rna_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036c00)
#define is_v1_FNARROWWHQ_rnz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036500)
#define is_v1_FNARROWWHQ_rnz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036d00)
#define is_v1_FNARROWWHQ_rn__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036000)
#define is_v1_FNARROWWHQ_rn_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036800)
#define is_v1_FNARROWWHQ_ro__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036600)
#define is_v1_FNARROWWHQ_ro_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036e00)
#define is_v1_FNARROWWHQ_ru__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036100)
#define is_v1_FNARROWWHQ_ru_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036900)
#define is_v1_FNARROWWHQ_rz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036300)
#define is_v1_FNARROWWHQ_rz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036b00)
#define is_v1_FNARROWWHQ___registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036700)
#define is_v1_FNARROWWHQ__s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036f00)
#define v1_INSF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x60030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v1_INSF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v1_INSF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x60030000)
#define v1_EXTFZ_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x64030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v1_EXTFZ_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v1_EXTFZ_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x64030000)
#define v1_EXTFS_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x68030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v1_EXTFS_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v1_EXTFS_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x68030000)
#define v1_CLRF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x6c030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v1_CLRF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v1_CLRF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x6c030000)
#define v1_SATD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SATD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e026000)
#define v1_SATD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7e022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SATD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e022000)
#define v1_SRSD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRSD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78022000)
#define v1_SRSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78026000)
#define v1_SLLD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLLD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79022000)
#define v1_SLLD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLLD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79026000)
#define v1_SRAD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRAD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a022000)
#define v1_SRAD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRAD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a026000)
#define v1_SRLD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRLD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b022000)
#define v1_SRLD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRLD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b026000)
#define v1_SLSD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLSD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c022000)
#define v1_SLSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c026000)
#define v1_SRSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78027000)
#define v1_SRSW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRSW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78023000)
#define v1_SLLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79027000)
#define v1_SLLW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLLW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79023000)
#define v1_SRAW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRAW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a027000)
#define v1_SRAW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRAW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a023000)
#define v1_SRLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b027000)
#define v1_SRLW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRLW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b023000)
#define v1_SLSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c027000)
#define v1_SLSW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLSW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c023000)
#define v1_ROLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ROLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e027000)
#define v1_ROLW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7e023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_ROLW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e023000)
#define v1_RORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_RORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f027000)
#define v1_RORW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7f023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_RORW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f023000)
#define v1_SRSWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRSWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78024000)
#define v1_SRSWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRSWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78028000)
#define v1_SLLWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLLWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79024000)
#define v1_SLLWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLLWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79028000)
#define v1_SRAWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRAWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a024000)
#define v1_SRAWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRAWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a028000)
#define v1_SRLWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRLWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b024000)
#define v1_SRLWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRLWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b028000)
#define v1_SLSWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLSWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c024000)
#define v1_SLSWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLSWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c028000)
#define v1_ROLWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7e024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_ROLWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e024000)
#define v1_ROLWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_ROLWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e028000)
#define v1_RORWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7f024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_RORWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f024000)
#define v1_RORWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_RORWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f028000)
#define v1_SRSHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRSHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78025000)
#define v1_SRSHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRSHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78029000)
#define v1_SLLHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLLHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79025000)
#define v1_SLLHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLLHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79029000)
#define v1_SRAHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRAHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a025000)
#define v1_SRAHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRAHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a029000)
#define v1_SRLHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SRLHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b025000)
#define v1_SRLHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SRLHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b029000)
#define v1_SLSHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_SLSHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c025000)
#define v1_SLSHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_SLSHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c029000)
#define v1_COMPD_all_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_any_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6c010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_eq_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x61010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_ge_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x63010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_geu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x67010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_gt_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x65010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_gtu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x69010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_le_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x64010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_leu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x68010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_lt_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x62010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_ltu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x66010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_nall_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_ne_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x60010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v1_COMPD_none_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6d010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_COMPD_all_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6a010000)
#define is_v1_COMPD_any_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6c010000)
#define is_v1_COMPD_eq_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x61010000)
#define is_v1_COMPD_ge_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x63010000)
#define is_v1_COMPD_geu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x67010000)
#define is_v1_COMPD_gt_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x65010000)
#define is_v1_COMPD_gtu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x69010000)
#define is_v1_COMPD_le_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x64010000)
#define is_v1_COMPD_leu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x68010000)
#define is_v1_COMPD_lt_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x62010000)
#define is_v1_COMPD_ltu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x66010000)
#define is_v1_COMPD_nall_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6b010000)
#define is_v1_COMPD_ne_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x60010000)
#define is_v1_COMPD_none_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6d010000)
#define v1_COMPD_all_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xea010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_all_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_any_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xec010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_any_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_eq_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe1010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_eq_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_ge_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe3010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_ge_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_geu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe7010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_geu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_gt_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe5010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_gt_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_gtu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe9010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_gtu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_le_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe4010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_le_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_leu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe8010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_leu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_lt_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe2010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_lt_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_ltu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe6010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_ltu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_nall_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xeb010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_nall_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_ne_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe0010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_ne_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_COMPD_none_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xed010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_COMPD_none_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_COMPD_all_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea010000)
#define is_v1_COMPD_any_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec010000)
#define is_v1_COMPD_eq_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1010000)
#define is_v1_COMPD_ge_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3010000)
#define is_v1_COMPD_geu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7010000)
#define is_v1_COMPD_gt_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5010000)
#define is_v1_COMPD_gtu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9010000)
#define is_v1_COMPD_le_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4010000)
#define is_v1_COMPD_leu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8010000)
#define is_v1_COMPD_lt_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2010000)
#define is_v1_COMPD_ltu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6010000)
#define is_v1_COMPD_nall_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb010000)
#define is_v1_COMPD_ne_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0010000)
#define is_v1_COMPD_none_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed010000)
#define v1_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xea010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xec010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe1010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe3010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe7010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe5010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe9010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe4010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe8010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe2010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe6010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xeb010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe0010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xed010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea010000)
#define is_v1_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec010000)
#define is_v1_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1010000)
#define is_v1_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3010000)
#define is_v1_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7010000)
#define is_v1_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5010000)
#define is_v1_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9010000)
#define is_v1_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4010000)
#define is_v1_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8010000)
#define is_v1_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2010000)
#define is_v1_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6010000)
#define is_v1_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb010000)
#define is_v1_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0010000)
#define is_v1_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed010000)
#define v1_COMPD_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPD_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_COMPD_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01a000)
#define is_v1_COMPD_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01a000)
#define is_v1_COMPD_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101a000)
#define is_v1_COMPD_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301a000)
#define is_v1_COMPD_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701a000)
#define is_v1_COMPD_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501a000)
#define is_v1_COMPD_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901a000)
#define is_v1_COMPD_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401a000)
#define is_v1_COMPD_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801a000)
#define is_v1_COMPD_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201a000)
#define is_v1_COMPD_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601a000)
#define is_v1_COMPD_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01a000)
#define is_v1_COMPD_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001a000)
#define is_v1_COMPD_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01a000)
#define v1_COMPW_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPW_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_COMPW_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01b000)
#define is_v1_COMPW_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01b000)
#define is_v1_COMPW_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101b000)
#define is_v1_COMPW_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301b000)
#define is_v1_COMPW_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701b000)
#define is_v1_COMPW_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501b000)
#define is_v1_COMPW_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901b000)
#define is_v1_COMPW_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401b000)
#define is_v1_COMPW_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801b000)
#define is_v1_COMPW_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201b000)
#define is_v1_COMPW_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601b000)
#define is_v1_COMPW_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01b000)
#define is_v1_COMPW_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001b000)
#define is_v1_COMPW_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01b000)
#define v1_COMPW_all_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_all_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_any_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_any_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_eq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf101b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_eq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_ge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf301b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_ge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_geu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf701b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_geu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_gt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf501b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_gt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_gtu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf901b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_gtu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_le_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf401b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_le_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_leu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf801b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_leu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_lt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf201b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_lt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_ltu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf601b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_ltu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_nall_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_nall_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_ne_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf001b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_ne_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_COMPW_none_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_COMPW_none_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_COMPW_all_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa01b000)
#define is_v1_COMPW_any_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc01b000)
#define is_v1_COMPW_eq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf101b000)
#define is_v1_COMPW_ge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf301b000)
#define is_v1_COMPW_geu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf701b000)
#define is_v1_COMPW_gt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf501b000)
#define is_v1_COMPW_gtu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf901b000)
#define is_v1_COMPW_le_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf401b000)
#define is_v1_COMPW_leu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf801b000)
#define is_v1_COMPW_lt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf201b000)
#define is_v1_COMPW_ltu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf601b000)
#define is_v1_COMPW_nall_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb01b000)
#define is_v1_COMPW_ne_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf001b000)
#define is_v1_COMPW_none_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd01b000)
#define v1_COMPWD_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPWD_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_COMPWD_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01c000)
#define is_v1_COMPWD_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01c000)
#define is_v1_COMPWD_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101c000)
#define is_v1_COMPWD_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301c000)
#define is_v1_COMPWD_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701c000)
#define is_v1_COMPWD_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501c000)
#define is_v1_COMPWD_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901c000)
#define is_v1_COMPWD_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401c000)
#define is_v1_COMPWD_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801c000)
#define is_v1_COMPWD_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201c000)
#define is_v1_COMPWD_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601c000)
#define is_v1_COMPWD_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01c000)
#define is_v1_COMPWD_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001c000)
#define is_v1_COMPWD_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01c000)
#define v1_COMPUWD_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPUWD_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_COMPUWD_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01d000)
#define is_v1_COMPUWD_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01d000)
#define is_v1_COMPUWD_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101d000)
#define is_v1_COMPUWD_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301d000)
#define is_v1_COMPUWD_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701d000)
#define is_v1_COMPUWD_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501d000)
#define is_v1_COMPUWD_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901d000)
#define is_v1_COMPUWD_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401d000)
#define is_v1_COMPUWD_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801d000)
#define is_v1_COMPUWD_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201d000)
#define is_v1_COMPUWD_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601d000)
#define is_v1_COMPUWD_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01d000)
#define is_v1_COMPUWD_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001d000)
#define is_v1_COMPUWD_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01d000)
#define v1_COMPNWP_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNWP_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_COMPNWP_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01e000)
#define is_v1_COMPNWP_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01e000)
#define is_v1_COMPNWP_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101e000)
#define is_v1_COMPNWP_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301e000)
#define is_v1_COMPNWP_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701e000)
#define is_v1_COMPNWP_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501e000)
#define is_v1_COMPNWP_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901e000)
#define is_v1_COMPNWP_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401e000)
#define is_v1_COMPNWP_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801e000)
#define is_v1_COMPNWP_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201e000)
#define is_v1_COMPNWP_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601e000)
#define is_v1_COMPNWP_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01e000)
#define is_v1_COMPNWP_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001e000)
#define is_v1_COMPNWP_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01e000)
#define v1_COMPNHQ_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_COMPNHQ_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_COMPNHQ_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01f000)
#define is_v1_COMPNHQ_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01f000)
#define is_v1_COMPNHQ_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101f000)
#define is_v1_COMPNHQ_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301f000)
#define is_v1_COMPNHQ_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701f000)
#define is_v1_COMPNHQ_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501f000)
#define is_v1_COMPNHQ_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901f000)
#define is_v1_COMPNHQ_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401f000)
#define is_v1_COMPNHQ_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801f000)
#define is_v1_COMPNHQ_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201f000)
#define is_v1_COMPNHQ_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601f000)
#define is_v1_COMPNHQ_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01f000)
#define is_v1_COMPNHQ_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001f000)
#define is_v1_COMPNHQ_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01f000)
#define v1_CMOVED_deqz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x61020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_dgez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x63020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_dgtz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x65020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_dlez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x64020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_dltz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x62020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_dnez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x60020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_even_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x67020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_odd_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x66020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_weqz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x69020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_wgez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6b020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_wgtz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6d020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_wlez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6c020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_wltz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6a020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v1_CMOVED_wnez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x68020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_CMOVED_deqz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x61020000)
#define is_v1_CMOVED_dgez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x63020000)
#define is_v1_CMOVED_dgtz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x65020000)
#define is_v1_CMOVED_dlez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x64020000)
#define is_v1_CMOVED_dltz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x62020000)
#define is_v1_CMOVED_dnez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x60020000)
#define is_v1_CMOVED_even_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x67020000)
#define is_v1_CMOVED_odd_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x66020000)
#define is_v1_CMOVED_weqz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x69020000)
#define is_v1_CMOVED_wgez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6b020000)
#define is_v1_CMOVED_wgtz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6d020000)
#define is_v1_CMOVED_wlez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6c020000)
#define is_v1_CMOVED_wltz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6a020000)
#define is_v1_CMOVED_wnez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x68020000)
#define v1_CMOVED_deqz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe1020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_deqz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dgez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe3020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dgez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dgtz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe5020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dgtz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dlez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe4020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dlez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dltz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe2020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dltz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dnez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe0020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dnez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_even_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe7020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_even_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_odd_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe6020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_odd_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_weqz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe9020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_weqz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wgez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xeb020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wgez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wgtz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xed020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wgtz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wlez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xec020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wlez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wltz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xea020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wltz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wnez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe8020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wnez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_CMOVED_deqz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1020000)
#define is_v1_CMOVED_dgez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3020000)
#define is_v1_CMOVED_dgtz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5020000)
#define is_v1_CMOVED_dlez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4020000)
#define is_v1_CMOVED_dltz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2020000)
#define is_v1_CMOVED_dnez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0020000)
#define is_v1_CMOVED_even_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7020000)
#define is_v1_CMOVED_odd_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6020000)
#define is_v1_CMOVED_weqz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9020000)
#define is_v1_CMOVED_wgez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb020000)
#define is_v1_CMOVED_wgtz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed020000)
#define is_v1_CMOVED_wlez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec020000)
#define is_v1_CMOVED_wltz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea020000)
#define is_v1_CMOVED_wnez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8020000)
#define v1_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe1020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe3020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe5020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe4020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe2020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe0020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe7020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe6020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe9020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xeb020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xed020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xec020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xea020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v1_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe8020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1020000)
#define is_v1_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3020000)
#define is_v1_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5020000)
#define is_v1_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4020000)
#define is_v1_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2020000)
#define is_v1_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0020000)
#define is_v1_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7020000)
#define is_v1_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6020000)
#define is_v1_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9020000)
#define is_v1_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb020000)
#define is_v1_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed020000)
#define is_v1_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec020000)
#define is_v1_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea020000)
#define is_v1_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8020000)
#define v1_CMOVED_deqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x71020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_dgez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x73020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_dgtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x75020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_dlez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x74020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_dltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x72020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_dnez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x70020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_even_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x77020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_odd_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x76020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_weqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x79020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_wgez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7b020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_wgtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7d020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_wlez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7c020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_wltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7a020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVED_wnez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x78020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMOVED_deqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71020000)
#define is_v1_CMOVED_dgez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73020000)
#define is_v1_CMOVED_dgtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75020000)
#define is_v1_CMOVED_dlez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74020000)
#define is_v1_CMOVED_dltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72020000)
#define is_v1_CMOVED_dnez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70020000)
#define is_v1_CMOVED_even_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77020000)
#define is_v1_CMOVED_odd_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76020000)
#define is_v1_CMOVED_weqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79020000)
#define is_v1_CMOVED_wgez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b020000)
#define is_v1_CMOVED_wgtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d020000)
#define is_v1_CMOVED_wlez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c020000)
#define is_v1_CMOVED_wltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a020000)
#define is_v1_CMOVED_wnez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78020000)
#define v1_CMOVEWP_eqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x71021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEWP_even_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x77021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEWP_gez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x73021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEWP_gtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x75021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEWP_lez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x74021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEWP_ltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x72021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEWP_nez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x70021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEWP_odd_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x76021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMOVEWP_eqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71021000)
#define is_v1_CMOVEWP_even_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77021000)
#define is_v1_CMOVEWP_gez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73021000)
#define is_v1_CMOVEWP_gtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75021000)
#define is_v1_CMOVEWP_lez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74021000)
#define is_v1_CMOVEWP_ltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72021000)
#define is_v1_CMOVEWP_nez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70021000)
#define is_v1_CMOVEWP_odd_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76021000)
#define v1_MAKE_registerW_signed16_simple_insn(registerW,signed16) (unsigned int)(0x60000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(signed16) & 0xffff))
#define is_v1_MAKE_registerW_signed16_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x60000000)
#define v1_MAKE_registerW_extend6_upper27_lower10_double_insn_0(registerW,extend6_upper27_lower10) (unsigned int)(0xe0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend6_upper27_lower10) >> 6) >> 27) & 0x3ff) << 6))
#define v1_MAKE_registerW_extend6_upper27_lower10_double_insn_1(registerW,extend6_upper27_lower10) (unsigned int)((((int)(extend6_upper27_lower10) >> 6) & 0x7ffffff))
#define v1_MAKE_registerW_extend6_upper27_lower10_double_insn_2(registerW,extend6_upper27_lower10) (unsigned int)(((int)(extend6_upper27_lower10) & 0x3f))
#define is_v1_MAKE_registerW_extend6_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0000000)
#define v1_MAKE_registerW_extend27_upper27_lower10_triple_insn_0(registerW,extend27_upper27_lower10) (unsigned int)(0xe0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MAKE_registerW_extend27_upper27_lower10_triple_insn_1(registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MAKE_registerW_extend27_upper27_lower10_triple_insn_2(registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MAKE_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0000000)
#define v1_PCREL_registerW_signed16_simple_insn(registerW,signed16) (unsigned int)(0x70000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(signed16) & 0xffff))
#define is_v1_PCREL_registerW_signed16_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x70000000)
#define v1_PCREL_registerW_extend6_upper27_lower10_double_insn_0(registerW,extend6_upper27_lower10) (unsigned int)(0xf0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend6_upper27_lower10) >> 6) >> 27) & 0x3ff) << 6))
#define v1_PCREL_registerW_extend6_upper27_lower10_double_insn_1(registerW,extend6_upper27_lower10) (unsigned int)((((int)(extend6_upper27_lower10) >> 6) & 0x7ffffff))
#define v1_PCREL_registerW_extend6_upper27_lower10_double_insn_2(registerW,extend6_upper27_lower10) (unsigned int)(((int)(extend6_upper27_lower10) & 0x3f))
#define is_v1_PCREL_registerW_extend6_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf0000000)
#define v1_PCREL_registerW_extend27_upper27_lower10_triple_insn_0(registerW,extend27_upper27_lower10) (unsigned int)(0xf0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_PCREL_registerW_extend27_upper27_lower10_triple_insn_1(registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_PCREL_registerW_extend27_upper27_lower10_triple_insn_2(registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_PCREL_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf0000000)
#define v1_CLZW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CLZW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70023000)
#define v1_CLSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CLSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71023000)
#define v1_CBSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CBSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72023000)
#define v1_CTZW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CTZW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73023000)
#define v1_CLZD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CLZD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70022000)
#define v1_CLSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CLSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71022000)
#define v1_CBSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CBSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72022000)
#define v1_CTZD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CTZD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73022000)
#define v1_CLZWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CLZWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70024000)
#define v1_CLSWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CLSWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71024000)
#define v1_CBSWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CBSWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72024000)
#define v1_CTZWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_CTZWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73024000)
#define v1_SXLHWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x76024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SXLHWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76024000)
#define v1_SXMHWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x77024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SXMHWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77024000)
#define v1_SXLBHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x76025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SXLBHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76025000)
#define v1_SXMBHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x77025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SXMBHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77025000)
#define v1_CMOVEHQ_eqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x79021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEHQ_even_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7f021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEHQ_gez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7b021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEHQ_gtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7d021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEHQ_lez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7c021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEHQ_ltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7a021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEHQ_nez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x78021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v1_CMOVEHQ_odd_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7e021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMOVEHQ_eqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79021000)
#define is_v1_CMOVEHQ_even_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f021000)
#define is_v1_CMOVEHQ_gez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b021000)
#define is_v1_CMOVEHQ_gtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d021000)
#define is_v1_CMOVEHQ_lez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c021000)
#define is_v1_CMOVEHQ_ltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a021000)
#define is_v1_CMOVEHQ_nez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78021000)
#define is_v1_CMOVEHQ_odd_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e021000)
#define v1_FCOMPW_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPW_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPW_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPW_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPW_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPW_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPW_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPW_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FCOMPW_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72030000)
#define is_v1_FCOMPW_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76030000)
#define is_v1_FCOMPW_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74030000)
#define is_v1_FCOMPW_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70030000)
#define is_v1_FCOMPW_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71030000)
#define is_v1_FCOMPW_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75030000)
#define is_v1_FCOMPW_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77030000)
#define is_v1_FCOMPW_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73030000)
#define v1_FCOMPW_oeq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_oeq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPW_oge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_oge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPW_olt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_olt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPW_one_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_one_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPW_ueq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_ueq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPW_uge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_uge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPW_ult_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_ult_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPW_une_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPW_une_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_FCOMPW_oeq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2030000)
#define is_v1_FCOMPW_oge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6030000)
#define is_v1_FCOMPW_olt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4030000)
#define is_v1_FCOMPW_one_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0030000)
#define is_v1_FCOMPW_ueq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1030000)
#define is_v1_FCOMPW_uge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5030000)
#define is_v1_FCOMPW_ult_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7030000)
#define is_v1_FCOMPW_une_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3030000)
#define v1_FCOMPD_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPD_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPD_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPD_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPD_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPD_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPD_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPD_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FCOMPD_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a030000)
#define is_v1_FCOMPD_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e030000)
#define is_v1_FCOMPD_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c030000)
#define is_v1_FCOMPD_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78030000)
#define is_v1_FCOMPD_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79030000)
#define is_v1_FCOMPD_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d030000)
#define is_v1_FCOMPD_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f030000)
#define is_v1_FCOMPD_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b030000)
#define v1_FCOMPD_oeq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_oeq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPD_oge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_oge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPD_olt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_olt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPD_one_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_one_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPD_ueq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_ueq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPD_uge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_uge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPD_ult_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_ult_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v1_FCOMPD_une_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_FCOMPD_une_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_FCOMPD_oeq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa030000)
#define is_v1_FCOMPD_oge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe030000)
#define is_v1_FCOMPD_olt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc030000)
#define is_v1_FCOMPD_one_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8030000)
#define is_v1_FCOMPD_ueq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9030000)
#define is_v1_FCOMPD_uge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd030000)
#define is_v1_FCOMPD_ult_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff030000)
#define is_v1_FCOMPD_une_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb030000)
#define v1_FCOMPNWP_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNWP_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNWP_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNWP_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNWP_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNWP_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNWP_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNWP_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FCOMPNWP_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72031000)
#define is_v1_FCOMPNWP_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76031000)
#define is_v1_FCOMPNWP_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74031000)
#define is_v1_FCOMPNWP_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70031000)
#define is_v1_FCOMPNWP_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71031000)
#define is_v1_FCOMPNWP_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75031000)
#define is_v1_FCOMPNWP_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77031000)
#define is_v1_FCOMPNWP_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73031000)
#define v1_FCOMPNHQ_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNHQ_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNHQ_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNHQ_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNHQ_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNHQ_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNHQ_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FCOMPNHQ_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FCOMPNHQ_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a031000)
#define is_v1_FCOMPNHQ_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e031000)
#define is_v1_FCOMPNHQ_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c031000)
#define is_v1_FCOMPNHQ_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78031000)
#define is_v1_FCOMPNHQ_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79031000)
#define is_v1_FCOMPNHQ_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d031000)
#define is_v1_FCOMPNHQ_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f031000)
#define is_v1_FCOMPNHQ_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b031000)
#define v1_FSDIVD__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x70035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FSDIVD_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x70035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FSDIVD__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70035000)
#define is_v1_FSDIVD_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70035800)
#define v1_FCDIVD__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x71035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FCDIVD_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x71035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FCDIVD__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x71035000)
#define is_v1_FCDIVD_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x71035800)
#define v1_FSDIVW__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x72035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FSDIVW_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x72035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FSDIVW__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72035000)
#define is_v1_FSDIVW_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72035800)
#define v1_FCDIVW__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x73035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FCDIVW_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x73035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FCDIVW__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x73035000)
#define is_v1_FCDIVW_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x73035800)
#define v1_FSDIVWP__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x74035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FSDIVWP_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x74035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FSDIVWP__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74035000)
#define is_v1_FSDIVWP_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74035800)
#define v1_FCDIVWP__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x75035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v1_FCDIVWP_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x75035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v1_FCDIVWP__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x75035000)
#define is_v1_FCDIVWP_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x75035800)
#define v1_FMIND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMIND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70038000)
#define v1_FMAXD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMAXD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71038000)
#define v1_FMINW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMINW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72038000)
#define v1_FMAXW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMAXW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73038000)
#define v1_FMINWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMINWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74038000)
#define v1_FMAXWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMAXWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75038000)
#define v1_FMINHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMINHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76038000)
#define v1_FMAXHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMAXHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77038000)
#define v1_FLOATD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FLOATD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44032000)
#define is_v1_FLOATD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403a000)
#define is_v1_FLOATD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44034000)
#define is_v1_FLOATD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403c000)
#define is_v1_FLOATD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44035000)
#define is_v1_FLOATD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403d000)
#define is_v1_FLOATD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44030000)
#define is_v1_FLOATD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44038000)
#define is_v1_FLOATD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44036000)
#define is_v1_FLOATD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403e000)
#define is_v1_FLOATD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44031000)
#define is_v1_FLOATD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44039000)
#define is_v1_FLOATD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44033000)
#define is_v1_FLOATD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403b000)
#define is_v1_FLOATD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44037000)
#define is_v1_FLOATD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403f000)
#define v1_FLOATWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FLOATWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c032000)
#define is_v1_FLOATWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03a000)
#define is_v1_FLOATWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c034000)
#define is_v1_FLOATWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03c000)
#define is_v1_FLOATWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c035000)
#define is_v1_FLOATWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03d000)
#define is_v1_FLOATWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c030000)
#define is_v1_FLOATWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c038000)
#define is_v1_FLOATWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c036000)
#define is_v1_FLOATWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03e000)
#define is_v1_FLOATWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c031000)
#define is_v1_FLOATWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c039000)
#define is_v1_FLOATWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c033000)
#define is_v1_FLOATWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03b000)
#define is_v1_FLOATWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c037000)
#define is_v1_FLOATWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03f000)
#define v1_FLOATW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FLOATW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48032000)
#define is_v1_FLOATW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803a000)
#define is_v1_FLOATW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48034000)
#define is_v1_FLOATW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803c000)
#define is_v1_FLOATW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48035000)
#define is_v1_FLOATW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803d000)
#define is_v1_FLOATW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48030000)
#define is_v1_FLOATW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48038000)
#define is_v1_FLOATW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48036000)
#define is_v1_FLOATW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803e000)
#define is_v1_FLOATW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48031000)
#define is_v1_FLOATW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48039000)
#define is_v1_FLOATW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48033000)
#define is_v1_FLOATW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803b000)
#define is_v1_FLOATW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48037000)
#define is_v1_FLOATW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803f000)
#define v1_FLOATUD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FLOATUD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45032000)
#define is_v1_FLOATUD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503a000)
#define is_v1_FLOATUD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45034000)
#define is_v1_FLOATUD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503c000)
#define is_v1_FLOATUD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45035000)
#define is_v1_FLOATUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503d000)
#define is_v1_FLOATUD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45030000)
#define is_v1_FLOATUD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45038000)
#define is_v1_FLOATUD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45036000)
#define is_v1_FLOATUD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503e000)
#define is_v1_FLOATUD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45031000)
#define is_v1_FLOATUD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45039000)
#define is_v1_FLOATUD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45033000)
#define is_v1_FLOATUD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503b000)
#define is_v1_FLOATUD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45037000)
#define is_v1_FLOATUD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503f000)
#define v1_FLOATUWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FLOATUWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d032000)
#define is_v1_FLOATUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03a000)
#define is_v1_FLOATUWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d034000)
#define is_v1_FLOATUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03c000)
#define is_v1_FLOATUWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d035000)
#define is_v1_FLOATUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03d000)
#define is_v1_FLOATUWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d030000)
#define is_v1_FLOATUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d038000)
#define is_v1_FLOATUWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d036000)
#define is_v1_FLOATUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03e000)
#define is_v1_FLOATUWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d031000)
#define is_v1_FLOATUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d039000)
#define is_v1_FLOATUWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d033000)
#define is_v1_FLOATUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03b000)
#define is_v1_FLOATUWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d037000)
#define is_v1_FLOATUWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03f000)
#define v1_FLOATUW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FLOATUW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FLOATUW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49032000)
#define is_v1_FLOATUW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903a000)
#define is_v1_FLOATUW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49034000)
#define is_v1_FLOATUW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903c000)
#define is_v1_FLOATUW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49035000)
#define is_v1_FLOATUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903d000)
#define is_v1_FLOATUW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49030000)
#define is_v1_FLOATUW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49038000)
#define is_v1_FLOATUW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49036000)
#define is_v1_FLOATUW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903e000)
#define is_v1_FLOATUW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49031000)
#define is_v1_FLOATUW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49039000)
#define is_v1_FLOATUW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49033000)
#define is_v1_FLOATUW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903b000)
#define is_v1_FLOATUW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49037000)
#define is_v1_FLOATUW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903f000)
#define v1_FIXEDD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FIXEDD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46032000)
#define is_v1_FIXEDD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603a000)
#define is_v1_FIXEDD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46034000)
#define is_v1_FIXEDD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603c000)
#define is_v1_FIXEDD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46035000)
#define is_v1_FIXEDD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603d000)
#define is_v1_FIXEDD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46030000)
#define is_v1_FIXEDD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46038000)
#define is_v1_FIXEDD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46036000)
#define is_v1_FIXEDD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603e000)
#define is_v1_FIXEDD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46031000)
#define is_v1_FIXEDD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46039000)
#define is_v1_FIXEDD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46033000)
#define is_v1_FIXEDD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603b000)
#define is_v1_FIXEDD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46037000)
#define is_v1_FIXEDD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603f000)
#define v1_FIXEDWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FIXEDWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e032000)
#define is_v1_FIXEDWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03a000)
#define is_v1_FIXEDWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e034000)
#define is_v1_FIXEDWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03c000)
#define is_v1_FIXEDWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e035000)
#define is_v1_FIXEDWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03d000)
#define is_v1_FIXEDWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e030000)
#define is_v1_FIXEDWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e038000)
#define is_v1_FIXEDWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e036000)
#define is_v1_FIXEDWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03e000)
#define is_v1_FIXEDWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e031000)
#define is_v1_FIXEDWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e039000)
#define is_v1_FIXEDWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e033000)
#define is_v1_FIXEDWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03b000)
#define is_v1_FIXEDWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e037000)
#define is_v1_FIXEDWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03f000)
#define v1_FIXEDW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FIXEDW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a032000)
#define is_v1_FIXEDW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03a000)
#define is_v1_FIXEDW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a034000)
#define is_v1_FIXEDW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03c000)
#define is_v1_FIXEDW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a035000)
#define is_v1_FIXEDW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03d000)
#define is_v1_FIXEDW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a030000)
#define is_v1_FIXEDW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a038000)
#define is_v1_FIXEDW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a036000)
#define is_v1_FIXEDW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03e000)
#define is_v1_FIXEDW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a031000)
#define is_v1_FIXEDW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a039000)
#define is_v1_FIXEDW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a033000)
#define is_v1_FIXEDW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03b000)
#define is_v1_FIXEDW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a037000)
#define is_v1_FIXEDW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03f000)
#define v1_FIXEDUD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FIXEDUD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47032000)
#define is_v1_FIXEDUD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703a000)
#define is_v1_FIXEDUD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47034000)
#define is_v1_FIXEDUD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703c000)
#define is_v1_FIXEDUD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47035000)
#define is_v1_FIXEDUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703d000)
#define is_v1_FIXEDUD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47030000)
#define is_v1_FIXEDUD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47038000)
#define is_v1_FIXEDUD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47036000)
#define is_v1_FIXEDUD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703e000)
#define is_v1_FIXEDUD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47031000)
#define is_v1_FIXEDUD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47039000)
#define is_v1_FIXEDUD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47033000)
#define is_v1_FIXEDUD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703b000)
#define is_v1_FIXEDUD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47037000)
#define is_v1_FIXEDUD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703f000)
#define v1_FIXEDUWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FIXEDUWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f032000)
#define is_v1_FIXEDUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03a000)
#define is_v1_FIXEDUWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f034000)
#define is_v1_FIXEDUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03c000)
#define is_v1_FIXEDUWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f035000)
#define is_v1_FIXEDUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03d000)
#define is_v1_FIXEDUWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f030000)
#define is_v1_FIXEDUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f038000)
#define is_v1_FIXEDUWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f036000)
#define is_v1_FIXEDUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03e000)
#define is_v1_FIXEDUWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f031000)
#define is_v1_FIXEDUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f039000)
#define is_v1_FIXEDUWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f033000)
#define is_v1_FIXEDUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03b000)
#define is_v1_FIXEDUWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f037000)
#define is_v1_FIXEDUWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03f000)
#define v1_FIXEDUW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v1_FIXEDUW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v1_FIXEDUW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b032000)
#define is_v1_FIXEDUW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03a000)
#define is_v1_FIXEDUW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b034000)
#define is_v1_FIXEDUW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03c000)
#define is_v1_FIXEDUW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b035000)
#define is_v1_FIXEDUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03d000)
#define is_v1_FIXEDUW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b030000)
#define is_v1_FIXEDUW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b038000)
#define is_v1_FIXEDUW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b036000)
#define is_v1_FIXEDUW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03e000)
#define is_v1_FIXEDUW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b031000)
#define is_v1_FIXEDUW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b039000)
#define is_v1_FIXEDUW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b033000)
#define is_v1_FIXEDUW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03b000)
#define is_v1_FIXEDUW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b037000)
#define is_v1_FIXEDUW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03f000)
#define v1_MADDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x40000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MADDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x40000000)
#define v1_MADDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc0000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MADDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MADDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0000000)
#define v1_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc0000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0000000)
#define v1_MADDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50000000)
#define v1_MSBFD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54000000)
#define v1_MADDWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x41000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MADDWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x41000000)
#define v1_MADDWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MADDWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MADDWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1000000)
#define v1_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1000000)
#define v1_MADDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51000000)
#define v1_MSBFWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55000000)
#define v1_MADDHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x42000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MADDHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x42000000)
#define v1_MADDHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MADDHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MADDHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2000000)
#define v1_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2000000)
#define v1_MADDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52000000)
#define v1_MSBFHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56000000)
#define v1_MADDHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50004000)
#define v1_MSBFHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54004000)
#define v1_MADDUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51004000)
#define v1_MSBFUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x55004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55004000)
#define v1_MADDSUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDSUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52004000)
#define v1_MSBFSUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFSUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56004000)
#define v1_MADDDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x48000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MADDDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x48000000)
#define v1_MADDDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc8000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MADDDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MADDDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8000000)
#define v1_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc8000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8000000)
#define v1_MADDDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58001000)
#define v1_MADDUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x49000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MADDUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x49000000)
#define v1_MADDUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc9000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MADDUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MADDUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9000000)
#define v1_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc9000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9000000)
#define v1_MADDUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59001000)
#define v1_MADDSUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4a000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MADDSUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4a000000)
#define v1_MADDSUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xca000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MADDSUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MADDSUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca000000)
#define v1_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xca000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca000000)
#define v1_MADDSUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDSUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a001000)
#define v1_MADDUZDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4b000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MADDUZDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4b000000)
#define v1_MADDUZDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xcb000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MADDUZDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MADDUZDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb000000)
#define v1_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xcb000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb000000)
#define v1_MADDUZDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5b001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDUZDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b001000)
#define v1_MSBFDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5c001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c001000)
#define v1_MSBFUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5d001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d001000)
#define v1_MSBFSUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5e001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFSUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e001000)
#define v1_MSBFUZDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFUZDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f001000)
#define v1_MULD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x44000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MULD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x44000000)
#define v1_MULD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MULD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MULD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4000000)
#define v1_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4000000)
#define v1_MULD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54001000)
#define v1_MULDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x48040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MULDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x48040000)
#define v1_MULDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc8040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MULDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MULDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8040000)
#define v1_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc8040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8040000)
#define v1_MULDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58041000)
#define v1_MULUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x49040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MULUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x49040000)
#define v1_MULUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc9040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MULUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MULUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9040000)
#define v1_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc9040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9040000)
#define v1_MULUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59041000)
#define v1_MULSUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4a040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MULSUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4a040000)
#define v1_MULSUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xca040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MULSUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MULSUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca040000)
#define v1_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xca040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca040000)
#define v1_MULSUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULSUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a041000)
#define v1_CMULDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4b040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_CMULDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4b040000)
#define v1_CMULDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xcb040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_CMULDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_CMULDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb040000)
#define v1_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xcb040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb040000)
#define v1_CMULDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5b041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMULDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b041000)
#define v1_CMULXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5c041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMULXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c041000)
#define v1_CMULGLXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5d041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMULGLXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d041000)
#define v1_CMULGMXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5e041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMULGMXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e041000)
#define v1_CMULGHXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CMULGHXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f041000)
#define v1_CRCBELMW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CRCBELMW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58002000)
#define v1_CRCBELMW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd8002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_CRCBELMW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_CRCBELMW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd8002000)
#define v1_CRCBELLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CRCBELLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59002000)
#define v1_CRCBELLW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd9002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_CRCBELLW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_CRCBELLW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd9002000)
#define v1_CRCLELMW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CRCLELMW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a002000)
#define v1_CRCLELMW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xda002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_CRCLELMW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_CRCLELMW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xda002000)
#define v1_CRCLELLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_CRCLELLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b002000)
#define v1_CRCLELLW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xdb002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_CRCLELLW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_CRCLELLW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xdb002000)
#define v1_DOT2WD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4c000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_DOT2WD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4c000000)
#define v1_DOT2WD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcc000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_DOT2WD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2WD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc000000)
#define v1_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcc000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc000000)
#define v1_DOT2WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_DOT2WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c002000)
#define v1_DOT2UWD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4d000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_DOT2UWD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4d000000)
#define v1_DOT2UWD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcd000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_DOT2UWD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2UWD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd000000)
#define v1_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcd000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd000000)
#define v1_DOT2UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_DOT2UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d002000)
#define v1_DOT2SUWD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4e000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_DOT2SUWD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4e000000)
#define v1_DOT2SUWD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xce000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_DOT2SUWD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2SUWD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce000000)
#define v1_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xce000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce000000)
#define v1_DOT2SUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_DOT2SUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e002000)
#define v1_DOT2W_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4f000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_DOT2W_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4f000000)
#define v1_DOT2W_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcf000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_DOT2W_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2W_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf000000)
#define v1_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcf000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf000000)
#define v1_DOT2W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_DOT2W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f002000)
#define v1_DOT2WDP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x50002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_DOT2WDP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50002000)
#define v1_DOT2UWDP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x51002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_DOT2UWDP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51002000)
#define v1_DOT2SUWDP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x52002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_DOT2SUWDP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52002000)
#define v1_MULWQ_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x54002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_MULWQ_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54002000)
#define v1_DOT2WZP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x53002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_DOT2WZP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53002000)
#define v1_MULWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x45000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MULWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x45000000)
#define v1_MULWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MULWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MULWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5000000)
#define v1_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5000000)
#define v1_MULWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55001000)
#define v1_MULHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x46000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MULHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x46000000)
#define v1_MULHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MULHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MULHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6000000)
#define v1_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6000000)
#define v1_MULHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56001000)
#define v1_MULWC_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x47000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_MULWC_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x47000000)
#define v1_MULWC_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_MULWC_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_MULWC_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7000000)
#define v1_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7000000)
#define v1_MULWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57001000)
#define v1_MULCWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULCWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55002000)
#define v1_MULWDC_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56002000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULWDC_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56002000)
#define v1_MULCWDC_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x57002000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULCWDC_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57002000)
#define v1_MULHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58004000)
#define v1_MULUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59004000)
#define v1_MULSUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULSUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a004000)
#define v1_MADDWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50003000)
#define v1_MADDWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd0003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MADDWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MADDWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd0003000)
#define v1_MADDUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51003000)
#define v1_MADDUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd1003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MADDUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MADDUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd1003000)
#define v1_MADDSUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDSUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52003000)
#define v1_MADDSUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd2003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MADDSUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MADDSUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd2003000)
#define v1_MADDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53003000)
#define v1_MADDW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd3003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MADDW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MADDW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd3003000)
#define v1_MSBFWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54003000)
#define v1_MSBFWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd4003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MSBFWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MSBFWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd4003000)
#define v1_MSBFUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55003000)
#define v1_MSBFUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd5003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MSBFUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MSBFUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd5003000)
#define v1_MSBFSUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFSUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56003000)
#define v1_MSBFSUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd6003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MSBFSUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MSBFSUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd6003000)
#define v1_MSBFW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57003000)
#define v1_MSBFW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd7003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MSBFW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MSBFW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd7003000)
#define v1_MULWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58003000)
#define v1_MULWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd8003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MULWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MULWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd8003000)
#define v1_MULUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59003000)
#define v1_MULUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd9003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MULUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MULUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd9003000)
#define v1_MULSUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULSUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a003000)
#define v1_MULSUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xda003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MULSUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MULSUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xda003000)
#define v1_MULW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b003000)
#define v1_MULW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xdb003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v1_MULW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v1_MULW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xdb003000)
#define v1_MADDWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58000000)
#define v1_MADDUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59000000)
#define v1_MADDSUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MADDSUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a000000)
#define v1_MMA212W_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5b000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MMA212W_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b000000)
#define v1_MSBFWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5c000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c000000)
#define v1_MSBFUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5d000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d000000)
#define v1_MSBFSUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5e000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MSBFSUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e000000)
#define v1_MMS212W_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MMS212W_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f000000)
#define v1_MULWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50001000)
#define v1_MULUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51001000)
#define v1_MULSUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MULSUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52001000)
#define v1_MM212W_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MM212W_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53001000)
#define v1_FFMAD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x40010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x40010000)
#define v1_FFMAD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc0010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0010000)
#define v1_FFMAD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc0010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0010000)
#define v1_FFMAD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50012000)
#define is_v1_FFMAD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001a000)
#define is_v1_FFMAD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50014000)
#define is_v1_FFMAD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001c000)
#define is_v1_FFMAD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50015000)
#define is_v1_FFMAD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001d000)
#define is_v1_FFMAD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50010000)
#define is_v1_FFMAD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50018000)
#define is_v1_FFMAD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50016000)
#define is_v1_FFMAD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001e000)
#define is_v1_FFMAD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50011000)
#define is_v1_FFMAD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50019000)
#define is_v1_FFMAD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50013000)
#define is_v1_FFMAD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001b000)
#define is_v1_FFMAD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50017000)
#define is_v1_FFMAD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001f000)
#define v1_FFMAWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x42010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x42010000)
#define v1_FFMAWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc2010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2010000)
#define v1_FFMAWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc2010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2010000)
#define v1_FFMAWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52012000)
#define is_v1_FFMAWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201a000)
#define is_v1_FFMAWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52014000)
#define is_v1_FFMAWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201c000)
#define is_v1_FFMAWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52015000)
#define is_v1_FFMAWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201d000)
#define is_v1_FFMAWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52010000)
#define is_v1_FFMAWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52018000)
#define is_v1_FFMAWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52016000)
#define is_v1_FFMAWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201e000)
#define is_v1_FFMAWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52011000)
#define is_v1_FFMAWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52019000)
#define is_v1_FFMAWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52013000)
#define is_v1_FFMAWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201b000)
#define is_v1_FFMAWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52017000)
#define is_v1_FFMAWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201f000)
#define v1_FFMAHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x43010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x43010000)
#define v1_FFMAHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc3010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc3010000)
#define v1_FFMAHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc3010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc3010000)
#define v1_FFMAHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53012000)
#define is_v1_FFMAHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301a000)
#define is_v1_FFMAHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53014000)
#define is_v1_FFMAHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301c000)
#define is_v1_FFMAHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53015000)
#define is_v1_FFMAHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301d000)
#define is_v1_FFMAHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53010000)
#define is_v1_FFMAHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53018000)
#define is_v1_FFMAHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53016000)
#define is_v1_FFMAHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301e000)
#define is_v1_FFMAHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53011000)
#define is_v1_FFMAHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53019000)
#define is_v1_FFMAHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53013000)
#define is_v1_FFMAHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301b000)
#define is_v1_FFMAHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53017000)
#define is_v1_FFMAHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301f000)
#define v1_FFMSD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x44010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x44010000)
#define v1_FFMSD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc4010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4010000)
#define v1_FFMSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc4010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4010000)
#define v1_FFMSD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54012000)
#define is_v1_FFMSD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401a000)
#define is_v1_FFMSD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54014000)
#define is_v1_FFMSD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401c000)
#define is_v1_FFMSD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54015000)
#define is_v1_FFMSD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401d000)
#define is_v1_FFMSD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54010000)
#define is_v1_FFMSD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54018000)
#define is_v1_FFMSD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54016000)
#define is_v1_FFMSD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401e000)
#define is_v1_FFMSD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54011000)
#define is_v1_FFMSD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54019000)
#define is_v1_FFMSD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54013000)
#define is_v1_FFMSD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401b000)
#define is_v1_FFMSD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54017000)
#define is_v1_FFMSD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401f000)
#define v1_FFMSWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x46010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x46010000)
#define v1_FFMSWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc6010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6010000)
#define v1_FFMSWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc6010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6010000)
#define v1_FFMSWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56012000)
#define is_v1_FFMSWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601a000)
#define is_v1_FFMSWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56014000)
#define is_v1_FFMSWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601c000)
#define is_v1_FFMSWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56015000)
#define is_v1_FFMSWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601d000)
#define is_v1_FFMSWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56010000)
#define is_v1_FFMSWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56018000)
#define is_v1_FFMSWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56016000)
#define is_v1_FFMSWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601e000)
#define is_v1_FFMSWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56011000)
#define is_v1_FFMSWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56019000)
#define is_v1_FFMSWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56013000)
#define is_v1_FFMSWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601b000)
#define is_v1_FFMSWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56017000)
#define is_v1_FFMSWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601f000)
#define v1_FFMSHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x47010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x47010000)
#define v1_FFMSHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc7010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7010000)
#define v1_FFMSHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc7010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7010000)
#define v1_FFMSHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57012000)
#define is_v1_FFMSHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701a000)
#define is_v1_FFMSHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57014000)
#define is_v1_FFMSHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701c000)
#define is_v1_FFMSHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57015000)
#define is_v1_FFMSHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701d000)
#define is_v1_FFMSHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57010000)
#define is_v1_FFMSHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57018000)
#define is_v1_FFMSHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57016000)
#define is_v1_FFMSHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701e000)
#define is_v1_FFMSHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57011000)
#define is_v1_FFMSHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57019000)
#define is_v1_FFMSHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57013000)
#define is_v1_FFMSHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701b000)
#define is_v1_FFMSHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57017000)
#define is_v1_FFMSHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701f000)
#define v1_FMULD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x48010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x48010000)
#define v1_FMULD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc8010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc8010000)
#define v1_FMULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc8010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc8010000)
#define v1_FMULD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58012000)
#define is_v1_FMULD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801a000)
#define is_v1_FMULD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58014000)
#define is_v1_FMULD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801c000)
#define is_v1_FMULD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58015000)
#define is_v1_FMULD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801d000)
#define is_v1_FMULD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58010000)
#define is_v1_FMULD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58018000)
#define is_v1_FMULD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58016000)
#define is_v1_FMULD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801e000)
#define is_v1_FMULD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58011000)
#define is_v1_FMULD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58019000)
#define is_v1_FMULD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58013000)
#define is_v1_FMULD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801b000)
#define is_v1_FMULD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58017000)
#define is_v1_FMULD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801f000)
#define v1_FMULWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4a010000)
#define v1_FMULWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xca010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xca010000)
#define v1_FMULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xca010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xca010000)
#define v1_FMULWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a012000)
#define is_v1_FMULWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01a000)
#define is_v1_FMULWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a014000)
#define is_v1_FMULWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01c000)
#define is_v1_FMULWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a015000)
#define is_v1_FMULWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01d000)
#define is_v1_FMULWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a010000)
#define is_v1_FMULWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a018000)
#define is_v1_FMULWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a016000)
#define is_v1_FMULWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01e000)
#define is_v1_FMULWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a011000)
#define is_v1_FMULWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a019000)
#define is_v1_FMULWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a013000)
#define is_v1_FMULWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01b000)
#define is_v1_FMULWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a017000)
#define is_v1_FMULWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01f000)
#define v1_FMULHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4b010000)
#define v1_FMULHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcb010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcb010000)
#define v1_FMULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcb010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcb010000)
#define v1_FMULHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b012000)
#define is_v1_FMULHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01a000)
#define is_v1_FMULHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b014000)
#define is_v1_FMULHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01c000)
#define is_v1_FMULHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b015000)
#define is_v1_FMULHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01d000)
#define is_v1_FMULHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b010000)
#define is_v1_FMULHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b018000)
#define is_v1_FMULHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b016000)
#define is_v1_FMULHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01e000)
#define is_v1_FMULHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b011000)
#define is_v1_FMULHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b019000)
#define is_v1_FMULHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b013000)
#define is_v1_FMULHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01b000)
#define is_v1_FMULHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b017000)
#define is_v1_FMULHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01f000)
#define v1_FMULWDP_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x40070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULWDP_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x40070000)
#define v1_FMULWDP_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc0070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULWDP_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWDP_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc0070000)
#define v1_FMULWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc0070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc0070000)
#define v1_FMULWDP_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50072000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50074000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50075000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50078000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50076000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50071000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50079000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50073000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50077000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDP__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULWDP_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50072000)
#define is_v1_FMULWDP_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007a000)
#define is_v1_FMULWDP_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50074000)
#define is_v1_FMULWDP_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007c000)
#define is_v1_FMULWDP_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50075000)
#define is_v1_FMULWDP_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007d000)
#define is_v1_FMULWDP_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50070000)
#define is_v1_FMULWDP_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50078000)
#define is_v1_FMULWDP_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50076000)
#define is_v1_FMULWDP_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007e000)
#define is_v1_FMULWDP_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50071000)
#define is_v1_FMULWDP_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50079000)
#define is_v1_FMULWDP_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50073000)
#define is_v1_FMULWDP_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007b000)
#define is_v1_FMULWDP___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50077000)
#define is_v1_FMULWDP__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007f000)
#define v1_FMULHWQ_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x41070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULHWQ_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x41070000)
#define v1_FMULHWQ_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc1070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULHWQ_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULHWQ_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc1070000)
#define v1_FMULHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc1070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc1070000)
#define v1_FMULHWQ_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51072000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51074000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51075000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51078000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51076000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51071000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51079000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51073000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51077000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHWQ__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULHWQ_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51072000)
#define is_v1_FMULHWQ_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107a000)
#define is_v1_FMULHWQ_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51074000)
#define is_v1_FMULHWQ_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107c000)
#define is_v1_FMULHWQ_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51075000)
#define is_v1_FMULHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107d000)
#define is_v1_FMULHWQ_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51070000)
#define is_v1_FMULHWQ_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51078000)
#define is_v1_FMULHWQ_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51076000)
#define is_v1_FMULHWQ_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107e000)
#define is_v1_FMULHWQ_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51071000)
#define is_v1_FMULHWQ_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51079000)
#define is_v1_FMULHWQ_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51073000)
#define is_v1_FMULHWQ_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107b000)
#define is_v1_FMULHWQ___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51077000)
#define is_v1_FMULHWQ__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107f000)
#define v1_FMULWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FMULWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FMULWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e072000)
#define is_v1_FMULWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07a000)
#define is_v1_FMULWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e074000)
#define is_v1_FMULWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07c000)
#define is_v1_FMULWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e075000)
#define is_v1_FMULWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07d000)
#define is_v1_FMULWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e070000)
#define is_v1_FMULWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e078000)
#define is_v1_FMULWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e076000)
#define is_v1_FMULWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07e000)
#define is_v1_FMULWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e071000)
#define is_v1_FMULWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e079000)
#define is_v1_FMULWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e073000)
#define is_v1_FMULWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07b000)
#define is_v1_FMULWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e077000)
#define is_v1_FMULWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07f000)
#define v1_FMULWC_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4e010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULWC_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4e010000)
#define v1_FMULWC_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xce010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULWC_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWC_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce010000)
#define v1_FMULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xce010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce010000)
#define v1_FMULWC_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWC__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULWC_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e012000)
#define is_v1_FMULWC_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e01a000)
#define is_v1_FMULWC_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e014000)
#define is_v1_FMULWC_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e01c000)
#define is_v1_FMULWC_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e015000)
#define is_v1_FMULWC_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e01d000)
#define is_v1_FMULWC_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e010000)
#define is_v1_FMULWC_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e018000)
#define is_v1_FMULWC_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e016000)
#define is_v1_FMULWC_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e01e000)
#define is_v1_FMULWC_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e011000)
#define is_v1_FMULWC_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e019000)
#define is_v1_FMULWC_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e013000)
#define is_v1_FMULWC_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e01b000)
#define is_v1_FMULWC___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e017000)
#define is_v1_FMULWC__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e01f000)
#define v1_FMULWDC_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x42070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULWDC_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x42070000)
#define v1_FMULWDC_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc2070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULWDC_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWDC_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc2070000)
#define v1_FMULWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc2070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc2070000)
#define v1_FMULWDC_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52072000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5207a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52074000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5207c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52075000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5207d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52078000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52076000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5207e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52071000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52079000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52073000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5207b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52077000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWDC__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5207f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULWDC_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52072000)
#define is_v1_FMULWDC_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5207a000)
#define is_v1_FMULWDC_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52074000)
#define is_v1_FMULWDC_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5207c000)
#define is_v1_FMULWDC_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52075000)
#define is_v1_FMULWDC_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5207d000)
#define is_v1_FMULWDC_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52070000)
#define is_v1_FMULWDC_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52078000)
#define is_v1_FMULWDC_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52076000)
#define is_v1_FMULWDC_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5207e000)
#define is_v1_FMULWDC_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52071000)
#define is_v1_FMULWDC_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52079000)
#define is_v1_FMULWDC_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52073000)
#define is_v1_FMULWDC_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5207b000)
#define is_v1_FMULWDC___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52077000)
#define is_v1_FMULWDC__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5207f000)
#define v1_FMULCWC_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4f010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULCWC_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4f010000)
#define v1_FMULCWC_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcf010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULCWC_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULCWC_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf010000)
#define v1_FMULCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcf010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf010000)
#define v1_FMULCWC_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWC__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULCWC_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f012000)
#define is_v1_FMULCWC_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f01a000)
#define is_v1_FMULCWC_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f014000)
#define is_v1_FMULCWC_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f01c000)
#define is_v1_FMULCWC_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f015000)
#define is_v1_FMULCWC_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f01d000)
#define is_v1_FMULCWC_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f010000)
#define is_v1_FMULCWC_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f018000)
#define is_v1_FMULCWC_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f016000)
#define is_v1_FMULCWC_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f01e000)
#define is_v1_FMULCWC_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f011000)
#define is_v1_FMULCWC_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f019000)
#define is_v1_FMULCWC_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f013000)
#define is_v1_FMULCWC_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f01b000)
#define is_v1_FMULCWC___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f017000)
#define is_v1_FMULCWC__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f01f000)
#define v1_FMULCWDC_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x43070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULCWDC_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x43070000)
#define v1_FMULCWDC_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc3070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULCWDC_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULCWDC_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc3070000)
#define v1_FMULCWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc3070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULCWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULCWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULCWDC_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc3070000)
#define v1_FMULCWDC_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53072000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5307a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53074000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5307c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53075000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5307d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53078000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53076000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5307e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53071000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53079000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53073000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5307b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53077000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULCWDC__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5307f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULCWDC_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53072000)
#define is_v1_FMULCWDC_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5307a000)
#define is_v1_FMULCWDC_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53074000)
#define is_v1_FMULCWDC_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5307c000)
#define is_v1_FMULCWDC_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53075000)
#define is_v1_FMULCWDC_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5307d000)
#define is_v1_FMULCWDC_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53070000)
#define is_v1_FMULCWDC_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53078000)
#define is_v1_FMULCWDC_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53076000)
#define is_v1_FMULCWDC_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5307e000)
#define is_v1_FMULCWDC_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53071000)
#define is_v1_FMULCWDC_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53079000)
#define is_v1_FMULCWDC_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53073000)
#define is_v1_FMULCWDC_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5307b000)
#define is_v1_FMULCWDC___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53077000)
#define is_v1_FMULCWDC__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5307f000)
#define v1_FADDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x40020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FADDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x40020000)
#define v1_FADDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc0020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FADDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FADDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0020000)
#define v1_FADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc0020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0020000)
#define v1_FADDD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FADDD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50022000)
#define is_v1_FADDD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002a000)
#define is_v1_FADDD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50024000)
#define is_v1_FADDD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002c000)
#define is_v1_FADDD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50025000)
#define is_v1_FADDD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002d000)
#define is_v1_FADDD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50020000)
#define is_v1_FADDD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50028000)
#define is_v1_FADDD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50026000)
#define is_v1_FADDD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002e000)
#define is_v1_FADDD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50021000)
#define is_v1_FADDD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50029000)
#define is_v1_FADDD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50023000)
#define is_v1_FADDD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002b000)
#define is_v1_FADDD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50027000)
#define is_v1_FADDD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002f000)
#define v1_FADDWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x41020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FADDWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x41020000)
#define v1_FADDWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc1020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FADDWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FADDWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1020000)
#define v1_FADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc1020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1020000)
#define v1_FADDWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FADDWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51022000)
#define is_v1_FADDWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102a000)
#define is_v1_FADDWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51024000)
#define is_v1_FADDWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102c000)
#define is_v1_FADDWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51025000)
#define is_v1_FADDWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102d000)
#define is_v1_FADDWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51020000)
#define is_v1_FADDWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51028000)
#define is_v1_FADDWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51026000)
#define is_v1_FADDWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102e000)
#define is_v1_FADDWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51021000)
#define is_v1_FADDWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51029000)
#define is_v1_FADDWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51023000)
#define is_v1_FADDWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102b000)
#define is_v1_FADDWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51027000)
#define is_v1_FADDWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102f000)
#define v1_FADDHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x42020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FADDHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x42020000)
#define v1_FADDHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc2020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FADDHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FADDHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2020000)
#define v1_FADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc2020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2020000)
#define v1_FADDHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FADDHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52022000)
#define is_v1_FADDHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202a000)
#define is_v1_FADDHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52024000)
#define is_v1_FADDHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202c000)
#define is_v1_FADDHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52025000)
#define is_v1_FADDHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202d000)
#define is_v1_FADDHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52020000)
#define is_v1_FADDHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52028000)
#define is_v1_FADDHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52026000)
#define is_v1_FADDHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202e000)
#define is_v1_FADDHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52021000)
#define is_v1_FADDHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52029000)
#define is_v1_FADDHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52023000)
#define is_v1_FADDHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202b000)
#define is_v1_FADDHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52027000)
#define is_v1_FADDHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202f000)
#define v1_FADDDP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDDP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FADDDP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c032000)
#define is_v1_FADDDP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03a000)
#define is_v1_FADDDP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c034000)
#define is_v1_FADDDP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03c000)
#define is_v1_FADDDP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c035000)
#define is_v1_FADDDP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03d000)
#define is_v1_FADDDP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c030000)
#define is_v1_FADDDP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c038000)
#define is_v1_FADDDP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c036000)
#define is_v1_FADDDP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03e000)
#define is_v1_FADDDP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c031000)
#define is_v1_FADDDP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c039000)
#define is_v1_FADDDP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c033000)
#define is_v1_FADDDP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03b000)
#define is_v1_FADDDP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c037000)
#define is_v1_FADDDP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03f000)
#define v1_FADDWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FADDWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58072000)
#define is_v1_FADDWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807a000)
#define is_v1_FADDWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58074000)
#define is_v1_FADDWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807c000)
#define is_v1_FADDWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58075000)
#define is_v1_FADDWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807d000)
#define is_v1_FADDWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58070000)
#define is_v1_FADDWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58078000)
#define is_v1_FADDWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58076000)
#define is_v1_FADDWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807e000)
#define is_v1_FADDWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58071000)
#define is_v1_FADDWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58079000)
#define is_v1_FADDWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58073000)
#define is_v1_FADDWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807b000)
#define is_v1_FADDWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58077000)
#define is_v1_FADDWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807f000)
#define v1_FADDCWC_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x43020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FADDCWC_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x43020000)
#define v1_FADDCWC_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc3020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FADDCWC_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FADDCWC_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc3020000)
#define v1_FADDCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc3020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FADDCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FADDCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FADDCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc3020000)
#define v1_FADDCWC_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDCWC__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FADDCWC_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53022000)
#define is_v1_FADDCWC_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302a000)
#define is_v1_FADDCWC_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53024000)
#define is_v1_FADDCWC_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302c000)
#define is_v1_FADDCWC_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53025000)
#define is_v1_FADDCWC_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302d000)
#define is_v1_FADDCWC_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53020000)
#define is_v1_FADDCWC_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53028000)
#define is_v1_FADDCWC_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53026000)
#define is_v1_FADDCWC_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302e000)
#define is_v1_FADDCWC_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53021000)
#define is_v1_FADDCWC_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53029000)
#define is_v1_FADDCWC_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53023000)
#define is_v1_FADDCWC_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302b000)
#define is_v1_FADDCWC___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53027000)
#define is_v1_FADDCWC__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302f000)
#define v1_FADDCDC_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCDC__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FADDCDC_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d032000)
#define is_v1_FADDCDC_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03a000)
#define is_v1_FADDCDC_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d034000)
#define is_v1_FADDCDC_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03c000)
#define is_v1_FADDCDC_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d035000)
#define is_v1_FADDCDC_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03d000)
#define is_v1_FADDCDC_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d030000)
#define is_v1_FADDCDC_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d038000)
#define is_v1_FADDCDC_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d036000)
#define is_v1_FADDCDC_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03e000)
#define is_v1_FADDCDC_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d031000)
#define is_v1_FADDCDC_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d039000)
#define is_v1_FADDCDC_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d033000)
#define is_v1_FADDCDC_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03b000)
#define is_v1_FADDCDC___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d037000)
#define is_v1_FADDCDC__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03f000)
#define v1_FADDCWCP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FADDCWCP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FADDCWCP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59072000)
#define is_v1_FADDCWCP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907a000)
#define is_v1_FADDCWCP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59074000)
#define is_v1_FADDCWCP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907c000)
#define is_v1_FADDCWCP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59075000)
#define is_v1_FADDCWCP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907d000)
#define is_v1_FADDCWCP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59070000)
#define is_v1_FADDCWCP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59078000)
#define is_v1_FADDCWCP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59076000)
#define is_v1_FADDCWCP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907e000)
#define is_v1_FADDCWCP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59071000)
#define is_v1_FADDCWCP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59079000)
#define is_v1_FADDCWCP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59073000)
#define is_v1_FADDCWCP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907b000)
#define is_v1_FADDCWCP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59077000)
#define is_v1_FADDCWCP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907f000)
#define v1_FSBFD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x44020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FSBFD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x44020000)
#define v1_FSBFD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc4020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FSBFD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4020000)
#define v1_FSBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc4020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FSBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FSBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4020000)
#define v1_FSBFD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FSBFD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54022000)
#define is_v1_FSBFD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402a000)
#define is_v1_FSBFD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54024000)
#define is_v1_FSBFD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402c000)
#define is_v1_FSBFD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54025000)
#define is_v1_FSBFD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402d000)
#define is_v1_FSBFD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54020000)
#define is_v1_FSBFD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54028000)
#define is_v1_FSBFD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54026000)
#define is_v1_FSBFD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402e000)
#define is_v1_FSBFD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54021000)
#define is_v1_FSBFD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54029000)
#define is_v1_FSBFD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54023000)
#define is_v1_FSBFD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402b000)
#define is_v1_FSBFD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54027000)
#define is_v1_FSBFD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402f000)
#define v1_FSBFWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x45020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FSBFWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x45020000)
#define v1_FSBFWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc5020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FSBFWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5020000)
#define v1_FSBFWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc5020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FSBFWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FSBFWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5020000)
#define v1_FSBFWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FSBFWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55022000)
#define is_v1_FSBFWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502a000)
#define is_v1_FSBFWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55024000)
#define is_v1_FSBFWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502c000)
#define is_v1_FSBFWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55025000)
#define is_v1_FSBFWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502d000)
#define is_v1_FSBFWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55020000)
#define is_v1_FSBFWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55028000)
#define is_v1_FSBFWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55026000)
#define is_v1_FSBFWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502e000)
#define is_v1_FSBFWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55021000)
#define is_v1_FSBFWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55029000)
#define is_v1_FSBFWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55023000)
#define is_v1_FSBFWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502b000)
#define is_v1_FSBFWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55027000)
#define is_v1_FSBFWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502f000)
#define v1_FSBFHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x46020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FSBFHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x46020000)
#define v1_FSBFHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc6020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FSBFHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6020000)
#define v1_FSBFHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc6020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FSBFHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FSBFHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6020000)
#define v1_FSBFHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FSBFHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56022000)
#define is_v1_FSBFHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602a000)
#define is_v1_FSBFHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56024000)
#define is_v1_FSBFHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602c000)
#define is_v1_FSBFHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56025000)
#define is_v1_FSBFHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602d000)
#define is_v1_FSBFHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56020000)
#define is_v1_FSBFHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56028000)
#define is_v1_FSBFHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56026000)
#define is_v1_FSBFHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602e000)
#define is_v1_FSBFHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56021000)
#define is_v1_FSBFHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56029000)
#define is_v1_FSBFHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56023000)
#define is_v1_FSBFHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602b000)
#define is_v1_FSBFHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56027000)
#define is_v1_FSBFHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602f000)
#define v1_FSBFDP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFDP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FSBFDP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e032000)
#define is_v1_FSBFDP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03a000)
#define is_v1_FSBFDP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e034000)
#define is_v1_FSBFDP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03c000)
#define is_v1_FSBFDP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e035000)
#define is_v1_FSBFDP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03d000)
#define is_v1_FSBFDP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e030000)
#define is_v1_FSBFDP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e038000)
#define is_v1_FSBFDP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e036000)
#define is_v1_FSBFDP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03e000)
#define is_v1_FSBFDP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e031000)
#define is_v1_FSBFDP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e039000)
#define is_v1_FSBFDP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e033000)
#define is_v1_FSBFDP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03b000)
#define is_v1_FSBFDP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e037000)
#define is_v1_FSBFDP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03f000)
#define v1_FSBFWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FSBFWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a072000)
#define is_v1_FSBFWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07a000)
#define is_v1_FSBFWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a074000)
#define is_v1_FSBFWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07c000)
#define is_v1_FSBFWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a075000)
#define is_v1_FSBFWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07d000)
#define is_v1_FSBFWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a070000)
#define is_v1_FSBFWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a078000)
#define is_v1_FSBFWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a076000)
#define is_v1_FSBFWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07e000)
#define is_v1_FSBFWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a071000)
#define is_v1_FSBFWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a079000)
#define is_v1_FSBFWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a073000)
#define is_v1_FSBFWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07b000)
#define is_v1_FSBFWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a077000)
#define is_v1_FSBFWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07f000)
#define v1_FSBFCWC_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x47020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FSBFCWC_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x47020000)
#define v1_FSBFCWC_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc7020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FSBFCWC_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFCWC_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7020000)
#define v1_FSBFCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc7020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FSBFCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FSBFCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFCWC_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7020000)
#define v1_FSBFCWC_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFCWC__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FSBFCWC_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57022000)
#define is_v1_FSBFCWC_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702a000)
#define is_v1_FSBFCWC_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57024000)
#define is_v1_FSBFCWC_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702c000)
#define is_v1_FSBFCWC_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57025000)
#define is_v1_FSBFCWC_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702d000)
#define is_v1_FSBFCWC_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57020000)
#define is_v1_FSBFCWC_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57028000)
#define is_v1_FSBFCWC_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57026000)
#define is_v1_FSBFCWC_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702e000)
#define is_v1_FSBFCWC_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57021000)
#define is_v1_FSBFCWC_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57029000)
#define is_v1_FSBFCWC_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57023000)
#define is_v1_FSBFCWC_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702b000)
#define is_v1_FSBFCWC___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57027000)
#define is_v1_FSBFCWC__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702f000)
#define v1_FSBFCDC_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCDC__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FSBFCDC_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f032000)
#define is_v1_FSBFCDC_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03a000)
#define is_v1_FSBFCDC_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f034000)
#define is_v1_FSBFCDC_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03c000)
#define is_v1_FSBFCDC_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f035000)
#define is_v1_FSBFCDC_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03d000)
#define is_v1_FSBFCDC_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f030000)
#define is_v1_FSBFCDC_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f038000)
#define is_v1_FSBFCDC_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f036000)
#define is_v1_FSBFCDC_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03e000)
#define is_v1_FSBFCDC_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f031000)
#define is_v1_FSBFCDC_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f039000)
#define is_v1_FSBFCDC_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f033000)
#define is_v1_FSBFCDC_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03b000)
#define is_v1_FSBFCDC___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f037000)
#define is_v1_FSBFCDC__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03f000)
#define v1_FSBFCWCP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FSBFCWCP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FSBFCWCP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b072000)
#define is_v1_FSBFCWCP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07a000)
#define is_v1_FSBFCWCP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b074000)
#define is_v1_FSBFCWCP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07c000)
#define is_v1_FSBFCWCP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b075000)
#define is_v1_FSBFCWCP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07d000)
#define is_v1_FSBFCWCP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b070000)
#define is_v1_FSBFCWCP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b078000)
#define is_v1_FSBFCWCP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b076000)
#define is_v1_FSBFCWCP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07e000)
#define is_v1_FSBFCWCP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b071000)
#define is_v1_FSBFCWCP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b079000)
#define is_v1_FSBFCWCP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b073000)
#define is_v1_FSBFCWCP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07b000)
#define is_v1_FSBFCWCP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b077000)
#define is_v1_FSBFCWCP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07f000)
#define v1_FFMAHW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x48020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAHW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x48020000)
#define v1_FFMAHW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc8020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAHW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAHW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc8020000)
#define v1_FFMAHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc8020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAHW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc8020000)
#define v1_FFMAHW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAHW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58022000)
#define is_v1_FFMAHW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802a000)
#define is_v1_FFMAHW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58024000)
#define is_v1_FFMAHW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802c000)
#define is_v1_FFMAHW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58025000)
#define is_v1_FFMAHW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802d000)
#define is_v1_FFMAHW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58020000)
#define is_v1_FFMAHW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58028000)
#define is_v1_FFMAHW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58026000)
#define is_v1_FFMAHW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802e000)
#define is_v1_FFMAHW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58021000)
#define is_v1_FFMAHW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58029000)
#define is_v1_FFMAHW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58023000)
#define is_v1_FFMAHW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802b000)
#define is_v1_FFMAHW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58027000)
#define is_v1_FFMAHW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802f000)
#define v1_FFMAW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x49020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x49020000)
#define v1_FFMAW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc9020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc9020000)
#define v1_FFMAW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc9020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc9020000)
#define v1_FFMAW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59022000)
#define is_v1_FFMAW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902a000)
#define is_v1_FFMAW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59024000)
#define is_v1_FFMAW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902c000)
#define is_v1_FFMAW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59025000)
#define is_v1_FFMAW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902d000)
#define is_v1_FFMAW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59020000)
#define is_v1_FFMAW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59028000)
#define is_v1_FFMAW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59026000)
#define is_v1_FFMAW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902e000)
#define is_v1_FFMAW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59021000)
#define is_v1_FFMAW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59029000)
#define is_v1_FFMAW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59023000)
#define is_v1_FFMAW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902b000)
#define is_v1_FFMAW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59027000)
#define is_v1_FFMAW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902f000)
#define v1_FFMAWD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x41010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAWD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x41010000)
#define v1_FFMAWD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc1010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAWD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAWD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1010000)
#define v1_FFMAWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc1010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAWD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1010000)
#define v1_FFMAWD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAWD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51012000)
#define is_v1_FFMAWD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101a000)
#define is_v1_FFMAWD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51014000)
#define is_v1_FFMAWD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101c000)
#define is_v1_FFMAWD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51015000)
#define is_v1_FFMAWD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101d000)
#define is_v1_FFMAWD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51010000)
#define is_v1_FFMAWD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51018000)
#define is_v1_FFMAWD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51016000)
#define is_v1_FFMAWD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101e000)
#define is_v1_FFMAWD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51011000)
#define is_v1_FFMAWD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51019000)
#define is_v1_FFMAWD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51013000)
#define is_v1_FFMAWD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101b000)
#define is_v1_FFMAWD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51017000)
#define is_v1_FFMAWD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101f000)
#define v1_FFMAWDP_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x40030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAWDP_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x40030000)
#define v1_FFMAWDP_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc0030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAWDP_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAWDP_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc0030000)
#define v1_FFMAWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc0030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc0030000)
#define v1_FFMAWDP_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAWDP__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAWDP_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50032000)
#define is_v1_FFMAWDP_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003a000)
#define is_v1_FFMAWDP_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50034000)
#define is_v1_FFMAWDP_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003c000)
#define is_v1_FFMAWDP_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50035000)
#define is_v1_FFMAWDP_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003d000)
#define is_v1_FFMAWDP_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50030000)
#define is_v1_FFMAWDP_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50038000)
#define is_v1_FFMAWDP_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50036000)
#define is_v1_FFMAWDP_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003e000)
#define is_v1_FFMAWDP_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50031000)
#define is_v1_FFMAWDP_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50039000)
#define is_v1_FFMAWDP_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50033000)
#define is_v1_FFMAWDP_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003b000)
#define is_v1_FFMAWDP___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50037000)
#define is_v1_FFMAWDP__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003f000)
#define v1_FFMAHWQ_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x41030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMAHWQ_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x41030000)
#define v1_FFMAHWQ_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc1030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMAHWQ_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAHWQ_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc1030000)
#define v1_FFMAHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc1030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMAHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMAHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMAHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc1030000)
#define v1_FFMAHWQ_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMAHWQ__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMAHWQ_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51032000)
#define is_v1_FFMAHWQ_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103a000)
#define is_v1_FFMAHWQ_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51034000)
#define is_v1_FFMAHWQ_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103c000)
#define is_v1_FFMAHWQ_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51035000)
#define is_v1_FFMAHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103d000)
#define is_v1_FFMAHWQ_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51030000)
#define is_v1_FFMAHWQ_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51038000)
#define is_v1_FFMAHWQ_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51036000)
#define is_v1_FFMAHWQ_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103e000)
#define is_v1_FFMAHWQ_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51031000)
#define is_v1_FFMAHWQ_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51039000)
#define is_v1_FFMAHWQ_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51033000)
#define is_v1_FFMAHWQ_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103b000)
#define is_v1_FFMAHWQ___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51037000)
#define is_v1_FFMAHWQ__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103f000)
#define v1_FFMSHW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4a020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSHW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4a020000)
#define v1_FFMSHW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xca020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSHW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSHW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xca020000)
#define v1_FFMSHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xca020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSHW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xca020000)
#define v1_FFMSHW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSHW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a022000)
#define is_v1_FFMSHW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02a000)
#define is_v1_FFMSHW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a024000)
#define is_v1_FFMSHW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02c000)
#define is_v1_FFMSHW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a025000)
#define is_v1_FFMSHW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02d000)
#define is_v1_FFMSHW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a020000)
#define is_v1_FFMSHW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a028000)
#define is_v1_FFMSHW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a026000)
#define is_v1_FFMSHW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02e000)
#define is_v1_FFMSHW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a021000)
#define is_v1_FFMSHW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a029000)
#define is_v1_FFMSHW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a023000)
#define is_v1_FFMSHW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02b000)
#define is_v1_FFMSHW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a027000)
#define is_v1_FFMSHW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02f000)
#define v1_FFMSW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4b020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4b020000)
#define v1_FFMSW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcb020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcb020000)
#define v1_FFMSW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcb020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcb020000)
#define v1_FFMSW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b022000)
#define is_v1_FFMSW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02a000)
#define is_v1_FFMSW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b024000)
#define is_v1_FFMSW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02c000)
#define is_v1_FFMSW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b025000)
#define is_v1_FFMSW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02d000)
#define is_v1_FFMSW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b020000)
#define is_v1_FFMSW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b028000)
#define is_v1_FFMSW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b026000)
#define is_v1_FFMSW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02e000)
#define is_v1_FFMSW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b021000)
#define is_v1_FFMSW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b029000)
#define is_v1_FFMSW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b023000)
#define is_v1_FFMSW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02b000)
#define is_v1_FFMSW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b027000)
#define is_v1_FFMSW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02f000)
#define v1_FFMSWD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x45010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSWD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x45010000)
#define v1_FFMSWD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc5010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSWD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSWD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5010000)
#define v1_FFMSWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc5010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSWD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5010000)
#define v1_FFMSWD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSWD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55012000)
#define is_v1_FFMSWD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501a000)
#define is_v1_FFMSWD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55014000)
#define is_v1_FFMSWD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501c000)
#define is_v1_FFMSWD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55015000)
#define is_v1_FFMSWD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501d000)
#define is_v1_FFMSWD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55010000)
#define is_v1_FFMSWD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55018000)
#define is_v1_FFMSWD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55016000)
#define is_v1_FFMSWD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501e000)
#define is_v1_FFMSWD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55011000)
#define is_v1_FFMSWD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55019000)
#define is_v1_FFMSWD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55013000)
#define is_v1_FFMSWD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501b000)
#define is_v1_FFMSWD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55017000)
#define is_v1_FFMSWD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501f000)
#define v1_FFMSWDP_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x42030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSWDP_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x42030000)
#define v1_FFMSWDP_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc2030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSWDP_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSWDP_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc2030000)
#define v1_FFMSWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc2030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSWDP_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc2030000)
#define v1_FFMSWDP_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSWDP__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSWDP_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52032000)
#define is_v1_FFMSWDP_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203a000)
#define is_v1_FFMSWDP_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52034000)
#define is_v1_FFMSWDP_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203c000)
#define is_v1_FFMSWDP_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52035000)
#define is_v1_FFMSWDP_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203d000)
#define is_v1_FFMSWDP_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52030000)
#define is_v1_FFMSWDP_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52038000)
#define is_v1_FFMSWDP_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52036000)
#define is_v1_FFMSWDP_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203e000)
#define is_v1_FFMSWDP_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52031000)
#define is_v1_FFMSWDP_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52039000)
#define is_v1_FFMSWDP_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52033000)
#define is_v1_FFMSWDP_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203b000)
#define is_v1_FFMSWDP___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52037000)
#define is_v1_FFMSWDP__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203f000)
#define v1_FFMSHWQ_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x43030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FFMSHWQ_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x43030000)
#define v1_FFMSHWQ_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc3030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FFMSHWQ_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSHWQ_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc3030000)
#define v1_FFMSHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc3030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FFMSHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FFMSHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FFMSHWQ_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc3030000)
#define v1_FFMSHWQ_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FFMSHWQ__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FFMSHWQ_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53032000)
#define is_v1_FFMSHWQ_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303a000)
#define is_v1_FFMSHWQ_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53034000)
#define is_v1_FFMSHWQ_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303c000)
#define is_v1_FFMSHWQ_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53035000)
#define is_v1_FFMSHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303d000)
#define is_v1_FFMSHWQ_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53030000)
#define is_v1_FFMSHWQ_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53038000)
#define is_v1_FFMSHWQ_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53036000)
#define is_v1_FFMSHWQ_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303e000)
#define is_v1_FFMSHWQ_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53031000)
#define is_v1_FFMSHWQ_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53039000)
#define is_v1_FFMSHWQ_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53033000)
#define is_v1_FFMSHWQ_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303b000)
#define is_v1_FFMSHWQ___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53037000)
#define is_v1_FFMSHWQ__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303f000)
#define v1_FADDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4c020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FADDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4c020000)
#define v1_FADDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcc020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FADDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FADDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc020000)
#define v1_FADDW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcc020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FADDW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FADDW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FADDW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc020000)
#define v1_FADDW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FADDW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FADDW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c022000)
#define is_v1_FADDW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02a000)
#define is_v1_FADDW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c024000)
#define is_v1_FADDW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02c000)
#define is_v1_FADDW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c025000)
#define is_v1_FADDW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02d000)
#define is_v1_FADDW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c020000)
#define is_v1_FADDW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c028000)
#define is_v1_FADDW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c026000)
#define is_v1_FADDW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02e000)
#define is_v1_FADDW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c021000)
#define is_v1_FADDW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c029000)
#define is_v1_FADDW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c023000)
#define is_v1_FADDW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02b000)
#define is_v1_FADDW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c027000)
#define is_v1_FADDW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02f000)
#define v1_FSBFW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4d020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FSBFW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4d020000)
#define v1_FSBFW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcd020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FSBFW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd020000)
#define v1_FSBFW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcd020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FSBFW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FSBFW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FSBFW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd020000)
#define v1_FSBFW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FSBFW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FSBFW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d022000)
#define is_v1_FSBFW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02a000)
#define is_v1_FSBFW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d024000)
#define is_v1_FSBFW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02c000)
#define is_v1_FSBFW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d025000)
#define is_v1_FSBFW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02d000)
#define is_v1_FSBFW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d020000)
#define is_v1_FSBFW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d028000)
#define is_v1_FSBFW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d026000)
#define is_v1_FSBFW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02e000)
#define is_v1_FSBFW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d021000)
#define is_v1_FSBFW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d029000)
#define is_v1_FSBFW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d023000)
#define is_v1_FSBFW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02b000)
#define is_v1_FSBFW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d027000)
#define is_v1_FSBFW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02f000)
#define v1_FMULHW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4f020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULHW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4f020000)
#define v1_FMULHW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcf020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULHW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULHW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf020000)
#define v1_FMULHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcf020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULHW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULHW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf020000)
#define v1_FMULHW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULHW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULHW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f022000)
#define is_v1_FMULHW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02a000)
#define is_v1_FMULHW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f024000)
#define is_v1_FMULHW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02c000)
#define is_v1_FMULHW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f025000)
#define is_v1_FMULHW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02d000)
#define is_v1_FMULHW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f020000)
#define is_v1_FMULHW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f028000)
#define is_v1_FMULHW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f026000)
#define is_v1_FMULHW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02e000)
#define is_v1_FMULHW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f021000)
#define is_v1_FMULHW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f029000)
#define is_v1_FMULHW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f023000)
#define is_v1_FMULHW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02b000)
#define is_v1_FMULHW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f027000)
#define is_v1_FMULHW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02f000)
#define v1_FMULW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4e020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4e020000)
#define v1_FMULW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xce020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce020000)
#define v1_FMULW_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xce020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULW_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULW_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULW_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce020000)
#define v1_FMULW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e022000)
#define is_v1_FMULW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02a000)
#define is_v1_FMULW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e024000)
#define is_v1_FMULW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02c000)
#define is_v1_FMULW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e025000)
#define is_v1_FMULW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02d000)
#define is_v1_FMULW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e020000)
#define is_v1_FMULW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e028000)
#define is_v1_FMULW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e026000)
#define is_v1_FMULW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02e000)
#define is_v1_FMULW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e021000)
#define is_v1_FMULW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e029000)
#define is_v1_FMULW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e023000)
#define is_v1_FMULW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02b000)
#define is_v1_FMULW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e027000)
#define is_v1_FMULW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02f000)
#define v1_FMULWD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x49010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FMULWD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x49010000)
#define v1_FMULWD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc9010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FMULWD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc9010000)
#define v1_FMULWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc9010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FMULWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FMULWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FMULWD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc9010000)
#define v1_FMULWD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMULWD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMULWD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59012000)
#define is_v1_FMULWD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901a000)
#define is_v1_FMULWD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59014000)
#define is_v1_FMULWD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901c000)
#define is_v1_FMULWD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59015000)
#define is_v1_FMULWD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901d000)
#define is_v1_FMULWD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59010000)
#define is_v1_FMULWD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59018000)
#define is_v1_FMULWD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59016000)
#define is_v1_FMULWD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901e000)
#define is_v1_FMULWD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59011000)
#define is_v1_FMULWD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59019000)
#define is_v1_FMULWD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59013000)
#define is_v1_FMULWD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901b000)
#define is_v1_FMULWD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59017000)
#define is_v1_FMULWD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901f000)
#define v1_FDOT2W_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4c010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FDOT2W_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4c010000)
#define v1_FDOT2W_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcc010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FDOT2W_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FDOT2W_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc010000)
#define v1_FDOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcc010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FDOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FDOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FDOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc010000)
#define v1_FDOT2W_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2W__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FDOT2W_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c012000)
#define is_v1_FDOT2W_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01a000)
#define is_v1_FDOT2W_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c014000)
#define is_v1_FDOT2W_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01c000)
#define is_v1_FDOT2W_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c015000)
#define is_v1_FDOT2W_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01d000)
#define is_v1_FDOT2W_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c010000)
#define is_v1_FDOT2W_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c018000)
#define is_v1_FDOT2W_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c016000)
#define is_v1_FDOT2W_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01e000)
#define is_v1_FDOT2W_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c011000)
#define is_v1_FDOT2W_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c019000)
#define is_v1_FDOT2W_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c013000)
#define is_v1_FDOT2W_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01b000)
#define is_v1_FDOT2W___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c017000)
#define is_v1_FDOT2W__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01f000)
#define v1_FDOT2WD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4d010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v1_FDOT2WD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4d010000)
#define v1_FDOT2WD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcd010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v1_FDOT2WD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v1_FDOT2WD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd010000)
#define v1_FDOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcd010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v1_FDOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v1_FDOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v1_FDOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd010000)
#define v1_FDOT2WD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FDOT2WD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FDOT2WD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d012000)
#define is_v1_FDOT2WD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01a000)
#define is_v1_FDOT2WD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d014000)
#define is_v1_FDOT2WD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01c000)
#define is_v1_FDOT2WD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d015000)
#define is_v1_FDOT2WD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01d000)
#define is_v1_FDOT2WD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d010000)
#define is_v1_FDOT2WD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d018000)
#define is_v1_FDOT2WD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d016000)
#define is_v1_FDOT2WD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01e000)
#define is_v1_FDOT2WD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d011000)
#define is_v1_FDOT2WD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d019000)
#define is_v1_FDOT2WD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d013000)
#define is_v1_FDOT2WD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01b000)
#define is_v1_FDOT2WD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d017000)
#define is_v1_FDOT2WD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01f000)
#define v1_FDOT2WZP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WZP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FDOT2WZP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d072000)
#define is_v1_FDOT2WZP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07a000)
#define is_v1_FDOT2WZP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d074000)
#define is_v1_FDOT2WZP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07c000)
#define is_v1_FDOT2WZP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d075000)
#define is_v1_FDOT2WZP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07d000)
#define is_v1_FDOT2WZP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d070000)
#define is_v1_FDOT2WZP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d078000)
#define is_v1_FDOT2WZP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d076000)
#define is_v1_FDOT2WZP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07e000)
#define is_v1_FDOT2WZP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d071000)
#define is_v1_FDOT2WZP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d079000)
#define is_v1_FDOT2WZP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d073000)
#define is_v1_FDOT2WZP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07b000)
#define is_v1_FDOT2WZP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d077000)
#define is_v1_FDOT2WZP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07f000)
#define v1_FDOT2WDP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v1_FDOT2WDP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v1_FDOT2WDP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c072000)
#define is_v1_FDOT2WDP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07a000)
#define is_v1_FDOT2WDP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c074000)
#define is_v1_FDOT2WDP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07c000)
#define is_v1_FDOT2WDP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c075000)
#define is_v1_FDOT2WDP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07d000)
#define is_v1_FDOT2WDP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c070000)
#define is_v1_FDOT2WDP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c078000)
#define is_v1_FDOT2WDP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c076000)
#define is_v1_FDOT2WDP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07e000)
#define is_v1_FDOT2WDP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c071000)
#define is_v1_FDOT2WDP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c079000)
#define is_v1_FDOT2WDP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c073000)
#define is_v1_FDOT2WDP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07b000)
#define is_v1_FDOT2WDP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c077000)
#define is_v1_FDOT2WDP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07f000)
#define v1_FMMA212W_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5403a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5403c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5403d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5403e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5403b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMA212W__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5403f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMMA212W_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54032000)
#define is_v1_FMMA212W_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5403a000)
#define is_v1_FMMA212W_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54034000)
#define is_v1_FMMA212W_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5403c000)
#define is_v1_FMMA212W_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54035000)
#define is_v1_FMMA212W_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5403d000)
#define is_v1_FMMA212W_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54030000)
#define is_v1_FMMA212W_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54038000)
#define is_v1_FMMA212W_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54036000)
#define is_v1_FMMA212W_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5403e000)
#define is_v1_FMMA212W_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54031000)
#define is_v1_FMMA212W_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54039000)
#define is_v1_FMMA212W_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54033000)
#define is_v1_FMMA212W_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5403b000)
#define is_v1_FMMA212W___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54037000)
#define is_v1_FMMA212W__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5403f000)
#define v1_FMMS212W_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5603a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5603c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5603d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5603e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5603b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMMS212W__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5603f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMMS212W_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56032000)
#define is_v1_FMMS212W_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5603a000)
#define is_v1_FMMS212W_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56034000)
#define is_v1_FMMS212W_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5603c000)
#define is_v1_FMMS212W_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56035000)
#define is_v1_FMMS212W_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5603d000)
#define is_v1_FMMS212W_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56030000)
#define is_v1_FMMS212W_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56038000)
#define is_v1_FMMS212W_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56036000)
#define is_v1_FMMS212W_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5603e000)
#define is_v1_FMMS212W_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56031000)
#define is_v1_FMMS212W_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56039000)
#define is_v1_FMMS212W_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56033000)
#define is_v1_FMMS212W_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5603b000)
#define is_v1_FMMS212W___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56037000)
#define is_v1_FMMS212W__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5603f000)
#define v1_FMM212W_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54072000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5407a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54074000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5407c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54075000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5407d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54078000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54076000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5407e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54071000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54079000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54073000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5407b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54077000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v1_FMM212W__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5407f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_FMM212W_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54072000)
#define is_v1_FMM212W_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5407a000)
#define is_v1_FMM212W_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54074000)
#define is_v1_FMM212W_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5407c000)
#define is_v1_FMM212W_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54075000)
#define is_v1_FMM212W_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5407d000)
#define is_v1_FMM212W_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54070000)
#define is_v1_FMM212W_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54078000)
#define is_v1_FMM212W_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54076000)
#define is_v1_FMM212W_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5407e000)
#define is_v1_FMM212W_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54071000)
#define is_v1_FMM212W_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54079000)
#define is_v1_FMM212W_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54073000)
#define is_v1_FMM212W_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5407b000)
#define is_v1_FMM212W___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x54077000)
#define is_v1_FMM212W__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5407f000)
#define v1_COPYQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f00f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_COPYQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f00f000)
#define v1_COPYO_registerN_registerR_simple_insn(registerN,registerR) (unsigned int)(0x3e050000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerR) & 0xf) << 2))
#define is_v1_COPYO_registerN_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3e050000)
#define v1_MOVETQ_registerAl_registerZ_registerY_simple_insn(registerAl,registerZ,registerY) (unsigned int)(0x7f02e000 | (((int)(registerAl) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MOVETQ_registerAl_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f02e000)
#define v1_MOVETQ_registerAh_registerZ_registerY_simple_insn(registerAh,registerZ,registerY) (unsigned int)(0x7f02f000 | (((int)(registerAh) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v1_MOVETQ_registerAh_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f02f000)
#define v1_ALIGNO_registerN_registerBe_registerCo_byteshift_simple_insn(registerN,registerBe,registerCo,byteshift) (unsigned int)(0x00081000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v1_ALIGNO_registerN_registerBe_registerCo_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x00081000)
#define v1_ALIGNO_registerN_registerBo_registerCe_byteshift_simple_insn(registerN,registerBo,registerCe,byteshift) (unsigned int)(0x000c1000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v1_ALIGNO_registerN_registerBo_registerCe_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x000c1000)
#define v1_ALIGNO_registerN_registerBe_registerCo_registerZ_simple_insn(registerN,registerBe,registerCo,registerZ) (unsigned int)(0x00081040 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v1_ALIGNO_registerN_registerBe_registerCo_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x00081040)
#define v1_ALIGNO_registerN_registerBo_registerCe_registerZ_simple_insn(registerN,registerBo,registerCe,registerZ) (unsigned int)(0x000c1040 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v1_ALIGNO_registerN_registerBo_registerCe_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x000c1040)
#define v1_ALIGNV_registerA_registerBe_registerCo_byteshift_simple_insn(registerA,registerBe,registerCo,byteshift) (unsigned int)(0x01000000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v1_ALIGNV_registerA_registerBe_registerCo_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01000000)
#define v1_ALIGNV_registerA_registerBo_registerCe_byteshift_simple_insn(registerA,registerBo,registerCe,byteshift) (unsigned int)(0x01001000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v1_ALIGNV_registerA_registerBo_registerCe_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01001000)
#define v1_ALIGNV_registerA_registerBe_registerCo_registerZ_simple_insn(registerA,registerBe,registerCo,registerZ) (unsigned int)(0x01000040 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v1_ALIGNV_registerA_registerBe_registerCo_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01000040)
#define v1_ALIGNV_registerA_registerBo_registerCe_registerZ_simple_insn(registerA,registerBo,registerCe,registerZ) (unsigned int)(0x01001040 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v1_ALIGNV_registerA_registerBo_registerCe_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01001040)
#define v1_MT44D_registerAq_registerBq_simple_insn(registerAq,registerBq) (unsigned int)(0x040c0000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14))
#define is_v1_MT44D_registerAq_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x040c0000)
#define v1_MMA484BW_registerAp_registerBp_registerC_registerD_simple_insn(registerAp,registerBp,registerC,registerD) (unsigned int)(0x02000000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA484BW_registerAp_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x02000000)
#define v1_MMA484UBW_registerAp_registerBp_registerC_registerD_simple_insn(registerAp,registerBp,registerC,registerD) (unsigned int)(0x02001000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA484UBW_registerAp_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x02001000)
#define v1_MMA484SUBW_registerAp_registerBp_registerC_registerD_simple_insn(registerAp,registerBp,registerC,registerD) (unsigned int)(0x02040000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA484SUBW_registerAp_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x02040000)
#define v1_MMA484USBW_registerAp_registerBp_registerC_registerD_simple_insn(registerAp,registerBp,registerC,registerD) (unsigned int)(0x02041000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA484USBW_registerAp_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x02041000)
#define v1_MMA444HBD0_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04000000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444HBD0_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04000000)
#define v1_MMA444UHBD0_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04001000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444UHBD0_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04001000)
#define v1_MMA444SUHBD0_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04002000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444SUHBD0_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04002000)
#define v1_MMA444USHBD0_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04003000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444USHBD0_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04003000)
#define v1_MMA444HBD1_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04040000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444HBD1_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04040000)
#define v1_MMA444UHBD1_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04041000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444UHBD1_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04041000)
#define v1_MMA444SUHBD1_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04042000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444SUHBD1_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04042000)
#define v1_MMA444USHBD1_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04043000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444USHBD1_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04043000)
#define v1_MMA444HD_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04080000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444HD_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04080000)
#define v1_MMA444UHD_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04081000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444UHD_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04081000)
#define v1_MMA444SUHD_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04082000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444SUHD_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04082000)
#define v1_MMA444USHD_registerAq_registerBq_registerC_registerD_simple_insn(registerAq,registerBq,registerC,registerD) (unsigned int)(0x04083000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_MMA444USHD_registerAq_registerBq_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3000) == 0x04083000)
#define v1_FMMA242HW0_registerA0_registerBp_registerC_registerD_simple_insn(registerA0,registerBp,registerC,registerD) (unsigned int)(0x03000000 | (((int)(registerA0) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_FMMA242HW0_registerA0_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x03000000)
#define v1_FMMA242HW1_registerA1_registerBp_registerC_registerD_simple_insn(registerA1,registerBp,registerC,registerD) (unsigned int)(0x03001000 | (((int)(registerA1) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_FMMA242HW1_registerA1_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x03001000)
#define v1_FMMA242HW2_registerA2_registerBp_registerC_registerD_simple_insn(registerA2,registerBp,registerC,registerD) (unsigned int)(0x03040000 | (((int)(registerA2) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_FMMA242HW2_registerA2_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x03040000)
#define v1_FMMA242HW3_registerA3_registerBp_registerC_registerD_simple_insn(registerA3,registerBp,registerC,registerD) (unsigned int)(0x03041000 | (((int)(registerA3) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerC) & 0x3f) << 6) | ((int)(registerD) & 0x3f))
#define is_v1_FMMA242HW3_registerA3_registerBp_registerC_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041000) == 0x03041000)
#define v1_CONVDHV0_rd_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000200 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_rd_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000280 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_rhu_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000400 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_rhu_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000480 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_rn_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000000 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_rn_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000080 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_ru_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000100 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_ru_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000180 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_rz_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000300 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV0_rz_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x05000380 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v1_CONVDHV0_rd_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000200)
#define is_v1_CONVDHV0_rd_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000280)
#define is_v1_CONVDHV0_rhu_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000400)
#define is_v1_CONVDHV0_rhu_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000480)
#define is_v1_CONVDHV0_rn_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000000)
#define is_v1_CONVDHV0_rn_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000080)
#define is_v1_CONVDHV0_ru_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000100)
#define is_v1_CONVDHV0_ru_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000180)
#define is_v1_CONVDHV0_rz_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000300)
#define is_v1_CONVDHV0_rz_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05000380)
#define v1_CONVDHV1_rd_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001200 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_rd_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001280 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_rhu_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001400 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_rhu_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001480 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_rn_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001000 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_rn_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001080 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_ru_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001100 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_ru_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001180 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_rz_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001300 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVDHV1_rz_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x05001380 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v1_CONVDHV1_rd_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001200)
#define is_v1_CONVDHV1_rd_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001280)
#define is_v1_CONVDHV1_rhu_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001400)
#define is_v1_CONVDHV1_rhu_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001480)
#define is_v1_CONVDHV1_rn_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001000)
#define is_v1_CONVDHV1_rn_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001080)
#define is_v1_CONVDHV1_ru_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001100)
#define is_v1_CONVDHV1_ru_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001180)
#define is_v1_CONVDHV1_rz_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001300)
#define is_v1_CONVDHV1_rz_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x05001380)
#define v1_CONVWBV0_rd_sat_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000200 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_rd_satu_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000280 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_rhu_sat_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000400 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_rhu_satu_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000480 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_rn_sat_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000000 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_rn_satu_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000080 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_ru_sat_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000100 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_ru_satu_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000180 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_rz_sat_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000300 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV0_rz_satu_registerAx_registerBq_simple_insn(registerAx,registerBq) (unsigned int)(0x06000380 | (((int)(registerAx) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v1_CONVWBV0_rd_sat_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000200)
#define is_v1_CONVWBV0_rd_satu_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000280)
#define is_v1_CONVWBV0_rhu_sat_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000400)
#define is_v1_CONVWBV0_rhu_satu_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000480)
#define is_v1_CONVWBV0_rn_sat_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000000)
#define is_v1_CONVWBV0_rn_satu_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000080)
#define is_v1_CONVWBV0_ru_sat_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000100)
#define is_v1_CONVWBV0_ru_satu_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000180)
#define is_v1_CONVWBV0_rz_sat_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000300)
#define is_v1_CONVWBV0_rz_satu_registerAx_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06000380)
#define v1_CONVWBV1_rd_sat_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001200 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_rd_satu_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001280 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_rhu_sat_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001400 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_rhu_satu_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001480 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_rn_sat_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001000 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_rn_satu_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001080 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_ru_sat_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001100 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_ru_satu_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001180 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_rz_sat_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001300 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV1_rz_satu_registerAy_registerBq_simple_insn(registerAy,registerBq) (unsigned int)(0x06001380 | (((int)(registerAy) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v1_CONVWBV1_rd_sat_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001200)
#define is_v1_CONVWBV1_rd_satu_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001280)
#define is_v1_CONVWBV1_rhu_sat_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001400)
#define is_v1_CONVWBV1_rhu_satu_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001480)
#define is_v1_CONVWBV1_rn_sat_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001000)
#define is_v1_CONVWBV1_rn_satu_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001080)
#define is_v1_CONVWBV1_ru_sat_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001100)
#define is_v1_CONVWBV1_ru_satu_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001180)
#define is_v1_CONVWBV1_rz_sat_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001300)
#define is_v1_CONVWBV1_rz_satu_registerAy_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06001380)
#define v1_CONVWBV2_rd_sat_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002200 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_rd_satu_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002280 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_rhu_sat_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002400 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_rhu_satu_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002480 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_rn_sat_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002000 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_rn_satu_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002080 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_ru_sat_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002100 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_ru_satu_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002180 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_rz_sat_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002300 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV2_rz_satu_registerAz_registerBq_simple_insn(registerAz,registerBq) (unsigned int)(0x06002380 | (((int)(registerAz) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v1_CONVWBV2_rd_sat_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002200)
#define is_v1_CONVWBV2_rd_satu_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002280)
#define is_v1_CONVWBV2_rhu_sat_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002400)
#define is_v1_CONVWBV2_rhu_satu_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002480)
#define is_v1_CONVWBV2_rn_sat_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002000)
#define is_v1_CONVWBV2_rn_satu_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002080)
#define is_v1_CONVWBV2_ru_sat_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002100)
#define is_v1_CONVWBV2_ru_satu_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002180)
#define is_v1_CONVWBV2_rz_sat_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002300)
#define is_v1_CONVWBV2_rz_satu_registerAz_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06002380)
#define v1_CONVWBV3_rd_sat_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003200 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_rd_satu_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003280 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_rhu_sat_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003400 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_rhu_satu_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003480 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_rn_sat_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003000 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_rn_satu_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003080 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_ru_sat_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003100 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_ru_satu_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003180 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_rz_sat_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003300 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v1_CONVWBV3_rz_satu_registerAt_registerBq_simple_insn(registerAt,registerBq) (unsigned int)(0x06003380 | (((int)(registerAt) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v1_CONVWBV3_rd_sat_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003200)
#define is_v1_CONVWBV3_rd_satu_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003280)
#define is_v1_CONVWBV3_rhu_sat_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003400)
#define is_v1_CONVWBV3_rhu_satu_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003480)
#define is_v1_CONVWBV3_rn_sat_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003000)
#define is_v1_CONVWBV3_rn_satu_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003080)
#define is_v1_CONVWBV3_ru_sat_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003100)
#define is_v1_CONVWBV3_ru_satu_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003180)
#define is_v1_CONVWBV3_rz_sat_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003300)
#define is_v1_CONVWBV3_rz_satu_registerAt_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0037c0) == 0x06003380)
#define v1_FSCALEWV_rd_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000a00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rd_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000a80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rd___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000200 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rd__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000280 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rna_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000c00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rna_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000c80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rna___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000400 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rna__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000480 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rnz_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000d00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rnz_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000d80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rnz___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000500 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rnz__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000580 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rn_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000800 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rn_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000880 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rn___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rn__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000080 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ro_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000e00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ro_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000e80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ro___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000600 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ro__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000680 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ru_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000900 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ru_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000980 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ru___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000100 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_ru__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000180 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rz_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000b00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rz_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000b80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rz___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000300 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV_rz__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000380 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV__s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000f00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV__s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000f80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV____registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000700 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v1_FSCALEWV___relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000780 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define is_v1_FSCALEWV_rd_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000a00)
#define is_v1_FSCALEWV_rd_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000a80)
#define is_v1_FSCALEWV_rd___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000200)
#define is_v1_FSCALEWV_rd__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000280)
#define is_v1_FSCALEWV_rna_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000c00)
#define is_v1_FSCALEWV_rna_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000c80)
#define is_v1_FSCALEWV_rna___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000400)
#define is_v1_FSCALEWV_rna__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000480)
#define is_v1_FSCALEWV_rnz_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000d00)
#define is_v1_FSCALEWV_rnz_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000d80)
#define is_v1_FSCALEWV_rnz___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000500)
#define is_v1_FSCALEWV_rnz__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000580)
#define is_v1_FSCALEWV_rn_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000800)
#define is_v1_FSCALEWV_rn_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000880)
#define is_v1_FSCALEWV_rn___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000000)
#define is_v1_FSCALEWV_rn__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000080)
#define is_v1_FSCALEWV_ro_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000e00)
#define is_v1_FSCALEWV_ro_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000e80)
#define is_v1_FSCALEWV_ro___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000600)
#define is_v1_FSCALEWV_ro__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000680)
#define is_v1_FSCALEWV_ru_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000900)
#define is_v1_FSCALEWV_ru_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000980)
#define is_v1_FSCALEWV_ru___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000100)
#define is_v1_FSCALEWV_ru__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000180)
#define is_v1_FSCALEWV_rz_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000b00)
#define is_v1_FSCALEWV_rz_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000b80)
#define is_v1_FSCALEWV_rz___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000300)
#define is_v1_FSCALEWV_rz__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000380)
#define is_v1_FSCALEWV__s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000f00)
#define is_v1_FSCALEWV__s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000f80)
#define is_v1_FSCALEWV____registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000700)
#define is_v1_FSCALEWV___relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000780)
#define v1_FNARROWWHV_rd__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000240 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rd_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000a40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rna__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000440 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rna_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000c40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rnz__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000540 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rnz_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000d40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rn__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000040 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rn_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000840 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_ro__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000640 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_ro_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000e40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_ru__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000140 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_ru_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000940 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rz__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000340 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV_rz_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000b40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV___registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000740 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v1_FNARROWWHV__s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000f40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define is_v1_FNARROWWHV_rd__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000240)
#define is_v1_FNARROWWHV_rd_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000a40)
#define is_v1_FNARROWWHV_rna__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000440)
#define is_v1_FNARROWWHV_rna_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000c40)
#define is_v1_FNARROWWHV_rnz__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000540)
#define is_v1_FNARROWWHV_rnz_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000d40)
#define is_v1_FNARROWWHV_rn__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000040)
#define is_v1_FNARROWWHV_rn_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000840)
#define is_v1_FNARROWWHV_ro__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000640)
#define is_v1_FNARROWWHV_ro_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000e40)
#define is_v1_FNARROWWHV_ru__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000140)
#define is_v1_FNARROWWHV_ru_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000940)
#define is_v1_FNARROWWHV_rz__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000340)
#define is_v1_FNARROWWHV_rz_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000b40)
#define is_v1_FNARROWWHV___registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000740)
#define is_v1_FNARROWWHV__s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000f40)
#define v1_COPYD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_COPYD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6a000000)
#define v1_NOTD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6c00ffc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_NOTD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6c00ffc0)
#define v1_NEGD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x65000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_NEGD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x65000000)
#define v1_ABSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x64000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ABSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x64000000)
#define v1_ZXBD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78003fc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ZXBD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x78003fc0)
#define v1_SXBD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x68037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SXBD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x68037000)
#define v1_ZXHD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6403f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ZXHD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6403f000)
#define v1_SXHD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6803f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SXHD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6803f000)
#define v1_ZXWD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7800ffc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ZXWD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7800ffc0)
#define v1_SXWD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6903f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SXWD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6903f000)
#define v1_SATDH_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7e022400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SATDH_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7e022400)
#define v1_SATDW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7e022800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_SATDW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7e022800)
#define v1_COPYW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_COPYW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7a000000)
#define v1_NOTW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c00ffc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_NOTW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7c00ffc0)
#define v1_NEGW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x75000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_NEGW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x75000000)
#define v1_ABSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ABSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x74000000)
#define v1_NEGWP_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf5012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_NEGWP_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf5012000)
#define v1_ABSWP_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf4012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ABSWP_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf4012000)
#define v1_NEGHQ_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf5013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_NEGHQ_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf5013000)
#define v1_ABSHQ_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf4013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v1_ABSHQ_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf4013000)
#define v1_COPYV_registerA_registerBe_simple_insn(registerA,registerBe) (unsigned int)(0x01000f80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBe) & 0x1f) << 13))
#define is_v1_COPYV_registerA_registerBe_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001fff) == 0x01000f80)
#define v1_COPYV_registerA_registerBo_simple_insn(registerA,registerBo) (unsigned int)(0x01001f80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBo) & 0x1f) << 13))
#define is_v1_COPYV_registerA_registerBo_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001fff) == 0x01001f80)
#define v1_MOVEFO_registerN_registerBe_simple_insn(registerN,registerBe) (unsigned int)(0x00081f80 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBe) & 0x1f) << 13))
#define is_v1_MOVEFO_registerN_registerBe_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1fff) == 0x00081f80)
#define v1_MOVEFO_registerN_registerBo_simple_insn(registerN,registerBo) (unsigned int)(0x000c1f80 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBo) & 0x1f) << 13))
#define is_v1_MOVEFO_registerN_registerBo_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1fff) == 0x000c1f80)
#define is_v2_AWAIT_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fa00000)
#define is_v2_SLEEP_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fa40000)
#define is_v2_STOP_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fa80000)
#define is_v2_BARRIER_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fac0000)
#define is_v2_TLBREAD_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f800000)
#define is_v2_TLBPROBE_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f840000)
#define is_v2_TLBWRITE_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f880000)
#define is_v2_TLBDINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f8c0000)
#define is_v2_TLBIINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0f900000)
#define is_v2_ERROP_simple_insn(x) (int)(((unsigned int)(x) & 0x7fffffc0) == 0x00000000)
#define v2_BREAK_brknumber_simple_insn(brknumber) (unsigned int)(0x00000040 | ((int)(brknumber) & 0x3))
#define is_v2_BREAK_brknumber_simple_insn(x) (int)(((unsigned int)(x) & 0x7fffffc0) == 0x00000040)
#define is_v2_RET_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fd00000)
#define is_v2_RFE_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fd40000)
#define v2_IGOTO_registerZ_simple_insn(registerZ) (unsigned int)(0x0fd80000 | ((int)(registerZ) & 0x3f))
#define is_v2_IGOTO_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fd80000)
#define v2_ICALL_registerZ_simple_insn(registerZ) (unsigned int)(0x0fdc0000 | ((int)(registerZ) & 0x3f))
#define is_v2_ICALL_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fdc0000)
#define v2_SCALL_registerZ_simple_insn(registerZ) (unsigned int)(0x0fe40000 | ((int)(registerZ) & 0x3f))
#define is_v2_SCALL_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fe40000)
#define v2_SCALL_sysnumber_simple_insn(sysnumber) (unsigned int)(0x0fe00000 | ((int)(sysnumber) & 0xfff))
#define is_v2_SCALL_sysnumber_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fe00000)
#define v2_LOOPDO_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0f000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define is_v2_LOOPDO_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0f000000)
#define v2_WFXL_systemT2_registerZ_simple_insn(systemT2,registerZ) (unsigned int)(0x0fb80000 | (((int)(systemT2) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_WFXL_systemT2_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb80000)
#define v2_WFXL_systemAlone_registerZ_simple_insn(systemAlone,registerZ) (unsigned int)(0x0fb80000 | (((int)(systemAlone) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_WFXL_systemAlone_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb80000)
#define v2_WFXL_systemT4_registerZ_simple_insn(systemT4,registerZ) (unsigned int)(0x0fb80000 | (((int)(systemT4) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_WFXL_systemT4_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb80000)
#define v2_WFXM_systemT2_registerZ_simple_insn(systemT2,registerZ) (unsigned int)(0x0fbc0000 | (((int)(systemT2) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_WFXM_systemT2_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fbc0000)
#define v2_WFXM_systemAlone_registerZ_simple_insn(systemAlone,registerZ) (unsigned int)(0x0fbc0000 | (((int)(systemAlone) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_WFXM_systemAlone_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fbc0000)
#define v2_WFXM_systemT4_registerZ_simple_insn(systemT4,registerZ) (unsigned int)(0x0fbc0000 | (((int)(systemT4) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_WFXM_systemT4_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fbc0000)
#define v2_GET_registerZ_systemS2_simple_insn(registerZ,systemS2) (unsigned int)(0x0fc40000 | ((int)(registerZ) & 0x3f) | (((int)(systemS2) & 0x1ff) << 6))
#define is_v2_GET_registerZ_systemS2_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc40000)
#define v2_GET_registerZ_systemS3_simple_insn(registerZ,systemS3) (unsigned int)(0x0fc40000 | ((int)(registerZ) & 0x3f) | (((int)(systemS3) & 0x1ff) << 6))
#define is_v2_GET_registerZ_systemS3_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc40000)
#define v2_IGET_registerZ_simple_insn(registerZ) (unsigned int)(0x0fcc0000 | ((int)(registerZ) & 0x3f))
#define is_v2_IGET_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fcc0000)
#define v2_SET_systemT3_registerZ_simple_insn(systemT3,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemT3) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_SET_systemT3_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v2_SET_systemAlone_registerZ_simple_insn(systemAlone,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemAlone) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_SET_systemAlone_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v2_SET_systemRA_registerZ_simple_insn(systemRA,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemRA) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_SET_systemRA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v2_SET_systemT4_registerZ_simple_insn(systemT4,registerZ) (unsigned int)(0x0fc00000 | (((int)(systemT4) & 0x1ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_SET_systemT4_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc00000)
#define v2_RSWAP_registerZ_systemS4_simple_insn(registerZ,systemS4) (unsigned int)(0x0fc80000 | ((int)(registerZ) & 0x3f) | (((int)(systemS4) & 0x1ff) << 6))
#define is_v2_RSWAP_registerZ_systemS4_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc80000)
#define v2_RSWAP_registerZ_systemAlone_simple_insn(registerZ,systemAlone) (unsigned int)(0x0fc80000 | ((int)(registerZ) & 0x3f) | (((int)(systemAlone) & 0x1ff) << 6))
#define is_v2_RSWAP_registerZ_systemAlone_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc80000)
#define v2_RSWAP_registerZ_systemS3_simple_insn(registerZ,systemS3) (unsigned int)(0x0fc80000 | ((int)(registerZ) & 0x3f) | (((int)(systemS3) & 0x1ff) << 6))
#define is_v2_RSWAP_registerZ_systemS3_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fc80000)
#define v2_WAITIT_registerZ_simple_insn(registerZ) (unsigned int)(0x0fb00000 | ((int)(registerZ) & 0x3f))
#define is_v2_WAITIT_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb00000)
#define v2_SYNCGROUP_registerZ_simple_insn(registerZ) (unsigned int)(0x0fb40000 | ((int)(registerZ) & 0x3f))
#define is_v2_SYNCGROUP_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffc0000) == 0x0fb40000)
#define v2_GOTO_pcrel27_simple_insn(pcrel27) (unsigned int)(0x10000000 | (((int)(pcrel27) >> 2) & 0x7ffffff))
#define is_v2_GOTO_pcrel27_simple_insn(x) (int)(((unsigned int)(x) & 0x78000000) == 0x10000000)
#define v2_CALL_pcrel27_simple_insn(pcrel27) (unsigned int)(0x18000000 | (((int)(pcrel27) >> 2) & 0x7ffffff))
#define is_v2_CALL_pcrel27_simple_insn(x) (int)(((unsigned int)(x) & 0x78000000) == 0x18000000)
#define v2_CB_deqz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x08800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_dgez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x09800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_dgtz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0a800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_dlez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0a000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_dltz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x09000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_dnez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x08000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_even_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0b800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_odd_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0b000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_weqz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0c800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_wgez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0d800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_wgtz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0e800000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_wlez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0e000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_wltz_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0d000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define v2_CB_wnez_registerZ_pcrel17_simple_insn(registerZ,pcrel17) (unsigned int)(0x0c000000 | ((int)(registerZ) & 0x3f) | ((((int)(pcrel17) >> 2) & 0x1ffff) << 6))
#define is_v2_CB_deqz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x08800000)
#define is_v2_CB_dgez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x09800000)
#define is_v2_CB_dgtz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0a800000)
#define is_v2_CB_dlez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0a000000)
#define is_v2_CB_dltz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x09000000)
#define is_v2_CB_dnez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x08000000)
#define is_v2_CB_even_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0b800000)
#define is_v2_CB_odd_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0b000000)
#define is_v2_CB_weqz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0c800000)
#define is_v2_CB_wgez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0d800000)
#define is_v2_CB_wgtz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0e800000)
#define is_v2_CB_wlez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0e000000)
#define is_v2_CB_wltz_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0d000000)
#define is_v2_CB_wnez_registerZ_pcrel17_simple_insn(x) (int)(((unsigned int)(x) & 0x7f800000) == 0x0c000000)
#define v2_LBZ__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x20000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x21000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x22000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x23000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LBZ__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x20000000)
#define is_v2_LBZ_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x21000000)
#define is_v2_LBZ_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x22000000)
#define is_v2_LBZ_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x23000000)
#define v2_LBZ__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa0000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LBZ_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa1000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LBZ_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa2000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LBZ_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa3000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LBZ__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0000000)
#define is_v2_LBZ_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1000000)
#define is_v2_LBZ_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa2000000)
#define is_v2_LBZ_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa3000000)
#define v2_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa1000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LBZ__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0000000)
#define is_v2_LBZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1000000)
#define is_v2_LBZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa2000000)
#define is_v2_LBZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa3000000)
#define v2_LBZ_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x21028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x23028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x22028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x20028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LBZ_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21021000)
#define is_v2_LBZ_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21023000)
#define is_v2_LBZ_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21025000)
#define is_v2_LBZ_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21024000)
#define is_v2_LBZ_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21022000)
#define is_v2_LBZ_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21020000)
#define is_v2_LBZ_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21027000)
#define is_v2_LBZ_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21026000)
#define is_v2_LBZ_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21029000)
#define is_v2_LBZ_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102b000)
#define is_v2_LBZ_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102d000)
#define is_v2_LBZ_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102c000)
#define is_v2_LBZ_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102a000)
#define is_v2_LBZ_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21028000)
#define is_v2_LBZ_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23021000)
#define is_v2_LBZ_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23023000)
#define is_v2_LBZ_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23025000)
#define is_v2_LBZ_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23024000)
#define is_v2_LBZ_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23022000)
#define is_v2_LBZ_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23020000)
#define is_v2_LBZ_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23027000)
#define is_v2_LBZ_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23026000)
#define is_v2_LBZ_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23029000)
#define is_v2_LBZ_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302b000)
#define is_v2_LBZ_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302d000)
#define is_v2_LBZ_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302c000)
#define is_v2_LBZ_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302a000)
#define is_v2_LBZ_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x23028000)
#define is_v2_LBZ_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22021000)
#define is_v2_LBZ_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22023000)
#define is_v2_LBZ_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22025000)
#define is_v2_LBZ_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22024000)
#define is_v2_LBZ_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22022000)
#define is_v2_LBZ_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22020000)
#define is_v2_LBZ_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22027000)
#define is_v2_LBZ_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22026000)
#define is_v2_LBZ_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22029000)
#define is_v2_LBZ_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202b000)
#define is_v2_LBZ_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202d000)
#define is_v2_LBZ_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202c000)
#define is_v2_LBZ_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202a000)
#define is_v2_LBZ_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x22028000)
#define is_v2_LBZ__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20021000)
#define is_v2_LBZ__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20023000)
#define is_v2_LBZ__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20025000)
#define is_v2_LBZ__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20024000)
#define is_v2_LBZ__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20022000)
#define is_v2_LBZ__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20020000)
#define is_v2_LBZ__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20027000)
#define is_v2_LBZ__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20026000)
#define is_v2_LBZ__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20029000)
#define is_v2_LBZ__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002b000)
#define is_v2_LBZ__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002d000)
#define is_v2_LBZ__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002c000)
#define is_v2_LBZ__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002a000)
#define is_v2_LBZ__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20028000)
#define v2_LBZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LBZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1021000)
#define is_v2_LBZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1023000)
#define is_v2_LBZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1025000)
#define is_v2_LBZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1024000)
#define is_v2_LBZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1022000)
#define is_v2_LBZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1020000)
#define is_v2_LBZ_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1027000)
#define is_v2_LBZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1026000)
#define is_v2_LBZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1029000)
#define is_v2_LBZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102b000)
#define is_v2_LBZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102d000)
#define is_v2_LBZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102c000)
#define is_v2_LBZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102a000)
#define is_v2_LBZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1028000)
#define is_v2_LBZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3021000)
#define is_v2_LBZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3023000)
#define is_v2_LBZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3025000)
#define is_v2_LBZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3024000)
#define is_v2_LBZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3022000)
#define is_v2_LBZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3020000)
#define is_v2_LBZ_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3027000)
#define is_v2_LBZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3026000)
#define is_v2_LBZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3029000)
#define is_v2_LBZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302b000)
#define is_v2_LBZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302d000)
#define is_v2_LBZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302c000)
#define is_v2_LBZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302a000)
#define is_v2_LBZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3028000)
#define is_v2_LBZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2021000)
#define is_v2_LBZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2023000)
#define is_v2_LBZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2025000)
#define is_v2_LBZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2024000)
#define is_v2_LBZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2022000)
#define is_v2_LBZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2020000)
#define is_v2_LBZ_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2027000)
#define is_v2_LBZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2026000)
#define is_v2_LBZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2029000)
#define is_v2_LBZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202b000)
#define is_v2_LBZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202d000)
#define is_v2_LBZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202c000)
#define is_v2_LBZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202a000)
#define is_v2_LBZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2028000)
#define is_v2_LBZ__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0021000)
#define is_v2_LBZ__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0023000)
#define is_v2_LBZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0025000)
#define is_v2_LBZ__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0024000)
#define is_v2_LBZ__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0022000)
#define is_v2_LBZ__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0020000)
#define is_v2_LBZ__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0027000)
#define is_v2_LBZ__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0026000)
#define is_v2_LBZ__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0029000)
#define is_v2_LBZ__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002b000)
#define is_v2_LBZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002d000)
#define is_v2_LBZ__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002c000)
#define is_v2_LBZ__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002a000)
#define is_v2_LBZ__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0028000)
#define v2_LBZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LBZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1021000)
#define is_v2_LBZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1023000)
#define is_v2_LBZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1025000)
#define is_v2_LBZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1024000)
#define is_v2_LBZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1022000)
#define is_v2_LBZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1020000)
#define is_v2_LBZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1027000)
#define is_v2_LBZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1026000)
#define is_v2_LBZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1029000)
#define is_v2_LBZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102b000)
#define is_v2_LBZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102d000)
#define is_v2_LBZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102c000)
#define is_v2_LBZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa102a000)
#define is_v2_LBZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1028000)
#define is_v2_LBZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3021000)
#define is_v2_LBZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3023000)
#define is_v2_LBZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3025000)
#define is_v2_LBZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3024000)
#define is_v2_LBZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3022000)
#define is_v2_LBZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3020000)
#define is_v2_LBZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3027000)
#define is_v2_LBZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3026000)
#define is_v2_LBZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3029000)
#define is_v2_LBZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302b000)
#define is_v2_LBZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302d000)
#define is_v2_LBZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302c000)
#define is_v2_LBZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa302a000)
#define is_v2_LBZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa3028000)
#define is_v2_LBZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2021000)
#define is_v2_LBZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2023000)
#define is_v2_LBZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2025000)
#define is_v2_LBZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2024000)
#define is_v2_LBZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2022000)
#define is_v2_LBZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2020000)
#define is_v2_LBZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2027000)
#define is_v2_LBZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2026000)
#define is_v2_LBZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2029000)
#define is_v2_LBZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202b000)
#define is_v2_LBZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202d000)
#define is_v2_LBZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202c000)
#define is_v2_LBZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa202a000)
#define is_v2_LBZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa2028000)
#define is_v2_LBZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0021000)
#define is_v2_LBZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0023000)
#define is_v2_LBZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0025000)
#define is_v2_LBZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0024000)
#define is_v2_LBZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0022000)
#define is_v2_LBZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0020000)
#define is_v2_LBZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0027000)
#define is_v2_LBZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0026000)
#define is_v2_LBZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0029000)
#define is_v2_LBZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002b000)
#define is_v2_LBZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002d000)
#define is_v2_LBZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002c000)
#define is_v2_LBZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa002a000)
#define is_v2_LBZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0028000)
#define v2_LBZ_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2102e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2102f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2302e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2302f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2202e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2202f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2002e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBZ__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2002f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LBZ_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102e000)
#define is_v2_LBZ_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2102f000)
#define is_v2_LBZ_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302e000)
#define is_v2_LBZ_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2302f000)
#define is_v2_LBZ_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202e000)
#define is_v2_LBZ_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2202f000)
#define is_v2_LBZ___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002e000)
#define is_v2_LBZ__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2002f000)
#define v2_LBS__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x24000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x25000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x26000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x27000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LBS__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x24000000)
#define is_v2_LBS_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x25000000)
#define is_v2_LBS_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x26000000)
#define is_v2_LBS_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x27000000)
#define v2_LBS__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa4000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LBS_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa5000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LBS_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa6000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LBS_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa7000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LBS__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4000000)
#define is_v2_LBS_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5000000)
#define is_v2_LBS_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6000000)
#define is_v2_LBS_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7000000)
#define v2_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa4000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa5000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa6000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa7000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LBS__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4000000)
#define is_v2_LBS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5000000)
#define is_v2_LBS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6000000)
#define is_v2_LBS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7000000)
#define v2_LBS_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x25028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x27028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x26028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x24028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LBS_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25021000)
#define is_v2_LBS_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25023000)
#define is_v2_LBS_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25025000)
#define is_v2_LBS_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25024000)
#define is_v2_LBS_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25022000)
#define is_v2_LBS_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25020000)
#define is_v2_LBS_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25027000)
#define is_v2_LBS_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25026000)
#define is_v2_LBS_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25029000)
#define is_v2_LBS_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502b000)
#define is_v2_LBS_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502d000)
#define is_v2_LBS_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502c000)
#define is_v2_LBS_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502a000)
#define is_v2_LBS_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25028000)
#define is_v2_LBS_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27021000)
#define is_v2_LBS_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27023000)
#define is_v2_LBS_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27025000)
#define is_v2_LBS_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27024000)
#define is_v2_LBS_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27022000)
#define is_v2_LBS_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27020000)
#define is_v2_LBS_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27027000)
#define is_v2_LBS_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27026000)
#define is_v2_LBS_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27029000)
#define is_v2_LBS_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702b000)
#define is_v2_LBS_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702d000)
#define is_v2_LBS_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702c000)
#define is_v2_LBS_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702a000)
#define is_v2_LBS_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27028000)
#define is_v2_LBS_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26021000)
#define is_v2_LBS_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26023000)
#define is_v2_LBS_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26025000)
#define is_v2_LBS_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26024000)
#define is_v2_LBS_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26022000)
#define is_v2_LBS_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26020000)
#define is_v2_LBS_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26027000)
#define is_v2_LBS_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26026000)
#define is_v2_LBS_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26029000)
#define is_v2_LBS_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602b000)
#define is_v2_LBS_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602d000)
#define is_v2_LBS_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602c000)
#define is_v2_LBS_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602a000)
#define is_v2_LBS_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26028000)
#define is_v2_LBS__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24021000)
#define is_v2_LBS__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24023000)
#define is_v2_LBS__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24025000)
#define is_v2_LBS__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24024000)
#define is_v2_LBS__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24022000)
#define is_v2_LBS__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24020000)
#define is_v2_LBS__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24027000)
#define is_v2_LBS__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24026000)
#define is_v2_LBS__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24029000)
#define is_v2_LBS__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402b000)
#define is_v2_LBS__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402d000)
#define is_v2_LBS__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402c000)
#define is_v2_LBS__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402a000)
#define is_v2_LBS__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24028000)
#define v2_LBS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LBS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5021000)
#define is_v2_LBS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5023000)
#define is_v2_LBS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5025000)
#define is_v2_LBS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5024000)
#define is_v2_LBS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5022000)
#define is_v2_LBS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5020000)
#define is_v2_LBS_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5027000)
#define is_v2_LBS_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5026000)
#define is_v2_LBS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5029000)
#define is_v2_LBS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502b000)
#define is_v2_LBS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502d000)
#define is_v2_LBS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502c000)
#define is_v2_LBS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502a000)
#define is_v2_LBS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5028000)
#define is_v2_LBS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7021000)
#define is_v2_LBS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7023000)
#define is_v2_LBS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7025000)
#define is_v2_LBS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7024000)
#define is_v2_LBS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7022000)
#define is_v2_LBS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7020000)
#define is_v2_LBS_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7027000)
#define is_v2_LBS_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7026000)
#define is_v2_LBS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7029000)
#define is_v2_LBS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702b000)
#define is_v2_LBS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702d000)
#define is_v2_LBS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702c000)
#define is_v2_LBS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702a000)
#define is_v2_LBS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7028000)
#define is_v2_LBS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6021000)
#define is_v2_LBS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6023000)
#define is_v2_LBS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6025000)
#define is_v2_LBS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6024000)
#define is_v2_LBS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6022000)
#define is_v2_LBS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6020000)
#define is_v2_LBS_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6027000)
#define is_v2_LBS_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6026000)
#define is_v2_LBS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6029000)
#define is_v2_LBS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602b000)
#define is_v2_LBS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602d000)
#define is_v2_LBS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602c000)
#define is_v2_LBS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602a000)
#define is_v2_LBS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6028000)
#define is_v2_LBS__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4021000)
#define is_v2_LBS__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4023000)
#define is_v2_LBS__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4025000)
#define is_v2_LBS__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4024000)
#define is_v2_LBS__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4022000)
#define is_v2_LBS__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4020000)
#define is_v2_LBS__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4027000)
#define is_v2_LBS__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4026000)
#define is_v2_LBS__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4029000)
#define is_v2_LBS__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402b000)
#define is_v2_LBS__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402d000)
#define is_v2_LBS__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402c000)
#define is_v2_LBS__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402a000)
#define is_v2_LBS__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4028000)
#define v2_LBS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LBS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LBS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LBS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5021000)
#define is_v2_LBS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5023000)
#define is_v2_LBS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5025000)
#define is_v2_LBS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5024000)
#define is_v2_LBS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5022000)
#define is_v2_LBS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5020000)
#define is_v2_LBS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5027000)
#define is_v2_LBS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5026000)
#define is_v2_LBS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5029000)
#define is_v2_LBS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502b000)
#define is_v2_LBS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502d000)
#define is_v2_LBS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502c000)
#define is_v2_LBS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa502a000)
#define is_v2_LBS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5028000)
#define is_v2_LBS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7021000)
#define is_v2_LBS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7023000)
#define is_v2_LBS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7025000)
#define is_v2_LBS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7024000)
#define is_v2_LBS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7022000)
#define is_v2_LBS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7020000)
#define is_v2_LBS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7027000)
#define is_v2_LBS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7026000)
#define is_v2_LBS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7029000)
#define is_v2_LBS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702b000)
#define is_v2_LBS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702d000)
#define is_v2_LBS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702c000)
#define is_v2_LBS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa702a000)
#define is_v2_LBS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7028000)
#define is_v2_LBS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6021000)
#define is_v2_LBS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6023000)
#define is_v2_LBS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6025000)
#define is_v2_LBS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6024000)
#define is_v2_LBS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6022000)
#define is_v2_LBS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6020000)
#define is_v2_LBS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6027000)
#define is_v2_LBS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6026000)
#define is_v2_LBS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6029000)
#define is_v2_LBS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602b000)
#define is_v2_LBS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602d000)
#define is_v2_LBS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602c000)
#define is_v2_LBS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa602a000)
#define is_v2_LBS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6028000)
#define is_v2_LBS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4021000)
#define is_v2_LBS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4023000)
#define is_v2_LBS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4025000)
#define is_v2_LBS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4024000)
#define is_v2_LBS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4022000)
#define is_v2_LBS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4020000)
#define is_v2_LBS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4027000)
#define is_v2_LBS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4026000)
#define is_v2_LBS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4029000)
#define is_v2_LBS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402b000)
#define is_v2_LBS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402d000)
#define is_v2_LBS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402c000)
#define is_v2_LBS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa402a000)
#define is_v2_LBS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4028000)
#define v2_LBS_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2502e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2502f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2702e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2702f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2602e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2602f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2402e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LBS__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2402f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LBS_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502e000)
#define is_v2_LBS_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2502f000)
#define is_v2_LBS_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702e000)
#define is_v2_LBS_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2702f000)
#define is_v2_LBS_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602e000)
#define is_v2_LBS_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2602f000)
#define is_v2_LBS___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402e000)
#define is_v2_LBS__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2402f000)
#define v2_LHZ__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x28000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x29000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2a000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2b000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LHZ__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x28000000)
#define is_v2_LHZ_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x29000000)
#define is_v2_LHZ_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2a000000)
#define is_v2_LHZ_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2b000000)
#define v2_LHZ__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa8000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LHZ_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xa9000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LHZ_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xaa000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LHZ_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xab000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LHZ__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa8000000)
#define is_v2_LHZ_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9000000)
#define is_v2_LHZ_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa000000)
#define is_v2_LHZ_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab000000)
#define v2_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa8000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xa9000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xaa000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xab000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LHZ__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa8000000)
#define is_v2_LHZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9000000)
#define is_v2_LHZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa000000)
#define is_v2_LHZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab000000)
#define v2_LHZ_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x29028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x28028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LHZ_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29021000)
#define is_v2_LHZ_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29023000)
#define is_v2_LHZ_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29025000)
#define is_v2_LHZ_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29024000)
#define is_v2_LHZ_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29022000)
#define is_v2_LHZ_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29020000)
#define is_v2_LHZ_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29027000)
#define is_v2_LHZ_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29026000)
#define is_v2_LHZ_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29029000)
#define is_v2_LHZ_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902b000)
#define is_v2_LHZ_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902d000)
#define is_v2_LHZ_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902c000)
#define is_v2_LHZ_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902a000)
#define is_v2_LHZ_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29028000)
#define is_v2_LHZ_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b021000)
#define is_v2_LHZ_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b023000)
#define is_v2_LHZ_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b025000)
#define is_v2_LHZ_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b024000)
#define is_v2_LHZ_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b022000)
#define is_v2_LHZ_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b020000)
#define is_v2_LHZ_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b027000)
#define is_v2_LHZ_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b026000)
#define is_v2_LHZ_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b029000)
#define is_v2_LHZ_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02b000)
#define is_v2_LHZ_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02d000)
#define is_v2_LHZ_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02c000)
#define is_v2_LHZ_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02a000)
#define is_v2_LHZ_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b028000)
#define is_v2_LHZ_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a021000)
#define is_v2_LHZ_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a023000)
#define is_v2_LHZ_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a025000)
#define is_v2_LHZ_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a024000)
#define is_v2_LHZ_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a022000)
#define is_v2_LHZ_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a020000)
#define is_v2_LHZ_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a027000)
#define is_v2_LHZ_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a026000)
#define is_v2_LHZ_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a029000)
#define is_v2_LHZ_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02b000)
#define is_v2_LHZ_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02d000)
#define is_v2_LHZ_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02c000)
#define is_v2_LHZ_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02a000)
#define is_v2_LHZ_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a028000)
#define is_v2_LHZ__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28021000)
#define is_v2_LHZ__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28023000)
#define is_v2_LHZ__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28025000)
#define is_v2_LHZ__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28024000)
#define is_v2_LHZ__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28022000)
#define is_v2_LHZ__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28020000)
#define is_v2_LHZ__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28027000)
#define is_v2_LHZ__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28026000)
#define is_v2_LHZ__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28029000)
#define is_v2_LHZ__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802b000)
#define is_v2_LHZ__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802d000)
#define is_v2_LHZ__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802c000)
#define is_v2_LHZ__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802a000)
#define is_v2_LHZ__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x28028000)
#define v2_LHZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xa8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LHZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9021000)
#define is_v2_LHZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9023000)
#define is_v2_LHZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9025000)
#define is_v2_LHZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9024000)
#define is_v2_LHZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9022000)
#define is_v2_LHZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9020000)
#define is_v2_LHZ_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9027000)
#define is_v2_LHZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9026000)
#define is_v2_LHZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9029000)
#define is_v2_LHZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902b000)
#define is_v2_LHZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902d000)
#define is_v2_LHZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902c000)
#define is_v2_LHZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902a000)
#define is_v2_LHZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9028000)
#define is_v2_LHZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab021000)
#define is_v2_LHZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab023000)
#define is_v2_LHZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab025000)
#define is_v2_LHZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab024000)
#define is_v2_LHZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab022000)
#define is_v2_LHZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab020000)
#define is_v2_LHZ_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab027000)
#define is_v2_LHZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab026000)
#define is_v2_LHZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab029000)
#define is_v2_LHZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02b000)
#define is_v2_LHZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02d000)
#define is_v2_LHZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02c000)
#define is_v2_LHZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02a000)
#define is_v2_LHZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab028000)
#define is_v2_LHZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa021000)
#define is_v2_LHZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa023000)
#define is_v2_LHZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa025000)
#define is_v2_LHZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa024000)
#define is_v2_LHZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa022000)
#define is_v2_LHZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa020000)
#define is_v2_LHZ_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa027000)
#define is_v2_LHZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa026000)
#define is_v2_LHZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa029000)
#define is_v2_LHZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02b000)
#define is_v2_LHZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02d000)
#define is_v2_LHZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02c000)
#define is_v2_LHZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02a000)
#define is_v2_LHZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa028000)
#define is_v2_LHZ__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8021000)
#define is_v2_LHZ__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8023000)
#define is_v2_LHZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8025000)
#define is_v2_LHZ__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8024000)
#define is_v2_LHZ__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8022000)
#define is_v2_LHZ__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8020000)
#define is_v2_LHZ__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8027000)
#define is_v2_LHZ__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8026000)
#define is_v2_LHZ__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8029000)
#define is_v2_LHZ__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802b000)
#define is_v2_LHZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802d000)
#define is_v2_LHZ__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802c000)
#define is_v2_LHZ__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802a000)
#define is_v2_LHZ__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8028000)
#define v2_LHZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xa8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LHZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9021000)
#define is_v2_LHZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9023000)
#define is_v2_LHZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9025000)
#define is_v2_LHZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9024000)
#define is_v2_LHZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9022000)
#define is_v2_LHZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9020000)
#define is_v2_LHZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9027000)
#define is_v2_LHZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9026000)
#define is_v2_LHZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9029000)
#define is_v2_LHZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902b000)
#define is_v2_LHZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902d000)
#define is_v2_LHZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902c000)
#define is_v2_LHZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa902a000)
#define is_v2_LHZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9028000)
#define is_v2_LHZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab021000)
#define is_v2_LHZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab023000)
#define is_v2_LHZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab025000)
#define is_v2_LHZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab024000)
#define is_v2_LHZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab022000)
#define is_v2_LHZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab020000)
#define is_v2_LHZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab027000)
#define is_v2_LHZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab026000)
#define is_v2_LHZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab029000)
#define is_v2_LHZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02b000)
#define is_v2_LHZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02d000)
#define is_v2_LHZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02c000)
#define is_v2_LHZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab02a000)
#define is_v2_LHZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab028000)
#define is_v2_LHZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa021000)
#define is_v2_LHZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa023000)
#define is_v2_LHZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa025000)
#define is_v2_LHZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa024000)
#define is_v2_LHZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa022000)
#define is_v2_LHZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa020000)
#define is_v2_LHZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa027000)
#define is_v2_LHZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa026000)
#define is_v2_LHZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa029000)
#define is_v2_LHZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02b000)
#define is_v2_LHZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02d000)
#define is_v2_LHZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02c000)
#define is_v2_LHZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa02a000)
#define is_v2_LHZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa028000)
#define is_v2_LHZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8021000)
#define is_v2_LHZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8023000)
#define is_v2_LHZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8025000)
#define is_v2_LHZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8024000)
#define is_v2_LHZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8022000)
#define is_v2_LHZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8020000)
#define is_v2_LHZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8027000)
#define is_v2_LHZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8026000)
#define is_v2_LHZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8029000)
#define is_v2_LHZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802b000)
#define is_v2_LHZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802d000)
#define is_v2_LHZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802c000)
#define is_v2_LHZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa802a000)
#define is_v2_LHZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa8028000)
#define v2_LHZ_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2902e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2902f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2802e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHZ__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2802f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LHZ_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902e000)
#define is_v2_LHZ_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2902f000)
#define is_v2_LHZ_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02e000)
#define is_v2_LHZ_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b02f000)
#define is_v2_LHZ_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02e000)
#define is_v2_LHZ_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a02f000)
#define is_v2_LHZ___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802e000)
#define is_v2_LHZ__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2802f000)
#define v2_LHS__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2c000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2d000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2e000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2f000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LHS__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2c000000)
#define is_v2_LHS_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2d000000)
#define is_v2_LHS_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2e000000)
#define is_v2_LHS_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2f000000)
#define v2_LHS__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xac000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LHS_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xad000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LHS_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xae000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LHS_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xaf000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LHS__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xac000000)
#define is_v2_LHS_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xad000000)
#define is_v2_LHS_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae000000)
#define is_v2_LHS_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf000000)
#define v2_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xac000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xad000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xae000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xaf000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LHS__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xac000000)
#define is_v2_LHS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xad000000)
#define is_v2_LHS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae000000)
#define is_v2_LHS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf000000)
#define v2_LHS_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2d028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2f028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2e028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2c028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LHS_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d021000)
#define is_v2_LHS_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d023000)
#define is_v2_LHS_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d025000)
#define is_v2_LHS_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d024000)
#define is_v2_LHS_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d022000)
#define is_v2_LHS_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d020000)
#define is_v2_LHS_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d027000)
#define is_v2_LHS_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d026000)
#define is_v2_LHS_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d029000)
#define is_v2_LHS_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02b000)
#define is_v2_LHS_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02d000)
#define is_v2_LHS_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02c000)
#define is_v2_LHS_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02a000)
#define is_v2_LHS_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d028000)
#define is_v2_LHS_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f021000)
#define is_v2_LHS_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f023000)
#define is_v2_LHS_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f025000)
#define is_v2_LHS_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f024000)
#define is_v2_LHS_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f022000)
#define is_v2_LHS_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f020000)
#define is_v2_LHS_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f027000)
#define is_v2_LHS_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f026000)
#define is_v2_LHS_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f029000)
#define is_v2_LHS_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02b000)
#define is_v2_LHS_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02d000)
#define is_v2_LHS_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02c000)
#define is_v2_LHS_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02a000)
#define is_v2_LHS_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f028000)
#define is_v2_LHS_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e021000)
#define is_v2_LHS_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e023000)
#define is_v2_LHS_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e025000)
#define is_v2_LHS_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e024000)
#define is_v2_LHS_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e022000)
#define is_v2_LHS_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e020000)
#define is_v2_LHS_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e027000)
#define is_v2_LHS_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e026000)
#define is_v2_LHS_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e029000)
#define is_v2_LHS_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02b000)
#define is_v2_LHS_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02d000)
#define is_v2_LHS_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02c000)
#define is_v2_LHS_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02a000)
#define is_v2_LHS_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e028000)
#define is_v2_LHS__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c021000)
#define is_v2_LHS__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c023000)
#define is_v2_LHS__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c025000)
#define is_v2_LHS__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c024000)
#define is_v2_LHS__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c022000)
#define is_v2_LHS__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c020000)
#define is_v2_LHS__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c027000)
#define is_v2_LHS__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c026000)
#define is_v2_LHS__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c029000)
#define is_v2_LHS__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02b000)
#define is_v2_LHS__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02d000)
#define is_v2_LHS__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02c000)
#define is_v2_LHS__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02a000)
#define is_v2_LHS__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c028000)
#define v2_LHS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xad028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xae028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xac028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LHS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad021000)
#define is_v2_LHS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad023000)
#define is_v2_LHS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad025000)
#define is_v2_LHS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad024000)
#define is_v2_LHS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad022000)
#define is_v2_LHS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad020000)
#define is_v2_LHS_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad027000)
#define is_v2_LHS_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad026000)
#define is_v2_LHS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad029000)
#define is_v2_LHS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02b000)
#define is_v2_LHS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02d000)
#define is_v2_LHS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02c000)
#define is_v2_LHS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02a000)
#define is_v2_LHS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad028000)
#define is_v2_LHS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf021000)
#define is_v2_LHS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf023000)
#define is_v2_LHS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf025000)
#define is_v2_LHS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf024000)
#define is_v2_LHS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf022000)
#define is_v2_LHS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf020000)
#define is_v2_LHS_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf027000)
#define is_v2_LHS_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf026000)
#define is_v2_LHS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf029000)
#define is_v2_LHS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02b000)
#define is_v2_LHS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02d000)
#define is_v2_LHS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02c000)
#define is_v2_LHS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02a000)
#define is_v2_LHS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf028000)
#define is_v2_LHS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae021000)
#define is_v2_LHS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae023000)
#define is_v2_LHS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae025000)
#define is_v2_LHS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae024000)
#define is_v2_LHS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae022000)
#define is_v2_LHS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae020000)
#define is_v2_LHS_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae027000)
#define is_v2_LHS_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae026000)
#define is_v2_LHS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae029000)
#define is_v2_LHS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02b000)
#define is_v2_LHS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02d000)
#define is_v2_LHS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02c000)
#define is_v2_LHS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02a000)
#define is_v2_LHS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae028000)
#define is_v2_LHS__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac021000)
#define is_v2_LHS__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac023000)
#define is_v2_LHS__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac025000)
#define is_v2_LHS__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac024000)
#define is_v2_LHS__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac022000)
#define is_v2_LHS__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac020000)
#define is_v2_LHS__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac027000)
#define is_v2_LHS__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac026000)
#define is_v2_LHS__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac029000)
#define is_v2_LHS__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02b000)
#define is_v2_LHS__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02d000)
#define is_v2_LHS__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02c000)
#define is_v2_LHS__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02a000)
#define is_v2_LHS__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac028000)
#define v2_LHS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xad028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xae028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LHS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xac028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LHS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LHS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad021000)
#define is_v2_LHS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad023000)
#define is_v2_LHS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad025000)
#define is_v2_LHS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad024000)
#define is_v2_LHS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad022000)
#define is_v2_LHS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad020000)
#define is_v2_LHS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad027000)
#define is_v2_LHS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad026000)
#define is_v2_LHS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad029000)
#define is_v2_LHS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02b000)
#define is_v2_LHS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02d000)
#define is_v2_LHS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02c000)
#define is_v2_LHS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad02a000)
#define is_v2_LHS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xad028000)
#define is_v2_LHS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf021000)
#define is_v2_LHS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf023000)
#define is_v2_LHS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf025000)
#define is_v2_LHS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf024000)
#define is_v2_LHS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf022000)
#define is_v2_LHS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf020000)
#define is_v2_LHS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf027000)
#define is_v2_LHS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf026000)
#define is_v2_LHS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf029000)
#define is_v2_LHS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02b000)
#define is_v2_LHS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02d000)
#define is_v2_LHS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02c000)
#define is_v2_LHS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf02a000)
#define is_v2_LHS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf028000)
#define is_v2_LHS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae021000)
#define is_v2_LHS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae023000)
#define is_v2_LHS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae025000)
#define is_v2_LHS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae024000)
#define is_v2_LHS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae022000)
#define is_v2_LHS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae020000)
#define is_v2_LHS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae027000)
#define is_v2_LHS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae026000)
#define is_v2_LHS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae029000)
#define is_v2_LHS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02b000)
#define is_v2_LHS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02d000)
#define is_v2_LHS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02c000)
#define is_v2_LHS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae02a000)
#define is_v2_LHS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae028000)
#define is_v2_LHS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac021000)
#define is_v2_LHS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac023000)
#define is_v2_LHS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac025000)
#define is_v2_LHS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac024000)
#define is_v2_LHS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac022000)
#define is_v2_LHS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac020000)
#define is_v2_LHS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac027000)
#define is_v2_LHS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac026000)
#define is_v2_LHS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac029000)
#define is_v2_LHS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02b000)
#define is_v2_LHS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02d000)
#define is_v2_LHS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02c000)
#define is_v2_LHS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac02a000)
#define is_v2_LHS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xac028000)
#define v2_LHS_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2d02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2d02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2f02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2f02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2e02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2e02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2c02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LHS__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2c02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LHS_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02e000)
#define is_v2_LHS_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2d02f000)
#define is_v2_LHS_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02e000)
#define is_v2_LHS_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f02f000)
#define is_v2_LHS_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02e000)
#define is_v2_LHS_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e02f000)
#define is_v2_LHS___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02e000)
#define is_v2_LHS__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2c02f000)
#define v2_LWZ__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x30000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x31000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x32000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x33000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LWZ__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x30000000)
#define is_v2_LWZ_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x31000000)
#define is_v2_LWZ_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x32000000)
#define is_v2_LWZ_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x33000000)
#define v2_LWZ__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb0000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LWZ_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb1000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LWZ_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb2000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LWZ_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb3000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LWZ__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb0000000)
#define is_v2_LWZ_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb1000000)
#define is_v2_LWZ_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb2000000)
#define is_v2_LWZ_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb3000000)
#define v2_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb1000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb2000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb3000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LWZ__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb0000000)
#define is_v2_LWZ_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb1000000)
#define is_v2_LWZ_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb2000000)
#define is_v2_LWZ_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb3000000)
#define v2_LWZ_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x31028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x33028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x32028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x30028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LWZ_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31021000)
#define is_v2_LWZ_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31023000)
#define is_v2_LWZ_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31025000)
#define is_v2_LWZ_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31024000)
#define is_v2_LWZ_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31022000)
#define is_v2_LWZ_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31020000)
#define is_v2_LWZ_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31027000)
#define is_v2_LWZ_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31026000)
#define is_v2_LWZ_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31029000)
#define is_v2_LWZ_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102b000)
#define is_v2_LWZ_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102d000)
#define is_v2_LWZ_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102c000)
#define is_v2_LWZ_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102a000)
#define is_v2_LWZ_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x31028000)
#define is_v2_LWZ_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33021000)
#define is_v2_LWZ_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33023000)
#define is_v2_LWZ_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33025000)
#define is_v2_LWZ_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33024000)
#define is_v2_LWZ_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33022000)
#define is_v2_LWZ_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33020000)
#define is_v2_LWZ_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33027000)
#define is_v2_LWZ_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33026000)
#define is_v2_LWZ_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33029000)
#define is_v2_LWZ_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302b000)
#define is_v2_LWZ_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302d000)
#define is_v2_LWZ_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302c000)
#define is_v2_LWZ_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302a000)
#define is_v2_LWZ_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x33028000)
#define is_v2_LWZ_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32021000)
#define is_v2_LWZ_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32023000)
#define is_v2_LWZ_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32025000)
#define is_v2_LWZ_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32024000)
#define is_v2_LWZ_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32022000)
#define is_v2_LWZ_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32020000)
#define is_v2_LWZ_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32027000)
#define is_v2_LWZ_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32026000)
#define is_v2_LWZ_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32029000)
#define is_v2_LWZ_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202b000)
#define is_v2_LWZ_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202d000)
#define is_v2_LWZ_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202c000)
#define is_v2_LWZ_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202a000)
#define is_v2_LWZ_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x32028000)
#define is_v2_LWZ__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30021000)
#define is_v2_LWZ__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30023000)
#define is_v2_LWZ__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30025000)
#define is_v2_LWZ__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30024000)
#define is_v2_LWZ__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30022000)
#define is_v2_LWZ__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30020000)
#define is_v2_LWZ__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30027000)
#define is_v2_LWZ__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30026000)
#define is_v2_LWZ__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30029000)
#define is_v2_LWZ__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002b000)
#define is_v2_LWZ__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002d000)
#define is_v2_LWZ__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002c000)
#define is_v2_LWZ__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002a000)
#define is_v2_LWZ__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x30028000)
#define v2_LWZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LWZ_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1021000)
#define is_v2_LWZ_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1023000)
#define is_v2_LWZ_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1025000)
#define is_v2_LWZ_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1024000)
#define is_v2_LWZ_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1022000)
#define is_v2_LWZ_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1020000)
#define is_v2_LWZ_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1027000)
#define is_v2_LWZ_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1026000)
#define is_v2_LWZ_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1029000)
#define is_v2_LWZ_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102b000)
#define is_v2_LWZ_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102d000)
#define is_v2_LWZ_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102c000)
#define is_v2_LWZ_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102a000)
#define is_v2_LWZ_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1028000)
#define is_v2_LWZ_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3021000)
#define is_v2_LWZ_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3023000)
#define is_v2_LWZ_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3025000)
#define is_v2_LWZ_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3024000)
#define is_v2_LWZ_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3022000)
#define is_v2_LWZ_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3020000)
#define is_v2_LWZ_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3027000)
#define is_v2_LWZ_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3026000)
#define is_v2_LWZ_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3029000)
#define is_v2_LWZ_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302b000)
#define is_v2_LWZ_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302d000)
#define is_v2_LWZ_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302c000)
#define is_v2_LWZ_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302a000)
#define is_v2_LWZ_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3028000)
#define is_v2_LWZ_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2021000)
#define is_v2_LWZ_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2023000)
#define is_v2_LWZ_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2025000)
#define is_v2_LWZ_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2024000)
#define is_v2_LWZ_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2022000)
#define is_v2_LWZ_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2020000)
#define is_v2_LWZ_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2027000)
#define is_v2_LWZ_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2026000)
#define is_v2_LWZ_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2029000)
#define is_v2_LWZ_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202b000)
#define is_v2_LWZ_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202d000)
#define is_v2_LWZ_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202c000)
#define is_v2_LWZ_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202a000)
#define is_v2_LWZ_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2028000)
#define is_v2_LWZ__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0021000)
#define is_v2_LWZ__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0023000)
#define is_v2_LWZ__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0025000)
#define is_v2_LWZ__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0024000)
#define is_v2_LWZ__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0022000)
#define is_v2_LWZ__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0020000)
#define is_v2_LWZ__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0027000)
#define is_v2_LWZ__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0026000)
#define is_v2_LWZ__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0029000)
#define is_v2_LWZ__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002b000)
#define is_v2_LWZ__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002d000)
#define is_v2_LWZ__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002c000)
#define is_v2_LWZ__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002a000)
#define is_v2_LWZ__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0028000)
#define v2_LWZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb102a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb1028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb302a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb3028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb202a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb2028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb002a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb0028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LWZ_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1021000)
#define is_v2_LWZ_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1023000)
#define is_v2_LWZ_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1025000)
#define is_v2_LWZ_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1024000)
#define is_v2_LWZ_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1022000)
#define is_v2_LWZ_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1020000)
#define is_v2_LWZ_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1027000)
#define is_v2_LWZ_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1026000)
#define is_v2_LWZ_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1029000)
#define is_v2_LWZ_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102b000)
#define is_v2_LWZ_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102d000)
#define is_v2_LWZ_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102c000)
#define is_v2_LWZ_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb102a000)
#define is_v2_LWZ_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb1028000)
#define is_v2_LWZ_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3021000)
#define is_v2_LWZ_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3023000)
#define is_v2_LWZ_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3025000)
#define is_v2_LWZ_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3024000)
#define is_v2_LWZ_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3022000)
#define is_v2_LWZ_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3020000)
#define is_v2_LWZ_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3027000)
#define is_v2_LWZ_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3026000)
#define is_v2_LWZ_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3029000)
#define is_v2_LWZ_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302b000)
#define is_v2_LWZ_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302d000)
#define is_v2_LWZ_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302c000)
#define is_v2_LWZ_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb302a000)
#define is_v2_LWZ_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb3028000)
#define is_v2_LWZ_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2021000)
#define is_v2_LWZ_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2023000)
#define is_v2_LWZ_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2025000)
#define is_v2_LWZ_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2024000)
#define is_v2_LWZ_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2022000)
#define is_v2_LWZ_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2020000)
#define is_v2_LWZ_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2027000)
#define is_v2_LWZ_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2026000)
#define is_v2_LWZ_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2029000)
#define is_v2_LWZ_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202b000)
#define is_v2_LWZ_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202d000)
#define is_v2_LWZ_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202c000)
#define is_v2_LWZ_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb202a000)
#define is_v2_LWZ_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb2028000)
#define is_v2_LWZ__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0021000)
#define is_v2_LWZ__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0023000)
#define is_v2_LWZ__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0025000)
#define is_v2_LWZ__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0024000)
#define is_v2_LWZ__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0022000)
#define is_v2_LWZ__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0020000)
#define is_v2_LWZ__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0027000)
#define is_v2_LWZ__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0026000)
#define is_v2_LWZ__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0029000)
#define is_v2_LWZ__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002b000)
#define is_v2_LWZ__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002d000)
#define is_v2_LWZ__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002c000)
#define is_v2_LWZ__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb002a000)
#define is_v2_LWZ__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb0028000)
#define v2_LWZ_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3102e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3102f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3302e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3302f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3202e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3202f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3002e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWZ__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3002f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LWZ_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102e000)
#define is_v2_LWZ_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3102f000)
#define is_v2_LWZ_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302e000)
#define is_v2_LWZ_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3302f000)
#define is_v2_LWZ_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202e000)
#define is_v2_LWZ_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3202f000)
#define is_v2_LWZ___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002e000)
#define is_v2_LWZ__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3002f000)
#define v2_LWS__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x34000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x35000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x36000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x37000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LWS__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x34000000)
#define is_v2_LWS_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x35000000)
#define is_v2_LWS_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x36000000)
#define is_v2_LWS_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x37000000)
#define v2_LWS__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb4000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LWS_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb5000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LWS_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb6000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LWS_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb7000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LWS__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb4000000)
#define is_v2_LWS_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb5000000)
#define is_v2_LWS_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb6000000)
#define is_v2_LWS_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb7000000)
#define v2_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb4000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb5000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb6000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb7000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LWS__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb4000000)
#define is_v2_LWS_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb5000000)
#define is_v2_LWS_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb6000000)
#define is_v2_LWS_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb7000000)
#define v2_LWS_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x35028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x37028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x36028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x34028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LWS_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35021000)
#define is_v2_LWS_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35023000)
#define is_v2_LWS_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35025000)
#define is_v2_LWS_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35024000)
#define is_v2_LWS_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35022000)
#define is_v2_LWS_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35020000)
#define is_v2_LWS_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35027000)
#define is_v2_LWS_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35026000)
#define is_v2_LWS_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35029000)
#define is_v2_LWS_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502b000)
#define is_v2_LWS_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502d000)
#define is_v2_LWS_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502c000)
#define is_v2_LWS_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502a000)
#define is_v2_LWS_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x35028000)
#define is_v2_LWS_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37021000)
#define is_v2_LWS_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37023000)
#define is_v2_LWS_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37025000)
#define is_v2_LWS_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37024000)
#define is_v2_LWS_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37022000)
#define is_v2_LWS_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37020000)
#define is_v2_LWS_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37027000)
#define is_v2_LWS_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37026000)
#define is_v2_LWS_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37029000)
#define is_v2_LWS_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702b000)
#define is_v2_LWS_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702d000)
#define is_v2_LWS_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702c000)
#define is_v2_LWS_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702a000)
#define is_v2_LWS_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x37028000)
#define is_v2_LWS_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36021000)
#define is_v2_LWS_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36023000)
#define is_v2_LWS_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36025000)
#define is_v2_LWS_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36024000)
#define is_v2_LWS_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36022000)
#define is_v2_LWS_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36020000)
#define is_v2_LWS_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36027000)
#define is_v2_LWS_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36026000)
#define is_v2_LWS_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36029000)
#define is_v2_LWS_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602b000)
#define is_v2_LWS_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602d000)
#define is_v2_LWS_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602c000)
#define is_v2_LWS_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602a000)
#define is_v2_LWS_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x36028000)
#define is_v2_LWS__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34021000)
#define is_v2_LWS__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34023000)
#define is_v2_LWS__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34025000)
#define is_v2_LWS__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34024000)
#define is_v2_LWS__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34022000)
#define is_v2_LWS__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34020000)
#define is_v2_LWS__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34027000)
#define is_v2_LWS__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34026000)
#define is_v2_LWS__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34029000)
#define is_v2_LWS__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402b000)
#define is_v2_LWS__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402d000)
#define is_v2_LWS__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402c000)
#define is_v2_LWS__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402a000)
#define is_v2_LWS__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x34028000)
#define v2_LWS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LWS_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5021000)
#define is_v2_LWS_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5023000)
#define is_v2_LWS_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5025000)
#define is_v2_LWS_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5024000)
#define is_v2_LWS_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5022000)
#define is_v2_LWS_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5020000)
#define is_v2_LWS_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5027000)
#define is_v2_LWS_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5026000)
#define is_v2_LWS_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5029000)
#define is_v2_LWS_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502b000)
#define is_v2_LWS_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502d000)
#define is_v2_LWS_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502c000)
#define is_v2_LWS_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502a000)
#define is_v2_LWS_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5028000)
#define is_v2_LWS_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7021000)
#define is_v2_LWS_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7023000)
#define is_v2_LWS_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7025000)
#define is_v2_LWS_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7024000)
#define is_v2_LWS_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7022000)
#define is_v2_LWS_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7020000)
#define is_v2_LWS_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7027000)
#define is_v2_LWS_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7026000)
#define is_v2_LWS_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7029000)
#define is_v2_LWS_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702b000)
#define is_v2_LWS_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702d000)
#define is_v2_LWS_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702c000)
#define is_v2_LWS_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702a000)
#define is_v2_LWS_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7028000)
#define is_v2_LWS_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6021000)
#define is_v2_LWS_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6023000)
#define is_v2_LWS_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6025000)
#define is_v2_LWS_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6024000)
#define is_v2_LWS_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6022000)
#define is_v2_LWS_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6020000)
#define is_v2_LWS_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6027000)
#define is_v2_LWS_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6026000)
#define is_v2_LWS_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6029000)
#define is_v2_LWS_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602b000)
#define is_v2_LWS_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602d000)
#define is_v2_LWS_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602c000)
#define is_v2_LWS_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602a000)
#define is_v2_LWS_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6028000)
#define is_v2_LWS__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4021000)
#define is_v2_LWS__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4023000)
#define is_v2_LWS__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4025000)
#define is_v2_LWS__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4024000)
#define is_v2_LWS__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4022000)
#define is_v2_LWS__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4020000)
#define is_v2_LWS__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4027000)
#define is_v2_LWS__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4026000)
#define is_v2_LWS__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4029000)
#define is_v2_LWS__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402b000)
#define is_v2_LWS__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402d000)
#define is_v2_LWS__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402c000)
#define is_v2_LWS__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402a000)
#define is_v2_LWS__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4028000)
#define v2_LWS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb502a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb5028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb702a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb7028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb602a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb6028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb402a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LWS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb4028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LWS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LWS_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5021000)
#define is_v2_LWS_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5023000)
#define is_v2_LWS_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5025000)
#define is_v2_LWS_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5024000)
#define is_v2_LWS_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5022000)
#define is_v2_LWS_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5020000)
#define is_v2_LWS_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5027000)
#define is_v2_LWS_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5026000)
#define is_v2_LWS_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5029000)
#define is_v2_LWS_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502b000)
#define is_v2_LWS_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502d000)
#define is_v2_LWS_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502c000)
#define is_v2_LWS_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb502a000)
#define is_v2_LWS_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb5028000)
#define is_v2_LWS_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7021000)
#define is_v2_LWS_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7023000)
#define is_v2_LWS_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7025000)
#define is_v2_LWS_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7024000)
#define is_v2_LWS_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7022000)
#define is_v2_LWS_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7020000)
#define is_v2_LWS_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7027000)
#define is_v2_LWS_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7026000)
#define is_v2_LWS_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7029000)
#define is_v2_LWS_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702b000)
#define is_v2_LWS_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702d000)
#define is_v2_LWS_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702c000)
#define is_v2_LWS_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb702a000)
#define is_v2_LWS_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb7028000)
#define is_v2_LWS_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6021000)
#define is_v2_LWS_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6023000)
#define is_v2_LWS_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6025000)
#define is_v2_LWS_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6024000)
#define is_v2_LWS_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6022000)
#define is_v2_LWS_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6020000)
#define is_v2_LWS_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6027000)
#define is_v2_LWS_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6026000)
#define is_v2_LWS_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6029000)
#define is_v2_LWS_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602b000)
#define is_v2_LWS_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602d000)
#define is_v2_LWS_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602c000)
#define is_v2_LWS_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb602a000)
#define is_v2_LWS_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb6028000)
#define is_v2_LWS__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4021000)
#define is_v2_LWS__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4023000)
#define is_v2_LWS__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4025000)
#define is_v2_LWS__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4024000)
#define is_v2_LWS__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4022000)
#define is_v2_LWS__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4020000)
#define is_v2_LWS__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4027000)
#define is_v2_LWS__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4026000)
#define is_v2_LWS__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4029000)
#define is_v2_LWS__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402b000)
#define is_v2_LWS__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402d000)
#define is_v2_LWS__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402c000)
#define is_v2_LWS__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb402a000)
#define is_v2_LWS__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb4028000)
#define v2_LWS_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3502e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3502f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3702e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3702f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3602e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3602f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3402e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LWS__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3402f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LWS_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502e000)
#define is_v2_LWS_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3502f000)
#define is_v2_LWS_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702e000)
#define is_v2_LWS_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3702f000)
#define is_v2_LWS_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602e000)
#define is_v2_LWS_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3602f000)
#define is_v2_LWS___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402e000)
#define is_v2_LWS__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3402f000)
#define v2_LD__registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x38000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x39000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x3a000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x3b000000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LD__registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x38000000)
#define is_v2_LD_s_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x39000000)
#define is_v2_LD_u_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x3a000000)
#define is_v2_LD_us_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x3b000000)
#define v2_LD__registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb8000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD__registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LD_s_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xb9000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LD_u_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xba000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LD_us_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xbb000000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LD__registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb8000000)
#define is_v2_LD_s_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb9000000)
#define is_v2_LD_u_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xba000000)
#define is_v2_LD_us_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xbb000000)
#define v2_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb8000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xb9000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xba000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xbb000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LD__registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb8000000)
#define is_v2_LD_s_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xb9000000)
#define is_v2_LD_u_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xba000000)
#define is_v2_LD_us_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xbb000000)
#define v2_LD_s_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x39028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3b028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3a028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x3802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LD__wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x38028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LD_s_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39021000)
#define is_v2_LD_s_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39023000)
#define is_v2_LD_s_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39025000)
#define is_v2_LD_s_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39024000)
#define is_v2_LD_s_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39022000)
#define is_v2_LD_s_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39020000)
#define is_v2_LD_s_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39027000)
#define is_v2_LD_s_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39026000)
#define is_v2_LD_s_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39029000)
#define is_v2_LD_s_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902b000)
#define is_v2_LD_s_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902d000)
#define is_v2_LD_s_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902c000)
#define is_v2_LD_s_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902a000)
#define is_v2_LD_s_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x39028000)
#define is_v2_LD_us_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b021000)
#define is_v2_LD_us_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b023000)
#define is_v2_LD_us_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b025000)
#define is_v2_LD_us_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b024000)
#define is_v2_LD_us_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b022000)
#define is_v2_LD_us_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b020000)
#define is_v2_LD_us_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b027000)
#define is_v2_LD_us_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b026000)
#define is_v2_LD_us_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b029000)
#define is_v2_LD_us_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02b000)
#define is_v2_LD_us_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02d000)
#define is_v2_LD_us_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02c000)
#define is_v2_LD_us_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02a000)
#define is_v2_LD_us_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b028000)
#define is_v2_LD_u_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a021000)
#define is_v2_LD_u_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a023000)
#define is_v2_LD_u_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a025000)
#define is_v2_LD_u_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a024000)
#define is_v2_LD_u_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a022000)
#define is_v2_LD_u_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a020000)
#define is_v2_LD_u_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a027000)
#define is_v2_LD_u_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a026000)
#define is_v2_LD_u_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a029000)
#define is_v2_LD_u_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02b000)
#define is_v2_LD_u_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02d000)
#define is_v2_LD_u_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02c000)
#define is_v2_LD_u_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02a000)
#define is_v2_LD_u_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a028000)
#define is_v2_LD__deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38021000)
#define is_v2_LD__dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38023000)
#define is_v2_LD__dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38025000)
#define is_v2_LD__dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38024000)
#define is_v2_LD__dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38022000)
#define is_v2_LD__dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38020000)
#define is_v2_LD__even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38027000)
#define is_v2_LD__odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38026000)
#define is_v2_LD__weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38029000)
#define is_v2_LD__wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802b000)
#define is_v2_LD__wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802d000)
#define is_v2_LD__wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802c000)
#define is_v2_LD__wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802a000)
#define is_v2_LD__wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x38028000)
#define v2_LD_s_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xbb028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xba028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xb8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LD_s_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9021000)
#define is_v2_LD_s_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9023000)
#define is_v2_LD_s_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9025000)
#define is_v2_LD_s_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9024000)
#define is_v2_LD_s_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9022000)
#define is_v2_LD_s_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9020000)
#define is_v2_LD_s_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9027000)
#define is_v2_LD_s_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9026000)
#define is_v2_LD_s_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9029000)
#define is_v2_LD_s_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902b000)
#define is_v2_LD_s_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902d000)
#define is_v2_LD_s_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902c000)
#define is_v2_LD_s_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902a000)
#define is_v2_LD_s_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9028000)
#define is_v2_LD_us_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb021000)
#define is_v2_LD_us_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb023000)
#define is_v2_LD_us_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb025000)
#define is_v2_LD_us_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb024000)
#define is_v2_LD_us_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb022000)
#define is_v2_LD_us_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb020000)
#define is_v2_LD_us_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb027000)
#define is_v2_LD_us_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb026000)
#define is_v2_LD_us_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb029000)
#define is_v2_LD_us_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02b000)
#define is_v2_LD_us_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02d000)
#define is_v2_LD_us_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02c000)
#define is_v2_LD_us_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02a000)
#define is_v2_LD_us_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb028000)
#define is_v2_LD_u_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba021000)
#define is_v2_LD_u_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba023000)
#define is_v2_LD_u_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba025000)
#define is_v2_LD_u_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba024000)
#define is_v2_LD_u_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba022000)
#define is_v2_LD_u_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba020000)
#define is_v2_LD_u_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba027000)
#define is_v2_LD_u_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba026000)
#define is_v2_LD_u_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba029000)
#define is_v2_LD_u_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02b000)
#define is_v2_LD_u_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02d000)
#define is_v2_LD_u_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02c000)
#define is_v2_LD_u_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02a000)
#define is_v2_LD_u_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba028000)
#define is_v2_LD__deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8021000)
#define is_v2_LD__dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8023000)
#define is_v2_LD__dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8025000)
#define is_v2_LD__dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8024000)
#define is_v2_LD__dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8022000)
#define is_v2_LD__dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8020000)
#define is_v2_LD__even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8027000)
#define is_v2_LD__odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8026000)
#define is_v2_LD__weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8029000)
#define is_v2_LD__wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802b000)
#define is_v2_LD__wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802d000)
#define is_v2_LD__wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802c000)
#define is_v2_LD__wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802a000)
#define is_v2_LD__wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8028000)
#define v2_LD_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb902a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb9028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xbb028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xba028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb802a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LD__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xb8028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LD__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LD_s_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9021000)
#define is_v2_LD_s_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9023000)
#define is_v2_LD_s_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9025000)
#define is_v2_LD_s_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9024000)
#define is_v2_LD_s_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9022000)
#define is_v2_LD_s_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9020000)
#define is_v2_LD_s_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9027000)
#define is_v2_LD_s_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9026000)
#define is_v2_LD_s_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9029000)
#define is_v2_LD_s_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902b000)
#define is_v2_LD_s_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902d000)
#define is_v2_LD_s_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902c000)
#define is_v2_LD_s_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb902a000)
#define is_v2_LD_s_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb9028000)
#define is_v2_LD_us_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb021000)
#define is_v2_LD_us_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb023000)
#define is_v2_LD_us_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb025000)
#define is_v2_LD_us_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb024000)
#define is_v2_LD_us_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb022000)
#define is_v2_LD_us_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb020000)
#define is_v2_LD_us_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb027000)
#define is_v2_LD_us_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb026000)
#define is_v2_LD_us_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb029000)
#define is_v2_LD_us_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02b000)
#define is_v2_LD_us_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02d000)
#define is_v2_LD_us_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02c000)
#define is_v2_LD_us_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb02a000)
#define is_v2_LD_us_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xbb028000)
#define is_v2_LD_u_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba021000)
#define is_v2_LD_u_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba023000)
#define is_v2_LD_u_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba025000)
#define is_v2_LD_u_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba024000)
#define is_v2_LD_u_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba022000)
#define is_v2_LD_u_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba020000)
#define is_v2_LD_u_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba027000)
#define is_v2_LD_u_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba026000)
#define is_v2_LD_u_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba029000)
#define is_v2_LD_u_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02b000)
#define is_v2_LD_u_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02d000)
#define is_v2_LD_u_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02c000)
#define is_v2_LD_u_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba02a000)
#define is_v2_LD_u_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xba028000)
#define is_v2_LD__deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8021000)
#define is_v2_LD__dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8023000)
#define is_v2_LD__dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8025000)
#define is_v2_LD__dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8024000)
#define is_v2_LD__dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8022000)
#define is_v2_LD__dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8020000)
#define is_v2_LD__even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8027000)
#define is_v2_LD__odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8026000)
#define is_v2_LD__weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8029000)
#define is_v2_LD__wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802b000)
#define is_v2_LD__wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802d000)
#define is_v2_LD__wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802c000)
#define is_v2_LD__wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb802a000)
#define is_v2_LD__wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xb8028000)
#define v2_LD_s__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3902e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_s_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3902f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_us__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3b02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_us_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3b02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_u__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3a02e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD_u_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3a02f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD___registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3802e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LD__xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x3802f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LD_s__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902e000)
#define is_v2_LD_s_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3902f000)
#define is_v2_LD_us__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02e000)
#define is_v2_LD_us_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3b02f000)
#define is_v2_LD_u__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02e000)
#define is_v2_LD_u_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3a02f000)
#define is_v2_LD___registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802e000)
#define is_v2_LD__xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x3802f000)
#define v2_LQ__registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3c000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3d000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3e000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_registerM_signed10_registerZ_simple_insn(registerM,signed10,registerZ) (unsigned int)(0x3f000000 | (((int)(registerM) & 0x1f) << 19) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LQ__registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3c000000)
#define is_v2_LQ_s_registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3d000000)
#define is_v2_LQ_u_registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3e000000)
#define is_v2_LQ_us_registerM_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3f000000)
#define v2_LQ__registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbc000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ__registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LQ_s_registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbd000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LQ_u_registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbe000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LQ_us_registerM_upper27_lower10_registerZ_double_insn_0(registerM,upper27_lower10,registerZ) (unsigned int)(0xbf000000 | (((int)(registerM) & 0x1f) << 19) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_registerM_upper27_lower10_registerZ_double_insn_1(registerM,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LQ__registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbc000000)
#define is_v2_LQ_s_registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbd000000)
#define is_v2_LQ_u_registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbe000000)
#define is_v2_LQ_us_registerM_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbf000000)
#define v2_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbc000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbd000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbe000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn_0(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(0xbf000000 | (((int)(registerM) & 0x1f) << 19) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn_1(registerM,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn_2(registerM,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LQ__registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbc000000)
#define is_v2_LQ_s_registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbd000000)
#define is_v2_LQ_u_registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbe000000)
#define is_v2_LQ_us_registerM_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbf000000)
#define v2_LQ_s_deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3d028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3f028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3e028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__deqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__even_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__odd_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__weqz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wgez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wgtz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wlez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wltz_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wnez_registerY_registerM_registerZ_simple_insn(registerY,registerM,registerZ) (unsigned int)(0x3c028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f))
#define is_v2_LQ_s_deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d021000)
#define is_v2_LQ_s_dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d023000)
#define is_v2_LQ_s_dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d025000)
#define is_v2_LQ_s_dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d024000)
#define is_v2_LQ_s_dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d022000)
#define is_v2_LQ_s_dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d020000)
#define is_v2_LQ_s_even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d027000)
#define is_v2_LQ_s_odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d026000)
#define is_v2_LQ_s_weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d029000)
#define is_v2_LQ_s_wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02b000)
#define is_v2_LQ_s_wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02d000)
#define is_v2_LQ_s_wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02c000)
#define is_v2_LQ_s_wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02a000)
#define is_v2_LQ_s_wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d028000)
#define is_v2_LQ_us_deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f021000)
#define is_v2_LQ_us_dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f023000)
#define is_v2_LQ_us_dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f025000)
#define is_v2_LQ_us_dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f024000)
#define is_v2_LQ_us_dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f022000)
#define is_v2_LQ_us_dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f020000)
#define is_v2_LQ_us_even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f027000)
#define is_v2_LQ_us_odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f026000)
#define is_v2_LQ_us_weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f029000)
#define is_v2_LQ_us_wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02b000)
#define is_v2_LQ_us_wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02d000)
#define is_v2_LQ_us_wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02c000)
#define is_v2_LQ_us_wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02a000)
#define is_v2_LQ_us_wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f028000)
#define is_v2_LQ_u_deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e021000)
#define is_v2_LQ_u_dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e023000)
#define is_v2_LQ_u_dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e025000)
#define is_v2_LQ_u_dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e024000)
#define is_v2_LQ_u_dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e022000)
#define is_v2_LQ_u_dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e020000)
#define is_v2_LQ_u_even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e027000)
#define is_v2_LQ_u_odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e026000)
#define is_v2_LQ_u_weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e029000)
#define is_v2_LQ_u_wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02b000)
#define is_v2_LQ_u_wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02d000)
#define is_v2_LQ_u_wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02c000)
#define is_v2_LQ_u_wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02a000)
#define is_v2_LQ_u_wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e028000)
#define is_v2_LQ__deqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c021000)
#define is_v2_LQ__dgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c023000)
#define is_v2_LQ__dgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c025000)
#define is_v2_LQ__dlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c024000)
#define is_v2_LQ__dltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c022000)
#define is_v2_LQ__dnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c020000)
#define is_v2_LQ__even_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c027000)
#define is_v2_LQ__odd_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c026000)
#define is_v2_LQ__weqz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c029000)
#define is_v2_LQ__wgez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02b000)
#define is_v2_LQ__wgtz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02d000)
#define is_v2_LQ__wlez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02c000)
#define is_v2_LQ__wltz_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02a000)
#define is_v2_LQ__wnez_registerY_registerM_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c028000)
#define v2_LQ_s_deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbd028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbe028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__deqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__even_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__odd_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__weqz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wgez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wgtz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wlez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wltz_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wnez_registerY_registerM_offset27_registerZ_double_insn(registerY,registerM,offset27,registerZ) (unsigned int)(0xbc028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LQ_s_deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd021000)
#define is_v2_LQ_s_dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd023000)
#define is_v2_LQ_s_dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd025000)
#define is_v2_LQ_s_dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd024000)
#define is_v2_LQ_s_dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd022000)
#define is_v2_LQ_s_dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd020000)
#define is_v2_LQ_s_even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd027000)
#define is_v2_LQ_s_odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd026000)
#define is_v2_LQ_s_weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd029000)
#define is_v2_LQ_s_wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02b000)
#define is_v2_LQ_s_wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02d000)
#define is_v2_LQ_s_wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02c000)
#define is_v2_LQ_s_wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02a000)
#define is_v2_LQ_s_wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd028000)
#define is_v2_LQ_us_deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf021000)
#define is_v2_LQ_us_dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf023000)
#define is_v2_LQ_us_dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf025000)
#define is_v2_LQ_us_dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf024000)
#define is_v2_LQ_us_dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf022000)
#define is_v2_LQ_us_dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf020000)
#define is_v2_LQ_us_even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf027000)
#define is_v2_LQ_us_odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf026000)
#define is_v2_LQ_us_weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf029000)
#define is_v2_LQ_us_wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02b000)
#define is_v2_LQ_us_wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02d000)
#define is_v2_LQ_us_wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02c000)
#define is_v2_LQ_us_wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02a000)
#define is_v2_LQ_us_wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf028000)
#define is_v2_LQ_u_deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe021000)
#define is_v2_LQ_u_dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe023000)
#define is_v2_LQ_u_dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe025000)
#define is_v2_LQ_u_dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe024000)
#define is_v2_LQ_u_dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe022000)
#define is_v2_LQ_u_dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe020000)
#define is_v2_LQ_u_even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe027000)
#define is_v2_LQ_u_odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe026000)
#define is_v2_LQ_u_weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe029000)
#define is_v2_LQ_u_wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02b000)
#define is_v2_LQ_u_wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02d000)
#define is_v2_LQ_u_wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02c000)
#define is_v2_LQ_u_wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02a000)
#define is_v2_LQ_u_wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe028000)
#define is_v2_LQ__deqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc021000)
#define is_v2_LQ__dgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc023000)
#define is_v2_LQ__dgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc025000)
#define is_v2_LQ__dlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc024000)
#define is_v2_LQ__dltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc022000)
#define is_v2_LQ__dnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc020000)
#define is_v2_LQ__even_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc027000)
#define is_v2_LQ__odd_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc026000)
#define is_v2_LQ__weqz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc029000)
#define is_v2_LQ__wgez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02b000)
#define is_v2_LQ__wgtz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02d000)
#define is_v2_LQ__wlez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02c000)
#define is_v2_LQ__wltz_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02a000)
#define is_v2_LQ__wnez_registerY_registerM_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc028000)
#define v2_LQ_s_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_s_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbd028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_us_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbf028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ_u_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbe028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc021000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc023000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc025000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc024000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc022000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc020000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__even_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc027000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__even_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc026000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__odd_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc029000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc02a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LQ__wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_0(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(0xbc028000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerM) & 0x1f) << 19) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LQ__wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn_1(registerY,registerM,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LQ_s_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd021000)
#define is_v2_LQ_s_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd023000)
#define is_v2_LQ_s_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd025000)
#define is_v2_LQ_s_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd024000)
#define is_v2_LQ_s_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd022000)
#define is_v2_LQ_s_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd020000)
#define is_v2_LQ_s_even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd027000)
#define is_v2_LQ_s_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd026000)
#define is_v2_LQ_s_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd029000)
#define is_v2_LQ_s_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02b000)
#define is_v2_LQ_s_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02d000)
#define is_v2_LQ_s_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02c000)
#define is_v2_LQ_s_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd02a000)
#define is_v2_LQ_s_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd028000)
#define is_v2_LQ_us_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf021000)
#define is_v2_LQ_us_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf023000)
#define is_v2_LQ_us_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf025000)
#define is_v2_LQ_us_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf024000)
#define is_v2_LQ_us_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf022000)
#define is_v2_LQ_us_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf020000)
#define is_v2_LQ_us_even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf027000)
#define is_v2_LQ_us_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf026000)
#define is_v2_LQ_us_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf029000)
#define is_v2_LQ_us_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02b000)
#define is_v2_LQ_us_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02d000)
#define is_v2_LQ_us_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02c000)
#define is_v2_LQ_us_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf02a000)
#define is_v2_LQ_us_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf028000)
#define is_v2_LQ_u_deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe021000)
#define is_v2_LQ_u_dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe023000)
#define is_v2_LQ_u_dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe025000)
#define is_v2_LQ_u_dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe024000)
#define is_v2_LQ_u_dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe022000)
#define is_v2_LQ_u_dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe020000)
#define is_v2_LQ_u_even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe027000)
#define is_v2_LQ_u_odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe026000)
#define is_v2_LQ_u_weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe029000)
#define is_v2_LQ_u_wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02b000)
#define is_v2_LQ_u_wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02d000)
#define is_v2_LQ_u_wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02c000)
#define is_v2_LQ_u_wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe02a000)
#define is_v2_LQ_u_wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe028000)
#define is_v2_LQ__deqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc021000)
#define is_v2_LQ__dgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc023000)
#define is_v2_LQ__dgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc025000)
#define is_v2_LQ__dlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc024000)
#define is_v2_LQ__dltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc022000)
#define is_v2_LQ__dnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc020000)
#define is_v2_LQ__even_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc027000)
#define is_v2_LQ__odd_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc026000)
#define is_v2_LQ__weqz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc029000)
#define is_v2_LQ__wgez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02b000)
#define is_v2_LQ__wgtz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02d000)
#define is_v2_LQ__wlez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02c000)
#define is_v2_LQ__wltz_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc02a000)
#define is_v2_LQ__wnez_registerY_registerM_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc028000)
#define v2_LQ_s__registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3d02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_s_xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3d02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us__registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3f02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_us_xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3f02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u__registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3e02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ_u_xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3e02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ___registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3c02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LQ__xs_registerM_registerY_registerZ_simple_insn(registerM,registerY,registerZ) (unsigned int)(0x3c02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LQ_s__registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02e000)
#define is_v2_LQ_s_xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d02f000)
#define is_v2_LQ_us__registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02e000)
#define is_v2_LQ_us_xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f02f000)
#define is_v2_LQ_u__registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02e000)
#define is_v2_LQ_u_xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e02f000)
#define is_v2_LQ___registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02e000)
#define is_v2_LQ__xs_registerM_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c02f000)
#define v2_LO__registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3c040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3d040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3e040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_registerN_signed10_registerZ_simple_insn(registerN,signed10,registerZ) (unsigned int)(0x3f040000 | (((int)(registerN) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LO__registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3c040000)
#define is_v2_LO_s_registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3d040000)
#define is_v2_LO_u_registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3e040000)
#define is_v2_LO_us_registerN_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3f040000)
#define v2_LO__registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbc040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO__registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LO_s_registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbd040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LO_u_registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbe040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LO_us_registerN_upper27_lower10_registerZ_double_insn_0(registerN,upper27_lower10,registerZ) (unsigned int)(0xbf040000 | (((int)(registerN) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_registerN_upper27_lower10_registerZ_double_insn_1(registerN,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LO__registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbc040000)
#define is_v2_LO_s_registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbd040000)
#define is_v2_LO_u_registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbe040000)
#define is_v2_LO_us_registerN_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbf040000)
#define v2_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbc040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbd040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbe040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn_0(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(0xbf040000 | (((int)(registerN) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn_1(registerN,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn_2(registerN,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LO__registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbc040000)
#define is_v2_LO_s_registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbd040000)
#define is_v2_LO_u_registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbe040000)
#define is_v2_LO_us_registerN_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xbf040000)
#define v2_LO_s_deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3d068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3f068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3e068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__deqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__dgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__dgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__dlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__dltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__dnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__even_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__odd_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__weqz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__wgez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__wgtz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__wlez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__wltz_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LO__wnez_registerY_registerN_registerZ_simple_insn(registerY,registerN,registerZ) (unsigned int)(0x3c068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define is_v2_LO_s_deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d061000)
#define is_v2_LO_s_dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d063000)
#define is_v2_LO_s_dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d065000)
#define is_v2_LO_s_dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d064000)
#define is_v2_LO_s_dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d062000)
#define is_v2_LO_s_dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d060000)
#define is_v2_LO_s_even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d067000)
#define is_v2_LO_s_odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d066000)
#define is_v2_LO_s_weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d069000)
#define is_v2_LO_s_wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06b000)
#define is_v2_LO_s_wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06d000)
#define is_v2_LO_s_wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06c000)
#define is_v2_LO_s_wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06a000)
#define is_v2_LO_s_wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d068000)
#define is_v2_LO_us_deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f061000)
#define is_v2_LO_us_dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f063000)
#define is_v2_LO_us_dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f065000)
#define is_v2_LO_us_dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f064000)
#define is_v2_LO_us_dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f062000)
#define is_v2_LO_us_dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f060000)
#define is_v2_LO_us_even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f067000)
#define is_v2_LO_us_odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f066000)
#define is_v2_LO_us_weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f069000)
#define is_v2_LO_us_wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06b000)
#define is_v2_LO_us_wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06d000)
#define is_v2_LO_us_wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06c000)
#define is_v2_LO_us_wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06a000)
#define is_v2_LO_us_wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f068000)
#define is_v2_LO_u_deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e061000)
#define is_v2_LO_u_dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e063000)
#define is_v2_LO_u_dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e065000)
#define is_v2_LO_u_dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e064000)
#define is_v2_LO_u_dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e062000)
#define is_v2_LO_u_dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e060000)
#define is_v2_LO_u_even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e067000)
#define is_v2_LO_u_odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e066000)
#define is_v2_LO_u_weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e069000)
#define is_v2_LO_u_wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06b000)
#define is_v2_LO_u_wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06d000)
#define is_v2_LO_u_wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06c000)
#define is_v2_LO_u_wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06a000)
#define is_v2_LO_u_wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e068000)
#define is_v2_LO__deqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c061000)
#define is_v2_LO__dgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c063000)
#define is_v2_LO__dgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c065000)
#define is_v2_LO__dlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c064000)
#define is_v2_LO__dltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c062000)
#define is_v2_LO__dnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c060000)
#define is_v2_LO__even_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c067000)
#define is_v2_LO__odd_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c066000)
#define is_v2_LO__weqz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c069000)
#define is_v2_LO__wgez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06b000)
#define is_v2_LO__wgtz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06d000)
#define is_v2_LO__wlez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06c000)
#define is_v2_LO__wltz_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06a000)
#define is_v2_LO__wnez_registerY_registerN_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c068000)
#define v2_LO_s_deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbd068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbf068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbe068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__deqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__even_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__odd_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__weqz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wgez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wgtz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wlez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wltz_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wnez_registerY_registerN_offset27_registerZ_double_insn(registerY,registerN,offset27,registerZ) (unsigned int)(0xbc068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LO_s_deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd061000)
#define is_v2_LO_s_dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd063000)
#define is_v2_LO_s_dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd065000)
#define is_v2_LO_s_dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd064000)
#define is_v2_LO_s_dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd062000)
#define is_v2_LO_s_dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd060000)
#define is_v2_LO_s_even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd067000)
#define is_v2_LO_s_odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd066000)
#define is_v2_LO_s_weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd069000)
#define is_v2_LO_s_wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06b000)
#define is_v2_LO_s_wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06d000)
#define is_v2_LO_s_wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06c000)
#define is_v2_LO_s_wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06a000)
#define is_v2_LO_s_wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd068000)
#define is_v2_LO_us_deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf061000)
#define is_v2_LO_us_dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf063000)
#define is_v2_LO_us_dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf065000)
#define is_v2_LO_us_dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf064000)
#define is_v2_LO_us_dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf062000)
#define is_v2_LO_us_dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf060000)
#define is_v2_LO_us_even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf067000)
#define is_v2_LO_us_odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf066000)
#define is_v2_LO_us_weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf069000)
#define is_v2_LO_us_wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06b000)
#define is_v2_LO_us_wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06d000)
#define is_v2_LO_us_wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06c000)
#define is_v2_LO_us_wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06a000)
#define is_v2_LO_us_wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf068000)
#define is_v2_LO_u_deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe061000)
#define is_v2_LO_u_dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe063000)
#define is_v2_LO_u_dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe065000)
#define is_v2_LO_u_dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe064000)
#define is_v2_LO_u_dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe062000)
#define is_v2_LO_u_dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe060000)
#define is_v2_LO_u_even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe067000)
#define is_v2_LO_u_odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe066000)
#define is_v2_LO_u_weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe069000)
#define is_v2_LO_u_wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06b000)
#define is_v2_LO_u_wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06d000)
#define is_v2_LO_u_wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06c000)
#define is_v2_LO_u_wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06a000)
#define is_v2_LO_u_wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe068000)
#define is_v2_LO__deqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc061000)
#define is_v2_LO__dgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc063000)
#define is_v2_LO__dgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc065000)
#define is_v2_LO__dlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc064000)
#define is_v2_LO__dltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc062000)
#define is_v2_LO__dnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc060000)
#define is_v2_LO__even_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc067000)
#define is_v2_LO__odd_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc066000)
#define is_v2_LO__weqz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc069000)
#define is_v2_LO__wgez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06b000)
#define is_v2_LO__wgtz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06d000)
#define is_v2_LO__wlez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06c000)
#define is_v2_LO__wltz_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06a000)
#define is_v2_LO__wnez_registerY_registerN_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc068000)
#define v2_LO_s_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_s_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbd068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_us_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbf068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO_u_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbe068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc061000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc063000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc065000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc064000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc062000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc060000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__even_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc067000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__even_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc066000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__odd_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc069000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc06a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LO__wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_0(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(0xbc068000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerN) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LO__wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn_1(registerY,registerN,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LO_s_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd061000)
#define is_v2_LO_s_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd063000)
#define is_v2_LO_s_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd065000)
#define is_v2_LO_s_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd064000)
#define is_v2_LO_s_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd062000)
#define is_v2_LO_s_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd060000)
#define is_v2_LO_s_even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd067000)
#define is_v2_LO_s_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd066000)
#define is_v2_LO_s_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd069000)
#define is_v2_LO_s_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06b000)
#define is_v2_LO_s_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06d000)
#define is_v2_LO_s_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06c000)
#define is_v2_LO_s_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd06a000)
#define is_v2_LO_s_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbd068000)
#define is_v2_LO_us_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf061000)
#define is_v2_LO_us_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf063000)
#define is_v2_LO_us_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf065000)
#define is_v2_LO_us_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf064000)
#define is_v2_LO_us_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf062000)
#define is_v2_LO_us_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf060000)
#define is_v2_LO_us_even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf067000)
#define is_v2_LO_us_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf066000)
#define is_v2_LO_us_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf069000)
#define is_v2_LO_us_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06b000)
#define is_v2_LO_us_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06d000)
#define is_v2_LO_us_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06c000)
#define is_v2_LO_us_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf06a000)
#define is_v2_LO_us_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbf068000)
#define is_v2_LO_u_deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe061000)
#define is_v2_LO_u_dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe063000)
#define is_v2_LO_u_dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe065000)
#define is_v2_LO_u_dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe064000)
#define is_v2_LO_u_dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe062000)
#define is_v2_LO_u_dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe060000)
#define is_v2_LO_u_even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe067000)
#define is_v2_LO_u_odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe066000)
#define is_v2_LO_u_weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe069000)
#define is_v2_LO_u_wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06b000)
#define is_v2_LO_u_wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06d000)
#define is_v2_LO_u_wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06c000)
#define is_v2_LO_u_wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe06a000)
#define is_v2_LO_u_wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbe068000)
#define is_v2_LO__deqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc061000)
#define is_v2_LO__dgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc063000)
#define is_v2_LO__dgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc065000)
#define is_v2_LO__dlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc064000)
#define is_v2_LO__dltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc062000)
#define is_v2_LO__dnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc060000)
#define is_v2_LO__even_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc067000)
#define is_v2_LO__odd_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc066000)
#define is_v2_LO__weqz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc069000)
#define is_v2_LO__wgez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06b000)
#define is_v2_LO__wgtz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06d000)
#define is_v2_LO__wlez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06c000)
#define is_v2_LO__wltz_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc06a000)
#define is_v2_LO__wnez_registerY_registerN_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xbc068000)
#define v2_LO_s__registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3d06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_s_xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3d06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_us__registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3f06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_us_xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3f06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_u__registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3e06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO_u_xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3e06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO___registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3c06e000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LO__xs_registerN_registerY_registerZ_simple_insn(registerN,registerY,registerZ) (unsigned int)(0x3c06f000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LO_s__registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06e000)
#define is_v2_LO_s_xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3d06f000)
#define is_v2_LO_us__registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06e000)
#define is_v2_LO_us_xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3f06f000)
#define is_v2_LO_u__registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06e000)
#define is_v2_LO_u_xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3e06f000)
#define is_v2_LO___registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06e000)
#define is_v2_LO__xs_registerN_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x3c06f000)
#define v2_LV__registerA_signed10_registerZ_simple_insn(registerA,signed10,registerZ) (unsigned int)(0x20010000 | (((int)(registerA) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_registerA_signed10_registerZ_simple_insn(registerA,signed10,registerZ) (unsigned int)(0x21010000 | (((int)(registerA) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LV__registerA_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x20010000)
#define is_v2_LV_s_registerA_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x21010000)
#define v2_LV__registerA_upper27_lower10_registerZ_double_insn_0(registerA,upper27_lower10,registerZ) (unsigned int)(0xa0010000 | (((int)(registerA) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV__registerA_upper27_lower10_registerZ_double_insn_1(registerA,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_s_registerA_upper27_lower10_registerZ_double_insn_0(registerA,upper27_lower10,registerZ) (unsigned int)(0xa1010000 | (((int)(registerA) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_registerA_upper27_lower10_registerZ_double_insn_1(registerA,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LV__registerA_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0010000)
#define is_v2_LV_s_registerA_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1010000)
#define v2_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn_0(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(0xa0010000 | (((int)(registerA) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn_1(registerA,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn_2(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn_0(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(0xa1010000 | (((int)(registerA) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn_1(registerA,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn_2(registerA,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LV__registerA_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa0010000)
#define is_v2_LV_s_registerA_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa1010000)
#define v2_LV_s_deqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_even_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_odd_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_weqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2103a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x21038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__deqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__dgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__dgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__dlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__dltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__dnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__even_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__odd_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__weqz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__wgez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__wgtz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__wlez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__wltz_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x2003a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_LV__wnez_registerY_registerA_registerZ_simple_insn(registerY,registerA,registerZ) (unsigned int)(0x20038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_LV_s_deqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21031000)
#define is_v2_LV_s_dgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21033000)
#define is_v2_LV_s_dgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21035000)
#define is_v2_LV_s_dlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21034000)
#define is_v2_LV_s_dltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21032000)
#define is_v2_LV_s_dnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21030000)
#define is_v2_LV_s_even_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21037000)
#define is_v2_LV_s_odd_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21036000)
#define is_v2_LV_s_weqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21039000)
#define is_v2_LV_s_wgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103b000)
#define is_v2_LV_s_wgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103d000)
#define is_v2_LV_s_wlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103c000)
#define is_v2_LV_s_wltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103a000)
#define is_v2_LV_s_wnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x21038000)
#define is_v2_LV__deqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20031000)
#define is_v2_LV__dgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20033000)
#define is_v2_LV__dgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20035000)
#define is_v2_LV__dlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20034000)
#define is_v2_LV__dltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20032000)
#define is_v2_LV__dnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20030000)
#define is_v2_LV__even_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20037000)
#define is_v2_LV__odd_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20036000)
#define is_v2_LV__weqz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20039000)
#define is_v2_LV__wgez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003b000)
#define is_v2_LV__wgtz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003d000)
#define is_v2_LV__wlez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003c000)
#define is_v2_LV__wltz_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003a000)
#define is_v2_LV__wnez_registerY_registerA_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x20038000)
#define v2_LV_s_deqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_even_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_odd_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_weqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa103a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa1038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__deqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__even_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__odd_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__weqz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wgez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wgtz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wlez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wltz_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa003a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wnez_registerY_registerA_offset27_registerZ_double_insn(registerY,registerA,offset27,registerZ) (unsigned int)(0xa0038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LV_s_deqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1031000)
#define is_v2_LV_s_dgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1033000)
#define is_v2_LV_s_dgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1035000)
#define is_v2_LV_s_dlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1034000)
#define is_v2_LV_s_dltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1032000)
#define is_v2_LV_s_dnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1030000)
#define is_v2_LV_s_even_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1037000)
#define is_v2_LV_s_odd_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1036000)
#define is_v2_LV_s_weqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1039000)
#define is_v2_LV_s_wgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103b000)
#define is_v2_LV_s_wgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103d000)
#define is_v2_LV_s_wlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103c000)
#define is_v2_LV_s_wltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103a000)
#define is_v2_LV_s_wnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1038000)
#define is_v2_LV__deqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0031000)
#define is_v2_LV__dgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0033000)
#define is_v2_LV__dgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0035000)
#define is_v2_LV__dlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0034000)
#define is_v2_LV__dltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0032000)
#define is_v2_LV__dnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0030000)
#define is_v2_LV__even_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0037000)
#define is_v2_LV__odd_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0036000)
#define is_v2_LV__weqz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0039000)
#define is_v2_LV__wgez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003b000)
#define is_v2_LV__wgtz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003d000)
#define is_v2_LV__wlez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003c000)
#define is_v2_LV__wltz_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003a000)
#define is_v2_LV__wnez_registerY_registerA_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0038000)
#define v2_LV_s_deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_even_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_even_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa103a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_s_wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa1038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__even_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__even_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__odd_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa003a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV__wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_0(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(0xa0038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerA) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV__wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn_1(registerY,registerA,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LV_s_deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1031000)
#define is_v2_LV_s_dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1033000)
#define is_v2_LV_s_dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1035000)
#define is_v2_LV_s_dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1034000)
#define is_v2_LV_s_dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1032000)
#define is_v2_LV_s_dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1030000)
#define is_v2_LV_s_even_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1037000)
#define is_v2_LV_s_odd_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1036000)
#define is_v2_LV_s_weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1039000)
#define is_v2_LV_s_wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103b000)
#define is_v2_LV_s_wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103d000)
#define is_v2_LV_s_wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103c000)
#define is_v2_LV_s_wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa103a000)
#define is_v2_LV_s_wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa1038000)
#define is_v2_LV__deqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0031000)
#define is_v2_LV__dgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0033000)
#define is_v2_LV__dgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0035000)
#define is_v2_LV__dlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0034000)
#define is_v2_LV__dltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0032000)
#define is_v2_LV__dnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0030000)
#define is_v2_LV__even_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0037000)
#define is_v2_LV__odd_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0036000)
#define is_v2_LV__weqz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0039000)
#define is_v2_LV__wgez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003b000)
#define is_v2_LV__wgtz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003d000)
#define is_v2_LV__wlez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003c000)
#define is_v2_LV__wltz_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa003a000)
#define is_v2_LV__wnez_registerY_registerA_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa0038000)
#define v2_LV_s__registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2103e000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_s_xs_registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2103f000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV___registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2003e000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV__xs_registerA_registerY_registerZ_simple_insn(registerA,registerY,registerZ) (unsigned int)(0x2003f000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LV_s__registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103e000)
#define is_v2_LV_s_xs_registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2103f000)
#define is_v2_LV___registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003e000)
#define is_v2_LV__xs_registerA_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2003f000)
#define v2_LV_c0__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x22010000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x23010000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x22050000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x23050000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x22090000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x23090000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x220d0000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_registerAq_signed10_registerZ_simple_insn(registerAq,signed10,registerZ) (unsigned int)(0x230d0000 | (((int)(registerAq) & 0xf) << 20) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LV_c0__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x22010000)
#define is_v2_LV_c0_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x23010000)
#define is_v2_LV_c1__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x22050000)
#define is_v2_LV_c1_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x23050000)
#define is_v2_LV_c2__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x22090000)
#define is_v2_LV_c2_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x23090000)
#define is_v2_LV_c3__registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x220d0000)
#define is_v2_LV_c3_s_registerAq_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x230d0000)
#define v2_LV_c0__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa2010000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_c0_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa3010000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_c1__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa2050000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_c1_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa3050000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_c2__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa2090000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_c2_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa3090000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_c3__registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa20d0000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_LV_c3_s_registerAq_upper27_lower10_registerZ_double_insn_0(registerAq,upper27_lower10,registerZ) (unsigned int)(0xa30d0000 | (((int)(registerAq) & 0xf) << 20) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_registerAq_upper27_lower10_registerZ_double_insn_1(registerAq,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_LV_c0__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2010000)
#define is_v2_LV_c0_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3010000)
#define is_v2_LV_c1__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2050000)
#define is_v2_LV_c1_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3050000)
#define is_v2_LV_c2__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2090000)
#define is_v2_LV_c2_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3090000)
#define is_v2_LV_c3__registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa20d0000)
#define is_v2_LV_c3_s_registerAq_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa30d0000)
#define v2_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2010000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3010000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2050000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3050000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa2090000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa3090000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa20d0000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_0(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(0xa30d0000 | (((int)(registerAq) & 0xf) << 20) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_1(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn_2(registerAq,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_LV_c0__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2010000)
#define is_v2_LV_c0_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3010000)
#define is_v2_LV_c1__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2050000)
#define is_v2_LV_c1_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3050000)
#define is_v2_LV_c2__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa2090000)
#define is_v2_LV_c2_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa3090000)
#define is_v2_LV_c3__registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa20d0000)
#define is_v2_LV_c3_s_registerAq_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa30d0000)
#define v2_LV_c0_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2303a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2203a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2307a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x23078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x2207a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x22078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x230f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__deqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__even_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__odd_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__weqz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wgez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wgtz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wlez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wltz_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wnez_registerY_registerAq_registerZ_simple_insn(registerY,registerAq,registerZ) (unsigned int)(0x220f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(registerZ) & 0x3f))
#define is_v2_LV_c0_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23031000)
#define is_v2_LV_c0_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23033000)
#define is_v2_LV_c0_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23035000)
#define is_v2_LV_c0_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23034000)
#define is_v2_LV_c0_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23032000)
#define is_v2_LV_c0_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23030000)
#define is_v2_LV_c0_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23037000)
#define is_v2_LV_c0_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23036000)
#define is_v2_LV_c0_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23039000)
#define is_v2_LV_c0_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303b000)
#define is_v2_LV_c0_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303d000)
#define is_v2_LV_c0_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303c000)
#define is_v2_LV_c0_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303a000)
#define is_v2_LV_c0_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23038000)
#define is_v2_LV_c0__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22031000)
#define is_v2_LV_c0__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22033000)
#define is_v2_LV_c0__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22035000)
#define is_v2_LV_c0__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22034000)
#define is_v2_LV_c0__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22032000)
#define is_v2_LV_c0__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22030000)
#define is_v2_LV_c0__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22037000)
#define is_v2_LV_c0__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22036000)
#define is_v2_LV_c0__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22039000)
#define is_v2_LV_c0__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203b000)
#define is_v2_LV_c0__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203d000)
#define is_v2_LV_c0__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203c000)
#define is_v2_LV_c0__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203a000)
#define is_v2_LV_c0__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22038000)
#define is_v2_LV_c1_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23071000)
#define is_v2_LV_c1_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23073000)
#define is_v2_LV_c1_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23075000)
#define is_v2_LV_c1_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23074000)
#define is_v2_LV_c1_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23072000)
#define is_v2_LV_c1_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23070000)
#define is_v2_LV_c1_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23077000)
#define is_v2_LV_c1_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23076000)
#define is_v2_LV_c1_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23079000)
#define is_v2_LV_c1_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307b000)
#define is_v2_LV_c1_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307d000)
#define is_v2_LV_c1_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307c000)
#define is_v2_LV_c1_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307a000)
#define is_v2_LV_c1_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x23078000)
#define is_v2_LV_c1__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22071000)
#define is_v2_LV_c1__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22073000)
#define is_v2_LV_c1__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22075000)
#define is_v2_LV_c1__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22074000)
#define is_v2_LV_c1__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22072000)
#define is_v2_LV_c1__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22070000)
#define is_v2_LV_c1__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22077000)
#define is_v2_LV_c1__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22076000)
#define is_v2_LV_c1__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22079000)
#define is_v2_LV_c1__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207b000)
#define is_v2_LV_c1__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207d000)
#define is_v2_LV_c1__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207c000)
#define is_v2_LV_c1__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207a000)
#define is_v2_LV_c1__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x22078000)
#define is_v2_LV_c2_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b1000)
#define is_v2_LV_c2_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b3000)
#define is_v2_LV_c2_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b5000)
#define is_v2_LV_c2_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b4000)
#define is_v2_LV_c2_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b2000)
#define is_v2_LV_c2_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b0000)
#define is_v2_LV_c2_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b7000)
#define is_v2_LV_c2_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b6000)
#define is_v2_LV_c2_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b9000)
#define is_v2_LV_c2_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bb000)
#define is_v2_LV_c2_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bd000)
#define is_v2_LV_c2_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bc000)
#define is_v2_LV_c2_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230ba000)
#define is_v2_LV_c2_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230b8000)
#define is_v2_LV_c2__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b1000)
#define is_v2_LV_c2__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b3000)
#define is_v2_LV_c2__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b5000)
#define is_v2_LV_c2__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b4000)
#define is_v2_LV_c2__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b2000)
#define is_v2_LV_c2__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b0000)
#define is_v2_LV_c2__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b7000)
#define is_v2_LV_c2__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b6000)
#define is_v2_LV_c2__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b9000)
#define is_v2_LV_c2__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bb000)
#define is_v2_LV_c2__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bd000)
#define is_v2_LV_c2__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bc000)
#define is_v2_LV_c2__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220ba000)
#define is_v2_LV_c2__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220b8000)
#define is_v2_LV_c3_s_deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f1000)
#define is_v2_LV_c3_s_dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f3000)
#define is_v2_LV_c3_s_dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f5000)
#define is_v2_LV_c3_s_dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f4000)
#define is_v2_LV_c3_s_dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f2000)
#define is_v2_LV_c3_s_dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f0000)
#define is_v2_LV_c3_s_even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f7000)
#define is_v2_LV_c3_s_odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f6000)
#define is_v2_LV_c3_s_weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f9000)
#define is_v2_LV_c3_s_wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fb000)
#define is_v2_LV_c3_s_wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fd000)
#define is_v2_LV_c3_s_wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fc000)
#define is_v2_LV_c3_s_wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fa000)
#define is_v2_LV_c3_s_wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230f8000)
#define is_v2_LV_c3__deqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f1000)
#define is_v2_LV_c3__dgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f3000)
#define is_v2_LV_c3__dgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f5000)
#define is_v2_LV_c3__dlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f4000)
#define is_v2_LV_c3__dltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f2000)
#define is_v2_LV_c3__dnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f0000)
#define is_v2_LV_c3__even_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f7000)
#define is_v2_LV_c3__odd_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f6000)
#define is_v2_LV_c3__weqz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f9000)
#define is_v2_LV_c3__wgez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fb000)
#define is_v2_LV_c3__wgtz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fd000)
#define is_v2_LV_c3__wlez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fc000)
#define is_v2_LV_c3__wltz_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fa000)
#define is_v2_LV_c3__wnez_registerY_registerAq_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220f8000)
#define v2_LV_c0_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa303a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa203a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa307a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa3078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa207a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa2078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa30f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__deqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__even_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__odd_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__weqz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wgez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wgtz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wlez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wltz_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wnez_registerY_registerAq_offset27_registerZ_double_insn(registerY,registerAq,offset27,registerZ) (unsigned int)(0xa20f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_LV_c0_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3031000)
#define is_v2_LV_c0_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3033000)
#define is_v2_LV_c0_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3035000)
#define is_v2_LV_c0_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3034000)
#define is_v2_LV_c0_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3032000)
#define is_v2_LV_c0_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3030000)
#define is_v2_LV_c0_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3037000)
#define is_v2_LV_c0_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3036000)
#define is_v2_LV_c0_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3039000)
#define is_v2_LV_c0_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303b000)
#define is_v2_LV_c0_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303d000)
#define is_v2_LV_c0_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303c000)
#define is_v2_LV_c0_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303a000)
#define is_v2_LV_c0_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3038000)
#define is_v2_LV_c0__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2031000)
#define is_v2_LV_c0__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2033000)
#define is_v2_LV_c0__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2035000)
#define is_v2_LV_c0__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2034000)
#define is_v2_LV_c0__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2032000)
#define is_v2_LV_c0__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2030000)
#define is_v2_LV_c0__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2037000)
#define is_v2_LV_c0__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2036000)
#define is_v2_LV_c0__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2039000)
#define is_v2_LV_c0__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203b000)
#define is_v2_LV_c0__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203d000)
#define is_v2_LV_c0__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203c000)
#define is_v2_LV_c0__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203a000)
#define is_v2_LV_c0__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2038000)
#define is_v2_LV_c1_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3071000)
#define is_v2_LV_c1_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3073000)
#define is_v2_LV_c1_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3075000)
#define is_v2_LV_c1_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3074000)
#define is_v2_LV_c1_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3072000)
#define is_v2_LV_c1_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3070000)
#define is_v2_LV_c1_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3077000)
#define is_v2_LV_c1_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3076000)
#define is_v2_LV_c1_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3079000)
#define is_v2_LV_c1_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307b000)
#define is_v2_LV_c1_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307d000)
#define is_v2_LV_c1_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307c000)
#define is_v2_LV_c1_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307a000)
#define is_v2_LV_c1_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3078000)
#define is_v2_LV_c1__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2071000)
#define is_v2_LV_c1__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2073000)
#define is_v2_LV_c1__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2075000)
#define is_v2_LV_c1__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2074000)
#define is_v2_LV_c1__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2072000)
#define is_v2_LV_c1__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2070000)
#define is_v2_LV_c1__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2077000)
#define is_v2_LV_c1__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2076000)
#define is_v2_LV_c1__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2079000)
#define is_v2_LV_c1__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207b000)
#define is_v2_LV_c1__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207d000)
#define is_v2_LV_c1__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207c000)
#define is_v2_LV_c1__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207a000)
#define is_v2_LV_c1__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2078000)
#define is_v2_LV_c2_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b1000)
#define is_v2_LV_c2_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b3000)
#define is_v2_LV_c2_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b5000)
#define is_v2_LV_c2_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b4000)
#define is_v2_LV_c2_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b2000)
#define is_v2_LV_c2_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b0000)
#define is_v2_LV_c2_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b7000)
#define is_v2_LV_c2_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b6000)
#define is_v2_LV_c2_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b9000)
#define is_v2_LV_c2_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bb000)
#define is_v2_LV_c2_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bd000)
#define is_v2_LV_c2_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bc000)
#define is_v2_LV_c2_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30ba000)
#define is_v2_LV_c2_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b8000)
#define is_v2_LV_c2__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b1000)
#define is_v2_LV_c2__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b3000)
#define is_v2_LV_c2__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b5000)
#define is_v2_LV_c2__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b4000)
#define is_v2_LV_c2__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b2000)
#define is_v2_LV_c2__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b0000)
#define is_v2_LV_c2__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b7000)
#define is_v2_LV_c2__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b6000)
#define is_v2_LV_c2__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b9000)
#define is_v2_LV_c2__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bb000)
#define is_v2_LV_c2__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bd000)
#define is_v2_LV_c2__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bc000)
#define is_v2_LV_c2__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20ba000)
#define is_v2_LV_c2__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b8000)
#define is_v2_LV_c3_s_deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f1000)
#define is_v2_LV_c3_s_dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f3000)
#define is_v2_LV_c3_s_dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f5000)
#define is_v2_LV_c3_s_dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f4000)
#define is_v2_LV_c3_s_dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f2000)
#define is_v2_LV_c3_s_dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f0000)
#define is_v2_LV_c3_s_even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f7000)
#define is_v2_LV_c3_s_odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f6000)
#define is_v2_LV_c3_s_weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f9000)
#define is_v2_LV_c3_s_wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fb000)
#define is_v2_LV_c3_s_wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fd000)
#define is_v2_LV_c3_s_wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fc000)
#define is_v2_LV_c3_s_wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fa000)
#define is_v2_LV_c3_s_wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f8000)
#define is_v2_LV_c3__deqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f1000)
#define is_v2_LV_c3__dgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f3000)
#define is_v2_LV_c3__dgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f5000)
#define is_v2_LV_c3__dlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f4000)
#define is_v2_LV_c3__dltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f2000)
#define is_v2_LV_c3__dnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f0000)
#define is_v2_LV_c3__even_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f7000)
#define is_v2_LV_c3__odd_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f6000)
#define is_v2_LV_c3__weqz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f9000)
#define is_v2_LV_c3__wgez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fb000)
#define is_v2_LV_c3__wgtz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fd000)
#define is_v2_LV_c3__wlez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fc000)
#define is_v2_LV_c3__wltz_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fa000)
#define is_v2_LV_c3__wnez_registerY_registerAq_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f8000)
#define v2_LV_c0_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa303a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa203a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c0__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa307a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa3078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa207a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c1__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa2078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20bb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20bd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20bc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20ba000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c2__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20b8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa30f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fd000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fc000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20fa000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_LV_c3__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_0(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(0xa20f8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerAq) & 0xf) << 20) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn_1(registerY,registerAq,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_LV_c0_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3031000)
#define is_v2_LV_c0_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3033000)
#define is_v2_LV_c0_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3035000)
#define is_v2_LV_c0_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3034000)
#define is_v2_LV_c0_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3032000)
#define is_v2_LV_c0_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3030000)
#define is_v2_LV_c0_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3037000)
#define is_v2_LV_c0_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3036000)
#define is_v2_LV_c0_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3039000)
#define is_v2_LV_c0_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303b000)
#define is_v2_LV_c0_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303d000)
#define is_v2_LV_c0_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303c000)
#define is_v2_LV_c0_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa303a000)
#define is_v2_LV_c0_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3038000)
#define is_v2_LV_c0__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2031000)
#define is_v2_LV_c0__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2033000)
#define is_v2_LV_c0__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2035000)
#define is_v2_LV_c0__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2034000)
#define is_v2_LV_c0__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2032000)
#define is_v2_LV_c0__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2030000)
#define is_v2_LV_c0__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2037000)
#define is_v2_LV_c0__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2036000)
#define is_v2_LV_c0__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2039000)
#define is_v2_LV_c0__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203b000)
#define is_v2_LV_c0__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203d000)
#define is_v2_LV_c0__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203c000)
#define is_v2_LV_c0__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa203a000)
#define is_v2_LV_c0__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2038000)
#define is_v2_LV_c1_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3071000)
#define is_v2_LV_c1_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3073000)
#define is_v2_LV_c1_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3075000)
#define is_v2_LV_c1_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3074000)
#define is_v2_LV_c1_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3072000)
#define is_v2_LV_c1_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3070000)
#define is_v2_LV_c1_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3077000)
#define is_v2_LV_c1_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3076000)
#define is_v2_LV_c1_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3079000)
#define is_v2_LV_c1_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307b000)
#define is_v2_LV_c1_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307d000)
#define is_v2_LV_c1_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307c000)
#define is_v2_LV_c1_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa307a000)
#define is_v2_LV_c1_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa3078000)
#define is_v2_LV_c1__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2071000)
#define is_v2_LV_c1__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2073000)
#define is_v2_LV_c1__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2075000)
#define is_v2_LV_c1__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2074000)
#define is_v2_LV_c1__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2072000)
#define is_v2_LV_c1__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2070000)
#define is_v2_LV_c1__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2077000)
#define is_v2_LV_c1__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2076000)
#define is_v2_LV_c1__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2079000)
#define is_v2_LV_c1__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207b000)
#define is_v2_LV_c1__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207d000)
#define is_v2_LV_c1__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207c000)
#define is_v2_LV_c1__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa207a000)
#define is_v2_LV_c1__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa2078000)
#define is_v2_LV_c2_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b1000)
#define is_v2_LV_c2_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b3000)
#define is_v2_LV_c2_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b5000)
#define is_v2_LV_c2_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b4000)
#define is_v2_LV_c2_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b2000)
#define is_v2_LV_c2_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b0000)
#define is_v2_LV_c2_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b7000)
#define is_v2_LV_c2_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b6000)
#define is_v2_LV_c2_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b9000)
#define is_v2_LV_c2_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bb000)
#define is_v2_LV_c2_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bd000)
#define is_v2_LV_c2_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30bc000)
#define is_v2_LV_c2_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30ba000)
#define is_v2_LV_c2_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30b8000)
#define is_v2_LV_c2__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b1000)
#define is_v2_LV_c2__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b3000)
#define is_v2_LV_c2__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b5000)
#define is_v2_LV_c2__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b4000)
#define is_v2_LV_c2__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b2000)
#define is_v2_LV_c2__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b0000)
#define is_v2_LV_c2__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b7000)
#define is_v2_LV_c2__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b6000)
#define is_v2_LV_c2__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b9000)
#define is_v2_LV_c2__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bb000)
#define is_v2_LV_c2__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bd000)
#define is_v2_LV_c2__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20bc000)
#define is_v2_LV_c2__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20ba000)
#define is_v2_LV_c2__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20b8000)
#define is_v2_LV_c3_s_deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f1000)
#define is_v2_LV_c3_s_dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f3000)
#define is_v2_LV_c3_s_dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f5000)
#define is_v2_LV_c3_s_dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f4000)
#define is_v2_LV_c3_s_dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f2000)
#define is_v2_LV_c3_s_dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f0000)
#define is_v2_LV_c3_s_even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f7000)
#define is_v2_LV_c3_s_odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f6000)
#define is_v2_LV_c3_s_weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f9000)
#define is_v2_LV_c3_s_wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fb000)
#define is_v2_LV_c3_s_wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fd000)
#define is_v2_LV_c3_s_wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fc000)
#define is_v2_LV_c3_s_wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30fa000)
#define is_v2_LV_c3_s_wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa30f8000)
#define is_v2_LV_c3__deqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f1000)
#define is_v2_LV_c3__dgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f3000)
#define is_v2_LV_c3__dgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f5000)
#define is_v2_LV_c3__dlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f4000)
#define is_v2_LV_c3__dltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f2000)
#define is_v2_LV_c3__dnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f0000)
#define is_v2_LV_c3__even_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f7000)
#define is_v2_LV_c3__odd_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f6000)
#define is_v2_LV_c3__weqz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f9000)
#define is_v2_LV_c3__wgez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fb000)
#define is_v2_LV_c3__wgtz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fd000)
#define is_v2_LV_c3__wlez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fc000)
#define is_v2_LV_c3__wltz_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20fa000)
#define is_v2_LV_c3__wnez_registerY_registerAq_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa20f8000)
#define v2_LV_c0_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2303e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2303f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2203e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c0__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2203f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2307e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2307f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2207e000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c1__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x2207f000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230be000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230bf000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220be000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c2__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220bf000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s__registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230fe000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3_s_xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x230ff000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3___registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220fe000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_LV_c3__xs_registerAq_registerY_registerZ_simple_insn(registerAq,registerY,registerZ) (unsigned int)(0x220ff000 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_LV_c0_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303e000)
#define is_v2_LV_c0_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2303f000)
#define is_v2_LV_c0___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203e000)
#define is_v2_LV_c0__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2203f000)
#define is_v2_LV_c1_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307e000)
#define is_v2_LV_c1_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2307f000)
#define is_v2_LV_c1___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207e000)
#define is_v2_LV_c1__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2207f000)
#define is_v2_LV_c2_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230be000)
#define is_v2_LV_c2_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230bf000)
#define is_v2_LV_c2___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220be000)
#define is_v2_LV_c2__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220bf000)
#define is_v2_LV_c3_s__registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230fe000)
#define is_v2_LV_c3_s_xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x230ff000)
#define is_v2_LV_c3___registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220fe000)
#define is_v2_LV_c3__xs_registerAq_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x220ff000)
#define v2_SB_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x24010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SB_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x24010000)
#define v2_SB_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa4010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SB_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4010000)
#define v2_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa4010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SB_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa4010000)
#define v2_SB__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SB__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403e000)
#define is_v2_SB_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403f000)
#define v2_SB_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2403a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x24038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SB_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24031000)
#define is_v2_SB_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24033000)
#define is_v2_SB_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24035000)
#define is_v2_SB_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24034000)
#define is_v2_SB_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24032000)
#define is_v2_SB_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24030000)
#define is_v2_SB_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24037000)
#define is_v2_SB_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24036000)
#define is_v2_SB_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24039000)
#define is_v2_SB_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403b000)
#define is_v2_SB_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403d000)
#define is_v2_SB_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403c000)
#define is_v2_SB_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2403a000)
#define is_v2_SB_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x24038000)
#define v2_SB_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa403a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa4038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SB_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4031000)
#define is_v2_SB_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4033000)
#define is_v2_SB_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4035000)
#define is_v2_SB_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4034000)
#define is_v2_SB_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4032000)
#define is_v2_SB_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4030000)
#define is_v2_SB_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4037000)
#define is_v2_SB_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4036000)
#define is_v2_SB_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4039000)
#define is_v2_SB_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403b000)
#define is_v2_SB_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403d000)
#define is_v2_SB_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403c000)
#define is_v2_SB_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403a000)
#define is_v2_SB_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4038000)
#define v2_SB_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa403a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SB_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa4038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SB_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_SB_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4031000)
#define is_v2_SB_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4033000)
#define is_v2_SB_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4035000)
#define is_v2_SB_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4034000)
#define is_v2_SB_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4032000)
#define is_v2_SB_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4030000)
#define is_v2_SB_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4037000)
#define is_v2_SB_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4036000)
#define is_v2_SB_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4039000)
#define is_v2_SB_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403b000)
#define is_v2_SB_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403d000)
#define is_v2_SB_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403c000)
#define is_v2_SB_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa403a000)
#define is_v2_SB_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa4038000)
#define v2_SH_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x25010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SH_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x25010000)
#define v2_SH_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa5010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SH_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5010000)
#define v2_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa5010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SH_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa5010000)
#define v2_SH__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SH__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503e000)
#define is_v2_SH_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503f000)
#define v2_SH_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2503a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x25038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SH_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25031000)
#define is_v2_SH_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25033000)
#define is_v2_SH_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25035000)
#define is_v2_SH_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25034000)
#define is_v2_SH_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25032000)
#define is_v2_SH_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25030000)
#define is_v2_SH_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25037000)
#define is_v2_SH_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25036000)
#define is_v2_SH_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25039000)
#define is_v2_SH_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503b000)
#define is_v2_SH_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503d000)
#define is_v2_SH_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503c000)
#define is_v2_SH_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2503a000)
#define is_v2_SH_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x25038000)
#define v2_SH_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa503a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa5038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SH_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5031000)
#define is_v2_SH_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5033000)
#define is_v2_SH_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5035000)
#define is_v2_SH_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5034000)
#define is_v2_SH_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5032000)
#define is_v2_SH_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5030000)
#define is_v2_SH_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5037000)
#define is_v2_SH_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5036000)
#define is_v2_SH_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5039000)
#define is_v2_SH_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503b000)
#define is_v2_SH_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503d000)
#define is_v2_SH_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503c000)
#define is_v2_SH_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503a000)
#define is_v2_SH_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5038000)
#define v2_SH_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa503a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SH_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa5038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SH_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_SH_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5031000)
#define is_v2_SH_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5033000)
#define is_v2_SH_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5035000)
#define is_v2_SH_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5034000)
#define is_v2_SH_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5032000)
#define is_v2_SH_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5030000)
#define is_v2_SH_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5037000)
#define is_v2_SH_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5036000)
#define is_v2_SH_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5039000)
#define is_v2_SH_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503b000)
#define is_v2_SH_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503d000)
#define is_v2_SH_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503c000)
#define is_v2_SH_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa503a000)
#define is_v2_SH_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa5038000)
#define v2_SW_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x26010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SW_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x26010000)
#define v2_SW_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa6010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SW_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6010000)
#define v2_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa6010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SW_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa6010000)
#define v2_SW__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SW__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603e000)
#define is_v2_SW_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603f000)
#define v2_SW_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2603a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x26038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SW_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26031000)
#define is_v2_SW_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26033000)
#define is_v2_SW_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26035000)
#define is_v2_SW_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26034000)
#define is_v2_SW_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26032000)
#define is_v2_SW_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26030000)
#define is_v2_SW_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26037000)
#define is_v2_SW_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26036000)
#define is_v2_SW_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26039000)
#define is_v2_SW_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603b000)
#define is_v2_SW_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603d000)
#define is_v2_SW_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603c000)
#define is_v2_SW_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2603a000)
#define is_v2_SW_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x26038000)
#define v2_SW_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa603a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa6038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SW_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6031000)
#define is_v2_SW_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6033000)
#define is_v2_SW_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6035000)
#define is_v2_SW_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6034000)
#define is_v2_SW_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6032000)
#define is_v2_SW_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6030000)
#define is_v2_SW_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6037000)
#define is_v2_SW_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6036000)
#define is_v2_SW_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6039000)
#define is_v2_SW_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603b000)
#define is_v2_SW_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603d000)
#define is_v2_SW_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603c000)
#define is_v2_SW_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603a000)
#define is_v2_SW_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6038000)
#define v2_SW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa603a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa6038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_SW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6031000)
#define is_v2_SW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6033000)
#define is_v2_SW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6035000)
#define is_v2_SW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6034000)
#define is_v2_SW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6032000)
#define is_v2_SW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6030000)
#define is_v2_SW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6037000)
#define is_v2_SW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6036000)
#define is_v2_SW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6039000)
#define is_v2_SW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603b000)
#define is_v2_SW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603d000)
#define is_v2_SW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603c000)
#define is_v2_SW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa603a000)
#define is_v2_SW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa6038000)
#define v2_SD_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x27010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SD_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x27010000)
#define v2_SD_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xa7010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SD_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7010000)
#define v2_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xa7010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SD_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa7010000)
#define v2_SD__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SD__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703e000)
#define is_v2_SD_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703f000)
#define v2_SD_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2703a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x27038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SD_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27031000)
#define is_v2_SD_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27033000)
#define is_v2_SD_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27035000)
#define is_v2_SD_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27034000)
#define is_v2_SD_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27032000)
#define is_v2_SD_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27030000)
#define is_v2_SD_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27037000)
#define is_v2_SD_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27036000)
#define is_v2_SD_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27039000)
#define is_v2_SD_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703b000)
#define is_v2_SD_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703d000)
#define is_v2_SD_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703c000)
#define is_v2_SD_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2703a000)
#define is_v2_SD_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x27038000)
#define v2_SD_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa703a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xa7038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_SD_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7031000)
#define is_v2_SD_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7033000)
#define is_v2_SD_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7035000)
#define is_v2_SD_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7034000)
#define is_v2_SD_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7032000)
#define is_v2_SD_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7030000)
#define is_v2_SD_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7037000)
#define is_v2_SD_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7036000)
#define is_v2_SD_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7039000)
#define is_v2_SD_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703b000)
#define is_v2_SD_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703d000)
#define is_v2_SD_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703c000)
#define is_v2_SD_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703a000)
#define is_v2_SD_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7038000)
#define v2_SD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa703a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xa7038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_SD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_SD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7031000)
#define is_v2_SD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7033000)
#define is_v2_SD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7035000)
#define is_v2_SD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7034000)
#define is_v2_SD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7032000)
#define is_v2_SD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7030000)
#define is_v2_SD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7037000)
#define is_v2_SD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7036000)
#define is_v2_SD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7039000)
#define is_v2_SD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703b000)
#define is_v2_SD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703d000)
#define is_v2_SD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703c000)
#define is_v2_SD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa703a000)
#define is_v2_SD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa7038000)
#define v2_SQ_signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x28010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_SQ_signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x28010000)
#define v2_SQ_upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xa8010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SQ_upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xa8010000)
#define v2_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn_0(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(0xa8010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn_1(extend27_upper27_lower10,registerZ,registerU) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn_2(extend27_upper27_lower10,registerZ,registerU) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SQ_extend27_upper27_lower10_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xa8010000)
#define v2_SQ__registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_SQ__registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803e000)
#define is_v2_SQ_xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803f000)
#define v2_SQ_deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x2803a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x28038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_SQ_deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28031000)
#define is_v2_SQ_dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28033000)
#define is_v2_SQ_dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28035000)
#define is_v2_SQ_dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28034000)
#define is_v2_SQ_dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28032000)
#define is_v2_SQ_dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28030000)
#define is_v2_SQ_even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28037000)
#define is_v2_SQ_odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28036000)
#define is_v2_SQ_weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28039000)
#define is_v2_SQ_wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803b000)
#define is_v2_SQ_wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803d000)
#define is_v2_SQ_wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803c000)
#define is_v2_SQ_wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x2803a000)
#define is_v2_SQ_wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x28038000)
#define v2_SQ_deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa803a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xa8038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_SQ_deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8031000)
#define is_v2_SQ_dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8033000)
#define is_v2_SQ_dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8035000)
#define is_v2_SQ_dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8034000)
#define is_v2_SQ_dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8032000)
#define is_v2_SQ_dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8030000)
#define is_v2_SQ_even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8037000)
#define is_v2_SQ_odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8036000)
#define is_v2_SQ_weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8039000)
#define is_v2_SQ_wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803b000)
#define is_v2_SQ_wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803d000)
#define is_v2_SQ_wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803c000)
#define is_v2_SQ_wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803a000)
#define is_v2_SQ_wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8038000)
#define v2_SQ_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa803a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SQ_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xa8038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_SQ_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_SQ_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8031000)
#define is_v2_SQ_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8033000)
#define is_v2_SQ_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8035000)
#define is_v2_SQ_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8034000)
#define is_v2_SQ_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8032000)
#define is_v2_SQ_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8030000)
#define is_v2_SQ_even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8037000)
#define is_v2_SQ_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8036000)
#define is_v2_SQ_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8039000)
#define is_v2_SQ_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803b000)
#define is_v2_SQ_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803d000)
#define is_v2_SQ_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803c000)
#define is_v2_SQ_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa803a000)
#define is_v2_SQ_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xa8038000)
#define v2_SO_signed10_registerZ_registerV_simple_insn(signed10,registerZ,registerV) (unsigned int)(0x28050000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v2_SO_signed10_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x28050000)
#define v2_SO_upper27_lower10_registerZ_registerV_double_insn_0(upper27_lower10,registerZ,registerV) (unsigned int)(0xa8050000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_upper27_lower10_registerZ_registerV_double_insn_1(upper27_lower10,registerZ,registerV) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SO_upper27_lower10_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa8050000)
#define v2_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn_0(extend27_upper27_lower10,registerZ,registerV) (unsigned int)(0xa8050000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn_1(extend27_upper27_lower10,registerZ,registerV) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn_2(extend27_upper27_lower10,registerZ,registerV) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SO_extend27_upper27_lower10_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0f0000) == 0xa8050000)
#define v2_SO__registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_xs_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v2_SO__registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807e000)
#define is_v2_SO_xs_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807f000)
#define v2_SO_deqz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28071000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dgez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28073000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dgtz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28075000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dlez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28074000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dltz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28072000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dnez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28070000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_even_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28077000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_odd_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28076000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_weqz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28079000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wgez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wgtz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wlez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wltz_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x2807a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wnez_registerY_registerZ_registerV_simple_insn(registerY,registerZ,registerV) (unsigned int)(0x28078000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v2_SO_deqz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28071000)
#define is_v2_SO_dgez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28073000)
#define is_v2_SO_dgtz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28075000)
#define is_v2_SO_dlez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28074000)
#define is_v2_SO_dltz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28072000)
#define is_v2_SO_dnez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28070000)
#define is_v2_SO_even_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28077000)
#define is_v2_SO_odd_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28076000)
#define is_v2_SO_weqz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28079000)
#define is_v2_SO_wgez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807b000)
#define is_v2_SO_wgtz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807d000)
#define is_v2_SO_wlez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807c000)
#define is_v2_SO_wltz_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x2807a000)
#define is_v2_SO_wnez_registerY_registerZ_registerV_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0ff000) == 0x28078000)
#define v2_SO_deqz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8071000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dgez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8073000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dgtz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8075000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dlez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8074000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dltz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8072000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dnez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8070000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_even_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8077000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_odd_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8076000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_weqz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8079000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wgez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wgtz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wlez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wltz_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa807a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wnez_registerY_offset27_registerZ_registerV_double_insn(registerY,offset27,registerZ,registerV) (unsigned int)(0xa8078000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define is_v2_SO_deqz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8071000)
#define is_v2_SO_dgez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8073000)
#define is_v2_SO_dgtz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8075000)
#define is_v2_SO_dlez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8074000)
#define is_v2_SO_dltz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8072000)
#define is_v2_SO_dnez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8070000)
#define is_v2_SO_even_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8077000)
#define is_v2_SO_odd_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8076000)
#define is_v2_SO_weqz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8079000)
#define is_v2_SO_wgez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807b000)
#define is_v2_SO_wgtz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807d000)
#define is_v2_SO_wlez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807c000)
#define is_v2_SO_wltz_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807a000)
#define is_v2_SO_wnez_registerY_offset27_registerZ_registerV_double_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8078000)
#define v2_SO_deqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8071000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_deqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_dgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8073000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_dgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8075000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_dlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8074000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_dltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8072000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_dnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8070000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_dnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_even_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8077000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_even_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_odd_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8076000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_odd_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_weqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8079000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_weqz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_wgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wgez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_wgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_wlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wlez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_wltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa807a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wltz_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SO_wnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_0(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(0xa8078000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerV) & 0xf) << 20))
#define v2_SO_wnez_registerY_extend27_offset27_registerZ_registerV_triple_insn_1(registerY,extend27_offset27,registerZ,registerV) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_SO_deqz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8071000)
#define is_v2_SO_dgez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8073000)
#define is_v2_SO_dgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8075000)
#define is_v2_SO_dlez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8074000)
#define is_v2_SO_dltz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8072000)
#define is_v2_SO_dnez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8070000)
#define is_v2_SO_even_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8077000)
#define is_v2_SO_odd_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8076000)
#define is_v2_SO_weqz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8079000)
#define is_v2_SO_wgez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807b000)
#define is_v2_SO_wgtz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807d000)
#define is_v2_SO_wlez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807c000)
#define is_v2_SO_wltz_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa807a000)
#define is_v2_SO_wnez_registerY_extend27_offset27_registerZ_registerV_triple_insn(x) (int)(((unsigned int)(x) & 0xff0ff000) == 0xa8078000)
#define v2_SV_signed10_registerZ_registerE_simple_insn(signed10,registerZ,registerE) (unsigned int)(0x29010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v2_SV_signed10_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x29010000)
#define v2_SV_upper27_lower10_registerZ_registerE_double_insn_0(upper27_lower10,registerZ,registerE) (unsigned int)(0xa9010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_upper27_lower10_registerZ_registerE_double_insn_1(upper27_lower10,registerZ,registerE) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SV_upper27_lower10_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9010000)
#define v2_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn_0(extend27_upper27_lower10,registerZ,registerE) (unsigned int)(0xa9010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn_1(extend27_upper27_lower10,registerZ,registerE) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn_2(extend27_upper27_lower10,registerZ,registerE) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SV_extend27_upper27_lower10_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xa9010000)
#define v2_SV__registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_xs_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v2_SV__registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903e000)
#define is_v2_SV_xs_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903f000)
#define v2_SV_deqz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dgez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dgtz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dlez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dltz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dnez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_even_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_odd_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_weqz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wgez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wgtz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wlez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wltz_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x2903a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wnez_registerY_registerZ_registerE_simple_insn(registerY,registerZ,registerE) (unsigned int)(0x29038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v2_SV_deqz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29031000)
#define is_v2_SV_dgez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29033000)
#define is_v2_SV_dgtz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29035000)
#define is_v2_SV_dlez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29034000)
#define is_v2_SV_dltz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29032000)
#define is_v2_SV_dnez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29030000)
#define is_v2_SV_even_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29037000)
#define is_v2_SV_odd_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29036000)
#define is_v2_SV_weqz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29039000)
#define is_v2_SV_wgez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903b000)
#define is_v2_SV_wgtz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903d000)
#define is_v2_SV_wlez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903c000)
#define is_v2_SV_wltz_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2903a000)
#define is_v2_SV_wnez_registerY_registerZ_registerE_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x29038000)
#define v2_SV_deqz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dgez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dgtz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dlez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dltz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dnez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_even_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_odd_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_weqz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wgez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wgtz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wlez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wltz_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa903a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wnez_registerY_offset27_registerZ_registerE_double_insn(registerY,offset27,registerZ,registerE) (unsigned int)(0xa9038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define is_v2_SV_deqz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9031000)
#define is_v2_SV_dgez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9033000)
#define is_v2_SV_dgtz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9035000)
#define is_v2_SV_dlez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9034000)
#define is_v2_SV_dltz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9032000)
#define is_v2_SV_dnez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9030000)
#define is_v2_SV_even_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9037000)
#define is_v2_SV_odd_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9036000)
#define is_v2_SV_weqz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9039000)
#define is_v2_SV_wgez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903b000)
#define is_v2_SV_wgtz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903d000)
#define is_v2_SV_wlez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903c000)
#define is_v2_SV_wltz_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903a000)
#define is_v2_SV_wnez_registerY_offset27_registerZ_registerE_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9038000)
#define v2_SV_deqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_deqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_dgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_dgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_dlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_dltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_dnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_dnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_even_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_even_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_odd_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_odd_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_weqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_weqz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_wgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wgez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_wgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_wlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wlez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_wltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa903a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wltz_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_SV_wnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_0(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(0xa9038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerE) & 0x3f) << 18))
#define v2_SV_wnez_registerY_extend27_offset27_registerZ_registerE_triple_insn_1(registerY,extend27_offset27,registerZ,registerE) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_SV_deqz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9031000)
#define is_v2_SV_dgez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9033000)
#define is_v2_SV_dgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9035000)
#define is_v2_SV_dlez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9034000)
#define is_v2_SV_dltz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9032000)
#define is_v2_SV_dnez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9030000)
#define is_v2_SV_even_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9037000)
#define is_v2_SV_odd_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9036000)
#define is_v2_SV_weqz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9039000)
#define is_v2_SV_wgez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903b000)
#define is_v2_SV_wgtz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903d000)
#define is_v2_SV_wlez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903c000)
#define is_v2_SV_wltz_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa903a000)
#define is_v2_SV_wnez_registerY_extend27_offset27_registerZ_registerE_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xa9038000)
#define v2_ACSWAPW__signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x3e010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x3c010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPW__signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3e010000)
#define is_v2_ACSWAPW_v_signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3c010000)
#define v2_ACSWAPW__upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xbe010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_ACSWAPW_v_upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xbc010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ACSWAPW__upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbe010000)
#define is_v2_ACSWAPW_v_upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbc010000)
#define v2_ACSWAPW_v__registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW___registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPW_v__registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c03e000)
#define is_v2_ACSWAPW_v_xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c03f000)
#define is_v2_ACSWAPW___registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03e000)
#define is_v2_ACSWAPW__xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03f000)
#define v2_ACSWAPW_v_deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3c038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3e038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPW_v_deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c031000)
#define is_v2_ACSWAPW_v_dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c033000)
#define is_v2_ACSWAPW_v_dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c035000)
#define is_v2_ACSWAPW_v_dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c034000)
#define is_v2_ACSWAPW_v_dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c032000)
#define is_v2_ACSWAPW_v_dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c030000)
#define is_v2_ACSWAPW_v_even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c037000)
#define is_v2_ACSWAPW_v_odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c036000)
#define is_v2_ACSWAPW_v_weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c039000)
#define is_v2_ACSWAPW_v_wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c03b000)
#define is_v2_ACSWAPW_v_wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c03d000)
#define is_v2_ACSWAPW_v_wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c03c000)
#define is_v2_ACSWAPW_v_wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c03a000)
#define is_v2_ACSWAPW_v_wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3c038000)
#define is_v2_ACSWAPW__deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e031000)
#define is_v2_ACSWAPW__dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e033000)
#define is_v2_ACSWAPW__dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e035000)
#define is_v2_ACSWAPW__dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e034000)
#define is_v2_ACSWAPW__dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e032000)
#define is_v2_ACSWAPW__dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e030000)
#define is_v2_ACSWAPW__even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e037000)
#define is_v2_ACSWAPW__odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e036000)
#define is_v2_ACSWAPW__weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e039000)
#define is_v2_ACSWAPW__wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03b000)
#define is_v2_ACSWAPW__wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03d000)
#define is_v2_ACSWAPW__wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03c000)
#define is_v2_ACSWAPW__wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e03a000)
#define is_v2_ACSWAPW__wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3e038000)
#define v2_ACSWAPW_v_deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbc038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbe038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPW_v_deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc031000)
#define is_v2_ACSWAPW_v_dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc033000)
#define is_v2_ACSWAPW_v_dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc035000)
#define is_v2_ACSWAPW_v_dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc034000)
#define is_v2_ACSWAPW_v_dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc032000)
#define is_v2_ACSWAPW_v_dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc030000)
#define is_v2_ACSWAPW_v_even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc037000)
#define is_v2_ACSWAPW_v_odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc036000)
#define is_v2_ACSWAPW_v_weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc039000)
#define is_v2_ACSWAPW_v_wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03b000)
#define is_v2_ACSWAPW_v_wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03d000)
#define is_v2_ACSWAPW_v_wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03c000)
#define is_v2_ACSWAPW_v_wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03a000)
#define is_v2_ACSWAPW_v_wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc038000)
#define is_v2_ACSWAPW__deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe031000)
#define is_v2_ACSWAPW__dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe033000)
#define is_v2_ACSWAPW__dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe035000)
#define is_v2_ACSWAPW__dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe034000)
#define is_v2_ACSWAPW__dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe032000)
#define is_v2_ACSWAPW__dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe030000)
#define is_v2_ACSWAPW__even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe037000)
#define is_v2_ACSWAPW__odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe036000)
#define is_v2_ACSWAPW__weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe039000)
#define is_v2_ACSWAPW__wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03b000)
#define is_v2_ACSWAPW__wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03d000)
#define is_v2_ACSWAPW__wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03c000)
#define is_v2_ACSWAPW__wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03a000)
#define is_v2_ACSWAPW__wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe038000)
#define v2_ACSWAPW_v_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW_v_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbc038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW_v_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPW__wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbe038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPW__wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_ACSWAPW_v_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc031000)
#define is_v2_ACSWAPW_v_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc033000)
#define is_v2_ACSWAPW_v_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc035000)
#define is_v2_ACSWAPW_v_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc034000)
#define is_v2_ACSWAPW_v_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc032000)
#define is_v2_ACSWAPW_v_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc030000)
#define is_v2_ACSWAPW_v_even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc037000)
#define is_v2_ACSWAPW_v_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc036000)
#define is_v2_ACSWAPW_v_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc039000)
#define is_v2_ACSWAPW_v_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03b000)
#define is_v2_ACSWAPW_v_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03d000)
#define is_v2_ACSWAPW_v_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03c000)
#define is_v2_ACSWAPW_v_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc03a000)
#define is_v2_ACSWAPW_v_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbc038000)
#define is_v2_ACSWAPW__deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe031000)
#define is_v2_ACSWAPW__dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe033000)
#define is_v2_ACSWAPW__dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe035000)
#define is_v2_ACSWAPW__dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe034000)
#define is_v2_ACSWAPW__dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe032000)
#define is_v2_ACSWAPW__dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe030000)
#define is_v2_ACSWAPW__even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe037000)
#define is_v2_ACSWAPW__odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe036000)
#define is_v2_ACSWAPW__weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe039000)
#define is_v2_ACSWAPW__wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03b000)
#define is_v2_ACSWAPW__wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03d000)
#define is_v2_ACSWAPW__wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03c000)
#define is_v2_ACSWAPW__wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe03a000)
#define is_v2_ACSWAPW__wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbe038000)
#define v2_ACSWAPD__signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x3f010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_signed10_registerZ_registerU_simple_insn(signed10,registerZ,registerU) (unsigned int)(0x3d010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPD__signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3f010000)
#define is_v2_ACSWAPD_v_signed10_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x3d010000)
#define v2_ACSWAPD__upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xbf010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_ACSWAPD_v_upper27_lower10_registerZ_registerU_double_insn_0(upper27_lower10,registerZ,registerU) (unsigned int)(0xbd010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_upper27_lower10_registerZ_registerU_double_insn_1(upper27_lower10,registerZ,registerU) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ACSWAPD__upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbf010000)
#define is_v2_ACSWAPD_v_upper27_lower10_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xbd010000)
#define v2_ACSWAPD_v__registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD___registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__xs_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPD_v__registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d03e000)
#define is_v2_ACSWAPD_v_xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d03f000)
#define is_v2_ACSWAPD___registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03e000)
#define is_v2_ACSWAPD__xs_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03f000)
#define v2_ACSWAPD_v_deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3d038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__deqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__even_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__odd_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__weqz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wgez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wgtz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wlez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wltz_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wnez_registerY_registerZ_registerU_simple_insn(registerY,registerZ,registerU) (unsigned int)(0x3f038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPD_v_deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d031000)
#define is_v2_ACSWAPD_v_dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d033000)
#define is_v2_ACSWAPD_v_dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d035000)
#define is_v2_ACSWAPD_v_dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d034000)
#define is_v2_ACSWAPD_v_dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d032000)
#define is_v2_ACSWAPD_v_dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d030000)
#define is_v2_ACSWAPD_v_even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d037000)
#define is_v2_ACSWAPD_v_odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d036000)
#define is_v2_ACSWAPD_v_weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d039000)
#define is_v2_ACSWAPD_v_wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d03b000)
#define is_v2_ACSWAPD_v_wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d03d000)
#define is_v2_ACSWAPD_v_wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d03c000)
#define is_v2_ACSWAPD_v_wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d03a000)
#define is_v2_ACSWAPD_v_wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3d038000)
#define is_v2_ACSWAPD__deqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f031000)
#define is_v2_ACSWAPD__dgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f033000)
#define is_v2_ACSWAPD__dgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f035000)
#define is_v2_ACSWAPD__dlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f034000)
#define is_v2_ACSWAPD__dltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f032000)
#define is_v2_ACSWAPD__dnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f030000)
#define is_v2_ACSWAPD__even_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f037000)
#define is_v2_ACSWAPD__odd_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f036000)
#define is_v2_ACSWAPD__weqz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f039000)
#define is_v2_ACSWAPD__wgez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03b000)
#define is_v2_ACSWAPD__wgtz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03d000)
#define is_v2_ACSWAPD__wlez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03c000)
#define is_v2_ACSWAPD__wltz_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f03a000)
#define is_v2_ACSWAPD__wnez_registerY_registerZ_registerU_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x3f038000)
#define v2_ACSWAPD_v_deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbd038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__deqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__even_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__odd_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__weqz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wgez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wgtz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wlez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wltz_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wnez_registerY_offset27_registerZ_registerU_double_insn(registerY,offset27,registerZ,registerU) (unsigned int)(0xbf038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define is_v2_ACSWAPD_v_deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd031000)
#define is_v2_ACSWAPD_v_dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd033000)
#define is_v2_ACSWAPD_v_dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd035000)
#define is_v2_ACSWAPD_v_dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd034000)
#define is_v2_ACSWAPD_v_dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd032000)
#define is_v2_ACSWAPD_v_dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd030000)
#define is_v2_ACSWAPD_v_even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd037000)
#define is_v2_ACSWAPD_v_odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd036000)
#define is_v2_ACSWAPD_v_weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd039000)
#define is_v2_ACSWAPD_v_wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03b000)
#define is_v2_ACSWAPD_v_wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03d000)
#define is_v2_ACSWAPD_v_wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03c000)
#define is_v2_ACSWAPD_v_wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03a000)
#define is_v2_ACSWAPD_v_wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd038000)
#define is_v2_ACSWAPD__deqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf031000)
#define is_v2_ACSWAPD__dgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf033000)
#define is_v2_ACSWAPD__dgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf035000)
#define is_v2_ACSWAPD__dlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf034000)
#define is_v2_ACSWAPD__dltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf032000)
#define is_v2_ACSWAPD__dnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf030000)
#define is_v2_ACSWAPD__even_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf037000)
#define is_v2_ACSWAPD__odd_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf036000)
#define is_v2_ACSWAPD__weqz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf039000)
#define is_v2_ACSWAPD__wgez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03b000)
#define is_v2_ACSWAPD__wgtz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03d000)
#define is_v2_ACSWAPD__wlez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03c000)
#define is_v2_ACSWAPD__wltz_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03a000)
#define is_v2_ACSWAPD__wnez_registerY_offset27_registerZ_registerU_double_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf038000)
#define v2_ACSWAPD_v_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD_v_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbd038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD_v_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__even_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__even_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__odd_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ACSWAPD__wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_0(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(0xbf038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerU) & 0x1f) << 19))
#define v2_ACSWAPD__wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn_1(registerY,extend27_offset27,registerZ,registerU) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_ACSWAPD_v_deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd031000)
#define is_v2_ACSWAPD_v_dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd033000)
#define is_v2_ACSWAPD_v_dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd035000)
#define is_v2_ACSWAPD_v_dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd034000)
#define is_v2_ACSWAPD_v_dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd032000)
#define is_v2_ACSWAPD_v_dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd030000)
#define is_v2_ACSWAPD_v_even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd037000)
#define is_v2_ACSWAPD_v_odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd036000)
#define is_v2_ACSWAPD_v_weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd039000)
#define is_v2_ACSWAPD_v_wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03b000)
#define is_v2_ACSWAPD_v_wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03d000)
#define is_v2_ACSWAPD_v_wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03c000)
#define is_v2_ACSWAPD_v_wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd03a000)
#define is_v2_ACSWAPD_v_wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbd038000)
#define is_v2_ACSWAPD__deqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf031000)
#define is_v2_ACSWAPD__dgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf033000)
#define is_v2_ACSWAPD__dgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf035000)
#define is_v2_ACSWAPD__dlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf034000)
#define is_v2_ACSWAPD__dltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf032000)
#define is_v2_ACSWAPD__dnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf030000)
#define is_v2_ACSWAPD__even_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf037000)
#define is_v2_ACSWAPD__odd_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf036000)
#define is_v2_ACSWAPD__weqz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf039000)
#define is_v2_ACSWAPD__wgez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03b000)
#define is_v2_ACSWAPD__wgtz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03d000)
#define is_v2_ACSWAPD__wlez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03c000)
#define is_v2_ACSWAPD__wltz_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf03a000)
#define is_v2_ACSWAPD__wnez_registerY_extend27_offset27_registerZ_registerU_triple_insn(x) (int)(((unsigned int)(x) & 0xff07f000) == 0xbf038000)
#define v2_ALCLRW_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2a010000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRW_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2a010000)
#define v2_ALCLRW_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xaa010000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ALCLRW_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa010000)
#define v2_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xaa010000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ALCLRW_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaa010000)
#define v2_ALCLRW__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a03e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2a03f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRW__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03e000)
#define is_v2_ALCLRW_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03f000)
#define v2_ALCLRW_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2a038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRW_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a031000)
#define is_v2_ALCLRW_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a033000)
#define is_v2_ALCLRW_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a035000)
#define is_v2_ALCLRW_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a034000)
#define is_v2_ALCLRW_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a032000)
#define is_v2_ALCLRW_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a030000)
#define is_v2_ALCLRW_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a037000)
#define is_v2_ALCLRW_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a036000)
#define is_v2_ALCLRW_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a039000)
#define is_v2_ALCLRW_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03b000)
#define is_v2_ALCLRW_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03d000)
#define is_v2_ALCLRW_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03c000)
#define is_v2_ALCLRW_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a03a000)
#define is_v2_ALCLRW_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2a038000)
#define v2_ALCLRW_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xaa038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRW_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa031000)
#define is_v2_ALCLRW_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa033000)
#define is_v2_ALCLRW_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa035000)
#define is_v2_ALCLRW_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa034000)
#define is_v2_ALCLRW_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa032000)
#define is_v2_ALCLRW_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa030000)
#define is_v2_ALCLRW_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa037000)
#define is_v2_ALCLRW_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa036000)
#define is_v2_ALCLRW_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa039000)
#define is_v2_ALCLRW_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03b000)
#define is_v2_ALCLRW_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03d000)
#define is_v2_ALCLRW_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03c000)
#define is_v2_ALCLRW_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03a000)
#define is_v2_ALCLRW_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa038000)
#define v2_ALCLRW_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRW_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xaa038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRW_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_ALCLRW_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa031000)
#define is_v2_ALCLRW_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa033000)
#define is_v2_ALCLRW_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa035000)
#define is_v2_ALCLRW_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa034000)
#define is_v2_ALCLRW_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa032000)
#define is_v2_ALCLRW_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa030000)
#define is_v2_ALCLRW_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa037000)
#define is_v2_ALCLRW_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa036000)
#define is_v2_ALCLRW_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa039000)
#define is_v2_ALCLRW_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03b000)
#define is_v2_ALCLRW_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03d000)
#define is_v2_ALCLRW_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03c000)
#define is_v2_ALCLRW_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa03a000)
#define is_v2_ALCLRW_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaa038000)
#define v2_ALCLRD_registerW_signed10_registerZ_simple_insn(registerW,signed10,registerZ) (unsigned int)(0x2b010000 | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRD_registerW_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2b010000)
#define v2_ALCLRD_registerW_upper27_lower10_registerZ_double_insn_0(registerW,upper27_lower10,registerZ) (unsigned int)(0xab010000 | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_registerW_upper27_lower10_registerZ_double_insn_1(registerW,upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ALCLRD_registerW_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab010000)
#define v2_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn_0(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(0xab010000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn_1(registerW,extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn_2(registerW,extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ALCLRD_registerW_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xab010000)
#define v2_ALCLRD__registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b03e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_xs_registerW_registerY_registerZ_simple_insn(registerW,registerY,registerZ) (unsigned int)(0x2b03f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRD__registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03e000)
#define is_v2_ALCLRD_xs_registerW_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03f000)
#define v2_ALCLRD_deqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_even_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_odd_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_weqz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wgez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wgtz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wlez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wltz_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wnez_registerY_registerW_registerZ_simple_insn(registerY,registerW,registerZ) (unsigned int)(0x2b038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRD_deqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b031000)
#define is_v2_ALCLRD_dgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b033000)
#define is_v2_ALCLRD_dgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b035000)
#define is_v2_ALCLRD_dlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b034000)
#define is_v2_ALCLRD_dltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b032000)
#define is_v2_ALCLRD_dnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b030000)
#define is_v2_ALCLRD_even_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b037000)
#define is_v2_ALCLRD_odd_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b036000)
#define is_v2_ALCLRD_weqz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b039000)
#define is_v2_ALCLRD_wgez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03b000)
#define is_v2_ALCLRD_wgtz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03d000)
#define is_v2_ALCLRD_wlez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03c000)
#define is_v2_ALCLRD_wltz_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b03a000)
#define is_v2_ALCLRD_wnez_registerY_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2b038000)
#define v2_ALCLRD_deqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_even_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_odd_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_weqz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wgez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wgtz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wlez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wltz_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wnez_registerY_registerW_offset27_registerZ_double_insn(registerY,registerW,offset27,registerZ) (unsigned int)(0xab038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_ALCLRD_deqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab031000)
#define is_v2_ALCLRD_dgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab033000)
#define is_v2_ALCLRD_dgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab035000)
#define is_v2_ALCLRD_dlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab034000)
#define is_v2_ALCLRD_dltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab032000)
#define is_v2_ALCLRD_dnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab030000)
#define is_v2_ALCLRD_even_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab037000)
#define is_v2_ALCLRD_odd_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab036000)
#define is_v2_ALCLRD_weqz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab039000)
#define is_v2_ALCLRD_wgez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03b000)
#define is_v2_ALCLRD_wgtz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03d000)
#define is_v2_ALCLRD_wlez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03c000)
#define is_v2_ALCLRD_wltz_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03a000)
#define is_v2_ALCLRD_wnez_registerY_registerW_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab038000)
#define v2_ALCLRD_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_even_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALCLRD_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_0(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(0xab038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(registerW) & 0x3f) << 18) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_ALCLRD_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn_1(registerY,registerW,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_ALCLRD_deqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab031000)
#define is_v2_ALCLRD_dgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab033000)
#define is_v2_ALCLRD_dgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab035000)
#define is_v2_ALCLRD_dlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab034000)
#define is_v2_ALCLRD_dltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab032000)
#define is_v2_ALCLRD_dnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab030000)
#define is_v2_ALCLRD_even_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab037000)
#define is_v2_ALCLRD_odd_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab036000)
#define is_v2_ALCLRD_weqz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab039000)
#define is_v2_ALCLRD_wgez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03b000)
#define is_v2_ALCLRD_wgtz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03d000)
#define is_v2_ALCLRD_wlez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03c000)
#define is_v2_ALCLRD_wltz_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab03a000)
#define is_v2_ALCLRD_wnez_registerY_registerW_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xab038000)
#define v2_ALADDW_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x2e010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDW_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2e010000)
#define v2_ALADDW_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xae010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ALADDW_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae010000)
#define v2_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xae010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ALADDW_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xae010000)
#define v2_ALADDW__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDW__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03e000)
#define is_v2_ALADDW_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03f000)
#define v2_ALADDW_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2e038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDW_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e031000)
#define is_v2_ALADDW_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e033000)
#define is_v2_ALADDW_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e035000)
#define is_v2_ALADDW_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e034000)
#define is_v2_ALADDW_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e032000)
#define is_v2_ALADDW_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e030000)
#define is_v2_ALADDW_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e037000)
#define is_v2_ALADDW_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e036000)
#define is_v2_ALADDW_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e039000)
#define is_v2_ALADDW_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03b000)
#define is_v2_ALADDW_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03d000)
#define is_v2_ALADDW_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03c000)
#define is_v2_ALADDW_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e03a000)
#define is_v2_ALADDW_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2e038000)
#define v2_ALADDW_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xae038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDW_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae031000)
#define is_v2_ALADDW_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae033000)
#define is_v2_ALADDW_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae035000)
#define is_v2_ALADDW_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae034000)
#define is_v2_ALADDW_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae032000)
#define is_v2_ALADDW_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae030000)
#define is_v2_ALADDW_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae037000)
#define is_v2_ALADDW_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae036000)
#define is_v2_ALADDW_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae039000)
#define is_v2_ALADDW_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03b000)
#define is_v2_ALADDW_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03d000)
#define is_v2_ALADDW_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03c000)
#define is_v2_ALADDW_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03a000)
#define is_v2_ALADDW_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae038000)
#define v2_ALADDW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xae038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_ALADDW_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae031000)
#define is_v2_ALADDW_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae033000)
#define is_v2_ALADDW_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae035000)
#define is_v2_ALADDW_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae034000)
#define is_v2_ALADDW_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae032000)
#define is_v2_ALADDW_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae030000)
#define is_v2_ALADDW_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae037000)
#define is_v2_ALADDW_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae036000)
#define is_v2_ALADDW_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae039000)
#define is_v2_ALADDW_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03b000)
#define is_v2_ALADDW_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03d000)
#define is_v2_ALADDW_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03c000)
#define is_v2_ALADDW_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae03a000)
#define is_v2_ALADDW_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xae038000)
#define v2_ALADDD_signed10_registerZ_registerT_simple_insn(signed10,registerZ,registerT) (unsigned int)(0x2f010000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDD_signed10_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x2f010000)
#define v2_ALADDD_upper27_lower10_registerZ_registerT_double_insn_0(upper27_lower10,registerZ,registerT) (unsigned int)(0xaf010000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_upper27_lower10_registerZ_registerT_double_insn_1(upper27_lower10,registerZ,registerT) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ALADDD_upper27_lower10_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf010000)
#define v2_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn_0(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(0xaf010000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn_1(extend27_upper27_lower10,registerZ,registerT) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn_2(extend27_upper27_lower10,registerZ,registerT) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ALADDD_extend27_upper27_lower10_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xaf010000)
#define v2_ALADDD__registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03e000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_xs_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03f000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDD__registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03e000)
#define is_v2_ALADDD_xs_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03f000)
#define v2_ALADDD_deqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_even_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_odd_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_weqz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wgez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wgtz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wlez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wltz_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wnez_registerY_registerZ_registerT_simple_insn(registerY,registerZ,registerT) (unsigned int)(0x2f038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDD_deqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f031000)
#define is_v2_ALADDD_dgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f033000)
#define is_v2_ALADDD_dgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f035000)
#define is_v2_ALADDD_dlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f034000)
#define is_v2_ALADDD_dltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f032000)
#define is_v2_ALADDD_dnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f030000)
#define is_v2_ALADDD_even_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f037000)
#define is_v2_ALADDD_odd_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f036000)
#define is_v2_ALADDD_weqz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f039000)
#define is_v2_ALADDD_wgez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03b000)
#define is_v2_ALADDD_wgtz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03d000)
#define is_v2_ALADDD_wlez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03c000)
#define is_v2_ALADDD_wltz_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f03a000)
#define is_v2_ALADDD_wnez_registerY_registerZ_registerT_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x2f038000)
#define v2_ALADDD_deqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf031000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf033000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf035000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf034000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf032000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf030000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_even_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf037000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_odd_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf036000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_weqz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf039000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wgez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03b000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wgtz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03d000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wlez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03c000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wltz_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf03a000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wnez_registerY_offset27_registerZ_registerT_double_insn(registerY,offset27,registerZ,registerT) (unsigned int)(0xaf038000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define is_v2_ALADDD_deqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf031000)
#define is_v2_ALADDD_dgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf033000)
#define is_v2_ALADDD_dgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf035000)
#define is_v2_ALADDD_dlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf034000)
#define is_v2_ALADDD_dltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf032000)
#define is_v2_ALADDD_dnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf030000)
#define is_v2_ALADDD_even_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf037000)
#define is_v2_ALADDD_odd_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf036000)
#define is_v2_ALADDD_weqz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf039000)
#define is_v2_ALADDD_wgez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03b000)
#define is_v2_ALADDD_wgtz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03d000)
#define is_v2_ALADDD_wlez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03c000)
#define is_v2_ALADDD_wltz_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03a000)
#define is_v2_ALADDD_wnez_registerY_offset27_registerZ_registerT_double_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf038000)
#define v2_ALADDD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf031000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf033000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf035000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf034000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf032000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf030000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf037000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf036000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf039000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03b000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03d000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03c000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf03a000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_ALADDD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_0(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(0xaf038000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f) | (((int)(registerT) & 0x3f) << 18))
#define v2_ALADDD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn_1(registerY,extend27_offset27,registerZ,registerT) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_ALADDD_deqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf031000)
#define is_v2_ALADDD_dgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf033000)
#define is_v2_ALADDD_dgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf035000)
#define is_v2_ALADDD_dlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf034000)
#define is_v2_ALADDD_dltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf032000)
#define is_v2_ALADDD_dnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf030000)
#define is_v2_ALADDD_even_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf037000)
#define is_v2_ALADDD_odd_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf036000)
#define is_v2_ALADDD_weqz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf039000)
#define is_v2_ALADDD_wgez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03b000)
#define is_v2_ALADDD_wgtz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03d000)
#define is_v2_ALADDD_wlez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03c000)
#define is_v2_ALADDD_wltz_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf03a000)
#define is_v2_ALADDD_wnez_registerY_extend27_offset27_registerZ_registerT_triple_insn(x) (int)(((unsigned int)(x) & 0xff03f000) == 0xaf038000)
#define v2_DTOUCHL__signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3c0c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3e0c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3d0c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_DTOUCHL__signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3c0c0000)
#define is_v2_DTOUCHL_l1l2_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3e0c0000)
#define is_v2_DTOUCHL_l2_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3d0c0000)
#define v2_DTOUCHL__upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbc0c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbe0c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_DTOUCHL_l2_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbd0c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_DTOUCHL__upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbc0c0000)
#define is_v2_DTOUCHL_l1l2_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbe0c0000)
#define is_v2_DTOUCHL_l2_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbd0c0000)
#define v2_DTOUCHL__extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbc0c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DTOUCHL__extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbe0c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_DTOUCHL_l2_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbd0c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DTOUCHL_l2_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_DTOUCHL__extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbc0c0000)
#define is_v2_DTOUCHL_l1l2_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbe0c0000)
#define is_v2_DTOUCHL_l2_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbd0c0000)
#define v2_DTOUCHL__registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_DTOUCHL__registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0ee000)
#define is_v2_DTOUCHL_l1l2_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0ee000)
#define is_v2_DTOUCHL_l2_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0ee000)
#define v2_DTOUCHL_l1l2_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_DTOUCHL_l1l2_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e1000)
#define is_v2_DTOUCHL_l1l2_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e3000)
#define is_v2_DTOUCHL_l1l2_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e5000)
#define is_v2_DTOUCHL_l1l2_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e4000)
#define is_v2_DTOUCHL_l1l2_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e2000)
#define is_v2_DTOUCHL_l1l2_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e0000)
#define is_v2_DTOUCHL_l1l2_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e7000)
#define is_v2_DTOUCHL_l1l2_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e6000)
#define is_v2_DTOUCHL_l1l2_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e9000)
#define is_v2_DTOUCHL_l1l2_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0eb000)
#define is_v2_DTOUCHL_l1l2_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0ed000)
#define is_v2_DTOUCHL_l1l2_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0ec000)
#define is_v2_DTOUCHL_l1l2_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0ea000)
#define is_v2_DTOUCHL_l1l2_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e0e8000)
#define is_v2_DTOUCHL_l2_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e1000)
#define is_v2_DTOUCHL_l2_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e3000)
#define is_v2_DTOUCHL_l2_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e5000)
#define is_v2_DTOUCHL_l2_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e4000)
#define is_v2_DTOUCHL_l2_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e2000)
#define is_v2_DTOUCHL_l2_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e0000)
#define is_v2_DTOUCHL_l2_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e7000)
#define is_v2_DTOUCHL_l2_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e6000)
#define is_v2_DTOUCHL_l2_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e9000)
#define is_v2_DTOUCHL_l2_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0eb000)
#define is_v2_DTOUCHL_l2_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0ed000)
#define is_v2_DTOUCHL_l2_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0ec000)
#define is_v2_DTOUCHL_l2_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0ea000)
#define is_v2_DTOUCHL_l2_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d0e8000)
#define is_v2_DTOUCHL__deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e1000)
#define is_v2_DTOUCHL__dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e3000)
#define is_v2_DTOUCHL__dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e5000)
#define is_v2_DTOUCHL__dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e4000)
#define is_v2_DTOUCHL__dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e2000)
#define is_v2_DTOUCHL__dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e0000)
#define is_v2_DTOUCHL__even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e7000)
#define is_v2_DTOUCHL__odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e6000)
#define is_v2_DTOUCHL__weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e9000)
#define is_v2_DTOUCHL__wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0eb000)
#define is_v2_DTOUCHL__wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0ed000)
#define is_v2_DTOUCHL__wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0ec000)
#define is_v2_DTOUCHL__wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0ea000)
#define is_v2_DTOUCHL__wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c0e8000)
#define v2_DTOUCHL_l1l2_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc0e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_DTOUCHL_l1l2_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e1000)
#define is_v2_DTOUCHL_l1l2_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e3000)
#define is_v2_DTOUCHL_l1l2_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e5000)
#define is_v2_DTOUCHL_l1l2_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e4000)
#define is_v2_DTOUCHL_l1l2_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e2000)
#define is_v2_DTOUCHL_l1l2_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e0000)
#define is_v2_DTOUCHL_l1l2_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e7000)
#define is_v2_DTOUCHL_l1l2_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e6000)
#define is_v2_DTOUCHL_l1l2_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e9000)
#define is_v2_DTOUCHL_l1l2_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0eb000)
#define is_v2_DTOUCHL_l1l2_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0ed000)
#define is_v2_DTOUCHL_l1l2_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0ec000)
#define is_v2_DTOUCHL_l1l2_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0ea000)
#define is_v2_DTOUCHL_l1l2_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e8000)
#define is_v2_DTOUCHL_l2_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e1000)
#define is_v2_DTOUCHL_l2_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e3000)
#define is_v2_DTOUCHL_l2_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e5000)
#define is_v2_DTOUCHL_l2_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e4000)
#define is_v2_DTOUCHL_l2_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e2000)
#define is_v2_DTOUCHL_l2_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e0000)
#define is_v2_DTOUCHL_l2_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e7000)
#define is_v2_DTOUCHL_l2_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e6000)
#define is_v2_DTOUCHL_l2_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e9000)
#define is_v2_DTOUCHL_l2_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0eb000)
#define is_v2_DTOUCHL_l2_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0ed000)
#define is_v2_DTOUCHL_l2_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0ec000)
#define is_v2_DTOUCHL_l2_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0ea000)
#define is_v2_DTOUCHL_l2_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e8000)
#define is_v2_DTOUCHL__deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e1000)
#define is_v2_DTOUCHL__dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e3000)
#define is_v2_DTOUCHL__dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e5000)
#define is_v2_DTOUCHL__dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e4000)
#define is_v2_DTOUCHL__dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e2000)
#define is_v2_DTOUCHL__dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e0000)
#define is_v2_DTOUCHL__even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e7000)
#define is_v2_DTOUCHL__odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e6000)
#define is_v2_DTOUCHL__weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e9000)
#define is_v2_DTOUCHL__wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0eb000)
#define is_v2_DTOUCHL__wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0ed000)
#define is_v2_DTOUCHL__wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0ec000)
#define is_v2_DTOUCHL__wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0ea000)
#define is_v2_DTOUCHL__wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e8000)
#define v2_DTOUCHL_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe0e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd0e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DTOUCHL__wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc0e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DTOUCHL__wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_DTOUCHL_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e1000)
#define is_v2_DTOUCHL_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e3000)
#define is_v2_DTOUCHL_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e5000)
#define is_v2_DTOUCHL_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e4000)
#define is_v2_DTOUCHL_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e2000)
#define is_v2_DTOUCHL_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e0000)
#define is_v2_DTOUCHL_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e7000)
#define is_v2_DTOUCHL_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e6000)
#define is_v2_DTOUCHL_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e9000)
#define is_v2_DTOUCHL_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0eb000)
#define is_v2_DTOUCHL_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0ed000)
#define is_v2_DTOUCHL_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0ec000)
#define is_v2_DTOUCHL_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0ea000)
#define is_v2_DTOUCHL_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe0e8000)
#define is_v2_DTOUCHL_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e1000)
#define is_v2_DTOUCHL_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e3000)
#define is_v2_DTOUCHL_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e5000)
#define is_v2_DTOUCHL_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e4000)
#define is_v2_DTOUCHL_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e2000)
#define is_v2_DTOUCHL_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e0000)
#define is_v2_DTOUCHL_l2_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e7000)
#define is_v2_DTOUCHL_l2_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e6000)
#define is_v2_DTOUCHL_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e9000)
#define is_v2_DTOUCHL_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0eb000)
#define is_v2_DTOUCHL_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0ed000)
#define is_v2_DTOUCHL_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0ec000)
#define is_v2_DTOUCHL_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0ea000)
#define is_v2_DTOUCHL_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd0e8000)
#define is_v2_DTOUCHL__deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e1000)
#define is_v2_DTOUCHL__dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e3000)
#define is_v2_DTOUCHL__dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e5000)
#define is_v2_DTOUCHL__dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e4000)
#define is_v2_DTOUCHL__dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e2000)
#define is_v2_DTOUCHL__dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e0000)
#define is_v2_DTOUCHL__even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e7000)
#define is_v2_DTOUCHL__odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e6000)
#define is_v2_DTOUCHL__weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e9000)
#define is_v2_DTOUCHL__wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0eb000)
#define is_v2_DTOUCHL__wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0ed000)
#define is_v2_DTOUCHL__wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0ec000)
#define is_v2_DTOUCHL__wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0ea000)
#define is_v2_DTOUCHL__wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc0e8000)
#define v2_DINVALL__signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3c1c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3e1c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3d1c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_DINVALL__signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3c1c0000)
#define is_v2_DINVALL_l1l2_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3e1c0000)
#define is_v2_DINVALL_l2_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3d1c0000)
#define v2_DINVALL__upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbc1c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_DINVALL_l1l2_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbe1c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_DINVALL_l2_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbd1c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_DINVALL__upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbc1c0000)
#define is_v2_DINVALL_l1l2_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbe1c0000)
#define is_v2_DINVALL_l2_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbd1c0000)
#define v2_DINVALL__extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbc1c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DINVALL__extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_DINVALL_l1l2_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbe1c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DINVALL_l1l2_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_DINVALL_l2_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbd1c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DINVALL_l2_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_DINVALL__extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbc1c0000)
#define is_v2_DINVALL_l1l2_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbe1c0000)
#define is_v2_DINVALL_l2_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbd1c0000)
#define v2_DINVALL__registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_DINVALL__registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1ee000)
#define is_v2_DINVALL_l1l2_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1ee000)
#define is_v2_DINVALL_l2_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1ee000)
#define v2_DINVALL_l1l2_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_DINVALL_l1l2_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e1000)
#define is_v2_DINVALL_l1l2_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e3000)
#define is_v2_DINVALL_l1l2_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e5000)
#define is_v2_DINVALL_l1l2_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e4000)
#define is_v2_DINVALL_l1l2_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e2000)
#define is_v2_DINVALL_l1l2_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e0000)
#define is_v2_DINVALL_l1l2_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e7000)
#define is_v2_DINVALL_l1l2_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e6000)
#define is_v2_DINVALL_l1l2_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e9000)
#define is_v2_DINVALL_l1l2_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1eb000)
#define is_v2_DINVALL_l1l2_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1ed000)
#define is_v2_DINVALL_l1l2_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1ec000)
#define is_v2_DINVALL_l1l2_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1ea000)
#define is_v2_DINVALL_l1l2_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e1e8000)
#define is_v2_DINVALL_l2_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e1000)
#define is_v2_DINVALL_l2_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e3000)
#define is_v2_DINVALL_l2_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e5000)
#define is_v2_DINVALL_l2_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e4000)
#define is_v2_DINVALL_l2_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e2000)
#define is_v2_DINVALL_l2_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e0000)
#define is_v2_DINVALL_l2_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e7000)
#define is_v2_DINVALL_l2_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e6000)
#define is_v2_DINVALL_l2_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e9000)
#define is_v2_DINVALL_l2_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1eb000)
#define is_v2_DINVALL_l2_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1ed000)
#define is_v2_DINVALL_l2_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1ec000)
#define is_v2_DINVALL_l2_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1ea000)
#define is_v2_DINVALL_l2_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d1e8000)
#define is_v2_DINVALL__deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e1000)
#define is_v2_DINVALL__dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e3000)
#define is_v2_DINVALL__dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e5000)
#define is_v2_DINVALL__dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e4000)
#define is_v2_DINVALL__dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e2000)
#define is_v2_DINVALL__dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e0000)
#define is_v2_DINVALL__even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e7000)
#define is_v2_DINVALL__odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e6000)
#define is_v2_DINVALL__weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e9000)
#define is_v2_DINVALL__wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1eb000)
#define is_v2_DINVALL__wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1ed000)
#define is_v2_DINVALL__wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1ec000)
#define is_v2_DINVALL__wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1ea000)
#define is_v2_DINVALL__wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c1e8000)
#define v2_DINVALL_l1l2_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc1e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_DINVALL_l1l2_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e1000)
#define is_v2_DINVALL_l1l2_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e3000)
#define is_v2_DINVALL_l1l2_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e5000)
#define is_v2_DINVALL_l1l2_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e4000)
#define is_v2_DINVALL_l1l2_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e2000)
#define is_v2_DINVALL_l1l2_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e0000)
#define is_v2_DINVALL_l1l2_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e7000)
#define is_v2_DINVALL_l1l2_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e6000)
#define is_v2_DINVALL_l1l2_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e9000)
#define is_v2_DINVALL_l1l2_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1eb000)
#define is_v2_DINVALL_l1l2_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1ed000)
#define is_v2_DINVALL_l1l2_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1ec000)
#define is_v2_DINVALL_l1l2_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1ea000)
#define is_v2_DINVALL_l1l2_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e8000)
#define is_v2_DINVALL_l2_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e1000)
#define is_v2_DINVALL_l2_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e3000)
#define is_v2_DINVALL_l2_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e5000)
#define is_v2_DINVALL_l2_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e4000)
#define is_v2_DINVALL_l2_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e2000)
#define is_v2_DINVALL_l2_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e0000)
#define is_v2_DINVALL_l2_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e7000)
#define is_v2_DINVALL_l2_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e6000)
#define is_v2_DINVALL_l2_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e9000)
#define is_v2_DINVALL_l2_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1eb000)
#define is_v2_DINVALL_l2_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1ed000)
#define is_v2_DINVALL_l2_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1ec000)
#define is_v2_DINVALL_l2_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1ea000)
#define is_v2_DINVALL_l2_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e8000)
#define is_v2_DINVALL__deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e1000)
#define is_v2_DINVALL__dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e3000)
#define is_v2_DINVALL__dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e5000)
#define is_v2_DINVALL__dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e4000)
#define is_v2_DINVALL__dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e2000)
#define is_v2_DINVALL__dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e0000)
#define is_v2_DINVALL__even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e7000)
#define is_v2_DINVALL__odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e6000)
#define is_v2_DINVALL__weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e9000)
#define is_v2_DINVALL__wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1eb000)
#define is_v2_DINVALL__wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1ed000)
#define is_v2_DINVALL__wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1ec000)
#define is_v2_DINVALL__wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1ea000)
#define is_v2_DINVALL__wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e8000)
#define v2_DINVALL_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe1e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd1e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_DINVALL__wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc1e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_DINVALL__wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_DINVALL_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e1000)
#define is_v2_DINVALL_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e3000)
#define is_v2_DINVALL_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e5000)
#define is_v2_DINVALL_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e4000)
#define is_v2_DINVALL_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e2000)
#define is_v2_DINVALL_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e0000)
#define is_v2_DINVALL_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e7000)
#define is_v2_DINVALL_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e6000)
#define is_v2_DINVALL_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e9000)
#define is_v2_DINVALL_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1eb000)
#define is_v2_DINVALL_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1ed000)
#define is_v2_DINVALL_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1ec000)
#define is_v2_DINVALL_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1ea000)
#define is_v2_DINVALL_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe1e8000)
#define is_v2_DINVALL_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e1000)
#define is_v2_DINVALL_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e3000)
#define is_v2_DINVALL_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e5000)
#define is_v2_DINVALL_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e4000)
#define is_v2_DINVALL_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e2000)
#define is_v2_DINVALL_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e0000)
#define is_v2_DINVALL_l2_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e7000)
#define is_v2_DINVALL_l2_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e6000)
#define is_v2_DINVALL_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e9000)
#define is_v2_DINVALL_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1eb000)
#define is_v2_DINVALL_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1ed000)
#define is_v2_DINVALL_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1ec000)
#define is_v2_DINVALL_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1ea000)
#define is_v2_DINVALL_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd1e8000)
#define is_v2_DINVALL__deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e1000)
#define is_v2_DINVALL__dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e3000)
#define is_v2_DINVALL__dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e5000)
#define is_v2_DINVALL__dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e4000)
#define is_v2_DINVALL__dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e2000)
#define is_v2_DINVALL__dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e0000)
#define is_v2_DINVALL__even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e7000)
#define is_v2_DINVALL__odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e6000)
#define is_v2_DINVALL__weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e9000)
#define is_v2_DINVALL__wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1eb000)
#define is_v2_DINVALL__wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1ed000)
#define is_v2_DINVALL__wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1ec000)
#define is_v2_DINVALL__wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1ea000)
#define is_v2_DINVALL__wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc1e8000)
#define v2_IINVALS__signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3c5c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3e5c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_signed10_registerZ_simple_insn(signed10,registerZ) (unsigned int)(0x3d5c0000 | (((int)(signed10) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_IINVALS__signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3c5c0000)
#define is_v2_IINVALS_l1l2_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3e5c0000)
#define is_v2_IINVALS_l2_signed10_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3d5c0000)
#define v2_IINVALS__upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbc5c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_IINVALS_l1l2_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbe5c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_IINVALS_l2_upper27_lower10_registerZ_double_insn_0(upper27_lower10,registerZ) (unsigned int)(0xbd5c0000 | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_upper27_lower10_registerZ_double_insn_1(upper27_lower10,registerZ) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_IINVALS__upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbc5c0000)
#define is_v2_IINVALS_l1l2_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbe5c0000)
#define is_v2_IINVALS_l2_upper27_lower10_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbd5c0000)
#define v2_IINVALS__extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbc5c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_IINVALS__extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_IINVALS_l1l2_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbe5c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_IINVALS_l1l2_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_IINVALS_l2_extend27_upper27_lower10_registerZ_triple_insn_0(extend27_upper27_lower10,registerZ) (unsigned int)(0xbd5c0000 | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_extend27_upper27_lower10_registerZ_triple_insn_1(extend27_upper27_lower10,registerZ) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_IINVALS_l2_extend27_upper27_lower10_registerZ_triple_insn_2(extend27_upper27_lower10,registerZ) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_IINVALS__extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbc5c0000)
#define is_v2_IINVALS_l1l2_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbe5c0000)
#define is_v2_IINVALS_l2_extend27_upper27_lower10_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xffff0000) == 0xbd5c0000)
#define v2_IINVALS__registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5ee000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_IINVALS__registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5ee000)
#define is_v2_IINVALS_l1l2_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5ee000)
#define is_v2_IINVALS_l2_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5ee000)
#define v2_IINVALS_l1l2_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3e5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3d5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__deqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__even_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__odd_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__weqz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wgez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wgtz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wlez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wltz_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wnez_registerY_registerZ_simple_insn(registerY,registerZ) (unsigned int)(0x3c5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(registerZ) & 0x3f))
#define is_v2_IINVALS_l1l2_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e1000)
#define is_v2_IINVALS_l1l2_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e3000)
#define is_v2_IINVALS_l1l2_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e5000)
#define is_v2_IINVALS_l1l2_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e4000)
#define is_v2_IINVALS_l1l2_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e2000)
#define is_v2_IINVALS_l1l2_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e0000)
#define is_v2_IINVALS_l1l2_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e7000)
#define is_v2_IINVALS_l1l2_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e6000)
#define is_v2_IINVALS_l1l2_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e9000)
#define is_v2_IINVALS_l1l2_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5eb000)
#define is_v2_IINVALS_l1l2_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5ed000)
#define is_v2_IINVALS_l1l2_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5ec000)
#define is_v2_IINVALS_l1l2_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5ea000)
#define is_v2_IINVALS_l1l2_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3e5e8000)
#define is_v2_IINVALS_l2_deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e1000)
#define is_v2_IINVALS_l2_dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e3000)
#define is_v2_IINVALS_l2_dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e5000)
#define is_v2_IINVALS_l2_dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e4000)
#define is_v2_IINVALS_l2_dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e2000)
#define is_v2_IINVALS_l2_dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e0000)
#define is_v2_IINVALS_l2_even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e7000)
#define is_v2_IINVALS_l2_odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e6000)
#define is_v2_IINVALS_l2_weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e9000)
#define is_v2_IINVALS_l2_wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5eb000)
#define is_v2_IINVALS_l2_wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5ed000)
#define is_v2_IINVALS_l2_wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5ec000)
#define is_v2_IINVALS_l2_wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5ea000)
#define is_v2_IINVALS_l2_wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3d5e8000)
#define is_v2_IINVALS__deqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e1000)
#define is_v2_IINVALS__dgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e3000)
#define is_v2_IINVALS__dgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e5000)
#define is_v2_IINVALS__dlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e4000)
#define is_v2_IINVALS__dltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e2000)
#define is_v2_IINVALS__dnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e0000)
#define is_v2_IINVALS__even_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e7000)
#define is_v2_IINVALS__odd_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e6000)
#define is_v2_IINVALS__weqz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e9000)
#define is_v2_IINVALS__wgez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5eb000)
#define is_v2_IINVALS__wgtz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5ed000)
#define is_v2_IINVALS__wlez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5ec000)
#define is_v2_IINVALS__wltz_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5ea000)
#define is_v2_IINVALS__wnez_registerY_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7ffff000) == 0x3c5e8000)
#define v2_IINVALS_l1l2_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbe5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbd5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__deqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e1000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e3000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e5000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e4000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e2000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e0000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__even_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e7000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__odd_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e6000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__weqz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e9000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wgez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5eb000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wgtz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5ed000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wlez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5ec000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wltz_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5ea000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wnez_registerY_offset27_registerZ_double_insn(registerY,offset27,registerZ) (unsigned int)(0xbc5e8000 | (((int)(registerY) & 0x3f) << 6) | ((int)(offset27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define is_v2_IINVALS_l1l2_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e1000)
#define is_v2_IINVALS_l1l2_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e3000)
#define is_v2_IINVALS_l1l2_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e5000)
#define is_v2_IINVALS_l1l2_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e4000)
#define is_v2_IINVALS_l1l2_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e2000)
#define is_v2_IINVALS_l1l2_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e0000)
#define is_v2_IINVALS_l1l2_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e7000)
#define is_v2_IINVALS_l1l2_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e6000)
#define is_v2_IINVALS_l1l2_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e9000)
#define is_v2_IINVALS_l1l2_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5eb000)
#define is_v2_IINVALS_l1l2_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5ed000)
#define is_v2_IINVALS_l1l2_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5ec000)
#define is_v2_IINVALS_l1l2_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5ea000)
#define is_v2_IINVALS_l1l2_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e8000)
#define is_v2_IINVALS_l2_deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e1000)
#define is_v2_IINVALS_l2_dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e3000)
#define is_v2_IINVALS_l2_dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e5000)
#define is_v2_IINVALS_l2_dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e4000)
#define is_v2_IINVALS_l2_dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e2000)
#define is_v2_IINVALS_l2_dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e0000)
#define is_v2_IINVALS_l2_even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e7000)
#define is_v2_IINVALS_l2_odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e6000)
#define is_v2_IINVALS_l2_weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e9000)
#define is_v2_IINVALS_l2_wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5eb000)
#define is_v2_IINVALS_l2_wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5ed000)
#define is_v2_IINVALS_l2_wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5ec000)
#define is_v2_IINVALS_l2_wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5ea000)
#define is_v2_IINVALS_l2_wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e8000)
#define is_v2_IINVALS__deqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e1000)
#define is_v2_IINVALS__dgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e3000)
#define is_v2_IINVALS__dgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e5000)
#define is_v2_IINVALS__dlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e4000)
#define is_v2_IINVALS__dltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e2000)
#define is_v2_IINVALS__dnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e0000)
#define is_v2_IINVALS__even_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e7000)
#define is_v2_IINVALS__odd_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e6000)
#define is_v2_IINVALS__weqz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e9000)
#define is_v2_IINVALS__wgez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5eb000)
#define is_v2_IINVALS__wgtz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5ed000)
#define is_v2_IINVALS__wlez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5ec000)
#define is_v2_IINVALS__wltz_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5ea000)
#define is_v2_IINVALS__wnez_registerY_offset27_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e8000)
#define v2_IINVALS_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbe5e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbd5e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__deqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e1000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__deqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__dgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e3000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__dgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e5000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__dlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e4000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__dltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e2000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__dnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e0000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__dnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__even_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e7000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__even_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__odd_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e6000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__odd_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__weqz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e9000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__weqz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__wgez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5eb000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wgez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__wgtz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5ed000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wgtz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__wlez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5ec000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wlez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__wltz_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5ea000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wltz_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define v2_IINVALS__wnez_registerY_extend27_offset27_registerZ_triple_insn_0(registerY,extend27_offset27,registerZ) (unsigned int)(0xbc5e8000 | (((int)(registerY) & 0x3f) << 6) | (((int)(extend27_offset27) >> 27) & 0x7ffffff) | ((int)(registerZ) & 0x3f))
#define v2_IINVALS__wnez_registerY_extend27_offset27_registerZ_triple_insn_1(registerY,extend27_offset27,registerZ) (unsigned int)(((int)(extend27_offset27) & 0x7ffffff))
#define is_v2_IINVALS_l1l2_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e1000)
#define is_v2_IINVALS_l1l2_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e3000)
#define is_v2_IINVALS_l1l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e5000)
#define is_v2_IINVALS_l1l2_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e4000)
#define is_v2_IINVALS_l1l2_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e2000)
#define is_v2_IINVALS_l1l2_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e0000)
#define is_v2_IINVALS_l1l2_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e7000)
#define is_v2_IINVALS_l1l2_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e6000)
#define is_v2_IINVALS_l1l2_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e9000)
#define is_v2_IINVALS_l1l2_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5eb000)
#define is_v2_IINVALS_l1l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5ed000)
#define is_v2_IINVALS_l1l2_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5ec000)
#define is_v2_IINVALS_l1l2_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5ea000)
#define is_v2_IINVALS_l1l2_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbe5e8000)
#define is_v2_IINVALS_l2_deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e1000)
#define is_v2_IINVALS_l2_dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e3000)
#define is_v2_IINVALS_l2_dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e5000)
#define is_v2_IINVALS_l2_dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e4000)
#define is_v2_IINVALS_l2_dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e2000)
#define is_v2_IINVALS_l2_dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e0000)
#define is_v2_IINVALS_l2_even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e7000)
#define is_v2_IINVALS_l2_odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e6000)
#define is_v2_IINVALS_l2_weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e9000)
#define is_v2_IINVALS_l2_wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5eb000)
#define is_v2_IINVALS_l2_wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5ed000)
#define is_v2_IINVALS_l2_wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5ec000)
#define is_v2_IINVALS_l2_wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5ea000)
#define is_v2_IINVALS_l2_wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbd5e8000)
#define is_v2_IINVALS__deqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e1000)
#define is_v2_IINVALS__dgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e3000)
#define is_v2_IINVALS__dgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e5000)
#define is_v2_IINVALS__dlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e4000)
#define is_v2_IINVALS__dltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e2000)
#define is_v2_IINVALS__dnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e0000)
#define is_v2_IINVALS__even_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e7000)
#define is_v2_IINVALS__odd_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e6000)
#define is_v2_IINVALS__weqz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e9000)
#define is_v2_IINVALS__wgez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5eb000)
#define is_v2_IINVALS__wgtz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5ed000)
#define is_v2_IINVALS__wlez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5ec000)
#define is_v2_IINVALS__wltz_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5ea000)
#define is_v2_IINVALS__wnez_registerY_extend27_offset27_registerZ_triple_insn(x) (int)(((unsigned int)(x) & 0xfffff000) == 0xbc5e8000)
#define is_v2_DINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3f8d0000)
#define is_v2_IINVAL_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3f9d0000)
#define is_v2_FENCE_simple_insn(x) (int)(((unsigned int)(x) & 0x7fff0000) == 0x3fcd0000)
#define is_v2_NOP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f03f000)
#define v2_STSUD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_STSUD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70010000)
#define v2_ADDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71010000)
#define v2_ADDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x61000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ADDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x61000000)
#define v2_ADDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ADDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ADDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1000000)
#define v2_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1000000)
#define v2_SBFD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75010000)
#define v2_SBFD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x65000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_SBFD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x65000000)
#define v2_SBFD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_SBFD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SBFD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5000000)
#define v2_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SBFD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5000000)
#define v2_ABDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ABDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74010000)
#define v2_ABDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x64000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ABDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x64000000)
#define v2_ABDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ABDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ABDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4000000)
#define v2_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ABDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4000000)
#define v2_MIND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MIND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72010000)
#define v2_MIND_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x62000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MIND_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x62000000)
#define v2_MIND_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MIND_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MIND_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2000000)
#define v2_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MIND_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2000000)
#define v2_MINUD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MINUD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76010000)
#define v2_MINUD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x66000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MINUD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x66000000)
#define v2_MINUD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MINUD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MINUD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6000000)
#define v2_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MINUD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6000000)
#define v2_MAXD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73010000)
#define v2_MAXD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x63000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MAXD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x63000000)
#define v2_MAXD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe3000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MAXD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MAXD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3000000)
#define v2_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe3000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MAXD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3000000)
#define v2_MAXUD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXUD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77010000)
#define v2_MAXUD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x67000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MAXUD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x67000000)
#define v2_MAXUD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MAXUD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MAXUD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7000000)
#define v2_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MAXUD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7000000)
#define v2_ANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78010000)
#define v2_ANDD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ANDD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ANDD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8010800)
#define v2_ANDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x68000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ANDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x68000000)
#define v2_ANDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe8000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ANDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ANDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8000000)
#define v2_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe8000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8000000)
#define v2_NANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_NANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79010000)
#define v2_NANDD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_NANDD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_NANDD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9010800)
#define v2_NANDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x69000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_NANDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x69000000)
#define v2_NANDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe9000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_NANDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_NANDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9000000)
#define v2_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe9000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_NANDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9000000)
#define v2_ORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a010000)
#define v2_ORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa010800)
#define v2_ORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6a000000)
#define v2_ORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xea000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea000000)
#define v2_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xea000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea000000)
#define v2_NORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_NORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b010000)
#define v2_NORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_NORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_NORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb010800)
#define v2_NORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6b000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_NORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6b000000)
#define v2_NORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xeb000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_NORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_NORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb000000)
#define v2_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xeb000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_NORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb000000)
#define v2_XORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_XORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c010000)
#define v2_XORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_XORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_XORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc010800)
#define v2_XORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6c000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_XORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6c000000)
#define v2_XORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xec000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_XORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_XORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec000000)
#define v2_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xec000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_XORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec000000)
#define v2_NXORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_NXORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d010000)
#define v2_NXORD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_NXORD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_NXORD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd010800)
#define v2_NXORD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6d000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_NXORD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6d000000)
#define v2_NXORD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xed000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_NXORD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_NXORD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed000000)
#define v2_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xed000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_NXORD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed000000)
#define v2_ANDND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ANDND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e010000)
#define v2_ANDND_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ANDND_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ANDND_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe010800)
#define v2_ANDND_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6e000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ANDND_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6e000000)
#define v2_ANDND_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xee000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ANDND_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ANDND_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee000000)
#define v2_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xee000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ANDND_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee000000)
#define v2_ORND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ORND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f010000)
#define v2_ORND_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff010800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ORND_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ORND_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff010800)
#define v2_ORND_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6f000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ORND_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6f000000)
#define v2_ORND_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xef000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ORND_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ORND_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef000000)
#define v2_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xef000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ORND_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef000000)
#define v2_SBMM8_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBMM8_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e020000)
#define v2_SBMM8_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe020800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBMM8_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBMM8_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe020800)
#define v2_SBMM8_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6e020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_SBMM8_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6e020000)
#define v2_SBMM8_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xee020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_SBMM8_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SBMM8_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee020000)
#define v2_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xee020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SBMM8_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee020000)
#define v2_SBMMT8_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBMMT8_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f020000)
#define v2_SBMMT8_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff020800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBMMT8_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBMMT8_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff020800)
#define v2_SBMMT8_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6f020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_SBMMT8_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6f020000)
#define v2_SBMMT8_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xef020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_SBMMT8_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SBMMT8_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef020000)
#define v2_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xef020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SBMMT8_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef020000)
#define v2_ADDSD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6e010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ADDSD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6e010000)
#define v2_ADDSD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xee010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ADDSD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ADDSD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee010000)
#define v2_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xee010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_ADDSD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xee010000)
#define v2_ADDSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01a000)
#define v2_SBFSD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6f010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_SBFSD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6f010000)
#define v2_SBFSD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xef010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_SBFSD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SBFSD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef010000)
#define v2_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xef010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_SBFSD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xef010000)
#define v2_SBFSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01a000)
#define v2_ADDX2D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX2D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70014000)
#define v2_ADDX2D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX2D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX2D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0014000)
#define is_v2_ADDX2D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0014800)
#define v2_ADDX4D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX4D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72014000)
#define v2_ADDX4D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX4D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX4D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2014000)
#define is_v2_ADDX4D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2014800)
#define v2_ADDX8D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX8D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74014000)
#define v2_ADDX8D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX8D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX8D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4014000)
#define is_v2_ADDX8D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4014800)
#define v2_ADDX16D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX16D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76014000)
#define v2_ADDX16D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX16D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX16D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6014000)
#define is_v2_ADDX16D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6014800)
#define v2_SBFX2D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX2D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71014000)
#define v2_SBFX2D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX2D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX2D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1014000)
#define is_v2_SBFX2D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1014800)
#define v2_SBFX4D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX4D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73014000)
#define v2_SBFX4D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX4D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX4D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3014000)
#define is_v2_SBFX4D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3014800)
#define v2_SBFX8D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX8D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75014000)
#define v2_SBFX8D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX8D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX8D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5014000)
#define is_v2_SBFX8D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5014800)
#define v2_SBFX16D_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX16D_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77014000)
#define v2_SBFX16D__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16D__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX16D___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16D___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX16D__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7014000)
#define is_v2_SBFX16D___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7014800)
#define v2_ADDWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78014000)
#define v2_ADDWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8014000)
#define is_v2_ADDWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8014800)
#define v2_SBFWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79014000)
#define v2_SBFWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9014000)
#define is_v2_SBFWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9014800)
#define v2_ADDUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a014000)
#define v2_ADDUWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDUWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDUWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDUWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDUWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa014000)
#define is_v2_ADDUWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa014800)
#define v2_SBFUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b014000)
#define v2_SBFUWD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFUWD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFUWD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFUWD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFUWD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb014000)
#define is_v2_SBFUWD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb014800)
#define v2_LANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c014000)
#define v2_LANDD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LANDD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LANDD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LANDD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LANDD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc014000)
#define is_v2_LANDD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc014800)
#define v2_LNANDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNANDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d014000)
#define v2_LNANDD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNANDD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LNANDD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNANDD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNANDD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd014000)
#define is_v2_LNANDD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd014800)
#define v2_LORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e014000)
#define v2_LORD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LORD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LORD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LORD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LORD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe014000)
#define is_v2_LORD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe014800)
#define v2_LNORD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNORD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f014000)
#define v2_LNORD__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNORD__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LNORD___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff014800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNORD___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNORD__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff014000)
#define is_v2_LNORD___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff014800)
#define v2_STSUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_STSUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70011000)
#define v2_ADDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x71000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ADDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x71000000)
#define v2_ADDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ADDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ADDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf1000000)
#define v2_ADDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71011000)
#define v2_SBFW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x75000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_SBFW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x75000000)
#define v2_SBFW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_SBFW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_SBFW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf5000000)
#define v2_SBFW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75011000)
#define v2_ABDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x74000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ABDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x74000000)
#define v2_ABDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ABDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ABDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf4000000)
#define v2_ABDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ABDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74011000)
#define v2_MINW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x72000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MINW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x72000000)
#define v2_MINW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MINW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MINW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf2000000)
#define v2_MINW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MINW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72011000)
#define v2_MINUW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x76000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MINUW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x76000000)
#define v2_MINUW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MINUW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MINUW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf6000000)
#define v2_MINUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MINUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76011000)
#define v2_MAXW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x73000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MAXW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x73000000)
#define v2_MAXW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf3000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MAXW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MAXW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf3000000)
#define v2_MAXW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73011000)
#define v2_MAXUW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x77000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MAXUW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x77000000)
#define v2_MAXUW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MAXUW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MAXUW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf7000000)
#define v2_MAXUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77011000)
#define v2_ANDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x78000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ANDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x78000000)
#define v2_ANDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf8000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ANDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ANDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf8000000)
#define v2_ANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78011000)
#define v2_NANDW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x79000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_NANDW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x79000000)
#define v2_NANDW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xf9000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_NANDW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_NANDW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf9000000)
#define v2_NANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_NANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79011000)
#define v2_ORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7a000000)
#define v2_ORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfa000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfa000000)
#define v2_ORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a011000)
#define v2_NORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7b000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_NORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7b000000)
#define v2_NORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfb000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_NORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_NORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfb000000)
#define v2_NORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_NORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b011000)
#define v2_XORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7c000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_XORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7c000000)
#define v2_XORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfc000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_XORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_XORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfc000000)
#define v2_XORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_XORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c011000)
#define v2_NXORW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7d000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_NXORW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7d000000)
#define v2_NXORW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfd000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_NXORW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_NXORW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfd000000)
#define v2_NXORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_NXORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d011000)
#define v2_ANDNW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7e000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ANDNW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7e000000)
#define v2_ANDNW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xfe000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ANDNW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ANDNW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xfe000000)
#define v2_ANDNW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ANDNW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e011000)
#define v2_ORNW_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x7f000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_ORNW_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x7f000000)
#define v2_ORNW_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xff000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_ORNW_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_ORNW_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xff000000)
#define v2_ORNW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ORNW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f011000)
#define v2_ADDSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01b000)
#define v2_ADDSW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDSW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDSW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01b000)
#define v2_SBFSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01b000)
#define v2_SBFSW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFSW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFSW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01b000)
#define v2_ADDX2W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX2W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70015000)
#define v2_ADDX2W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX2W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0015000)
#define v2_ADDX4W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX4W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72015000)
#define v2_ADDX4W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX4W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2015000)
#define v2_ADDX8W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX8W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74015000)
#define v2_ADDX8W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX8W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4015000)
#define v2_ADDX16W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX16W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76015000)
#define v2_ADDX16W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX16W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6015000)
#define v2_SBFX2W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX2W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71015000)
#define v2_SBFX2W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX2W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1015000)
#define v2_SBFX4W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX4W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73015000)
#define v2_SBFX4W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX4W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3015000)
#define v2_SBFX8W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX8W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75015000)
#define v2_SBFX8W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX8W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5015000)
#define v2_SBFX16W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX16W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77015000)
#define v2_SBFX16W_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16W_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX16W_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7015000)
#define v2_AVGW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78015000)
#define v2_AVGW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8015000)
#define v2_AVGUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79015000)
#define v2_AVGUW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGUW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGUW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9015000)
#define v2_AVGRW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGRW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a015000)
#define v2_AVGRW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGRW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa015000)
#define v2_AVGRUW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGRUW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b015000)
#define v2_AVGRUW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRUW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGRUW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb015000)
#define v2_LANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c015000)
#define v2_LANDW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LANDW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LANDW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc015000)
#define v2_LNANDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNANDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d015000)
#define v2_LNANDW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNANDW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNANDW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd015000)
#define v2_LORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e015000)
#define v2_LORW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LORW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LORW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe015000)
#define v2_LNORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f015000)
#define v2_LNORW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNORW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNORW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff015000)
#define v2_ADDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71012000)
#define v2_ADDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1012000)
#define is_v2_ADDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1012800)
#define v2_SBFWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75012000)
#define v2_SBFWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5012000)
#define is_v2_SBFWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5012800)
#define v2_ABDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ABDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74012000)
#define v2_ABDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ABDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ABDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ABDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ABDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4012000)
#define is_v2_ABDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4012800)
#define v2_MINWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MINWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72012000)
#define v2_MINWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MINWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MINWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2012000)
#define is_v2_MINWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2012800)
#define v2_MINUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MINUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76012000)
#define v2_MINUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MINUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MINUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6012000)
#define is_v2_MINUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6012800)
#define v2_MAXWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73012000)
#define v2_MAXWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MAXWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MAXWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3012000)
#define is_v2_MAXWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3012800)
#define v2_MAXUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77012000)
#define v2_MAXUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MAXUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MAXUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7012000)
#define is_v2_MAXUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7012800)
#define v2_ADDCWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDCWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c012000)
#define v2_ADDCWC__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDCWC__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDCWC___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDCWC___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDCWC__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc012000)
#define is_v2_ADDCWC___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc012800)
#define v2_SBFCWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFCWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d012000)
#define v2_SBFCWC__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFCWC__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFCWC___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd012800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFCWC___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFCWC__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd012000)
#define is_v2_SBFCWC___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd012800)
#define v2_ADDSWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDSWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01e000)
#define v2_ADDSWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDSWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDSWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01e800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDSWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDSWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01e000)
#define is_v2_ADDSWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01e800)
#define v2_SBFSWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFSWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01e000)
#define v2_SBFSWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFSWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFSWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01e800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFSWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFSWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01e000)
#define is_v2_SBFSWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01e800)
#define v2_ADDX2WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX2WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70016000)
#define v2_ADDX2WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX2WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX2WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0016000)
#define is_v2_ADDX2WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0016800)
#define v2_ADDX4WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX4WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72016000)
#define v2_ADDX4WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX4WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX4WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2016000)
#define is_v2_ADDX4WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2016800)
#define v2_ADDX8WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX8WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74016000)
#define v2_ADDX8WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX8WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX8WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4016000)
#define is_v2_ADDX8WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4016800)
#define v2_ADDX16WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX16WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76016000)
#define v2_ADDX16WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX16WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX16WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6016000)
#define is_v2_ADDX16WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6016800)
#define v2_SBFX2WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX2WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71016000)
#define v2_SBFX2WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX2WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX2WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1016000)
#define is_v2_SBFX2WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1016800)
#define v2_SBFX4WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX4WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73016000)
#define v2_SBFX4WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX4WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX4WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3016000)
#define is_v2_SBFX4WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3016800)
#define v2_SBFX8WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX8WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75016000)
#define v2_SBFX8WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX8WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX8WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5016000)
#define is_v2_SBFX8WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5016800)
#define v2_SBFX16WP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX16WP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77016000)
#define v2_SBFX16WP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16WP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX16WP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16WP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX16WP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7016000)
#define is_v2_SBFX16WP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7016800)
#define v2_AVGWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78016000)
#define v2_AVGWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8016000)
#define is_v2_AVGWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8016800)
#define v2_AVGUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79016000)
#define v2_AVGUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9016000)
#define is_v2_AVGUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9016800)
#define v2_AVGRWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGRWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a016000)
#define v2_AVGRWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGRWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGRWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa016000)
#define is_v2_AVGRWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa016800)
#define v2_AVGRUWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGRUWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b016000)
#define v2_AVGRUWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRUWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGRUWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRUWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGRUWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb016000)
#define is_v2_AVGRUWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb016800)
#define v2_LANDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LANDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c016000)
#define v2_LANDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LANDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LANDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LANDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LANDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc016000)
#define is_v2_LANDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc016800)
#define v2_LNANDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNANDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d016000)
#define v2_LNANDWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNANDWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LNANDWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNANDWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNANDWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd016000)
#define is_v2_LNANDWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd016800)
#define v2_LORWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LORWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e016000)
#define v2_LORWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LORWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LORWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LORWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LORWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe016000)
#define is_v2_LORWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe016800)
#define v2_LNORWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNORWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f016000)
#define v2_LNORWP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNORWP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LNORWP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff016800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNORWP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNORWP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff016000)
#define is_v2_LNORWP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff016800)
#define v2_ADDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71013000)
#define v2_ADDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1013000)
#define is_v2_ADDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1013800)
#define v2_SBFHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75013000)
#define v2_SBFHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5013000)
#define is_v2_SBFHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5013800)
#define v2_ABDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ABDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74013000)
#define v2_ABDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ABDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ABDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ABDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ABDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4013000)
#define is_v2_ABDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4013800)
#define v2_MINHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MINHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72013000)
#define v2_MINHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MINHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MINHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2013000)
#define is_v2_MINHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2013800)
#define v2_MINUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MINUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76013000)
#define v2_MINUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MINUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MINUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MINUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6013000)
#define is_v2_MINUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6013800)
#define v2_MAXHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73013000)
#define v2_MAXHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MAXHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MAXHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3013000)
#define is_v2_MAXHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3013800)
#define v2_MAXUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MAXUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77013000)
#define v2_MAXUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_MAXUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MAXUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MAXUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7013000)
#define is_v2_MAXUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7013800)
#define v2_ADDCHCP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDCHCP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c013000)
#define v2_ADDCHCP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDCHCP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDCHCP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDCHCP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDCHCP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc013000)
#define is_v2_ADDCHCP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc013800)
#define v2_SBFCHCP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFCHCP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d013000)
#define v2_SBFCHCP__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFCHCP__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFCHCP___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd013800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFCHCP___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFCHCP__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd013000)
#define is_v2_SBFCHCP___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd013800)
#define v2_ADDSHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDSHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01f000)
#define v2_ADDSHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDSHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDSHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01f800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDSHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDSHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01f000)
#define is_v2_ADDSHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01f800)
#define v2_SBFSHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFSHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01f000)
#define v2_SBFSHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFSHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFSHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01f800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFSHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFSHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01f000)
#define is_v2_SBFSHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01f800)
#define v2_ADDX2HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX2HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70017000)
#define v2_ADDX2HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX2HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX2HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0017000)
#define is_v2_ADDX2HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0017800)
#define v2_ADDX4HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX4HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72017000)
#define v2_ADDX4HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX4HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX4HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2017000)
#define is_v2_ADDX4HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2017800)
#define v2_ADDX8HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX8HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74017000)
#define v2_ADDX8HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX8HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX8HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4017000)
#define is_v2_ADDX8HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4017800)
#define v2_ADDX16HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX16HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76017000)
#define v2_ADDX16HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_ADDX16HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX16HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6017000)
#define is_v2_ADDX16HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6017800)
#define v2_SBFX2HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX2HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71017000)
#define v2_SBFX2HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX2HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX2HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1017000)
#define is_v2_SBFX2HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1017800)
#define v2_SBFX4HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX4HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73017000)
#define v2_SBFX4HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX4HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX4HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3017000)
#define is_v2_SBFX4HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3017800)
#define v2_SBFX8HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX8HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75017000)
#define v2_SBFX8HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX8HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX8HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5017000)
#define is_v2_SBFX8HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5017800)
#define v2_SBFX16HQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX16HQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77017000)
#define v2_SBFX16HQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16HQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_SBFX16HQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16HQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX16HQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7017000)
#define is_v2_SBFX16HQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7017800)
#define v2_AVGHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78017000)
#define v2_AVGHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8017000)
#define is_v2_AVGHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8017800)
#define v2_AVGUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79017000)
#define v2_AVGUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9017000)
#define is_v2_AVGUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9017800)
#define v2_AVGRHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGRHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a017000)
#define v2_AVGRHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGRHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGRHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa017000)
#define is_v2_AVGRHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa017800)
#define v2_AVGRUHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_AVGRUHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b017000)
#define v2_AVGRUHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRUHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_AVGRUHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_AVGRUHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_AVGRUHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb017000)
#define is_v2_AVGRUHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb017800)
#define v2_LANDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LANDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c017000)
#define v2_LANDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LANDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LANDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LANDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LANDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc017000)
#define is_v2_LANDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc017800)
#define v2_LNANDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNANDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d017000)
#define v2_LNANDHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNANDHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LNANDHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNANDHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNANDHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd017000)
#define is_v2_LNANDHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd017800)
#define v2_LORHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LORHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e017000)
#define v2_LORHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LORHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LORHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LORHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LORHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe017000)
#define is_v2_LORHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe017800)
#define v2_LNORHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_LNORHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f017000)
#define v2_LNORHQ__registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNORHQ__registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_LNORHQ___registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff017800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_LNORHQ___registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_LNORHQ__registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff017000)
#define is_v2_LNORHQ___registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff017800)
#define v2_ADDX2WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX2WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70018000)
#define v2_ADDX2WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX2WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0018000)
#define v2_ADDX4WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX4WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72018000)
#define v2_ADDX4WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX4WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2018000)
#define v2_ADDX8WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX8WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74018000)
#define v2_ADDX8WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX8WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4018000)
#define v2_ADDX16WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX16WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76018000)
#define v2_ADDX16WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX16WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6018000)
#define v2_SBFX2WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX2WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71018000)
#define v2_SBFX2WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX2WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1018000)
#define v2_SBFX4WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX4WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73018000)
#define v2_SBFX4WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX4WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3018000)
#define v2_SBFX8WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX8WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75018000)
#define v2_SBFX8WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX8WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5018000)
#define v2_SBFX16WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX16WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77018000)
#define v2_SBFX16WD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16WD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX16WD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7018000)
#define v2_ADDX2UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX2UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78018000)
#define v2_ADDX2UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX2UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX2UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8018000)
#define v2_ADDX4UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX4UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a018000)
#define v2_ADDX4UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX4UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX4UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa018000)
#define v2_ADDX8UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX8UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c018000)
#define v2_ADDX8UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX8UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX8UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc018000)
#define v2_ADDX16UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDX16UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e018000)
#define v2_ADDX16UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDX16UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDX16UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe018000)
#define v2_SBFX2UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX2UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79018000)
#define v2_SBFX2UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX2UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX2UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9018000)
#define v2_SBFX4UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX4UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b018000)
#define v2_SBFX4UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX4UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX4UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb018000)
#define v2_SBFX8UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX8UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d018000)
#define v2_SBFX8UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX8UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX8UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd018000)
#define v2_SBFX16UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFX16UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f018000)
#define v2_SBFX16UWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFX16UWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFX16UWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff018000)
#define v2_ADDCD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDCD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01c000)
#define v2_ADDCD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDCD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDCD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01c000)
#define v2_SBFCD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFCD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01c000)
#define v2_SBFCD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFCD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFCD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01c000)
#define v2_ADDCD_I_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ADDCD_I_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e01d000)
#define v2_ADDCD_I_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_ADDCD_I_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_ADDCD_I_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe01d000)
#define v2_SBFCD_I_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SBFCD_I_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f01d000)
#define v2_SBFCD_I_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_SBFCD_I_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_SBFCD_I_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff01d000)
#define v2_FNEGD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FNEGD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70032000)
#define v2_FABSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FABSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71032000)
#define v2_FSRECD__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FSRECD_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70034800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FSRECD__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70034000)
#define is_v2_FSRECD_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70034800)
#define v2_FSRSRD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FSRSRD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78032000)
#define v2_FWIDENLWD__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FWIDENLWD_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FWIDENLWD__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x78033000)
#define is_v2_FWIDENLWD_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x78033800)
#define v2_FWIDENMWD__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x79033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FWIDENMWD_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x79033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FWIDENMWD__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x79033000)
#define is_v2_FWIDENMWD_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x79033800)
#define v2_FNARROWDW_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWDW__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FNARROWDW_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036200)
#define is_v2_FNARROWDW_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036a00)
#define is_v2_FNARROWDW_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036400)
#define is_v2_FNARROWDW_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036c00)
#define is_v2_FNARROWDW_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036500)
#define is_v2_FNARROWDW_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036d00)
#define is_v2_FNARROWDW_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036000)
#define is_v2_FNARROWDW_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036800)
#define is_v2_FNARROWDW_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036600)
#define is_v2_FNARROWDW_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036e00)
#define is_v2_FNARROWDW_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036100)
#define is_v2_FNARROWDW_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036900)
#define is_v2_FNARROWDW_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036300)
#define is_v2_FNARROWDW_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036b00)
#define is_v2_FNARROWDW___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036700)
#define is_v2_FNARROWDW__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x78036f00)
#define v2_FNEGW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FNEGW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72032000)
#define v2_FABSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FABSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73032000)
#define v2_FRECW_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRECW__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FRECW_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036200)
#define is_v2_FRECW_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036a00)
#define is_v2_FRECW_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036400)
#define is_v2_FRECW_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036c00)
#define is_v2_FRECW_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036500)
#define is_v2_FRECW_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036d00)
#define is_v2_FRECW_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036000)
#define is_v2_FRECW_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036800)
#define is_v2_FRECW_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036600)
#define is_v2_FRECW_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036e00)
#define is_v2_FRECW_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036100)
#define is_v2_FRECW_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036900)
#define is_v2_FRECW_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036300)
#define is_v2_FRECW_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036b00)
#define is_v2_FRECW___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036700)
#define is_v2_FRECW__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x72036f00)
#define v2_FRSRW_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FRSRW__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FRSRW_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036200)
#define is_v2_FRSRW_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036a00)
#define is_v2_FRSRW_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036400)
#define is_v2_FRSRW_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036c00)
#define is_v2_FRSRW_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036500)
#define is_v2_FRSRW_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036d00)
#define is_v2_FRSRW_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036000)
#define is_v2_FRSRW_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036800)
#define is_v2_FRSRW_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036600)
#define is_v2_FRSRW_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036e00)
#define is_v2_FRSRW_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036100)
#define is_v2_FRSRW_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036900)
#define is_v2_FRSRW_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036300)
#define is_v2_FRSRW_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036b00)
#define is_v2_FRSRW___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036700)
#define is_v2_FRSRW__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x73036f00)
#define v2_FSRECW__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FSRECW_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72034800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FSRECW__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72034000)
#define is_v2_FSRECW_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72034800)
#define v2_FSRSRW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FSRSRW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a032000)
#define v2_FWIDENLHW__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FWIDENLHW_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FWIDENLHW__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7a033000)
#define is_v2_FWIDENLHW_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7a033800)
#define v2_FWIDENMHW__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7b033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FWIDENMHW_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7b033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FWIDENMHW__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7b033000)
#define is_v2_FWIDENMHW_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7b033800)
#define v2_FNARROWWH_rd__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036200 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rd_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036a00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rna__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rna_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036c00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rnz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036500 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rnz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036d00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rn__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rn_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_ro__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036600 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_ro_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036e00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_ru__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036100 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_ru_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036900 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rz__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036300 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH_rz_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036b00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH___registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036700 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FNARROWWH__s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a036f00 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FNARROWWH_rd__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036200)
#define is_v2_FNARROWWH_rd_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036a00)
#define is_v2_FNARROWWH_rna__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036400)
#define is_v2_FNARROWWH_rna_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036c00)
#define is_v2_FNARROWWH_rnz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036500)
#define is_v2_FNARROWWH_rnz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036d00)
#define is_v2_FNARROWWH_rn__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036000)
#define is_v2_FNARROWWH_rn_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036800)
#define is_v2_FNARROWWH_ro__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036600)
#define is_v2_FNARROWWH_ro_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036e00)
#define is_v2_FNARROWWH_ru__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036100)
#define is_v2_FNARROWWH_ru_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036900)
#define is_v2_FNARROWWH_rz__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036300)
#define is_v2_FNARROWWH_rz_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036b00)
#define is_v2_FNARROWWH___registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036700)
#define is_v2_FNARROWWH__s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7a036f00)
#define v2_FNEGWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FNEGWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74032000)
#define v2_FABSWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x75032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FABSWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75032000)
#define v2_FSRECWP__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FSRECWP_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74034800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FSRECWP__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74034000)
#define is_v2_FSRECWP_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74034800)
#define v2_FSRSRWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FSRSRWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c032000)
#define v2_FWIDENLHWP__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FWIDENLHWP_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FWIDENLHWP__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7c033000)
#define is_v2_FWIDENLHWP_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7c033800)
#define v2_FWIDENMHWP__registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7d033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define v2_FWIDENMHWP_s_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7d033800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FWIDENMHWP__registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7d033000)
#define is_v2_FWIDENMHWP_s_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x7d033800)
#define v2_FNARROWDWP_rd__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036200 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rd_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036a00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rna__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036400 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rna_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036c00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rnz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036500 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rnz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036d00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rn__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rn_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_ro__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036600 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_ro_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036e00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_ru__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036100 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_ru_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036900 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036300 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP_rz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036b00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP___registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036700 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWDWP__s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7c036f00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FNARROWDWP_rd__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036200)
#define is_v2_FNARROWDWP_rd_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036a00)
#define is_v2_FNARROWDWP_rna__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036400)
#define is_v2_FNARROWDWP_rna_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036c00)
#define is_v2_FNARROWDWP_rnz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036500)
#define is_v2_FNARROWDWP_rnz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036d00)
#define is_v2_FNARROWDWP_rn__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036000)
#define is_v2_FNARROWDWP_rn_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036800)
#define is_v2_FNARROWDWP_ro__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036600)
#define is_v2_FNARROWDWP_ro_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036e00)
#define is_v2_FNARROWDWP_ru__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036100)
#define is_v2_FNARROWDWP_ru_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036900)
#define is_v2_FNARROWDWP_rz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036300)
#define is_v2_FNARROWDWP_rz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036b00)
#define is_v2_FNARROWDWP___registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036700)
#define is_v2_FNARROWDWP__s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7c036f00)
#define v2_FNEGHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x76032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FNEGHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76032000)
#define v2_FABSHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x77032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_FABSHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77032000)
#define v2_FNARROWWHQ_rd__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036200 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rd_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036a00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rna__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036400 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rna_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036c00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rnz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036500 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rnz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036d00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rn__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rn_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_ro__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036600 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_ro_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036e00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_ru__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036100 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_ru_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036900 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rz__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036300 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ_rz_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036b00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ___registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036700 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FNARROWWHQ__s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x7e036f00 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FNARROWWHQ_rd__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036200)
#define is_v2_FNARROWWHQ_rd_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036a00)
#define is_v2_FNARROWWHQ_rna__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036400)
#define is_v2_FNARROWWHQ_rna_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036c00)
#define is_v2_FNARROWWHQ_rnz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036500)
#define is_v2_FNARROWWHQ_rnz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036d00)
#define is_v2_FNARROWWHQ_rn__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036000)
#define is_v2_FNARROWWHQ_rn_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036800)
#define is_v2_FNARROWWHQ_ro__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036600)
#define is_v2_FNARROWWHQ_ro_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036e00)
#define is_v2_FNARROWWHQ_ru__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036100)
#define is_v2_FNARROWWHQ_ru_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036900)
#define is_v2_FNARROWWHQ_rz__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036300)
#define is_v2_FNARROWWHQ_rz_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036b00)
#define is_v2_FNARROWWHQ___registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036700)
#define is_v2_FNARROWWHQ__s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ff00) == 0x7e036f00)
#define v2_INSF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x60030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v2_INSF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v2_INSF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x60030000)
#define v2_EXTFZ_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x64030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v2_EXTFZ_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v2_EXTFZ_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x64030000)
#define v2_EXTFS_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x68030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v2_EXTFS_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v2_EXTFS_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x68030000)
#define v2_CLRF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_0(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)(0x6c030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(stopbit2_stopbit4) >> 2) & 0xf) << 12) | (((int)(startbit) & 0x3f) << 6))
#define v2_CLRF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn_1(registerW,registerZ,stopbit2_stopbit4,startbit) (unsigned int)((((int)(stopbit2_stopbit4) & 0x3) << 24))
#define is_v2_CLRF_registerW_registerZ_stopbit2_stopbit4_startbit_simple_insn(x) (int)(((unsigned int)(x) & 0x7c030000) == 0x6c030000)
#define v2_SATD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SATD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e026000)
#define v2_SATD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7e022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SATD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e022000)
#define v2_SATUD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SATUD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f026000)
#define v2_SATUD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7f022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SATUD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f022000)
#define v2_SRSD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRSD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78022000)
#define v2_SRSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78026000)
#define v2_SLLD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLLD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79022000)
#define v2_SLLD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLLD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79026000)
#define v2_SRAD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRAD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a022000)
#define v2_SRAD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRAD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a026000)
#define v2_SRLD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRLD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b022000)
#define v2_SRLD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRLD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b026000)
#define v2_SLSD_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLSD_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c022000)
#define v2_SLSD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLSD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c026000)
#define v2_SRSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78027000)
#define v2_SRSW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRSW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78023000)
#define v2_SLLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79027000)
#define v2_SLLW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLLW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79023000)
#define v2_SRAW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRAW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a027000)
#define v2_SRAW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRAW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a023000)
#define v2_SRLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b027000)
#define v2_SRLW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRLW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b023000)
#define v2_SLSW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLSW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c027000)
#define v2_SLSW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLSW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c023000)
#define v2_ROLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ROLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e027000)
#define v2_ROLW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7e023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_ROLW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e023000)
#define v2_RORW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_RORW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f027000)
#define v2_RORW_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7f023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_RORW_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f023000)
#define v2_SRSWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRSWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78024000)
#define v2_SRSWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRSWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78028000)
#define v2_SLLWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLLWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79024000)
#define v2_SLLWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLLWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79028000)
#define v2_SRAWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRAWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a024000)
#define v2_SRAWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRAWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a028000)
#define v2_SRLWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRLWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b024000)
#define v2_SRLWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRLWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b028000)
#define v2_SLSWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLSWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c024000)
#define v2_SLSWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLSWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c028000)
#define v2_ROLWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7e024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_ROLWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e024000)
#define v2_ROLWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_ROLWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e028000)
#define v2_RORWPS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7f024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_RORWPS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f024000)
#define v2_RORWPS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_RORWPS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f028000)
#define v2_SRSHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x78025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRSHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78025000)
#define v2_SRSHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRSHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78029000)
#define v2_SLLHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x79025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLLHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79025000)
#define v2_SLLHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLLHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79029000)
#define v2_SRAHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7a025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRAHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a025000)
#define v2_SRAHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRAHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a029000)
#define v2_SRLHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7b025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SRLHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b025000)
#define v2_SRLHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SRLHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b029000)
#define v2_SLSHQS_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x7c025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_SLSHQS_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c025000)
#define v2_SLSHQS_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_SLSHQS_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c029000)
#define v2_COMPD_all_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_any_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6c010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_eq_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x61010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_ge_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x63010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_geu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x67010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_gt_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x65010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_gtu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x69010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_le_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x64010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_leu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x68010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_lt_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x62010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_ltu_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x66010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_nall_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_ne_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x60010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define v2_COMPD_none_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x6d010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_COMPD_all_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6a010000)
#define is_v2_COMPD_any_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6c010000)
#define is_v2_COMPD_eq_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x61010000)
#define is_v2_COMPD_ge_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x63010000)
#define is_v2_COMPD_geu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x67010000)
#define is_v2_COMPD_gt_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x65010000)
#define is_v2_COMPD_gtu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x69010000)
#define is_v2_COMPD_le_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x64010000)
#define is_v2_COMPD_leu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x68010000)
#define is_v2_COMPD_lt_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x62010000)
#define is_v2_COMPD_ltu_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x66010000)
#define is_v2_COMPD_nall_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6b010000)
#define is_v2_COMPD_ne_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x60010000)
#define is_v2_COMPD_none_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6d010000)
#define v2_COMPD_all_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xea010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_all_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_any_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xec010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_any_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_eq_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe1010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_eq_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_ge_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe3010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_ge_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_geu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe7010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_geu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_gt_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe5010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_gt_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_gtu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe9010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_gtu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_le_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe4010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_le_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_leu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe8010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_leu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_lt_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe2010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_lt_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_ltu_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe6010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_ltu_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_nall_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xeb010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_nall_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_ne_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xe0010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_ne_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_COMPD_none_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xed010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_COMPD_none_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_COMPD_all_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea010000)
#define is_v2_COMPD_any_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec010000)
#define is_v2_COMPD_eq_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1010000)
#define is_v2_COMPD_ge_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3010000)
#define is_v2_COMPD_geu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7010000)
#define is_v2_COMPD_gt_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5010000)
#define is_v2_COMPD_gtu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9010000)
#define is_v2_COMPD_le_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4010000)
#define is_v2_COMPD_leu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8010000)
#define is_v2_COMPD_lt_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2010000)
#define is_v2_COMPD_ltu_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6010000)
#define is_v2_COMPD_nall_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb010000)
#define is_v2_COMPD_ne_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0010000)
#define is_v2_COMPD_none_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed010000)
#define v2_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xea010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xec010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe1010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe3010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe7010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe5010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe9010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe4010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe8010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe2010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe6010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xeb010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xe0010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xed010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_COMPD_all_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea010000)
#define is_v2_COMPD_any_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec010000)
#define is_v2_COMPD_eq_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1010000)
#define is_v2_COMPD_ge_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3010000)
#define is_v2_COMPD_geu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7010000)
#define is_v2_COMPD_gt_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5010000)
#define is_v2_COMPD_gtu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9010000)
#define is_v2_COMPD_le_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4010000)
#define is_v2_COMPD_leu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8010000)
#define is_v2_COMPD_lt_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2010000)
#define is_v2_COMPD_ltu_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6010000)
#define is_v2_COMPD_nall_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb010000)
#define is_v2_COMPD_ne_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0010000)
#define is_v2_COMPD_none_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed010000)
#define v2_COMPD_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPD_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_COMPD_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01a000)
#define is_v2_COMPD_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01a000)
#define is_v2_COMPD_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101a000)
#define is_v2_COMPD_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301a000)
#define is_v2_COMPD_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701a000)
#define is_v2_COMPD_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501a000)
#define is_v2_COMPD_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901a000)
#define is_v2_COMPD_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401a000)
#define is_v2_COMPD_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801a000)
#define is_v2_COMPD_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201a000)
#define is_v2_COMPD_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601a000)
#define is_v2_COMPD_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01a000)
#define is_v2_COMPD_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001a000)
#define is_v2_COMPD_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01a000)
#define v2_COMPW_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPW_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_COMPW_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01b000)
#define is_v2_COMPW_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01b000)
#define is_v2_COMPW_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101b000)
#define is_v2_COMPW_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301b000)
#define is_v2_COMPW_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701b000)
#define is_v2_COMPW_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501b000)
#define is_v2_COMPW_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901b000)
#define is_v2_COMPW_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401b000)
#define is_v2_COMPW_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801b000)
#define is_v2_COMPW_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201b000)
#define is_v2_COMPW_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601b000)
#define is_v2_COMPW_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01b000)
#define is_v2_COMPW_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001b000)
#define is_v2_COMPW_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01b000)
#define v2_COMPW_all_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_all_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_any_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_any_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_eq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf101b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_eq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_ge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf301b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_ge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_geu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf701b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_geu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_gt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf501b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_gt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_gtu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf901b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_gtu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_le_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf401b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_le_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_leu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf801b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_leu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_lt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf201b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_lt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_ltu_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf601b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_ltu_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_nall_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_nall_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_ne_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf001b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_ne_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_COMPW_none_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_COMPW_none_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_COMPW_all_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa01b000)
#define is_v2_COMPW_any_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc01b000)
#define is_v2_COMPW_eq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf101b000)
#define is_v2_COMPW_ge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf301b000)
#define is_v2_COMPW_geu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf701b000)
#define is_v2_COMPW_gt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf501b000)
#define is_v2_COMPW_gtu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf901b000)
#define is_v2_COMPW_le_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf401b000)
#define is_v2_COMPW_leu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf801b000)
#define is_v2_COMPW_lt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf201b000)
#define is_v2_COMPW_ltu_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf601b000)
#define is_v2_COMPW_nall_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb01b000)
#define is_v2_COMPW_ne_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf001b000)
#define is_v2_COMPW_none_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd01b000)
#define v2_COMPWD_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPWD_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_COMPWD_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01c000)
#define is_v2_COMPWD_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01c000)
#define is_v2_COMPWD_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101c000)
#define is_v2_COMPWD_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301c000)
#define is_v2_COMPWD_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701c000)
#define is_v2_COMPWD_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501c000)
#define is_v2_COMPWD_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901c000)
#define is_v2_COMPWD_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401c000)
#define is_v2_COMPWD_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801c000)
#define is_v2_COMPWD_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201c000)
#define is_v2_COMPWD_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601c000)
#define is_v2_COMPWD_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01c000)
#define is_v2_COMPWD_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001c000)
#define is_v2_COMPWD_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01c000)
#define v2_COMPUWD_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPUWD_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_COMPUWD_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01d000)
#define is_v2_COMPUWD_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01d000)
#define is_v2_COMPUWD_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101d000)
#define is_v2_COMPUWD_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301d000)
#define is_v2_COMPUWD_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701d000)
#define is_v2_COMPUWD_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501d000)
#define is_v2_COMPUWD_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901d000)
#define is_v2_COMPUWD_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401d000)
#define is_v2_COMPUWD_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801d000)
#define is_v2_COMPUWD_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201d000)
#define is_v2_COMPUWD_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601d000)
#define is_v2_COMPUWD_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01d000)
#define is_v2_COMPUWD_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001d000)
#define is_v2_COMPUWD_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01d000)
#define v2_COMPNWP_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNWP_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_COMPNWP_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01e000)
#define is_v2_COMPNWP_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01e000)
#define is_v2_COMPNWP_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101e000)
#define is_v2_COMPNWP_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301e000)
#define is_v2_COMPNWP_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701e000)
#define is_v2_COMPNWP_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501e000)
#define is_v2_COMPNWP_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901e000)
#define is_v2_COMPNWP_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401e000)
#define is_v2_COMPNWP_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801e000)
#define is_v2_COMPNWP_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201e000)
#define is_v2_COMPNWP_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601e000)
#define is_v2_COMPNWP_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01e000)
#define is_v2_COMPNWP_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001e000)
#define is_v2_COMPNWP_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01e000)
#define v2_COMPNHQ_all_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_any_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_eq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7101f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_ge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7301f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_geu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7701f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_gt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7501f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_gtu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7901f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_le_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7401f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_leu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7801f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_lt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7201f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_ltu_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7601f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_nall_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_ne_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7001f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_COMPNHQ_none_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_COMPNHQ_all_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a01f000)
#define is_v2_COMPNHQ_any_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c01f000)
#define is_v2_COMPNHQ_eq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7101f000)
#define is_v2_COMPNHQ_ge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7301f000)
#define is_v2_COMPNHQ_geu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7701f000)
#define is_v2_COMPNHQ_gt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7501f000)
#define is_v2_COMPNHQ_gtu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7901f000)
#define is_v2_COMPNHQ_le_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7401f000)
#define is_v2_COMPNHQ_leu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7801f000)
#define is_v2_COMPNHQ_lt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7201f000)
#define is_v2_COMPNHQ_ltu_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7601f000)
#define is_v2_COMPNHQ_nall_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b01f000)
#define is_v2_COMPNHQ_ne_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7001f000)
#define is_v2_COMPNHQ_none_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d01f000)
#define v2_CMOVED_deqz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x61020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_dgez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x63020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_dgtz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x65020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_dlez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x64020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_dltz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x62020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_dnez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x60020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_even_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x67020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_odd_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x66020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_weqz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x69020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_wgez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6b020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_wgtz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6d020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_wlez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6c020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_wltz_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x6a020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define v2_CMOVED_wnez_registerZ_registerW_signed10_simple_insn(registerZ,registerW,signed10) (unsigned int)(0x68020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_CMOVED_deqz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x61020000)
#define is_v2_CMOVED_dgez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x63020000)
#define is_v2_CMOVED_dgtz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x65020000)
#define is_v2_CMOVED_dlez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x64020000)
#define is_v2_CMOVED_dltz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x62020000)
#define is_v2_CMOVED_dnez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x60020000)
#define is_v2_CMOVED_even_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x67020000)
#define is_v2_CMOVED_odd_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x66020000)
#define is_v2_CMOVED_weqz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x69020000)
#define is_v2_CMOVED_wgez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6b020000)
#define is_v2_CMOVED_wgtz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6d020000)
#define is_v2_CMOVED_wlez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6c020000)
#define is_v2_CMOVED_wltz_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x6a020000)
#define is_v2_CMOVED_wnez_registerZ_registerW_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x68020000)
#define v2_CMOVED_deqz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe1020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_deqz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dgez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe3020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dgez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dgtz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe5020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dgtz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dlez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe4020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dlez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dltz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe2020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dltz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dnez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe0020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dnez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_even_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe7020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_even_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_odd_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe6020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_odd_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_weqz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe9020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_weqz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wgez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xeb020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wgez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wgtz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xed020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wgtz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wlez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xec020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wlez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wltz_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xea020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wltz_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wnez_registerZ_registerW_upper27_lower10_double_insn_0(registerZ,registerW,upper27_lower10) (unsigned int)(0xe8020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wnez_registerZ_registerW_upper27_lower10_double_insn_1(registerZ,registerW,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_CMOVED_deqz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1020000)
#define is_v2_CMOVED_dgez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3020000)
#define is_v2_CMOVED_dgtz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5020000)
#define is_v2_CMOVED_dlez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4020000)
#define is_v2_CMOVED_dltz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2020000)
#define is_v2_CMOVED_dnez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0020000)
#define is_v2_CMOVED_even_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7020000)
#define is_v2_CMOVED_odd_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6020000)
#define is_v2_CMOVED_weqz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9020000)
#define is_v2_CMOVED_wgez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb020000)
#define is_v2_CMOVED_wgtz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed020000)
#define is_v2_CMOVED_wlez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec020000)
#define is_v2_CMOVED_wltz_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea020000)
#define is_v2_CMOVED_wnez_registerZ_registerW_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8020000)
#define v2_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe1020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe3020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe5020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe4020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe2020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe0020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe7020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe6020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe9020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xeb020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xed020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xec020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xea020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define v2_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_0(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(0xe8020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_1(registerZ,registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn_2(registerZ,registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_CMOVED_deqz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe1020000)
#define is_v2_CMOVED_dgez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe3020000)
#define is_v2_CMOVED_dgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe5020000)
#define is_v2_CMOVED_dlez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe4020000)
#define is_v2_CMOVED_dltz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe2020000)
#define is_v2_CMOVED_dnez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0020000)
#define is_v2_CMOVED_even_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe7020000)
#define is_v2_CMOVED_odd_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe6020000)
#define is_v2_CMOVED_weqz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe9020000)
#define is_v2_CMOVED_wgez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xeb020000)
#define is_v2_CMOVED_wgtz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xed020000)
#define is_v2_CMOVED_wlez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xec020000)
#define is_v2_CMOVED_wltz_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xea020000)
#define is_v2_CMOVED_wnez_registerZ_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe8020000)
#define v2_CMOVED_deqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x71020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_dgez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x73020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_dgtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x75020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_dlez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x74020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_dltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x72020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_dnez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x70020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_even_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x77020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_odd_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x76020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_weqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x79020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_wgez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7b020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_wgtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7d020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_wlez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7c020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_wltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7a020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVED_wnez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x78020000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMOVED_deqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71020000)
#define is_v2_CMOVED_dgez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73020000)
#define is_v2_CMOVED_dgtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75020000)
#define is_v2_CMOVED_dlez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74020000)
#define is_v2_CMOVED_dltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72020000)
#define is_v2_CMOVED_dnez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70020000)
#define is_v2_CMOVED_even_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77020000)
#define is_v2_CMOVED_odd_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76020000)
#define is_v2_CMOVED_weqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79020000)
#define is_v2_CMOVED_wgez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b020000)
#define is_v2_CMOVED_wgtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d020000)
#define is_v2_CMOVED_wlez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c020000)
#define is_v2_CMOVED_wltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a020000)
#define is_v2_CMOVED_wnez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78020000)
#define v2_CMOVEWP_eqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x71021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEWP_even_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x77021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEWP_gez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x73021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEWP_gtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x75021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEWP_lez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x74021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEWP_ltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x72021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEWP_nez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x70021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEWP_odd_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x76021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMOVEWP_eqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71021000)
#define is_v2_CMOVEWP_even_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77021000)
#define is_v2_CMOVEWP_gez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73021000)
#define is_v2_CMOVEWP_gtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75021000)
#define is_v2_CMOVEWP_lez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74021000)
#define is_v2_CMOVEWP_ltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72021000)
#define is_v2_CMOVEWP_nez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70021000)
#define is_v2_CMOVEWP_odd_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76021000)
#define v2_MAKE_registerW_signed16_simple_insn(registerW,signed16) (unsigned int)(0x60000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(signed16) & 0xffff))
#define is_v2_MAKE_registerW_signed16_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x60000000)
#define v2_MAKE_registerW_extend6_upper27_lower10_double_insn_0(registerW,extend6_upper27_lower10) (unsigned int)(0xe0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend6_upper27_lower10) >> 6) >> 27) & 0x3ff) << 6))
#define v2_MAKE_registerW_extend6_upper27_lower10_double_insn_1(registerW,extend6_upper27_lower10) (unsigned int)((((int)(extend6_upper27_lower10) >> 6) & 0x7ffffff))
#define v2_MAKE_registerW_extend6_upper27_lower10_double_insn_2(registerW,extend6_upper27_lower10) (unsigned int)(((int)(extend6_upper27_lower10) & 0x3f))
#define is_v2_MAKE_registerW_extend6_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0000000)
#define v2_MAKE_registerW_extend27_upper27_lower10_triple_insn_0(registerW,extend27_upper27_lower10) (unsigned int)(0xe0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MAKE_registerW_extend27_upper27_lower10_triple_insn_1(registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MAKE_registerW_extend27_upper27_lower10_triple_insn_2(registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MAKE_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xe0000000)
#define v2_PCREL_registerW_signed16_simple_insn(registerW,signed16) (unsigned int)(0x70000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(signed16) & 0xffff))
#define is_v2_PCREL_registerW_signed16_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x70000000)
#define v2_PCREL_registerW_extend6_upper27_lower10_double_insn_0(registerW,extend6_upper27_lower10) (unsigned int)(0xf0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend6_upper27_lower10) >> 6) >> 27) & 0x3ff) << 6))
#define v2_PCREL_registerW_extend6_upper27_lower10_double_insn_1(registerW,extend6_upper27_lower10) (unsigned int)((((int)(extend6_upper27_lower10) >> 6) & 0x7ffffff))
#define v2_PCREL_registerW_extend6_upper27_lower10_double_insn_2(registerW,extend6_upper27_lower10) (unsigned int)(((int)(extend6_upper27_lower10) & 0x3f))
#define is_v2_PCREL_registerW_extend6_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf0000000)
#define v2_PCREL_registerW_extend27_upper27_lower10_triple_insn_0(registerW,extend27_upper27_lower10) (unsigned int)(0xf0000000 | (((int)(registerW) & 0x3f) << 18) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_PCREL_registerW_extend27_upper27_lower10_triple_insn_1(registerW,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_PCREL_registerW_extend27_upper27_lower10_triple_insn_2(registerW,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_PCREL_registerW_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xf0000000)
#define v2_CLZW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CLZW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70023000)
#define v2_CLSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CLSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71023000)
#define v2_CBSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CBSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72023000)
#define v2_CTZW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CTZW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73023000)
#define v2_CLZD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CLZD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70022000)
#define v2_CLSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CLSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71022000)
#define v2_CBSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CBSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72022000)
#define v2_CTZD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CTZD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73022000)
#define v2_CLZWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x70024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CLZWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70024000)
#define v2_CLSWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x71024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CLSWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71024000)
#define v2_CBSWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x72024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CBSWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72024000)
#define v2_CTZWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x73024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_CTZWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73024000)
#define v2_SXLHWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x76024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SXLHWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76024000)
#define v2_SXMHWP_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x77024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SXMHWP_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77024000)
#define v2_SXLBHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x76025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SXLBHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76025000)
#define v2_SXMBHQ_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x77025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SXMBHQ_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77025000)
#define v2_CMOVEHQ_eqz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x79021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEHQ_even_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7f021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEHQ_gez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7b021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEHQ_gtz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7d021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEHQ_lez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7c021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEHQ_ltz_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7a021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEHQ_nez_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x78021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define v2_CMOVEHQ_odd_registerZ_registerW_registerY_simple_insn(registerZ,registerW,registerY) (unsigned int)(0x7e021000 | ((int)(registerZ) & 0x3f) | (((int)(registerW) & 0x3f) << 18) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMOVEHQ_eqz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79021000)
#define is_v2_CMOVEHQ_even_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f021000)
#define is_v2_CMOVEHQ_gez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b021000)
#define is_v2_CMOVEHQ_gtz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d021000)
#define is_v2_CMOVEHQ_lez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c021000)
#define is_v2_CMOVEHQ_ltz_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a021000)
#define is_v2_CMOVEHQ_nez_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78021000)
#define is_v2_CMOVEHQ_odd_registerZ_registerW_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e021000)
#define v2_FCOMPW_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPW_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPW_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPW_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPW_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPW_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPW_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPW_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FCOMPW_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72030000)
#define is_v2_FCOMPW_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76030000)
#define is_v2_FCOMPW_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74030000)
#define is_v2_FCOMPW_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70030000)
#define is_v2_FCOMPW_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71030000)
#define is_v2_FCOMPW_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75030000)
#define is_v2_FCOMPW_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77030000)
#define is_v2_FCOMPW_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73030000)
#define v2_FCOMPW_oeq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf2030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_oeq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPW_oge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf6030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_oge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPW_olt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf4030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_olt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPW_one_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf0030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_one_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPW_ueq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf1030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_ueq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPW_uge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf5030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_uge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPW_ult_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf7030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_ult_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPW_une_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf3030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPW_une_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_FCOMPW_oeq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf2030000)
#define is_v2_FCOMPW_oge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf6030000)
#define is_v2_FCOMPW_olt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf4030000)
#define is_v2_FCOMPW_one_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf0030000)
#define is_v2_FCOMPW_ueq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf1030000)
#define is_v2_FCOMPW_uge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf5030000)
#define is_v2_FCOMPW_ult_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf7030000)
#define is_v2_FCOMPW_une_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf3030000)
#define v2_FCOMPD_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPD_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPD_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPD_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPD_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPD_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPD_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPD_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FCOMPD_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a030000)
#define is_v2_FCOMPD_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e030000)
#define is_v2_FCOMPD_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c030000)
#define is_v2_FCOMPD_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78030000)
#define is_v2_FCOMPD_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79030000)
#define is_v2_FCOMPD_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d030000)
#define is_v2_FCOMPD_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f030000)
#define is_v2_FCOMPD_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b030000)
#define v2_FCOMPD_oeq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfa030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_oeq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPD_oge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfe030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_oge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPD_olt_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfc030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_olt_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPD_one_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf8030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_one_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPD_ueq_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xf9030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_ueq_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPD_uge_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfd030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_uge_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPD_ult_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xff030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_ult_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define v2_FCOMPD_une_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xfb030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_FCOMPD_une_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_FCOMPD_oeq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfa030000)
#define is_v2_FCOMPD_oge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfe030000)
#define is_v2_FCOMPD_olt_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfc030000)
#define is_v2_FCOMPD_one_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf8030000)
#define is_v2_FCOMPD_ueq_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xf9030000)
#define is_v2_FCOMPD_uge_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfd030000)
#define is_v2_FCOMPD_ult_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xff030000)
#define is_v2_FCOMPD_une_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xfb030000)
#define v2_FCOMPNWP_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNWP_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNWP_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNWP_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNWP_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNWP_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNWP_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNWP_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FCOMPNWP_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72031000)
#define is_v2_FCOMPNWP_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76031000)
#define is_v2_FCOMPNWP_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74031000)
#define is_v2_FCOMPNWP_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70031000)
#define is_v2_FCOMPNWP_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71031000)
#define is_v2_FCOMPNWP_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75031000)
#define is_v2_FCOMPNWP_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77031000)
#define is_v2_FCOMPNWP_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73031000)
#define v2_FCOMPNHQ_oeq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7a031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNHQ_oge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7e031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNHQ_olt_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7c031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNHQ_one_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x78031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNHQ_ueq_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x79031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNHQ_uge_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7d031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNHQ_ult_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7f031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FCOMPNHQ_une_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x7b031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FCOMPNHQ_oeq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7a031000)
#define is_v2_FCOMPNHQ_oge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7e031000)
#define is_v2_FCOMPNHQ_olt_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7c031000)
#define is_v2_FCOMPNHQ_one_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x78031000)
#define is_v2_FCOMPNHQ_ueq_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x79031000)
#define is_v2_FCOMPNHQ_uge_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7d031000)
#define is_v2_FCOMPNHQ_ult_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f031000)
#define is_v2_FCOMPNHQ_une_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7b031000)
#define v2_FSDIVD__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x70035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FSDIVD_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x70035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FSDIVD__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70035000)
#define is_v2_FSDIVD_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x70035800)
#define v2_FCDIVD__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x71035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FCDIVD_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x71035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FCDIVD__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x71035000)
#define is_v2_FCDIVD_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x71035800)
#define v2_FSDIVW__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x72035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FSDIVW_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x72035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FSDIVW__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72035000)
#define is_v2_FSDIVW_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x72035800)
#define v2_FCDIVW__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x73035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FCDIVW_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x73035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FCDIVW__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x73035000)
#define is_v2_FCDIVW_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x73035800)
#define v2_FSDIVWP__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x74035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FSDIVWP_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x74035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FSDIVWP__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74035000)
#define is_v2_FSDIVWP_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x74035800)
#define v2_FCDIVWP__registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x75035000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define v2_FCDIVWP_s_registerW_registerP_simple_insn(registerW,registerP) (unsigned int)(0x75035800 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1))
#define is_v2_FCDIVWP__registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x75035000)
#define is_v2_FCDIVWP_s_registerW_registerP_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f800) == 0x75035800)
#define v2_FMIND_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x70038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMIND_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x70038000)
#define v2_FMAXD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x71038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMAXD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x71038000)
#define v2_FMINW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x72038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMINW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x72038000)
#define v2_FMAXW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x73038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMAXW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x73038000)
#define v2_FMINWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x74038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMINWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x74038000)
#define v2_FMAXWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x75038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMAXWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x75038000)
#define v2_FMINHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x76038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMINHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x76038000)
#define v2_FMAXHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x77038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMAXHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x77038000)
#define v2_FLOATD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x44037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4403f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FLOATD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44032000)
#define is_v2_FLOATD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403a000)
#define is_v2_FLOATD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44034000)
#define is_v2_FLOATD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403c000)
#define is_v2_FLOATD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44035000)
#define is_v2_FLOATD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403d000)
#define is_v2_FLOATD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44030000)
#define is_v2_FLOATD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44038000)
#define is_v2_FLOATD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44036000)
#define is_v2_FLOATD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403e000)
#define is_v2_FLOATD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44031000)
#define is_v2_FLOATD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44039000)
#define is_v2_FLOATD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44033000)
#define is_v2_FLOATD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403b000)
#define is_v2_FLOATD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44037000)
#define is_v2_FLOATD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4403f000)
#define v2_FLOATWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4c03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FLOATWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c032000)
#define is_v2_FLOATWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03a000)
#define is_v2_FLOATWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c034000)
#define is_v2_FLOATWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03c000)
#define is_v2_FLOATWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c035000)
#define is_v2_FLOATWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03d000)
#define is_v2_FLOATWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c030000)
#define is_v2_FLOATWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c038000)
#define is_v2_FLOATWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c036000)
#define is_v2_FLOATWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03e000)
#define is_v2_FLOATWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c031000)
#define is_v2_FLOATWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c039000)
#define is_v2_FLOATWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c033000)
#define is_v2_FLOATWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03b000)
#define is_v2_FLOATWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c037000)
#define is_v2_FLOATWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4c03f000)
#define v2_FLOATW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x48037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4803f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FLOATW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48032000)
#define is_v2_FLOATW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803a000)
#define is_v2_FLOATW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48034000)
#define is_v2_FLOATW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803c000)
#define is_v2_FLOATW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48035000)
#define is_v2_FLOATW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803d000)
#define is_v2_FLOATW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48030000)
#define is_v2_FLOATW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48038000)
#define is_v2_FLOATW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48036000)
#define is_v2_FLOATW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803e000)
#define is_v2_FLOATW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48031000)
#define is_v2_FLOATW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48039000)
#define is_v2_FLOATW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48033000)
#define is_v2_FLOATW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803b000)
#define is_v2_FLOATW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48037000)
#define is_v2_FLOATW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4803f000)
#define v2_FLOATUD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x45037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4503f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FLOATUD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45032000)
#define is_v2_FLOATUD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503a000)
#define is_v2_FLOATUD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45034000)
#define is_v2_FLOATUD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503c000)
#define is_v2_FLOATUD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45035000)
#define is_v2_FLOATUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503d000)
#define is_v2_FLOATUD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45030000)
#define is_v2_FLOATUD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45038000)
#define is_v2_FLOATUD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45036000)
#define is_v2_FLOATUD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503e000)
#define is_v2_FLOATUD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45031000)
#define is_v2_FLOATUD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45039000)
#define is_v2_FLOATUD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45033000)
#define is_v2_FLOATUD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503b000)
#define is_v2_FLOATUD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45037000)
#define is_v2_FLOATUD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4503f000)
#define v2_FLOATUWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4d03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FLOATUWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d032000)
#define is_v2_FLOATUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03a000)
#define is_v2_FLOATUWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d034000)
#define is_v2_FLOATUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03c000)
#define is_v2_FLOATUWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d035000)
#define is_v2_FLOATUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03d000)
#define is_v2_FLOATUWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d030000)
#define is_v2_FLOATUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d038000)
#define is_v2_FLOATUWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d036000)
#define is_v2_FLOATUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03e000)
#define is_v2_FLOATUWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d031000)
#define is_v2_FLOATUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d039000)
#define is_v2_FLOATUWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d033000)
#define is_v2_FLOATUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03b000)
#define is_v2_FLOATUWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d037000)
#define is_v2_FLOATUWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4d03f000)
#define v2_FLOATUW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x49037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FLOATUW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4903f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FLOATUW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49032000)
#define is_v2_FLOATUW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903a000)
#define is_v2_FLOATUW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49034000)
#define is_v2_FLOATUW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903c000)
#define is_v2_FLOATUW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49035000)
#define is_v2_FLOATUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903d000)
#define is_v2_FLOATUW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49030000)
#define is_v2_FLOATUW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49038000)
#define is_v2_FLOATUW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49036000)
#define is_v2_FLOATUW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903e000)
#define is_v2_FLOATUW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49031000)
#define is_v2_FLOATUW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49039000)
#define is_v2_FLOATUW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49033000)
#define is_v2_FLOATUW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903b000)
#define is_v2_FLOATUW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49037000)
#define is_v2_FLOATUW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4903f000)
#define v2_FIXEDD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x46037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4603f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FIXEDD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46032000)
#define is_v2_FIXEDD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603a000)
#define is_v2_FIXEDD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46034000)
#define is_v2_FIXEDD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603c000)
#define is_v2_FIXEDD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46035000)
#define is_v2_FIXEDD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603d000)
#define is_v2_FIXEDD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46030000)
#define is_v2_FIXEDD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46038000)
#define is_v2_FIXEDD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46036000)
#define is_v2_FIXEDD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603e000)
#define is_v2_FIXEDD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46031000)
#define is_v2_FIXEDD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46039000)
#define is_v2_FIXEDD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46033000)
#define is_v2_FIXEDD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603b000)
#define is_v2_FIXEDD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46037000)
#define is_v2_FIXEDD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4603f000)
#define v2_FIXEDWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4e03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FIXEDWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e032000)
#define is_v2_FIXEDWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03a000)
#define is_v2_FIXEDWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e034000)
#define is_v2_FIXEDWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03c000)
#define is_v2_FIXEDWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e035000)
#define is_v2_FIXEDWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03d000)
#define is_v2_FIXEDWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e030000)
#define is_v2_FIXEDWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e038000)
#define is_v2_FIXEDWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e036000)
#define is_v2_FIXEDWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03e000)
#define is_v2_FIXEDWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e031000)
#define is_v2_FIXEDWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e039000)
#define is_v2_FIXEDWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e033000)
#define is_v2_FIXEDWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03b000)
#define is_v2_FIXEDWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e037000)
#define is_v2_FIXEDWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4e03f000)
#define v2_FIXEDW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4a03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FIXEDW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a032000)
#define is_v2_FIXEDW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03a000)
#define is_v2_FIXEDW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a034000)
#define is_v2_FIXEDW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03c000)
#define is_v2_FIXEDW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a035000)
#define is_v2_FIXEDW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03d000)
#define is_v2_FIXEDW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a030000)
#define is_v2_FIXEDW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a038000)
#define is_v2_FIXEDW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a036000)
#define is_v2_FIXEDW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03e000)
#define is_v2_FIXEDW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a031000)
#define is_v2_FIXEDW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a039000)
#define is_v2_FIXEDW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a033000)
#define is_v2_FIXEDW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03b000)
#define is_v2_FIXEDW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a037000)
#define is_v2_FIXEDW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a03f000)
#define v2_FIXEDUD_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x47037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUD__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4703f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FIXEDUD_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47032000)
#define is_v2_FIXEDUD_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703a000)
#define is_v2_FIXEDUD_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47034000)
#define is_v2_FIXEDUD_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703c000)
#define is_v2_FIXEDUD_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47035000)
#define is_v2_FIXEDUD_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703d000)
#define is_v2_FIXEDUD_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47030000)
#define is_v2_FIXEDUD_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47038000)
#define is_v2_FIXEDUD_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47036000)
#define is_v2_FIXEDUD_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703e000)
#define is_v2_FIXEDUD_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47031000)
#define is_v2_FIXEDUD_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47039000)
#define is_v2_FIXEDUD_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47033000)
#define is_v2_FIXEDUD_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703b000)
#define is_v2_FIXEDUD___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47037000)
#define is_v2_FIXEDUD__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4703f000)
#define v2_FIXEDUWP_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUWP__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4f03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FIXEDUWP_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f032000)
#define is_v2_FIXEDUWP_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03a000)
#define is_v2_FIXEDUWP_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f034000)
#define is_v2_FIXEDUWP_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03c000)
#define is_v2_FIXEDUWP_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f035000)
#define is_v2_FIXEDUWP_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03d000)
#define is_v2_FIXEDUWP_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f030000)
#define is_v2_FIXEDUWP_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f038000)
#define is_v2_FIXEDUWP_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f036000)
#define is_v2_FIXEDUWP_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03e000)
#define is_v2_FIXEDUWP_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f031000)
#define is_v2_FIXEDUWP_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f039000)
#define is_v2_FIXEDUWP_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f033000)
#define is_v2_FIXEDUWP_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03b000)
#define is_v2_FIXEDUWP___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f037000)
#define is_v2_FIXEDUWP__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4f03f000)
#define v2_FIXEDUW_rd__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b032000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rd_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rna__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b034000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rna_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rnz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b035000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rn__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b030000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rn_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b038000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_ro__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b036000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_ro_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_ru__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b031000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_ru_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b039000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rz__registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b033000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW_rz_s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW___registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define v2_FIXEDUW__s_registerW_registerZ_unsigned6_simple_insn(registerW,registerZ,unsigned6) (unsigned int)(0x4b03f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(unsigned6) & 0x3f) << 6))
#define is_v2_FIXEDUW_rd__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b032000)
#define is_v2_FIXEDUW_rd_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03a000)
#define is_v2_FIXEDUW_rna__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b034000)
#define is_v2_FIXEDUW_rna_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03c000)
#define is_v2_FIXEDUW_rnz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b035000)
#define is_v2_FIXEDUW_rnz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03d000)
#define is_v2_FIXEDUW_rn__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b030000)
#define is_v2_FIXEDUW_rn_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b038000)
#define is_v2_FIXEDUW_ro__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b036000)
#define is_v2_FIXEDUW_ro_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03e000)
#define is_v2_FIXEDUW_ru__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b031000)
#define is_v2_FIXEDUW_ru_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b039000)
#define is_v2_FIXEDUW_rz__registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b033000)
#define is_v2_FIXEDUW_rz_s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03b000)
#define is_v2_FIXEDUW___registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b037000)
#define is_v2_FIXEDUW__s_registerW_registerZ_unsigned6_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b03f000)
#define v2_MADDD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x40000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MADDD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x40000000)
#define v2_MADDD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc0000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MADDD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MADDD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0000000)
#define v2_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc0000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MADDD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc0000000)
#define v2_MADDD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50000000)
#define v2_MSBFD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54000000)
#define v2_MADDWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x41000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MADDWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x41000000)
#define v2_MADDWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MADDWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MADDWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1000000)
#define v2_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc1000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MADDWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc1000000)
#define v2_MADDWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51000000)
#define v2_MSBFWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55000000)
#define v2_MADDHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x42000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MADDHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x42000000)
#define v2_MADDHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MADDHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MADDHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2000000)
#define v2_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc2000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MADDHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc2000000)
#define v2_MADDHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52000000)
#define v2_MSBFHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56000000)
#define v2_MADDHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50004000)
#define v2_MSBFHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x54004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54004000)
#define v2_MADDUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51004000)
#define v2_MSBFUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x55004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55004000)
#define v2_MADDSUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDSUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52004000)
#define v2_MSBFSUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFSUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56004000)
#define v2_MADDDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x48000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MADDDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x48000000)
#define v2_MADDDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc8000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MADDDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MADDDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8000000)
#define v2_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc8000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MADDDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8000000)
#define v2_MADDDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58001000)
#define v2_MADDUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x49000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MADDUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x49000000)
#define v2_MADDUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc9000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MADDUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MADDUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9000000)
#define v2_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc9000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MADDUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9000000)
#define v2_MADDUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59001000)
#define v2_MADDSUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4a000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MADDSUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4a000000)
#define v2_MADDSUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xca000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MADDSUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MADDSUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca000000)
#define v2_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xca000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MADDSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca000000)
#define v2_MADDSUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDSUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a001000)
#define v2_MADDUZDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4b000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MADDUZDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4b000000)
#define v2_MADDUZDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xcb000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MADDUZDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MADDUZDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb000000)
#define v2_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xcb000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MADDUZDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb000000)
#define v2_MADDUZDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5b001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDUZDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b001000)
#define v2_MSBFDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5c001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c001000)
#define v2_MSBFUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5d001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d001000)
#define v2_MSBFSUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5e001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFSUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e001000)
#define v2_MSBFUZDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFUZDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f001000)
#define v2_MULD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x44000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MULD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x44000000)
#define v2_MULD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MULD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MULD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4000000)
#define v2_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc4000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MULD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc4000000)
#define v2_MULD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54001000)
#define v2_MULDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x48040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MULDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x48040000)
#define v2_MULDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc8040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MULDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MULDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8040000)
#define v2_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc8040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc8040000)
#define v2_MULDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58041000)
#define v2_MULUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x49040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MULUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x49040000)
#define v2_MULUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xc9040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MULUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MULUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9040000)
#define v2_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xc9040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MULUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xc9040000)
#define v2_MULUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59041000)
#define v2_MULSUDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4a040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MULSUDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4a040000)
#define v2_MULSUDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xca040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MULSUDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MULSUDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca040000)
#define v2_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xca040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MULSUDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xca040000)
#define v2_MULSUDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULSUDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a041000)
#define v2_CMULDT_registerM_registerZ_signed10_simple_insn(registerM,registerZ,signed10) (unsigned int)(0x4b040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_CMULDT_registerM_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f070000) == 0x4b040000)
#define v2_CMULDT_registerM_registerZ_upper27_lower10_double_insn_0(registerM,registerZ,upper27_lower10) (unsigned int)(0xcb040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_CMULDT_registerM_registerZ_upper27_lower10_double_insn_1(registerM,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_CMULDT_registerM_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb040000)
#define v2_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_0(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(0xcb040000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_1(registerM,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn_2(registerM,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_CMULDT_registerM_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff070000) == 0xcb040000)
#define v2_CMULDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5b041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMULDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b041000)
#define v2_CMULXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5c041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMULXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c041000)
#define v2_CMULGLXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5d041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMULGLXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d041000)
#define v2_CMULGMXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5e041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMULGMXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e041000)
#define v2_CMULGHXDT_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f041000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CMULGHXDT_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f041000)
#define v2_CRCBELMW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CRCBELMW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58002000)
#define v2_CRCBELMW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd8002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_CRCBELMW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_CRCBELMW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd8002000)
#define v2_CRCBELLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CRCBELLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59002000)
#define v2_CRCBELLW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd9002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_CRCBELLW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_CRCBELLW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd9002000)
#define v2_CRCLELMW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CRCLELMW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a002000)
#define v2_CRCLELMW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xda002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_CRCLELMW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_CRCLELMW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xda002000)
#define v2_CRCLELLW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_CRCLELLW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b002000)
#define v2_CRCLELLW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xdb002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_CRCLELLW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_CRCLELLW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xdb002000)
#define v2_DOT2WD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4c000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_DOT2WD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4c000000)
#define v2_DOT2WD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcc000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_DOT2WD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2WD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc000000)
#define v2_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcc000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2WD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcc000000)
#define v2_DOT2WD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_DOT2WD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c002000)
#define v2_DOT2UWD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4d000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_DOT2UWD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4d000000)
#define v2_DOT2UWD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcd000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_DOT2UWD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2UWD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd000000)
#define v2_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcd000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2UWD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcd000000)
#define v2_DOT2UWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_DOT2UWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d002000)
#define v2_DOT2SUWD_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4e000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_DOT2SUWD_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4e000000)
#define v2_DOT2SUWD_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xce000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_DOT2SUWD_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2SUWD_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce000000)
#define v2_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xce000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2SUWD_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xce000000)
#define v2_DOT2SUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_DOT2SUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e002000)
#define v2_DOT2W_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x4f000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_DOT2W_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x4f000000)
#define v2_DOT2W_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xcf000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_DOT2W_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2W_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf000000)
#define v2_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xcf000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_DOT2W_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xcf000000)
#define v2_DOT2W_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_DOT2W_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f002000)
#define v2_DOT2WDP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x50002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_DOT2WDP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50002000)
#define v2_DOT2UWDP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x51002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_DOT2UWDP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51002000)
#define v2_DOT2SUWDP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x52002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_DOT2SUWDP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52002000)
#define v2_MULWQ_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x54002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_MULWQ_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54002000)
#define v2_DOT2WZP_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x53002000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_DOT2WZP_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53002000)
#define v2_MULWP_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x45000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MULWP_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x45000000)
#define v2_MULWP_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MULWP_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MULWP_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5000000)
#define v2_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc5000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MULWP_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc5000000)
#define v2_MULWP_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULWP_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55001000)
#define v2_MULHQ_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x46000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MULHQ_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x46000000)
#define v2_MULHQ_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MULHQ_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MULHQ_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6000000)
#define v2_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc6000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MULHQ_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc6000000)
#define v2_MULHQ_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULHQ_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56001000)
#define v2_MULWC_registerW_registerZ_signed10_simple_insn(registerW,registerZ,signed10) (unsigned int)(0x47000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(signed10) & 0x3ff) << 6))
#define is_v2_MULWC_registerW_registerZ_signed10_simple_insn(x) (int)(((unsigned int)(x) & 0x7f030000) == 0x47000000)
#define v2_MULWC_registerW_registerZ_upper27_lower10_double_insn_0(registerW,registerZ,upper27_lower10) (unsigned int)(0xc7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower10) >> 27) & 0x3ff) << 6))
#define v2_MULWC_registerW_registerZ_upper27_lower10_double_insn_1(registerW,registerZ,upper27_lower10) (unsigned int)(((int)(upper27_lower10) & 0x7ffffff))
#define is_v2_MULWC_registerW_registerZ_upper27_lower10_double_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7000000)
#define v2_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_0(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(0xc7000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((((int)(extend27_upper27_lower10) >> 27) >> 27) & 0x3ff) << 6))
#define v2_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_1(registerW,registerZ,extend27_upper27_lower10) (unsigned int)((((int)(extend27_upper27_lower10) >> 27) & 0x7ffffff))
#define v2_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn_2(registerW,registerZ,extend27_upper27_lower10) (unsigned int)(((int)(extend27_upper27_lower10) & 0x7ffffff))
#define is_v2_MULWC_registerW_registerZ_extend27_upper27_lower10_triple_insn(x) (int)(((unsigned int)(x) & 0xff030000) == 0xc7000000)
#define v2_MULWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57001000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57001000)
#define v2_MULCWC_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55002000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULCWC_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55002000)
#define v2_MULWDC_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x56002000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULWDC_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56002000)
#define v2_MULCWDC_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x57002000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULCWDC_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57002000)
#define v2_MULHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58004000)
#define v2_MULUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59004000)
#define v2_MULSUHWQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a004000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULSUHWQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a004000)
#define v2_MADDWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50003000)
#define v2_MADDWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd0003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MADDWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MADDWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd0003000)
#define v2_MADDUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51003000)
#define v2_MADDUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd1003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MADDUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MADDUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd1003000)
#define v2_MADDSUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDSUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52003000)
#define v2_MADDSUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd2003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MADDSUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MADDSUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd2003000)
#define v2_MADDW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53003000)
#define v2_MADDW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd3003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MADDW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MADDW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd3003000)
#define v2_MSBFWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54003000)
#define v2_MSBFWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd4003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MSBFWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MSBFWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd4003000)
#define v2_MSBFUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55003000)
#define v2_MSBFUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd5003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MSBFUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MSBFUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd5003000)
#define v2_MSBFSUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFSUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56003000)
#define v2_MSBFSUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd6003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MSBFSUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MSBFSUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd6003000)
#define v2_MSBFW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57003000)
#define v2_MSBFW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd7003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MSBFW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MSBFW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd7003000)
#define v2_MULWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58003000)
#define v2_MULWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd8003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MULWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MULWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd8003000)
#define v2_MULUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59003000)
#define v2_MULUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xd9003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MULUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MULUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xd9003000)
#define v2_MULSUWD_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULSUWD_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a003000)
#define v2_MULSUWD_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xda003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MULSUWD_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MULSUWD_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xda003000)
#define v2_MULW_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULW_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b003000)
#define v2_MULW_registerW_registerZ_upper27_lower5_double_insn_0(registerW,registerZ,upper27_lower5) (unsigned int)(0xdb003000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | ((((int)(upper27_lower5) >> 27) & 0x1f) << 6))
#define v2_MULW_registerW_registerZ_upper27_lower5_double_insn_1(registerW,registerZ,upper27_lower5) (unsigned int)(((int)(upper27_lower5) & 0x7ffffff))
#define is_v2_MULW_registerW_registerZ_upper27_lower5_double_insn(x) (int)(((unsigned int)(x) & 0xff03f800) == 0xdb003000)
#define v2_MADDWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x58000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58000000)
#define v2_MADDUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x59000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59000000)
#define v2_MADDSUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5a000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MADDSUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a000000)
#define v2_MMA212W_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5b000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MMA212W_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b000000)
#define v2_MSBFWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5c000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c000000)
#define v2_MSBFUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5d000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d000000)
#define v2_MSBFSUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5e000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MSBFSUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e000000)
#define v2_MMS212W_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f000000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MMS212W_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f000000)
#define v2_MULWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50001000)
#define v2_MULUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51001000)
#define v2_MULSUWDP_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MULSUWDP_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52001000)
#define v2_MM212W_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53001000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MM212W_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53001000)
#define v2_FFMAD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5001f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50012000)
#define is_v2_FFMAD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001a000)
#define is_v2_FFMAD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50014000)
#define is_v2_FFMAD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001c000)
#define is_v2_FFMAD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50015000)
#define is_v2_FFMAD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001d000)
#define is_v2_FFMAD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50010000)
#define is_v2_FFMAD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50018000)
#define is_v2_FFMAD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50016000)
#define is_v2_FFMAD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001e000)
#define is_v2_FFMAD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50011000)
#define is_v2_FFMAD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50019000)
#define is_v2_FFMAD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50013000)
#define is_v2_FFMAD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001b000)
#define is_v2_FFMAD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50017000)
#define is_v2_FFMAD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5001f000)
#define v2_FFMAWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5201f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52012000)
#define is_v2_FFMAWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201a000)
#define is_v2_FFMAWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52014000)
#define is_v2_FFMAWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201c000)
#define is_v2_FFMAWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52015000)
#define is_v2_FFMAWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201d000)
#define is_v2_FFMAWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52010000)
#define is_v2_FFMAWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52018000)
#define is_v2_FFMAWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52016000)
#define is_v2_FFMAWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201e000)
#define is_v2_FFMAWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52011000)
#define is_v2_FFMAWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52019000)
#define is_v2_FFMAWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52013000)
#define is_v2_FFMAWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201b000)
#define is_v2_FFMAWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52017000)
#define is_v2_FFMAWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5201f000)
#define v2_FFMAHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5301f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53012000)
#define is_v2_FFMAHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301a000)
#define is_v2_FFMAHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53014000)
#define is_v2_FFMAHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301c000)
#define is_v2_FFMAHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53015000)
#define is_v2_FFMAHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301d000)
#define is_v2_FFMAHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53010000)
#define is_v2_FFMAHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53018000)
#define is_v2_FFMAHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53016000)
#define is_v2_FFMAHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301e000)
#define is_v2_FFMAHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53011000)
#define is_v2_FFMAHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53019000)
#define is_v2_FFMAHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53013000)
#define is_v2_FFMAHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301b000)
#define is_v2_FFMAHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53017000)
#define is_v2_FFMAHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5301f000)
#define v2_FFMAWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5803a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5803c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5803d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5803e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5803b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5803f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFMAWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58032000)
#define is_v2_FFMAWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5803a000)
#define is_v2_FFMAWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58034000)
#define is_v2_FFMAWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5803c000)
#define is_v2_FFMAWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58035000)
#define is_v2_FFMAWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5803d000)
#define is_v2_FFMAWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58030000)
#define is_v2_FFMAWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58038000)
#define is_v2_FFMAWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58036000)
#define is_v2_FFMAWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5803e000)
#define is_v2_FFMAWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58031000)
#define is_v2_FFMAWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58039000)
#define is_v2_FFMAWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58033000)
#define is_v2_FFMAWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5803b000)
#define is_v2_FFMAWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58037000)
#define is_v2_FFMAWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5803f000)
#define v2_FFMAHO_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAHO__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFMAHO_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a032000)
#define is_v2_FFMAHO_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a03a000)
#define is_v2_FFMAHO_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a034000)
#define is_v2_FFMAHO_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a03c000)
#define is_v2_FFMAHO_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a035000)
#define is_v2_FFMAHO_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a03d000)
#define is_v2_FFMAHO_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a030000)
#define is_v2_FFMAHO_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a038000)
#define is_v2_FFMAHO_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a036000)
#define is_v2_FFMAHO_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a03e000)
#define is_v2_FFMAHO_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a031000)
#define is_v2_FFMAHO_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a039000)
#define is_v2_FFMAHO_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a033000)
#define is_v2_FFMAHO_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a03b000)
#define is_v2_FFMAHO___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a037000)
#define is_v2_FFMAHO__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a03f000)
#define v2_FFMSD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5401f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54012000)
#define is_v2_FFMSD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401a000)
#define is_v2_FFMSD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54014000)
#define is_v2_FFMSD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401c000)
#define is_v2_FFMSD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54015000)
#define is_v2_FFMSD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401d000)
#define is_v2_FFMSD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54010000)
#define is_v2_FFMSD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54018000)
#define is_v2_FFMSD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54016000)
#define is_v2_FFMSD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401e000)
#define is_v2_FFMSD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54011000)
#define is_v2_FFMSD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54019000)
#define is_v2_FFMSD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54013000)
#define is_v2_FFMSD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401b000)
#define is_v2_FFMSD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54017000)
#define is_v2_FFMSD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5401f000)
#define v2_FFMSWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5601f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56012000)
#define is_v2_FFMSWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601a000)
#define is_v2_FFMSWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56014000)
#define is_v2_FFMSWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601c000)
#define is_v2_FFMSWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56015000)
#define is_v2_FFMSWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601d000)
#define is_v2_FFMSWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56010000)
#define is_v2_FFMSWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56018000)
#define is_v2_FFMSWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56016000)
#define is_v2_FFMSWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601e000)
#define is_v2_FFMSWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56011000)
#define is_v2_FFMSWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56019000)
#define is_v2_FFMSWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56013000)
#define is_v2_FFMSWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601b000)
#define is_v2_FFMSWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56017000)
#define is_v2_FFMSWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5601f000)
#define v2_FFMSHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5701f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57012000)
#define is_v2_FFMSHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701a000)
#define is_v2_FFMSHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57014000)
#define is_v2_FFMSHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701c000)
#define is_v2_FFMSHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57015000)
#define is_v2_FFMSHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701d000)
#define is_v2_FFMSHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57010000)
#define is_v2_FFMSHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57018000)
#define is_v2_FFMSHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57016000)
#define is_v2_FFMSHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701e000)
#define is_v2_FFMSHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57011000)
#define is_v2_FFMSHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57019000)
#define is_v2_FFMSHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57013000)
#define is_v2_FFMSHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701b000)
#define is_v2_FFMSHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57017000)
#define is_v2_FFMSHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5701f000)
#define v2_FFMSWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5903a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5903c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5903d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5903e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5903b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5903f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFMSWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59032000)
#define is_v2_FFMSWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5903a000)
#define is_v2_FFMSWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59034000)
#define is_v2_FFMSWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5903c000)
#define is_v2_FFMSWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59035000)
#define is_v2_FFMSWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5903d000)
#define is_v2_FFMSWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59030000)
#define is_v2_FFMSWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59038000)
#define is_v2_FFMSWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59036000)
#define is_v2_FFMSWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5903e000)
#define is_v2_FFMSWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59031000)
#define is_v2_FFMSWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59039000)
#define is_v2_FFMSWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59033000)
#define is_v2_FFMSWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5903b000)
#define is_v2_FFMSWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59037000)
#define is_v2_FFMSWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5903f000)
#define v2_FFMSHO_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSHO__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFMSHO_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b032000)
#define is_v2_FFMSHO_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b03a000)
#define is_v2_FFMSHO_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b034000)
#define is_v2_FFMSHO_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b03c000)
#define is_v2_FFMSHO_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b035000)
#define is_v2_FFMSHO_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b03d000)
#define is_v2_FFMSHO_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b030000)
#define is_v2_FFMSHO_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b038000)
#define is_v2_FFMSHO_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b036000)
#define is_v2_FFMSHO_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b03e000)
#define is_v2_FFMSHO_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b031000)
#define is_v2_FFMSHO_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b039000)
#define is_v2_FFMSHO_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b033000)
#define is_v2_FFMSHO_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b03b000)
#define is_v2_FFMSHO___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b037000)
#define is_v2_FFMSHO__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b03f000)
#define v2_FMULD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5801f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58012000)
#define is_v2_FMULD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801a000)
#define is_v2_FMULD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58014000)
#define is_v2_FMULD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801c000)
#define is_v2_FMULD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58015000)
#define is_v2_FMULD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801d000)
#define is_v2_FMULD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58010000)
#define is_v2_FMULD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58018000)
#define is_v2_FMULD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58016000)
#define is_v2_FMULD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801e000)
#define is_v2_FMULD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58011000)
#define is_v2_FMULD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58019000)
#define is_v2_FMULD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58013000)
#define is_v2_FMULD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801b000)
#define is_v2_FMULD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58017000)
#define is_v2_FMULD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5801f000)
#define v2_FMULWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a012000)
#define is_v2_FMULWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01a000)
#define is_v2_FMULWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a014000)
#define is_v2_FMULWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01c000)
#define is_v2_FMULWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a015000)
#define is_v2_FMULWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01d000)
#define is_v2_FMULWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a010000)
#define is_v2_FMULWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a018000)
#define is_v2_FMULWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a016000)
#define is_v2_FMULWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01e000)
#define is_v2_FMULWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a011000)
#define is_v2_FMULWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a019000)
#define is_v2_FMULWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a013000)
#define is_v2_FMULWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01b000)
#define is_v2_FMULWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a017000)
#define is_v2_FMULWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a01f000)
#define v2_FMULHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b012000)
#define is_v2_FMULHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01a000)
#define is_v2_FMULHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b014000)
#define is_v2_FMULHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01c000)
#define is_v2_FMULHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b015000)
#define is_v2_FMULHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01d000)
#define is_v2_FMULHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b010000)
#define is_v2_FMULHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b018000)
#define is_v2_FMULHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b016000)
#define is_v2_FMULHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01e000)
#define is_v2_FMULHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b011000)
#define is_v2_FMULHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b019000)
#define is_v2_FMULHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b013000)
#define is_v2_FMULHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01b000)
#define is_v2_FMULHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b017000)
#define is_v2_FMULHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b01f000)
#define v2_FMULWDP_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50072000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50074000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50075000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50078000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50076000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50071000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50079000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50073000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50077000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWDP__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5007f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULWDP_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50072000)
#define is_v2_FMULWDP_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007a000)
#define is_v2_FMULWDP_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50074000)
#define is_v2_FMULWDP_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007c000)
#define is_v2_FMULWDP_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50075000)
#define is_v2_FMULWDP_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007d000)
#define is_v2_FMULWDP_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50070000)
#define is_v2_FMULWDP_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50078000)
#define is_v2_FMULWDP_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50076000)
#define is_v2_FMULWDP_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007e000)
#define is_v2_FMULWDP_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50071000)
#define is_v2_FMULWDP_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50079000)
#define is_v2_FMULWDP_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50073000)
#define is_v2_FMULWDP_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007b000)
#define is_v2_FMULWDP___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50077000)
#define is_v2_FMULWDP__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5007f000)
#define v2_FMULHWQ_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51072000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51074000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51075000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51070000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51078000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51076000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51071000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51079000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51073000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51077000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHWQ__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5107f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULHWQ_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51072000)
#define is_v2_FMULHWQ_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107a000)
#define is_v2_FMULHWQ_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51074000)
#define is_v2_FMULHWQ_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107c000)
#define is_v2_FMULHWQ_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51075000)
#define is_v2_FMULHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107d000)
#define is_v2_FMULHWQ_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51070000)
#define is_v2_FMULHWQ_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51078000)
#define is_v2_FMULHWQ_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51076000)
#define is_v2_FMULHWQ_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107e000)
#define is_v2_FMULHWQ_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51071000)
#define is_v2_FMULHWQ_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51079000)
#define is_v2_FMULHWQ_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51073000)
#define is_v2_FMULHWQ_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107b000)
#define is_v2_FMULHWQ___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51077000)
#define is_v2_FMULHWQ__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5107f000)
#define v2_FMULWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FMULWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e072000)
#define is_v2_FMULWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07a000)
#define is_v2_FMULWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e074000)
#define is_v2_FMULWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07c000)
#define is_v2_FMULWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e075000)
#define is_v2_FMULWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07d000)
#define is_v2_FMULWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e070000)
#define is_v2_FMULWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e078000)
#define is_v2_FMULWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e076000)
#define is_v2_FMULWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07e000)
#define is_v2_FMULWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e071000)
#define is_v2_FMULWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e079000)
#define is_v2_FMULWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e073000)
#define is_v2_FMULWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07b000)
#define is_v2_FMULWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e077000)
#define is_v2_FMULWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e07f000)
#define v2_FMULHO_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5507a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5507c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5507d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5507e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5507b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x55077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULHO__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5507f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FMULHO_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55072000)
#define is_v2_FMULHO_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5507a000)
#define is_v2_FMULHO_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55074000)
#define is_v2_FMULHO_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5507c000)
#define is_v2_FMULHO_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55075000)
#define is_v2_FMULHO_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5507d000)
#define is_v2_FMULHO_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55070000)
#define is_v2_FMULHO_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55078000)
#define is_v2_FMULHO_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55076000)
#define is_v2_FMULHO_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5507e000)
#define is_v2_FMULHO_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55071000)
#define is_v2_FMULHO_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55079000)
#define is_v2_FMULHO_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55073000)
#define is_v2_FMULHO_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5507b000)
#define is_v2_FMULHO___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x55077000)
#define is_v2_FMULHO__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5507f000)
#define v2_FMULWC_c_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4101a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4101c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4101d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4101e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4101b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x41017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC_c__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4101f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4001a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4001c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4001d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4001e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC__rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4001b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC____registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x40017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWC___s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4001f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULWC_c_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41012000)
#define is_v2_FMULWC_c_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4101a000)
#define is_v2_FMULWC_c_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41014000)
#define is_v2_FMULWC_c_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4101c000)
#define is_v2_FMULWC_c_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41015000)
#define is_v2_FMULWC_c_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4101d000)
#define is_v2_FMULWC_c_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41010000)
#define is_v2_FMULWC_c_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41018000)
#define is_v2_FMULWC_c_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41016000)
#define is_v2_FMULWC_c_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4101e000)
#define is_v2_FMULWC_c_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41011000)
#define is_v2_FMULWC_c_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41019000)
#define is_v2_FMULWC_c_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41013000)
#define is_v2_FMULWC_c_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4101b000)
#define is_v2_FMULWC_c___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x41017000)
#define is_v2_FMULWC_c__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4101f000)
#define is_v2_FMULWC__rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40012000)
#define is_v2_FMULWC__rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4001a000)
#define is_v2_FMULWC__rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40014000)
#define is_v2_FMULWC__rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4001c000)
#define is_v2_FMULWC__rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40015000)
#define is_v2_FMULWC__rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4001d000)
#define is_v2_FMULWC__rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40010000)
#define is_v2_FMULWC__rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40018000)
#define is_v2_FMULWC__ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40016000)
#define is_v2_FMULWC__ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4001e000)
#define is_v2_FMULWC__ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40011000)
#define is_v2_FMULWC__ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40019000)
#define is_v2_FMULWC__rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40013000)
#define is_v2_FMULWC__rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4001b000)
#define is_v2_FMULWC____registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x40017000)
#define is_v2_FMULWC___s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4001f000)
#define v2_FMULWCP_c_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4102a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4102c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4102d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4102e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4102b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x41027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP_c__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4102f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4002a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4002c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4002d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4002e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP__rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4002b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP____registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x40027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMULWCP___s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4002f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FMULWCP_c_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41022000)
#define is_v2_FMULWCP_c_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4102a000)
#define is_v2_FMULWCP_c_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41024000)
#define is_v2_FMULWCP_c_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4102c000)
#define is_v2_FMULWCP_c_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41025000)
#define is_v2_FMULWCP_c_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4102d000)
#define is_v2_FMULWCP_c_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41020000)
#define is_v2_FMULWCP_c_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41028000)
#define is_v2_FMULWCP_c_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41026000)
#define is_v2_FMULWCP_c_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4102e000)
#define is_v2_FMULWCP_c_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41021000)
#define is_v2_FMULWCP_c_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41029000)
#define is_v2_FMULWCP_c_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41023000)
#define is_v2_FMULWCP_c_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4102b000)
#define is_v2_FMULWCP_c___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x41027000)
#define is_v2_FMULWCP_c__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4102f000)
#define is_v2_FMULWCP__rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40022000)
#define is_v2_FMULWCP__rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4002a000)
#define is_v2_FMULWCP__rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40024000)
#define is_v2_FMULWCP__rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4002c000)
#define is_v2_FMULWCP__rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40025000)
#define is_v2_FMULWCP__rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4002d000)
#define is_v2_FMULWCP__rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40020000)
#define is_v2_FMULWCP__rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40028000)
#define is_v2_FMULWCP__ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40026000)
#define is_v2_FMULWCP__ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4002e000)
#define is_v2_FMULWCP__ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40021000)
#define is_v2_FMULWCP__ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40029000)
#define is_v2_FMULWCP__rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40023000)
#define is_v2_FMULWCP__rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4002b000)
#define is_v2_FMULWCP____registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x40027000)
#define is_v2_FMULWCP___s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4002f000)
#define v2_FFDMAW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4201a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4201c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4201d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4201e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4201b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x42017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMAW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4201f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFDMAW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42012000)
#define is_v2_FFDMAW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4201a000)
#define is_v2_FFDMAW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42014000)
#define is_v2_FFDMAW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4201c000)
#define is_v2_FFDMAW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42015000)
#define is_v2_FFDMAW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4201d000)
#define is_v2_FFDMAW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42010000)
#define is_v2_FFDMAW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42018000)
#define is_v2_FFDMAW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42016000)
#define is_v2_FFDMAW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4201e000)
#define is_v2_FFDMAW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42011000)
#define is_v2_FFDMAW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42019000)
#define is_v2_FFDMAW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42013000)
#define is_v2_FFDMAW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4201b000)
#define is_v2_FFDMAW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x42017000)
#define is_v2_FFDMAW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4201f000)
#define v2_FFDMAWP_rd__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42022000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rd_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4202a000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rna__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42024000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rna_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4202c000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rnz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42025000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rnz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4202d000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rn__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42020000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rn_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42028000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_ro__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42026000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_ro_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4202e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_ru__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42021000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_ru_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42029000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42023000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP_rz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4202b000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP___registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x42027000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMAWP__s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4202f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFDMAWP_rd__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42022000)
#define is_v2_FFDMAWP_rd_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4202a000)
#define is_v2_FFDMAWP_rna__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42024000)
#define is_v2_FFDMAWP_rna_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4202c000)
#define is_v2_FFDMAWP_rnz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42025000)
#define is_v2_FFDMAWP_rnz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4202d000)
#define is_v2_FFDMAWP_rn__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42020000)
#define is_v2_FFDMAWP_rn_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42028000)
#define is_v2_FFDMAWP_ro__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42026000)
#define is_v2_FFDMAWP_ro_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4202e000)
#define is_v2_FFDMAWP_ru__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42021000)
#define is_v2_FFDMAWP_ru_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42029000)
#define is_v2_FFDMAWP_rz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42023000)
#define is_v2_FFDMAWP_rz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4202b000)
#define is_v2_FFDMAWP___registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x42027000)
#define is_v2_FFDMAWP__s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4202f000)
#define v2_FFDMAWQ_rd__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rd_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4202a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rna__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rna_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4202c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rnz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rnz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4202d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rn__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rn_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_ro__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_ro_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4202e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_ru__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_ru_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ_rz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4202b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ___registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x42027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMAWQ__s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4202f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define is_v2_FFDMAWQ_rd__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42022040)
#define is_v2_FFDMAWQ_rd_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4202a040)
#define is_v2_FFDMAWQ_rna__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42024040)
#define is_v2_FFDMAWQ_rna_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4202c040)
#define is_v2_FFDMAWQ_rnz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42025040)
#define is_v2_FFDMAWQ_rnz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4202d040)
#define is_v2_FFDMAWQ_rn__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42020040)
#define is_v2_FFDMAWQ_rn_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42028040)
#define is_v2_FFDMAWQ_ro__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42026040)
#define is_v2_FFDMAWQ_ro_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4202e040)
#define is_v2_FFDMAWQ_ru__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42021040)
#define is_v2_FFDMAWQ_ru_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42029040)
#define is_v2_FFDMAWQ_rz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42023040)
#define is_v2_FFDMAWQ_rz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4202b040)
#define is_v2_FFDMAWQ___registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x42027040)
#define is_v2_FFDMAWQ__s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4202f040)
#define v2_FFDMSW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4301a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4301c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4301d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4301e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4301b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x43017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4301f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFDMSW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43012000)
#define is_v2_FFDMSW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4301a000)
#define is_v2_FFDMSW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43014000)
#define is_v2_FFDMSW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4301c000)
#define is_v2_FFDMSW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43015000)
#define is_v2_FFDMSW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4301d000)
#define is_v2_FFDMSW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43010000)
#define is_v2_FFDMSW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43018000)
#define is_v2_FFDMSW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43016000)
#define is_v2_FFDMSW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4301e000)
#define is_v2_FFDMSW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43011000)
#define is_v2_FFDMSW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43019000)
#define is_v2_FFDMSW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43013000)
#define is_v2_FFDMSW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4301b000)
#define is_v2_FFDMSW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x43017000)
#define is_v2_FFDMSW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4301f000)
#define v2_FFDMSWP_rd__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43022000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rd_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4302a000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rna__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43024000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rna_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4302c000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rnz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43025000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rnz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4302d000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rn__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43020000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rn_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43028000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_ro__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43026000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_ro_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4302e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_ru__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43021000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_ru_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43029000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43023000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP_rz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4302b000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP___registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x43027000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSWP__s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4302f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFDMSWP_rd__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43022000)
#define is_v2_FFDMSWP_rd_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4302a000)
#define is_v2_FFDMSWP_rna__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43024000)
#define is_v2_FFDMSWP_rna_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4302c000)
#define is_v2_FFDMSWP_rnz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43025000)
#define is_v2_FFDMSWP_rnz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4302d000)
#define is_v2_FFDMSWP_rn__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43020000)
#define is_v2_FFDMSWP_rn_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43028000)
#define is_v2_FFDMSWP_ro__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43026000)
#define is_v2_FFDMSWP_ro_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4302e000)
#define is_v2_FFDMSWP_ru__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43021000)
#define is_v2_FFDMSWP_ru_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43029000)
#define is_v2_FFDMSWP_rz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43023000)
#define is_v2_FFDMSWP_rz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4302b000)
#define is_v2_FFDMSWP___registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x43027000)
#define is_v2_FFDMSWP__s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4302f000)
#define v2_FFDMSWQ_rd__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rd_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4302a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rna__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rna_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4302c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rnz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rnz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4302d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rn__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rn_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_ro__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_ro_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4302e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_ru__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_ru_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ_rz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4302b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ___registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x43027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSWQ__s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4302f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define is_v2_FFDMSWQ_rd__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43022040)
#define is_v2_FFDMSWQ_rd_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4302a040)
#define is_v2_FFDMSWQ_rna__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43024040)
#define is_v2_FFDMSWQ_rna_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4302c040)
#define is_v2_FFDMSWQ_rnz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43025040)
#define is_v2_FFDMSWQ_rnz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4302d040)
#define is_v2_FFDMSWQ_rn__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43020040)
#define is_v2_FFDMSWQ_rn_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43028040)
#define is_v2_FFDMSWQ_ro__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43026040)
#define is_v2_FFDMSWQ_ro_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4302e040)
#define is_v2_FFDMSWQ_ru__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43021040)
#define is_v2_FFDMSWQ_ru_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43029040)
#define is_v2_FFDMSWQ_rz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43023040)
#define is_v2_FFDMSWQ_rz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4302b040)
#define is_v2_FFDMSWQ___registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x43027040)
#define is_v2_FFDMSWQ__s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4302f040)
#define v2_FFDMDAW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4401a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4401c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4401d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4401e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4401b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x44017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDAW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4401f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFDMDAW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44012000)
#define is_v2_FFDMDAW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4401a000)
#define is_v2_FFDMDAW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44014000)
#define is_v2_FFDMDAW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4401c000)
#define is_v2_FFDMDAW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44015000)
#define is_v2_FFDMDAW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4401d000)
#define is_v2_FFDMDAW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44010000)
#define is_v2_FFDMDAW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44018000)
#define is_v2_FFDMDAW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44016000)
#define is_v2_FFDMDAW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4401e000)
#define is_v2_FFDMDAW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44011000)
#define is_v2_FFDMDAW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44019000)
#define is_v2_FFDMDAW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44013000)
#define is_v2_FFDMDAW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4401b000)
#define is_v2_FFDMDAW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x44017000)
#define is_v2_FFDMDAW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4401f000)
#define v2_FFDMDAWP_rd__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44022000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rd_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4402a000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rna__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44024000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rna_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4402c000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rnz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44025000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rnz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4402d000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rn__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44020000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rn_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44028000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_ro__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44026000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_ro_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4402e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_ru__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44021000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_ru_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44029000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44023000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP_rz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4402b000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP___registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x44027000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDAWP__s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4402f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFDMDAWP_rd__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44022000)
#define is_v2_FFDMDAWP_rd_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4402a000)
#define is_v2_FFDMDAWP_rna__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44024000)
#define is_v2_FFDMDAWP_rna_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4402c000)
#define is_v2_FFDMDAWP_rnz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44025000)
#define is_v2_FFDMDAWP_rnz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4402d000)
#define is_v2_FFDMDAWP_rn__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44020000)
#define is_v2_FFDMDAWP_rn_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44028000)
#define is_v2_FFDMDAWP_ro__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44026000)
#define is_v2_FFDMDAWP_ro_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4402e000)
#define is_v2_FFDMDAWP_ru__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44021000)
#define is_v2_FFDMDAWP_ru_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44029000)
#define is_v2_FFDMDAWP_rz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44023000)
#define is_v2_FFDMDAWP_rz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4402b000)
#define is_v2_FFDMDAWP___registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x44027000)
#define is_v2_FFDMDAWP__s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4402f000)
#define v2_FFDMDAWQ_rd__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rd_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4402a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rna__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rna_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4402c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rnz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rnz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4402d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rn__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rn_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_ro__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_ro_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4402e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_ru__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_ru_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ_rz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4402b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ___registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x44027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDAWQ__s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4402f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define is_v2_FFDMDAWQ_rd__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44022040)
#define is_v2_FFDMDAWQ_rd_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4402a040)
#define is_v2_FFDMDAWQ_rna__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44024040)
#define is_v2_FFDMDAWQ_rna_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4402c040)
#define is_v2_FFDMDAWQ_rnz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44025040)
#define is_v2_FFDMDAWQ_rnz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4402d040)
#define is_v2_FFDMDAWQ_rn__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44020040)
#define is_v2_FFDMDAWQ_rn_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44028040)
#define is_v2_FFDMDAWQ_ro__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44026040)
#define is_v2_FFDMDAWQ_ro_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4402e040)
#define is_v2_FFDMDAWQ_ru__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44021040)
#define is_v2_FFDMDAWQ_ru_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44029040)
#define is_v2_FFDMDAWQ_rz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44023040)
#define is_v2_FFDMDAWQ_rz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4402b040)
#define is_v2_FFDMDAWQ___registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x44027040)
#define is_v2_FFDMDAWQ__s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4402f040)
#define v2_FFDMSAW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4501a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4501c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4501d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4501e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4501b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x45017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMSAW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4501f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFDMSAW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45012000)
#define is_v2_FFDMSAW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4501a000)
#define is_v2_FFDMSAW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45014000)
#define is_v2_FFDMSAW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4501c000)
#define is_v2_FFDMSAW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45015000)
#define is_v2_FFDMSAW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4501d000)
#define is_v2_FFDMSAW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45010000)
#define is_v2_FFDMSAW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45018000)
#define is_v2_FFDMSAW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45016000)
#define is_v2_FFDMSAW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4501e000)
#define is_v2_FFDMSAW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45011000)
#define is_v2_FFDMSAW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45019000)
#define is_v2_FFDMSAW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45013000)
#define is_v2_FFDMSAW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4501b000)
#define is_v2_FFDMSAW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x45017000)
#define is_v2_FFDMSAW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4501f000)
#define v2_FFDMSAWP_rd__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45022000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rd_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4502a000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rna__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45024000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rna_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4502c000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rnz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45025000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rnz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4502d000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rn__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45020000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rn_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45028000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_ro__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45026000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_ro_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4502e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_ru__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45021000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_ru_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45029000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45023000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP_rz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4502b000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP___registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x45027000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMSAWP__s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4502f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFDMSAWP_rd__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45022000)
#define is_v2_FFDMSAWP_rd_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4502a000)
#define is_v2_FFDMSAWP_rna__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45024000)
#define is_v2_FFDMSAWP_rna_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4502c000)
#define is_v2_FFDMSAWP_rnz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45025000)
#define is_v2_FFDMSAWP_rnz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4502d000)
#define is_v2_FFDMSAWP_rn__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45020000)
#define is_v2_FFDMSAWP_rn_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45028000)
#define is_v2_FFDMSAWP_ro__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45026000)
#define is_v2_FFDMSAWP_ro_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4502e000)
#define is_v2_FFDMSAWP_ru__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45021000)
#define is_v2_FFDMSAWP_ru_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45029000)
#define is_v2_FFDMSAWP_rz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45023000)
#define is_v2_FFDMSAWP_rz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4502b000)
#define is_v2_FFDMSAWP___registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x45027000)
#define is_v2_FFDMSAWP__s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4502f000)
#define v2_FFDMSAWQ_rd__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rd_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4502a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rna__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rna_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4502c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rnz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rnz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4502d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rn__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rn_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_ro__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_ro_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4502e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_ru__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_ru_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ_rz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4502b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ___registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x45027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMSAWQ__s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4502f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define is_v2_FFDMSAWQ_rd__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45022040)
#define is_v2_FFDMSAWQ_rd_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4502a040)
#define is_v2_FFDMSAWQ_rna__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45024040)
#define is_v2_FFDMSAWQ_rna_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4502c040)
#define is_v2_FFDMSAWQ_rnz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45025040)
#define is_v2_FFDMSAWQ_rnz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4502d040)
#define is_v2_FFDMSAWQ_rn__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45020040)
#define is_v2_FFDMSAWQ_rn_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45028040)
#define is_v2_FFDMSAWQ_ro__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45026040)
#define is_v2_FFDMSAWQ_ro_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4502e040)
#define is_v2_FFDMSAWQ_ru__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45021040)
#define is_v2_FFDMSAWQ_ru_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45029040)
#define is_v2_FFDMSAWQ_rz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45023040)
#define is_v2_FFDMSAWQ_rz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4502b040)
#define is_v2_FFDMSAWQ___registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x45027040)
#define is_v2_FFDMSAWQ__s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4502f040)
#define v2_FFDMDSW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4601a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4601c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4601d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4601e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4601b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x46017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMDSW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4601f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFDMDSW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46012000)
#define is_v2_FFDMDSW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4601a000)
#define is_v2_FFDMDSW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46014000)
#define is_v2_FFDMDSW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4601c000)
#define is_v2_FFDMDSW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46015000)
#define is_v2_FFDMDSW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4601d000)
#define is_v2_FFDMDSW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46010000)
#define is_v2_FFDMDSW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46018000)
#define is_v2_FFDMDSW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46016000)
#define is_v2_FFDMDSW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4601e000)
#define is_v2_FFDMDSW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46011000)
#define is_v2_FFDMDSW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46019000)
#define is_v2_FFDMDSW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46013000)
#define is_v2_FFDMDSW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4601b000)
#define is_v2_FFDMDSW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x46017000)
#define is_v2_FFDMDSW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4601f000)
#define v2_FFDMDSWP_rd__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46022000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rd_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4602a000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rna__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46024000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rna_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4602c000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rnz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46025000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rnz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4602d000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rn__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46020000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rn_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46028000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_ro__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46026000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_ro_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4602e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_ru__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46021000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_ru_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46029000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46023000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP_rz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4602b000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP___registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x46027000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMDSWP__s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4602f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFDMDSWP_rd__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46022000)
#define is_v2_FFDMDSWP_rd_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4602a000)
#define is_v2_FFDMDSWP_rna__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46024000)
#define is_v2_FFDMDSWP_rna_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4602c000)
#define is_v2_FFDMDSWP_rnz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46025000)
#define is_v2_FFDMDSWP_rnz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4602d000)
#define is_v2_FFDMDSWP_rn__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46020000)
#define is_v2_FFDMDSWP_rn_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46028000)
#define is_v2_FFDMDSWP_ro__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46026000)
#define is_v2_FFDMDSWP_ro_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4602e000)
#define is_v2_FFDMDSWP_ru__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46021000)
#define is_v2_FFDMDSWP_ru_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46029000)
#define is_v2_FFDMDSWP_rz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46023000)
#define is_v2_FFDMDSWP_rz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4602b000)
#define is_v2_FFDMDSWP___registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x46027000)
#define is_v2_FFDMDSWP__s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4602f000)
#define v2_FFDMDSWQ_rd__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rd_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4602a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rna__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rna_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4602c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rnz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rnz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4602d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rn__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rn_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_ro__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_ro_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4602e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_ru__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_ru_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ_rz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4602b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ___registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x46027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMDSWQ__s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4602f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define is_v2_FFDMDSWQ_rd__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46022040)
#define is_v2_FFDMDSWQ_rd_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4602a040)
#define is_v2_FFDMDSWQ_rna__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46024040)
#define is_v2_FFDMDSWQ_rna_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4602c040)
#define is_v2_FFDMDSWQ_rnz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46025040)
#define is_v2_FFDMDSWQ_rnz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4602d040)
#define is_v2_FFDMDSWQ_rn__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46020040)
#define is_v2_FFDMDSWQ_rn_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46028040)
#define is_v2_FFDMDSWQ_ro__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46026040)
#define is_v2_FFDMDSWQ_ro_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4602e040)
#define is_v2_FFDMDSWQ_ru__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46021040)
#define is_v2_FFDMDSWQ_ru_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46029040)
#define is_v2_FFDMDSWQ_rz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46023040)
#define is_v2_FFDMDSWQ_rz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4602b040)
#define is_v2_FFDMDSWQ___registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x46027040)
#define is_v2_FFDMDSWQ__s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4602f040)
#define v2_FFDMASW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4701a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4701c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4701d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4701e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4701b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x47017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFDMASW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4701f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFDMASW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47012000)
#define is_v2_FFDMASW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4701a000)
#define is_v2_FFDMASW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47014000)
#define is_v2_FFDMASW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4701c000)
#define is_v2_FFDMASW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47015000)
#define is_v2_FFDMASW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4701d000)
#define is_v2_FFDMASW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47010000)
#define is_v2_FFDMASW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47018000)
#define is_v2_FFDMASW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47016000)
#define is_v2_FFDMASW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4701e000)
#define is_v2_FFDMASW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47011000)
#define is_v2_FFDMASW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47019000)
#define is_v2_FFDMASW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47013000)
#define is_v2_FFDMASW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4701b000)
#define is_v2_FFDMASW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x47017000)
#define is_v2_FFDMASW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4701f000)
#define v2_FFDMASWP_rd__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47022000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rd_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4702a000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rna__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47024000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rna_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4702c000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rnz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47025000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rnz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4702d000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rn__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47020000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rn_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47028000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_ro__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47026000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_ro_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4702e000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_ru__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47021000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_ru_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47029000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rz__registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47023000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP_rz_s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4702b000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP___registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x47027000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFDMASWP__s_registerW_registerP_registerO_simple_insn(registerW,registerP,registerO) (unsigned int)(0x4702f000 | (((int)(registerW) & 0x3f) << 18) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFDMASWP_rd__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47022000)
#define is_v2_FFDMASWP_rd_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4702a000)
#define is_v2_FFDMASWP_rna__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47024000)
#define is_v2_FFDMASWP_rna_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4702c000)
#define is_v2_FFDMASWP_rnz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47025000)
#define is_v2_FFDMASWP_rnz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4702d000)
#define is_v2_FFDMASWP_rn__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47020000)
#define is_v2_FFDMASWP_rn_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47028000)
#define is_v2_FFDMASWP_ro__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47026000)
#define is_v2_FFDMASWP_ro_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4702e000)
#define is_v2_FFDMASWP_ru__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47021000)
#define is_v2_FFDMASWP_ru_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47029000)
#define is_v2_FFDMASWP_rz__registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47023000)
#define is_v2_FFDMASWP_rz_s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4702b000)
#define is_v2_FFDMASWP___registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x47027000)
#define is_v2_FFDMASWP__s_registerW_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f040) == 0x4702f000)
#define v2_FFDMASWQ_rd__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rd_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4702a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rna__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rna_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4702c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rnz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rnz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4702d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rn__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rn_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_ro__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_ro_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4702e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_ru__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_ru_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rz__registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ_rz_s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4702b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ___registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x47027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define v2_FFDMASWQ__s_registerM_registerQ_registerR_simple_insn(registerM,registerQ,registerR) (unsigned int)(0x4702f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerQ) & 0xf) << 8) | (((int)(registerR) & 0xf) << 2))
#define is_v2_FFDMASWQ_rd__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47022040)
#define is_v2_FFDMASWQ_rd_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4702a040)
#define is_v2_FFDMASWQ_rna__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47024040)
#define is_v2_FFDMASWQ_rna_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4702c040)
#define is_v2_FFDMASWQ_rnz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47025040)
#define is_v2_FFDMASWQ_rnz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4702d040)
#define is_v2_FFDMASWQ_rn__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47020040)
#define is_v2_FFDMASWQ_rn_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47028040)
#define is_v2_FFDMASWQ_ro__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47026040)
#define is_v2_FFDMASWQ_ro_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4702e040)
#define is_v2_FFDMASWQ_ru__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47021040)
#define is_v2_FFDMASWQ_ru_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47029040)
#define is_v2_FFDMASWQ_rz__registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47023040)
#define is_v2_FFDMASWQ_rz_s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4702b040)
#define is_v2_FFDMASWQ___registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x47027040)
#define is_v2_FFDMASWQ__s_registerM_registerQ_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4702f040)
#define v2_FMM212W_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c012000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c01a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c014000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c01c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c015000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c01d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c010000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c018000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c016000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c01e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c011000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c019000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c013000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c01b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c017000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMM212W__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4c01f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMM212W_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c012000)
#define is_v2_FMM212W_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c01a000)
#define is_v2_FMM212W_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c014000)
#define is_v2_FMM212W_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c01c000)
#define is_v2_FMM212W_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c015000)
#define is_v2_FMM212W_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c01d000)
#define is_v2_FMM212W_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c010000)
#define is_v2_FMM212W_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c018000)
#define is_v2_FMM212W_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c016000)
#define is_v2_FMM212W_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c01e000)
#define is_v2_FMM212W_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c011000)
#define is_v2_FMM212W_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c019000)
#define is_v2_FMM212W_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c013000)
#define is_v2_FMM212W_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c01b000)
#define is_v2_FMM212W___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c017000)
#define is_v2_FMM212W__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4c01f000)
#define v2_FMM222W_nt_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c062000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c064000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c065000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c060000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c068000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c066000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c061000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c069000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c063000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c067000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_nt__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tn__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c062040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c064040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c065040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c060040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c068040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c066040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c061040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c069040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c063040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c067040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W_tt__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c06f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W__rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W____registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMM222W___s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4c02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FMM222W_nt_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c062000)
#define is_v2_FMM222W_nt_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06a000)
#define is_v2_FMM222W_nt_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c064000)
#define is_v2_FMM222W_nt_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06c000)
#define is_v2_FMM222W_nt_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c065000)
#define is_v2_FMM222W_nt_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06d000)
#define is_v2_FMM222W_nt_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c060000)
#define is_v2_FMM222W_nt_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c068000)
#define is_v2_FMM222W_nt_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c066000)
#define is_v2_FMM222W_nt_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06e000)
#define is_v2_FMM222W_nt_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c061000)
#define is_v2_FMM222W_nt_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c069000)
#define is_v2_FMM222W_nt_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c063000)
#define is_v2_FMM222W_nt_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06b000)
#define is_v2_FMM222W_nt___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c067000)
#define is_v2_FMM222W_nt__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06f000)
#define is_v2_FMM222W_tn_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c022040)
#define is_v2_FMM222W_tn_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02a040)
#define is_v2_FMM222W_tn_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c024040)
#define is_v2_FMM222W_tn_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02c040)
#define is_v2_FMM222W_tn_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c025040)
#define is_v2_FMM222W_tn_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02d040)
#define is_v2_FMM222W_tn_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c020040)
#define is_v2_FMM222W_tn_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c028040)
#define is_v2_FMM222W_tn_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c026040)
#define is_v2_FMM222W_tn_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02e040)
#define is_v2_FMM222W_tn_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c021040)
#define is_v2_FMM222W_tn_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c029040)
#define is_v2_FMM222W_tn_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c023040)
#define is_v2_FMM222W_tn_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02b040)
#define is_v2_FMM222W_tn___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c027040)
#define is_v2_FMM222W_tn__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02f040)
#define is_v2_FMM222W_tt_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c062040)
#define is_v2_FMM222W_tt_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06a040)
#define is_v2_FMM222W_tt_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c064040)
#define is_v2_FMM222W_tt_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06c040)
#define is_v2_FMM222W_tt_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c065040)
#define is_v2_FMM222W_tt_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06d040)
#define is_v2_FMM222W_tt_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c060040)
#define is_v2_FMM222W_tt_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c068040)
#define is_v2_FMM222W_tt_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c066040)
#define is_v2_FMM222W_tt_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06e040)
#define is_v2_FMM222W_tt_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c061040)
#define is_v2_FMM222W_tt_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c069040)
#define is_v2_FMM222W_tt_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c063040)
#define is_v2_FMM222W_tt_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06b040)
#define is_v2_FMM222W_tt___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c067040)
#define is_v2_FMM222W_tt__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c06f040)
#define is_v2_FMM222W__rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c022000)
#define is_v2_FMM222W__rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02a000)
#define is_v2_FMM222W__rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c024000)
#define is_v2_FMM222W__rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02c000)
#define is_v2_FMM222W__rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c025000)
#define is_v2_FMM222W__rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02d000)
#define is_v2_FMM222W__rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c020000)
#define is_v2_FMM222W__rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c028000)
#define is_v2_FMM222W__ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c026000)
#define is_v2_FMM222W__ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02e000)
#define is_v2_FMM222W__ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c021000)
#define is_v2_FMM222W__ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c029000)
#define is_v2_FMM222W__rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c023000)
#define is_v2_FMM222W__rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02b000)
#define is_v2_FMM222W____registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c027000)
#define is_v2_FMM222W___s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4c02f000)
#define v2_FMMA212W_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e012000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e01a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e014000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e01c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e015000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e01d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e010000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e018000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e016000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e01e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e011000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e019000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e013000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e01b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e017000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMA212W__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4e01f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMMA212W_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e012000)
#define is_v2_FMMA212W_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e01a000)
#define is_v2_FMMA212W_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e014000)
#define is_v2_FMMA212W_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e01c000)
#define is_v2_FMMA212W_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e015000)
#define is_v2_FMMA212W_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e01d000)
#define is_v2_FMMA212W_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e010000)
#define is_v2_FMMA212W_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e018000)
#define is_v2_FMMA212W_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e016000)
#define is_v2_FMMA212W_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e01e000)
#define is_v2_FMMA212W_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e011000)
#define is_v2_FMMA212W_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e019000)
#define is_v2_FMMA212W_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e013000)
#define is_v2_FMMA212W_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e01b000)
#define is_v2_FMMA212W___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e017000)
#define is_v2_FMMA212W__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4e01f000)
#define v2_FMMA222W_nt_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e062000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e064000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e065000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e060000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e068000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e066000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e061000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e069000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e063000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e067000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_nt__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tn__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e062040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e064040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e065040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e060040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e068040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e066040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e061040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e069040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e063040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e067040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W_tt__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e06f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W__rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W____registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMA222W___s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4e02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FMMA222W_nt_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e062000)
#define is_v2_FMMA222W_nt_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06a000)
#define is_v2_FMMA222W_nt_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e064000)
#define is_v2_FMMA222W_nt_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06c000)
#define is_v2_FMMA222W_nt_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e065000)
#define is_v2_FMMA222W_nt_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06d000)
#define is_v2_FMMA222W_nt_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e060000)
#define is_v2_FMMA222W_nt_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e068000)
#define is_v2_FMMA222W_nt_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e066000)
#define is_v2_FMMA222W_nt_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06e000)
#define is_v2_FMMA222W_nt_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e061000)
#define is_v2_FMMA222W_nt_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e069000)
#define is_v2_FMMA222W_nt_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e063000)
#define is_v2_FMMA222W_nt_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06b000)
#define is_v2_FMMA222W_nt___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e067000)
#define is_v2_FMMA222W_nt__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06f000)
#define is_v2_FMMA222W_tn_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e022040)
#define is_v2_FMMA222W_tn_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02a040)
#define is_v2_FMMA222W_tn_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e024040)
#define is_v2_FMMA222W_tn_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02c040)
#define is_v2_FMMA222W_tn_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e025040)
#define is_v2_FMMA222W_tn_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02d040)
#define is_v2_FMMA222W_tn_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e020040)
#define is_v2_FMMA222W_tn_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e028040)
#define is_v2_FMMA222W_tn_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e026040)
#define is_v2_FMMA222W_tn_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02e040)
#define is_v2_FMMA222W_tn_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e021040)
#define is_v2_FMMA222W_tn_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e029040)
#define is_v2_FMMA222W_tn_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e023040)
#define is_v2_FMMA222W_tn_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02b040)
#define is_v2_FMMA222W_tn___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e027040)
#define is_v2_FMMA222W_tn__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02f040)
#define is_v2_FMMA222W_tt_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e062040)
#define is_v2_FMMA222W_tt_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06a040)
#define is_v2_FMMA222W_tt_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e064040)
#define is_v2_FMMA222W_tt_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06c040)
#define is_v2_FMMA222W_tt_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e065040)
#define is_v2_FMMA222W_tt_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06d040)
#define is_v2_FMMA222W_tt_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e060040)
#define is_v2_FMMA222W_tt_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e068040)
#define is_v2_FMMA222W_tt_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e066040)
#define is_v2_FMMA222W_tt_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06e040)
#define is_v2_FMMA222W_tt_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e061040)
#define is_v2_FMMA222W_tt_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e069040)
#define is_v2_FMMA222W_tt_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e063040)
#define is_v2_FMMA222W_tt_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06b040)
#define is_v2_FMMA222W_tt___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e067040)
#define is_v2_FMMA222W_tt__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e06f040)
#define is_v2_FMMA222W__rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e022000)
#define is_v2_FMMA222W__rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02a000)
#define is_v2_FMMA222W__rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e024000)
#define is_v2_FMMA222W__rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02c000)
#define is_v2_FMMA222W__rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e025000)
#define is_v2_FMMA222W__rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02d000)
#define is_v2_FMMA222W__rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e020000)
#define is_v2_FMMA222W__rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e028000)
#define is_v2_FMMA222W__ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e026000)
#define is_v2_FMMA222W__ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02e000)
#define is_v2_FMMA222W__ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e021000)
#define is_v2_FMMA222W__ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e029000)
#define is_v2_FMMA222W__rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e023000)
#define is_v2_FMMA222W__rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02b000)
#define is_v2_FMMA222W____registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e027000)
#define is_v2_FMMA222W___s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4e02f000)
#define v2_FMMS212W_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f012000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f01a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f014000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f01c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f015000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f01d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f010000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f018000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f016000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f01e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f011000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f019000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f013000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f01b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f017000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMMS212W__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x4f01f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMMS212W_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f012000)
#define is_v2_FMMS212W_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f01a000)
#define is_v2_FMMS212W_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f014000)
#define is_v2_FMMS212W_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f01c000)
#define is_v2_FMMS212W_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f015000)
#define is_v2_FMMS212W_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f01d000)
#define is_v2_FMMS212W_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f010000)
#define is_v2_FMMS212W_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f018000)
#define is_v2_FMMS212W_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f016000)
#define is_v2_FMMS212W_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f01e000)
#define is_v2_FMMS212W_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f011000)
#define is_v2_FMMS212W_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f019000)
#define is_v2_FMMS212W_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f013000)
#define is_v2_FMMS212W_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f01b000)
#define is_v2_FMMS212W___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f017000)
#define is_v2_FMMS212W__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x4f01f000)
#define v2_FMMS222W_nt_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f062000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f064000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f065000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f060000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f068000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f066000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f061000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f069000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f063000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f067000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_nt__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f022040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f024040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f025040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f020040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f028040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f026040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f021040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f029040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f023040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f027040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tn__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f062040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06a040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f064040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06c040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f065040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06d040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f060040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f068040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f066040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06e040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f061040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f069040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f063040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06b040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f067040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W_tt__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f06f040 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W__rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W____registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FMMS222W___s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4f02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FMMS222W_nt_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f062000)
#define is_v2_FMMS222W_nt_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06a000)
#define is_v2_FMMS222W_nt_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f064000)
#define is_v2_FMMS222W_nt_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06c000)
#define is_v2_FMMS222W_nt_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f065000)
#define is_v2_FMMS222W_nt_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06d000)
#define is_v2_FMMS222W_nt_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f060000)
#define is_v2_FMMS222W_nt_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f068000)
#define is_v2_FMMS222W_nt_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f066000)
#define is_v2_FMMS222W_nt_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06e000)
#define is_v2_FMMS222W_nt_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f061000)
#define is_v2_FMMS222W_nt_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f069000)
#define is_v2_FMMS222W_nt_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f063000)
#define is_v2_FMMS222W_nt_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06b000)
#define is_v2_FMMS222W_nt___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f067000)
#define is_v2_FMMS222W_nt__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06f000)
#define is_v2_FMMS222W_tn_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f022040)
#define is_v2_FMMS222W_tn_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02a040)
#define is_v2_FMMS222W_tn_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f024040)
#define is_v2_FMMS222W_tn_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02c040)
#define is_v2_FMMS222W_tn_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f025040)
#define is_v2_FMMS222W_tn_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02d040)
#define is_v2_FMMS222W_tn_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f020040)
#define is_v2_FMMS222W_tn_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f028040)
#define is_v2_FMMS222W_tn_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f026040)
#define is_v2_FMMS222W_tn_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02e040)
#define is_v2_FMMS222W_tn_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f021040)
#define is_v2_FMMS222W_tn_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f029040)
#define is_v2_FMMS222W_tn_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f023040)
#define is_v2_FMMS222W_tn_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02b040)
#define is_v2_FMMS222W_tn___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f027040)
#define is_v2_FMMS222W_tn__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02f040)
#define is_v2_FMMS222W_tt_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f062040)
#define is_v2_FMMS222W_tt_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06a040)
#define is_v2_FMMS222W_tt_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f064040)
#define is_v2_FMMS222W_tt_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06c040)
#define is_v2_FMMS222W_tt_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f065040)
#define is_v2_FMMS222W_tt_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06d040)
#define is_v2_FMMS222W_tt_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f060040)
#define is_v2_FMMS222W_tt_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f068040)
#define is_v2_FMMS222W_tt_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f066040)
#define is_v2_FMMS222W_tt_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06e040)
#define is_v2_FMMS222W_tt_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f061040)
#define is_v2_FMMS222W_tt_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f069040)
#define is_v2_FMMS222W_tt_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f063040)
#define is_v2_FMMS222W_tt_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06b040)
#define is_v2_FMMS222W_tt___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f067040)
#define is_v2_FMMS222W_tt__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f06f040)
#define is_v2_FMMS222W__rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f022000)
#define is_v2_FMMS222W__rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02a000)
#define is_v2_FMMS222W__rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f024000)
#define is_v2_FMMS222W__rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02c000)
#define is_v2_FMMS222W__rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f025000)
#define is_v2_FMMS222W__rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02d000)
#define is_v2_FMMS222W__rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f020000)
#define is_v2_FMMS222W__rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f028000)
#define is_v2_FMMS222W__ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f026000)
#define is_v2_FMMS222W__ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02e000)
#define is_v2_FMMS222W__ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f021000)
#define is_v2_FMMS222W__ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f029000)
#define is_v2_FMMS222W__rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f023000)
#define is_v2_FMMS222W__rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02b000)
#define is_v2_FMMS222W____registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f027000)
#define is_v2_FMMS222W___s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4f02f000)
#define v2_FFMAWC_c_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4901a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4901c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4901d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4901e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4901b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x49017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC_c__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4901f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4801a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4801c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4801d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4801e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC__rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4801b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC____registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x48017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWC___s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4801f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAWC_c_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49012000)
#define is_v2_FFMAWC_c_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4901a000)
#define is_v2_FFMAWC_c_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49014000)
#define is_v2_FFMAWC_c_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4901c000)
#define is_v2_FFMAWC_c_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49015000)
#define is_v2_FFMAWC_c_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4901d000)
#define is_v2_FFMAWC_c_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49010000)
#define is_v2_FFMAWC_c_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49018000)
#define is_v2_FFMAWC_c_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49016000)
#define is_v2_FFMAWC_c_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4901e000)
#define is_v2_FFMAWC_c_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49011000)
#define is_v2_FFMAWC_c_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49019000)
#define is_v2_FFMAWC_c_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49013000)
#define is_v2_FFMAWC_c_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4901b000)
#define is_v2_FFMAWC_c___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x49017000)
#define is_v2_FFMAWC_c__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4901f000)
#define is_v2_FFMAWC__rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48012000)
#define is_v2_FFMAWC__rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4801a000)
#define is_v2_FFMAWC__rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48014000)
#define is_v2_FFMAWC__rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4801c000)
#define is_v2_FFMAWC__rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48015000)
#define is_v2_FFMAWC__rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4801d000)
#define is_v2_FFMAWC__rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48010000)
#define is_v2_FFMAWC__rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48018000)
#define is_v2_FFMAWC__ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48016000)
#define is_v2_FFMAWC__ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4801e000)
#define is_v2_FFMAWC__ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48011000)
#define is_v2_FFMAWC__ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48019000)
#define is_v2_FFMAWC__rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48013000)
#define is_v2_FFMAWC__rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4801b000)
#define is_v2_FFMAWC____registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x48017000)
#define is_v2_FFMAWC___s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4801f000)
#define v2_FFMAWCP_c_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4902a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4902c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4902d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4902e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4902b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x49027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP_c__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4902f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4802a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4802c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4802d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4802e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP__rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4802b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP____registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x48027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMAWCP___s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4802f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFMAWCP_c_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49022000)
#define is_v2_FFMAWCP_c_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4902a000)
#define is_v2_FFMAWCP_c_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49024000)
#define is_v2_FFMAWCP_c_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4902c000)
#define is_v2_FFMAWCP_c_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49025000)
#define is_v2_FFMAWCP_c_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4902d000)
#define is_v2_FFMAWCP_c_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49020000)
#define is_v2_FFMAWCP_c_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49028000)
#define is_v2_FFMAWCP_c_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49026000)
#define is_v2_FFMAWCP_c_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4902e000)
#define is_v2_FFMAWCP_c_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49021000)
#define is_v2_FFMAWCP_c_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49029000)
#define is_v2_FFMAWCP_c_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49023000)
#define is_v2_FFMAWCP_c_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4902b000)
#define is_v2_FFMAWCP_c___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x49027000)
#define is_v2_FFMAWCP_c__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4902f000)
#define is_v2_FFMAWCP__rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48022000)
#define is_v2_FFMAWCP__rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4802a000)
#define is_v2_FFMAWCP__rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48024000)
#define is_v2_FFMAWCP__rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4802c000)
#define is_v2_FFMAWCP__rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48025000)
#define is_v2_FFMAWCP__rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4802d000)
#define is_v2_FFMAWCP__rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48020000)
#define is_v2_FFMAWCP__rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48028000)
#define is_v2_FFMAWCP__ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48026000)
#define is_v2_FFMAWCP__ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4802e000)
#define is_v2_FFMAWCP__ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48021000)
#define is_v2_FFMAWCP__ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48029000)
#define is_v2_FFMAWCP__rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48023000)
#define is_v2_FFMAWCP__rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4802b000)
#define is_v2_FFMAWCP____registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x48027000)
#define is_v2_FFMAWCP___s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4802f000)
#define v2_FFMSWC_c_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC_c__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4b01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC__rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC____registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWC___s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x4a01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSWC_c_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b012000)
#define is_v2_FFMSWC_c_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b01a000)
#define is_v2_FFMSWC_c_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b014000)
#define is_v2_FFMSWC_c_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b01c000)
#define is_v2_FFMSWC_c_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b015000)
#define is_v2_FFMSWC_c_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b01d000)
#define is_v2_FFMSWC_c_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b010000)
#define is_v2_FFMSWC_c_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b018000)
#define is_v2_FFMSWC_c_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b016000)
#define is_v2_FFMSWC_c_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b01e000)
#define is_v2_FFMSWC_c_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b011000)
#define is_v2_FFMSWC_c_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b019000)
#define is_v2_FFMSWC_c_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b013000)
#define is_v2_FFMSWC_c_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b01b000)
#define is_v2_FFMSWC_c___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b017000)
#define is_v2_FFMSWC_c__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4b01f000)
#define is_v2_FFMSWC__rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a012000)
#define is_v2_FFMSWC__rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a01a000)
#define is_v2_FFMSWC__rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a014000)
#define is_v2_FFMSWC__rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a01c000)
#define is_v2_FFMSWC__rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a015000)
#define is_v2_FFMSWC__rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a01d000)
#define is_v2_FFMSWC__rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a010000)
#define is_v2_FFMSWC__rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a018000)
#define is_v2_FFMSWC__ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a016000)
#define is_v2_FFMSWC__ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a01e000)
#define is_v2_FFMSWC__ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a011000)
#define is_v2_FFMSWC__ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a019000)
#define is_v2_FFMSWC__rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a013000)
#define is_v2_FFMSWC__rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a01b000)
#define is_v2_FFMSWC____registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a017000)
#define is_v2_FFMSWC___s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x4a01f000)
#define v2_FFMSWCP_c_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b02a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b02c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b02d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b02b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP_c__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4b02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a022000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a02a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a024000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a02c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a025000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a02d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a020000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a028000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a026000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a02e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a021000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a029000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a023000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP__rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a02b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP____registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a027000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FFMSWCP___s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x4a02f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FFMSWCP_c_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b022000)
#define is_v2_FFMSWCP_c_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b02a000)
#define is_v2_FFMSWCP_c_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b024000)
#define is_v2_FFMSWCP_c_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b02c000)
#define is_v2_FFMSWCP_c_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b025000)
#define is_v2_FFMSWCP_c_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b02d000)
#define is_v2_FFMSWCP_c_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b020000)
#define is_v2_FFMSWCP_c_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b028000)
#define is_v2_FFMSWCP_c_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b026000)
#define is_v2_FFMSWCP_c_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b02e000)
#define is_v2_FFMSWCP_c_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b021000)
#define is_v2_FFMSWCP_c_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b029000)
#define is_v2_FFMSWCP_c_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b023000)
#define is_v2_FFMSWCP_c_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b02b000)
#define is_v2_FFMSWCP_c___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b027000)
#define is_v2_FFMSWCP_c__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4b02f000)
#define is_v2_FFMSWCP__rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a022000)
#define is_v2_FFMSWCP__rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a02a000)
#define is_v2_FFMSWCP__rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a024000)
#define is_v2_FFMSWCP__rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a02c000)
#define is_v2_FFMSWCP__rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a025000)
#define is_v2_FFMSWCP__rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a02d000)
#define is_v2_FFMSWCP__rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a020000)
#define is_v2_FFMSWCP__rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a028000)
#define is_v2_FFMSWCP__ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a026000)
#define is_v2_FFMSWCP__ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a02e000)
#define is_v2_FFMSWCP__ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a021000)
#define is_v2_FFMSWCP__ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a029000)
#define is_v2_FFMSWCP__rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a023000)
#define is_v2_FFMSWCP__rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a02b000)
#define is_v2_FFMSWCP____registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a027000)
#define is_v2_FFMSWCP___s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f040) == 0x4a02f000)
#define v2_FADDD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x50027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5002f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FADDD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50022000)
#define is_v2_FADDD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002a000)
#define is_v2_FADDD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50024000)
#define is_v2_FADDD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002c000)
#define is_v2_FADDD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50025000)
#define is_v2_FADDD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002d000)
#define is_v2_FADDD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50020000)
#define is_v2_FADDD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50028000)
#define is_v2_FADDD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50026000)
#define is_v2_FADDD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002e000)
#define is_v2_FADDD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50021000)
#define is_v2_FADDD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50029000)
#define is_v2_FADDD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50023000)
#define is_v2_FADDD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002b000)
#define is_v2_FADDD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x50027000)
#define is_v2_FADDD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5002f000)
#define v2_FADDWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5102f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FADDWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51022000)
#define is_v2_FADDWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102a000)
#define is_v2_FADDWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51024000)
#define is_v2_FADDWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102c000)
#define is_v2_FADDWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51025000)
#define is_v2_FADDWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102d000)
#define is_v2_FADDWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51020000)
#define is_v2_FADDWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51028000)
#define is_v2_FADDWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51026000)
#define is_v2_FADDWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102e000)
#define is_v2_FADDWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51021000)
#define is_v2_FADDWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51029000)
#define is_v2_FADDWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51023000)
#define is_v2_FADDWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102b000)
#define is_v2_FADDWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51027000)
#define is_v2_FADDWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5102f000)
#define v2_FADDHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x52027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5202f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FADDHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52022000)
#define is_v2_FADDHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202a000)
#define is_v2_FADDHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52024000)
#define is_v2_FADDHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202c000)
#define is_v2_FADDHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52025000)
#define is_v2_FADDHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202d000)
#define is_v2_FADDHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52020000)
#define is_v2_FADDHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52028000)
#define is_v2_FADDHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52026000)
#define is_v2_FADDHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202e000)
#define is_v2_FADDHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52021000)
#define is_v2_FADDHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52029000)
#define is_v2_FADDHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52023000)
#define is_v2_FADDHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202b000)
#define is_v2_FADDHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x52027000)
#define is_v2_FADDHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5202f000)
#define v2_FADDDP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDDP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FADDDP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c032000)
#define is_v2_FADDDP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03a000)
#define is_v2_FADDDP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c034000)
#define is_v2_FADDDP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03c000)
#define is_v2_FADDDP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c035000)
#define is_v2_FADDDP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03d000)
#define is_v2_FADDDP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c030000)
#define is_v2_FADDDP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c038000)
#define is_v2_FADDDP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c036000)
#define is_v2_FADDDP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03e000)
#define is_v2_FADDDP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c031000)
#define is_v2_FADDDP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c039000)
#define is_v2_FADDDP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c033000)
#define is_v2_FADDDP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03b000)
#define is_v2_FADDDP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c037000)
#define is_v2_FADDDP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c03f000)
#define v2_FADDWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x58077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5807f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FADDWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58072000)
#define is_v2_FADDWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807a000)
#define is_v2_FADDWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58074000)
#define is_v2_FADDWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807c000)
#define is_v2_FADDWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58075000)
#define is_v2_FADDWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807d000)
#define is_v2_FADDWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58070000)
#define is_v2_FADDWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58078000)
#define is_v2_FADDWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58076000)
#define is_v2_FADDWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807e000)
#define is_v2_FADDWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58071000)
#define is_v2_FADDWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58079000)
#define is_v2_FADDWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58073000)
#define is_v2_FADDWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807b000)
#define is_v2_FADDWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x58077000)
#define is_v2_FADDWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5807f000)
#define v2_FADDHO_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5607a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5607c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5607d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5607e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5607b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x56077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDHO__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5607f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FADDHO_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56072000)
#define is_v2_FADDHO_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5607a000)
#define is_v2_FADDHO_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56074000)
#define is_v2_FADDHO_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5607c000)
#define is_v2_FADDHO_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56075000)
#define is_v2_FADDHO_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5607d000)
#define is_v2_FADDHO_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56070000)
#define is_v2_FADDHO_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56078000)
#define is_v2_FADDHO_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56076000)
#define is_v2_FADDHO_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5607e000)
#define is_v2_FADDHO_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56071000)
#define is_v2_FADDHO_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56079000)
#define is_v2_FADDHO_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56073000)
#define is_v2_FADDHO_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5607b000)
#define is_v2_FADDHO___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x56077000)
#define is_v2_FADDHO__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5607f000)
#define v2_FADDCWC_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x53027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDCWC__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5302f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FADDCWC_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53022000)
#define is_v2_FADDCWC_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302a000)
#define is_v2_FADDCWC_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53024000)
#define is_v2_FADDCWC_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302c000)
#define is_v2_FADDCWC_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53025000)
#define is_v2_FADDCWC_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302d000)
#define is_v2_FADDCWC_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53020000)
#define is_v2_FADDCWC_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53028000)
#define is_v2_FADDCWC_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53026000)
#define is_v2_FADDCWC_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302e000)
#define is_v2_FADDCWC_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53021000)
#define is_v2_FADDCWC_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53029000)
#define is_v2_FADDCWC_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53023000)
#define is_v2_FADDCWC_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302b000)
#define is_v2_FADDCWC___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x53027000)
#define is_v2_FADDCWC__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5302f000)
#define v2_FADDCDC_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCDC__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FADDCDC_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d032000)
#define is_v2_FADDCDC_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03a000)
#define is_v2_FADDCDC_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d034000)
#define is_v2_FADDCDC_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03c000)
#define is_v2_FADDCDC_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d035000)
#define is_v2_FADDCDC_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03d000)
#define is_v2_FADDCDC_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d030000)
#define is_v2_FADDCDC_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d038000)
#define is_v2_FADDCDC_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d036000)
#define is_v2_FADDCDC_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03e000)
#define is_v2_FADDCDC_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d031000)
#define is_v2_FADDCDC_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d039000)
#define is_v2_FADDCDC_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d033000)
#define is_v2_FADDCDC_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03b000)
#define is_v2_FADDCDC___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d037000)
#define is_v2_FADDCDC__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d03f000)
#define v2_FADDCWCP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x59077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FADDCWCP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5907f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FADDCWCP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59072000)
#define is_v2_FADDCWCP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907a000)
#define is_v2_FADDCWCP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59074000)
#define is_v2_FADDCWCP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907c000)
#define is_v2_FADDCWCP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59075000)
#define is_v2_FADDCWCP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907d000)
#define is_v2_FADDCWCP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59070000)
#define is_v2_FADDCWCP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59078000)
#define is_v2_FADDCWCP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59076000)
#define is_v2_FADDCWCP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907e000)
#define is_v2_FADDCWCP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59071000)
#define is_v2_FADDCWCP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59079000)
#define is_v2_FADDCWCP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59073000)
#define is_v2_FADDCWCP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907b000)
#define is_v2_FADDCWCP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x59077000)
#define is_v2_FADDCWCP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5907f000)
#define v2_FSBFD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x54027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5402f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FSBFD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54022000)
#define is_v2_FSBFD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402a000)
#define is_v2_FSBFD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54024000)
#define is_v2_FSBFD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402c000)
#define is_v2_FSBFD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54025000)
#define is_v2_FSBFD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402d000)
#define is_v2_FSBFD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54020000)
#define is_v2_FSBFD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54028000)
#define is_v2_FSBFD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54026000)
#define is_v2_FSBFD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402e000)
#define is_v2_FSBFD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54021000)
#define is_v2_FSBFD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54029000)
#define is_v2_FSBFD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54023000)
#define is_v2_FSBFD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402b000)
#define is_v2_FSBFD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x54027000)
#define is_v2_FSBFD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5402f000)
#define v2_FSBFWP_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFWP__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5502f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FSBFWP_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55022000)
#define is_v2_FSBFWP_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502a000)
#define is_v2_FSBFWP_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55024000)
#define is_v2_FSBFWP_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502c000)
#define is_v2_FSBFWP_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55025000)
#define is_v2_FSBFWP_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502d000)
#define is_v2_FSBFWP_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55020000)
#define is_v2_FSBFWP_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55028000)
#define is_v2_FSBFWP_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55026000)
#define is_v2_FSBFWP_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502e000)
#define is_v2_FSBFWP_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55021000)
#define is_v2_FSBFWP_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55029000)
#define is_v2_FSBFWP_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55023000)
#define is_v2_FSBFWP_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502b000)
#define is_v2_FSBFWP___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55027000)
#define is_v2_FSBFWP__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5502f000)
#define v2_FSBFHQ_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x56027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFHQ__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5602f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FSBFHQ_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56022000)
#define is_v2_FSBFHQ_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602a000)
#define is_v2_FSBFHQ_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56024000)
#define is_v2_FSBFHQ_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602c000)
#define is_v2_FSBFHQ_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56025000)
#define is_v2_FSBFHQ_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602d000)
#define is_v2_FSBFHQ_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56020000)
#define is_v2_FSBFHQ_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56028000)
#define is_v2_FSBFHQ_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56026000)
#define is_v2_FSBFHQ_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602e000)
#define is_v2_FSBFHQ_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56021000)
#define is_v2_FSBFHQ_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56029000)
#define is_v2_FSBFHQ_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56023000)
#define is_v2_FSBFHQ_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602b000)
#define is_v2_FSBFHQ___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x56027000)
#define is_v2_FSBFHQ__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5602f000)
#define v2_FSBFDP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFDP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5e03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FSBFDP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e032000)
#define is_v2_FSBFDP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03a000)
#define is_v2_FSBFDP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e034000)
#define is_v2_FSBFDP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03c000)
#define is_v2_FSBFDP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e035000)
#define is_v2_FSBFDP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03d000)
#define is_v2_FSBFDP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e030000)
#define is_v2_FSBFDP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e038000)
#define is_v2_FSBFDP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e036000)
#define is_v2_FSBFDP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03e000)
#define is_v2_FSBFDP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e031000)
#define is_v2_FSBFDP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e039000)
#define is_v2_FSBFDP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e033000)
#define is_v2_FSBFDP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03b000)
#define is_v2_FSBFDP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e037000)
#define is_v2_FSBFDP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5e03f000)
#define v2_FSBFWQ_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFWQ__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5a07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FSBFWQ_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a072000)
#define is_v2_FSBFWQ_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07a000)
#define is_v2_FSBFWQ_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a074000)
#define is_v2_FSBFWQ_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07c000)
#define is_v2_FSBFWQ_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a075000)
#define is_v2_FSBFWQ_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07d000)
#define is_v2_FSBFWQ_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a070000)
#define is_v2_FSBFWQ_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a078000)
#define is_v2_FSBFWQ_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a076000)
#define is_v2_FSBFWQ_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07e000)
#define is_v2_FSBFWQ_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a071000)
#define is_v2_FSBFWQ_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a079000)
#define is_v2_FSBFWQ_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a073000)
#define is_v2_FSBFWQ_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07b000)
#define is_v2_FSBFWQ___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a077000)
#define is_v2_FSBFWQ__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5a07f000)
#define v2_FSBFHO_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5707a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5707c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5707d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5707e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5707b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x57077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFHO__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5707f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FSBFHO_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57072000)
#define is_v2_FSBFHO_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5707a000)
#define is_v2_FSBFHO_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57074000)
#define is_v2_FSBFHO_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5707c000)
#define is_v2_FSBFHO_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57075000)
#define is_v2_FSBFHO_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5707d000)
#define is_v2_FSBFHO_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57070000)
#define is_v2_FSBFHO_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57078000)
#define is_v2_FSBFHO_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57076000)
#define is_v2_FSBFHO_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5707e000)
#define is_v2_FSBFHO_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57071000)
#define is_v2_FSBFHO_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57079000)
#define is_v2_FSBFHO_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57073000)
#define is_v2_FSBFHO_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5707b000)
#define is_v2_FSBFHO___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x57077000)
#define is_v2_FSBFHO__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5707f000)
#define v2_FSBFCWC_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x57027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFCWC__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5702f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FSBFCWC_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57022000)
#define is_v2_FSBFCWC_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702a000)
#define is_v2_FSBFCWC_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57024000)
#define is_v2_FSBFCWC_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702c000)
#define is_v2_FSBFCWC_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57025000)
#define is_v2_FSBFCWC_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702d000)
#define is_v2_FSBFCWC_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57020000)
#define is_v2_FSBFCWC_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57028000)
#define is_v2_FSBFCWC_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57026000)
#define is_v2_FSBFCWC_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702e000)
#define is_v2_FSBFCWC_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57021000)
#define is_v2_FSBFCWC_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57029000)
#define is_v2_FSBFCWC_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57023000)
#define is_v2_FSBFCWC_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702b000)
#define is_v2_FSBFCWC___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x57027000)
#define is_v2_FSBFCWC__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5702f000)
#define v2_FSBFCDC_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f032000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f034000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f035000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f030000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f038000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f036000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f031000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f039000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f033000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f037000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCDC__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5f03f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FSBFCDC_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f032000)
#define is_v2_FSBFCDC_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03a000)
#define is_v2_FSBFCDC_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f034000)
#define is_v2_FSBFCDC_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03c000)
#define is_v2_FSBFCDC_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f035000)
#define is_v2_FSBFCDC_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03d000)
#define is_v2_FSBFCDC_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f030000)
#define is_v2_FSBFCDC_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f038000)
#define is_v2_FSBFCDC_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f036000)
#define is_v2_FSBFCDC_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03e000)
#define is_v2_FSBFCDC_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f031000)
#define is_v2_FSBFCDC_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f039000)
#define is_v2_FSBFCDC_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f033000)
#define is_v2_FSBFCDC_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03b000)
#define is_v2_FSBFCDC___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f037000)
#define is_v2_FSBFCDC__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5f03f000)
#define v2_FSBFCWCP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FSBFCWCP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5b07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FSBFCWCP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b072000)
#define is_v2_FSBFCWCP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07a000)
#define is_v2_FSBFCWCP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b074000)
#define is_v2_FSBFCWCP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07c000)
#define is_v2_FSBFCWCP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b075000)
#define is_v2_FSBFCWCP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07d000)
#define is_v2_FSBFCWCP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b070000)
#define is_v2_FSBFCWCP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b078000)
#define is_v2_FSBFCWCP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b076000)
#define is_v2_FSBFCWCP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07e000)
#define is_v2_FSBFCWCP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b071000)
#define is_v2_FSBFCWCP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b079000)
#define is_v2_FSBFCWCP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b073000)
#define is_v2_FSBFCWCP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07b000)
#define is_v2_FSBFCWCP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b077000)
#define is_v2_FSBFCWCP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5b07f000)
#define v2_FFMAHW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x58027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5802f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAHW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58022000)
#define is_v2_FFMAHW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802a000)
#define is_v2_FFMAHW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58024000)
#define is_v2_FFMAHW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802c000)
#define is_v2_FFMAHW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58025000)
#define is_v2_FFMAHW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802d000)
#define is_v2_FFMAHW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58020000)
#define is_v2_FFMAHW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58028000)
#define is_v2_FFMAHW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58026000)
#define is_v2_FFMAHW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802e000)
#define is_v2_FFMAHW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58021000)
#define is_v2_FFMAHW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58029000)
#define is_v2_FFMAHW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58023000)
#define is_v2_FFMAHW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802b000)
#define is_v2_FFMAHW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x58027000)
#define is_v2_FFMAHW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5802f000)
#define v2_FFMAW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5902f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59022000)
#define is_v2_FFMAW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902a000)
#define is_v2_FFMAW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59024000)
#define is_v2_FFMAW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902c000)
#define is_v2_FFMAW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59025000)
#define is_v2_FFMAW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902d000)
#define is_v2_FFMAW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59020000)
#define is_v2_FFMAW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59028000)
#define is_v2_FFMAW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59026000)
#define is_v2_FFMAW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902e000)
#define is_v2_FFMAW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59021000)
#define is_v2_FFMAW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59029000)
#define is_v2_FFMAW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59023000)
#define is_v2_FFMAW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902b000)
#define is_v2_FFMAW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59027000)
#define is_v2_FFMAW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5902f000)
#define v2_FFMAWD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x51017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5101f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAWD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51012000)
#define is_v2_FFMAWD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101a000)
#define is_v2_FFMAWD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51014000)
#define is_v2_FFMAWD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101c000)
#define is_v2_FFMAWD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51015000)
#define is_v2_FFMAWD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101d000)
#define is_v2_FFMAWD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51010000)
#define is_v2_FFMAWD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51018000)
#define is_v2_FFMAWD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51016000)
#define is_v2_FFMAWD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101e000)
#define is_v2_FFMAWD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51011000)
#define is_v2_FFMAWD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51019000)
#define is_v2_FFMAWD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51013000)
#define is_v2_FFMAWD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101b000)
#define is_v2_FFMAWD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x51017000)
#define is_v2_FFMAWD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5101f000)
#define v2_FFMAWDP_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x50037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAWDP__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5003f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAWDP_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50032000)
#define is_v2_FFMAWDP_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003a000)
#define is_v2_FFMAWDP_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50034000)
#define is_v2_FFMAWDP_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003c000)
#define is_v2_FFMAWDP_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50035000)
#define is_v2_FFMAWDP_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003d000)
#define is_v2_FFMAWDP_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50030000)
#define is_v2_FFMAWDP_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50038000)
#define is_v2_FFMAWDP_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50036000)
#define is_v2_FFMAWDP_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003e000)
#define is_v2_FFMAWDP_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50031000)
#define is_v2_FFMAWDP_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50039000)
#define is_v2_FFMAWDP_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50033000)
#define is_v2_FFMAWDP_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003b000)
#define is_v2_FFMAWDP___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x50037000)
#define is_v2_FFMAWDP__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5003f000)
#define v2_FFMAHWQ_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x51037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMAHWQ__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5103f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMAHWQ_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51032000)
#define is_v2_FFMAHWQ_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103a000)
#define is_v2_FFMAHWQ_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51034000)
#define is_v2_FFMAHWQ_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103c000)
#define is_v2_FFMAHWQ_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51035000)
#define is_v2_FFMAHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103d000)
#define is_v2_FFMAHWQ_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51030000)
#define is_v2_FFMAHWQ_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51038000)
#define is_v2_FFMAHWQ_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51036000)
#define is_v2_FFMAHWQ_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103e000)
#define is_v2_FFMAHWQ_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51031000)
#define is_v2_FFMAHWQ_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51039000)
#define is_v2_FFMAHWQ_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51033000)
#define is_v2_FFMAHWQ_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103b000)
#define is_v2_FFMAHWQ___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x51037000)
#define is_v2_FFMAHWQ__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5103f000)
#define v2_FFMSHW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5a02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSHW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a022000)
#define is_v2_FFMSHW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02a000)
#define is_v2_FFMSHW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a024000)
#define is_v2_FFMSHW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02c000)
#define is_v2_FFMSHW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a025000)
#define is_v2_FFMSHW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02d000)
#define is_v2_FFMSHW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a020000)
#define is_v2_FFMSHW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a028000)
#define is_v2_FFMSHW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a026000)
#define is_v2_FFMSHW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02e000)
#define is_v2_FFMSHW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a021000)
#define is_v2_FFMSHW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a029000)
#define is_v2_FFMSHW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a023000)
#define is_v2_FFMSHW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02b000)
#define is_v2_FFMSHW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a027000)
#define is_v2_FFMSHW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5a02f000)
#define v2_FFMSW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5b02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b022000)
#define is_v2_FFMSW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02a000)
#define is_v2_FFMSW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b024000)
#define is_v2_FFMSW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02c000)
#define is_v2_FFMSW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b025000)
#define is_v2_FFMSW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02d000)
#define is_v2_FFMSW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b020000)
#define is_v2_FFMSW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b028000)
#define is_v2_FFMSW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b026000)
#define is_v2_FFMSW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02e000)
#define is_v2_FFMSW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b021000)
#define is_v2_FFMSW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b029000)
#define is_v2_FFMSW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b023000)
#define is_v2_FFMSW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02b000)
#define is_v2_FFMSW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b027000)
#define is_v2_FFMSW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5b02f000)
#define v2_FFMSWD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x55017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5501f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSWD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55012000)
#define is_v2_FFMSWD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501a000)
#define is_v2_FFMSWD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55014000)
#define is_v2_FFMSWD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501c000)
#define is_v2_FFMSWD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55015000)
#define is_v2_FFMSWD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501d000)
#define is_v2_FFMSWD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55010000)
#define is_v2_FFMSWD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55018000)
#define is_v2_FFMSWD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55016000)
#define is_v2_FFMSWD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501e000)
#define is_v2_FFMSWD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55011000)
#define is_v2_FFMSWD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55019000)
#define is_v2_FFMSWD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55013000)
#define is_v2_FFMSWD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501b000)
#define is_v2_FFMSWD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x55017000)
#define is_v2_FFMSWD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5501f000)
#define v2_FFMSWDP_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x52037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSWDP__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5203f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSWDP_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52032000)
#define is_v2_FFMSWDP_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203a000)
#define is_v2_FFMSWDP_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52034000)
#define is_v2_FFMSWDP_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203c000)
#define is_v2_FFMSWDP_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52035000)
#define is_v2_FFMSWDP_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203d000)
#define is_v2_FFMSWDP_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52030000)
#define is_v2_FFMSWDP_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52038000)
#define is_v2_FFMSWDP_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52036000)
#define is_v2_FFMSWDP_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203e000)
#define is_v2_FFMSWDP_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52031000)
#define is_v2_FFMSWDP_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52039000)
#define is_v2_FFMSWDP_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52033000)
#define is_v2_FFMSWDP_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203b000)
#define is_v2_FFMSWDP___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x52037000)
#define is_v2_FFMSWDP__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5203f000)
#define v2_FFMSHWQ_rd__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53032000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rd_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303a000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rna__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53034000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rna_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303c000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rnz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53035000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303d000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rn__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53030000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rn_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53038000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_ro__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53036000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_ro_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303e000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_ru__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53031000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_ru_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53039000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rz__registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53033000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ_rz_s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303b000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ___registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x53037000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FFMSHWQ__s_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5303f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FFMSHWQ_rd__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53032000)
#define is_v2_FFMSHWQ_rd_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303a000)
#define is_v2_FFMSHWQ_rna__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53034000)
#define is_v2_FFMSHWQ_rna_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303c000)
#define is_v2_FFMSHWQ_rnz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53035000)
#define is_v2_FFMSHWQ_rnz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303d000)
#define is_v2_FFMSHWQ_rn__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53030000)
#define is_v2_FFMSHWQ_rn_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53038000)
#define is_v2_FFMSHWQ_ro__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53036000)
#define is_v2_FFMSHWQ_ro_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303e000)
#define is_v2_FFMSHWQ_ru__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53031000)
#define is_v2_FFMSHWQ_ru_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53039000)
#define is_v2_FFMSHWQ_rz__registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53033000)
#define is_v2_FFMSHWQ_rz_s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303b000)
#define is_v2_FFMSHWQ___registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x53037000)
#define is_v2_FFMSHWQ__s_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5303f000)
#define v2_FADDW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FADDW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FADDW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c022000)
#define is_v2_FADDW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02a000)
#define is_v2_FADDW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c024000)
#define is_v2_FADDW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02c000)
#define is_v2_FADDW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c025000)
#define is_v2_FADDW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02d000)
#define is_v2_FADDW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c020000)
#define is_v2_FADDW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c028000)
#define is_v2_FADDW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c026000)
#define is_v2_FADDW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02e000)
#define is_v2_FADDW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c021000)
#define is_v2_FADDW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c029000)
#define is_v2_FADDW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c023000)
#define is_v2_FADDW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02b000)
#define is_v2_FADDW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c027000)
#define is_v2_FADDW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c02f000)
#define v2_FSBFW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FSBFW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FSBFW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d022000)
#define is_v2_FSBFW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02a000)
#define is_v2_FSBFW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d024000)
#define is_v2_FSBFW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02c000)
#define is_v2_FSBFW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d025000)
#define is_v2_FSBFW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02d000)
#define is_v2_FSBFW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d020000)
#define is_v2_FSBFW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d028000)
#define is_v2_FSBFW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d026000)
#define is_v2_FSBFW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02e000)
#define is_v2_FSBFW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d021000)
#define is_v2_FSBFW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d029000)
#define is_v2_FSBFW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d023000)
#define is_v2_FSBFW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02b000)
#define is_v2_FSBFW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d027000)
#define is_v2_FSBFW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d02f000)
#define v2_FMULHW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULHW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5f02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULHW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f022000)
#define is_v2_FMULHW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02a000)
#define is_v2_FMULHW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f024000)
#define is_v2_FMULHW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02c000)
#define is_v2_FMULHW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f025000)
#define is_v2_FMULHW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02d000)
#define is_v2_FMULHW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f020000)
#define is_v2_FMULHW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f028000)
#define is_v2_FMULHW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f026000)
#define is_v2_FMULHW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02e000)
#define is_v2_FMULHW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f021000)
#define is_v2_FMULHW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f029000)
#define is_v2_FMULHW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f023000)
#define is_v2_FMULHW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02b000)
#define is_v2_FMULHW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f027000)
#define is_v2_FMULHW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f02f000)
#define v2_FMULW_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e022000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e024000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e025000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e020000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e028000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e026000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e021000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e029000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e023000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e027000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULW__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5e02f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULW_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e022000)
#define is_v2_FMULW_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02a000)
#define is_v2_FMULW_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e024000)
#define is_v2_FMULW_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02c000)
#define is_v2_FMULW_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e025000)
#define is_v2_FMULW_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02d000)
#define is_v2_FMULW_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e020000)
#define is_v2_FMULW_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e028000)
#define is_v2_FMULW_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e026000)
#define is_v2_FMULW_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02e000)
#define is_v2_FMULW_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e021000)
#define is_v2_FMULW_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e029000)
#define is_v2_FMULW_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e023000)
#define is_v2_FMULW_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02b000)
#define is_v2_FMULW___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e027000)
#define is_v2_FMULW__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5e02f000)
#define v2_FMULWD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x59017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FMULWD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5901f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FMULWD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59012000)
#define is_v2_FMULWD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901a000)
#define is_v2_FMULWD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59014000)
#define is_v2_FMULWD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901c000)
#define is_v2_FMULWD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59015000)
#define is_v2_FMULWD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901d000)
#define is_v2_FMULWD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59010000)
#define is_v2_FMULWD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59018000)
#define is_v2_FMULWD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59016000)
#define is_v2_FMULWD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901e000)
#define is_v2_FMULWD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59011000)
#define is_v2_FMULWD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59019000)
#define is_v2_FMULWD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59013000)
#define is_v2_FMULWD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901b000)
#define is_v2_FMULWD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x59017000)
#define is_v2_FMULWD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5901f000)
#define v2_FDOT2W_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2W__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5c01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FDOT2W_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c012000)
#define is_v2_FDOT2W_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01a000)
#define is_v2_FDOT2W_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c014000)
#define is_v2_FDOT2W_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01c000)
#define is_v2_FDOT2W_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c015000)
#define is_v2_FDOT2W_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01d000)
#define is_v2_FDOT2W_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c010000)
#define is_v2_FDOT2W_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c018000)
#define is_v2_FDOT2W_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c016000)
#define is_v2_FDOT2W_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01e000)
#define is_v2_FDOT2W_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c011000)
#define is_v2_FDOT2W_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c019000)
#define is_v2_FDOT2W_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c013000)
#define is_v2_FDOT2W_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01b000)
#define is_v2_FDOT2W___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c017000)
#define is_v2_FDOT2W__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5c01f000)
#define v2_FDOT2WD_rd__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rd_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01a000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rna__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d014000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rna_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01c000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rnz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d015000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rnz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01d000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rn__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d010000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rn_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d018000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_ro__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d016000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_ro_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01e000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_ru__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d011000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_ru_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d019000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rz__registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD_rz_s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01b000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD___registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d017000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define v2_FDOT2WD__s_registerW_registerZ_registerY_simple_insn(registerW,registerZ,registerY) (unsigned int)(0x5d01f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_FDOT2WD_rd__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d012000)
#define is_v2_FDOT2WD_rd_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01a000)
#define is_v2_FDOT2WD_rna__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d014000)
#define is_v2_FDOT2WD_rna_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01c000)
#define is_v2_FDOT2WD_rnz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d015000)
#define is_v2_FDOT2WD_rnz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01d000)
#define is_v2_FDOT2WD_rn__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d010000)
#define is_v2_FDOT2WD_rn_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d018000)
#define is_v2_FDOT2WD_ro__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d016000)
#define is_v2_FDOT2WD_ro_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01e000)
#define is_v2_FDOT2WD_ru__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d011000)
#define is_v2_FDOT2WD_ru_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d019000)
#define is_v2_FDOT2WD_rz__registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d013000)
#define is_v2_FDOT2WD_rz_s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01b000)
#define is_v2_FDOT2WD___registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d017000)
#define is_v2_FDOT2WD__s_registerW_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5d01f000)
#define v2_FDOT2WZP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WZP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5d07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FDOT2WZP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d072000)
#define is_v2_FDOT2WZP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07a000)
#define is_v2_FDOT2WZP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d074000)
#define is_v2_FDOT2WZP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07c000)
#define is_v2_FDOT2WZP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d075000)
#define is_v2_FDOT2WZP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07d000)
#define is_v2_FDOT2WZP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d070000)
#define is_v2_FDOT2WZP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d078000)
#define is_v2_FDOT2WZP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d076000)
#define is_v2_FDOT2WZP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07e000)
#define is_v2_FDOT2WZP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d071000)
#define is_v2_FDOT2WZP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d079000)
#define is_v2_FDOT2WZP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d073000)
#define is_v2_FDOT2WZP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07b000)
#define is_v2_FDOT2WZP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d077000)
#define is_v2_FDOT2WZP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5d07f000)
#define v2_FDOT2WDP_rd__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c072000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rd_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07a000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rna__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c074000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rna_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07c000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rnz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c075000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rnz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07d000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rn__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c070000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rn_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c078000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_ro__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c076000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_ro_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07e000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_ru__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c071000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_ru_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c079000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rz__registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c073000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP_rz_s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07b000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP___registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c077000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define v2_FDOT2WDP__s_registerM_registerP_registerO_simple_insn(registerM,registerP,registerO) (unsigned int)(0x5c07f000 | (((int)(registerM) & 0x1f) << 19) | (((int)(registerP) & 0x1f) << 1) | (((int)(registerO) & 0x1f) << 7))
#define is_v2_FDOT2WDP_rd__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c072000)
#define is_v2_FDOT2WDP_rd_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07a000)
#define is_v2_FDOT2WDP_rna__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c074000)
#define is_v2_FDOT2WDP_rna_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07c000)
#define is_v2_FDOT2WDP_rnz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c075000)
#define is_v2_FDOT2WDP_rnz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07d000)
#define is_v2_FDOT2WDP_rn__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c070000)
#define is_v2_FDOT2WDP_rn_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c078000)
#define is_v2_FDOT2WDP_ro__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c076000)
#define is_v2_FDOT2WDP_ro_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07e000)
#define is_v2_FDOT2WDP_ru__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c071000)
#define is_v2_FDOT2WDP_ru_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c079000)
#define is_v2_FDOT2WDP_rz__registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c073000)
#define is_v2_FDOT2WDP_rz_s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07b000)
#define is_v2_FDOT2WDP___registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c077000)
#define is_v2_FDOT2WDP__s_registerM_registerP_registerO_simple_insn(x) (int)(((unsigned int)(x) & 0x7f07f000) == 0x5c07f000)
#define v2_COPYQ_registerM_registerZ_registerY_simple_insn(registerM,registerZ,registerY) (unsigned int)(0x5f00f000 | (((int)(registerM) & 0x1f) << 19) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_COPYQ_registerM_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x5f00f000)
#define v2_COPYO_registerN_registerR_simple_insn(registerN,registerR) (unsigned int)(0x3e050000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerR) & 0xf) << 2))
#define is_v2_COPYO_registerN_registerR_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0f0000) == 0x3e050000)
#define v2_MOVETQ_registerAl_registerZ_registerY_simple_insn(registerAl,registerZ,registerY) (unsigned int)(0x7f02e000 | (((int)(registerAl) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MOVETQ_registerAl_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f02e000)
#define v2_MOVETQ_registerAh_registerZ_registerY_simple_insn(registerAh,registerZ,registerY) (unsigned int)(0x7f02f000 | (((int)(registerAh) & 0x3f) << 18) | ((int)(registerZ) & 0x3f) | (((int)(registerY) & 0x3f) << 6))
#define is_v2_MOVETQ_registerAh_registerZ_registerY_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7f02f000)
#define v2_SENDV_b_registerA_simple_insn(registerA) (unsigned int)(0x7103f000 | (((int)(registerA) & 0x3f) << 18))
#define v2_SENDV_f_registerA_simple_insn(registerA) (unsigned int)(0x7003f000 | (((int)(registerA) & 0x3f) << 18))
#define is_v2_SENDV_b_registerA_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7103f000)
#define is_v2_SENDV_f_registerA_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7003f000)
#define v2_RECVV_b_registerA_simple_insn(registerA) (unsigned int)(0x7303f000 | (((int)(registerA) & 0x3f) << 18))
#define v2_RECVV_f_registerA_simple_insn(registerA) (unsigned int)(0x7203f000 | (((int)(registerA) & 0x3f) << 18))
#define is_v2_RECVV_b_registerA_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7303f000)
#define is_v2_RECVV_f_registerA_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03f000) == 0x7203f000)
#define v2_ALIGNO_registerN_registerBe_registerCo_byteshift_simple_insn(registerN,registerBe,registerCo,byteshift) (unsigned int)(0x00081000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v2_ALIGNO_registerN_registerBe_registerCo_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x00081000)
#define v2_ALIGNO_registerN_registerBo_registerCe_byteshift_simple_insn(registerN,registerBo,registerCe,byteshift) (unsigned int)(0x000c1000 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v2_ALIGNO_registerN_registerBo_registerCe_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x000c1000)
#define v2_ALIGNO_registerN_registerBe_registerCo_registerZ_simple_insn(registerN,registerBe,registerCo,registerZ) (unsigned int)(0x00081040 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v2_ALIGNO_registerN_registerBe_registerCo_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x00081040)
#define v2_ALIGNO_registerN_registerBo_registerCe_registerZ_simple_insn(registerN,registerBo,registerCe,registerZ) (unsigned int)(0x000c1040 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v2_ALIGNO_registerN_registerBo_registerCe_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1040) == 0x000c1040)
#define v2_ALIGNV_registerA_registerBe_registerCo_byteshift_simple_insn(registerA,registerBe,registerCo,byteshift) (unsigned int)(0x01000000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v2_ALIGNV_registerA_registerBe_registerCo_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01000000)
#define v2_ALIGNV_registerA_registerBo_registerCe_byteshift_simple_insn(registerA,registerBo,registerCe,byteshift) (unsigned int)(0x01001000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(byteshift) & 0x3f))
#define is_v2_ALIGNV_registerA_registerBo_registerCe_byteshift_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01001000)
#define v2_ALIGNV_registerA_registerBe_registerCo_registerZ_simple_insn(registerA,registerBe,registerCo,registerZ) (unsigned int)(0x01000040 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBe) & 0x1f) << 13) | (((int)(registerCo) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v2_ALIGNV_registerA_registerBe_registerCo_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01000040)
#define v2_ALIGNV_registerA_registerBo_registerCe_registerZ_simple_insn(registerA,registerBo,registerCe,registerZ) (unsigned int)(0x01001040 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBo) & 0x1f) << 13) | (((int)(registerCe) & 0x1f) << 7) | ((int)(registerZ) & 0x3f))
#define is_v2_ALIGNV_registerA_registerBo_registerCe_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001040) == 0x01001040)
#define v2_MT44D_registerAq_registerBq_simple_insn(registerAq,registerBq) (unsigned int)(0x07001740 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerBq) & 0xf) << 14))
#define is_v2_MT44D_registerAq_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c3f40) == 0x07001740)
#define v2_MMA4164BW_registerAp_registerBp_registerDp_simple_insn(registerAp,registerBp,registerDp) (unsigned int)(0x02000000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define is_v2_MMA4164BW_registerAp_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0410c1) == 0x02000000)
#define v2_MMA4164UBW_registerAp_registerBp_registerDp_simple_insn(registerAp,registerBp,registerDp) (unsigned int)(0x02001000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define is_v2_MMA4164UBW_registerAp_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0410c1) == 0x02001000)
#define v2_MMA4164SUBW_registerAp_registerBp_registerDp_simple_insn(registerAp,registerBp,registerDp) (unsigned int)(0x02040000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define is_v2_MMA4164SUBW_registerAp_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0410c1) == 0x02040000)
#define v2_MMA4164USBW_registerAp_registerBp_registerDp_simple_insn(registerAp,registerBp,registerDp) (unsigned int)(0x02041000 | (((int)(registerAp) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define is_v2_MMA4164USBW_registerAp_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0410c1) == 0x02041000)
#define v2_MMA444HD_registerAq_registerB_registerD_simple_insn(registerAq,registerB,registerD) (unsigned int)(0x02000040 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerB) & 0x3f) << 12) | ((int)(registerD) & 0x3f))
#define is_v2_MMA444HD_registerAq_registerB_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c00c0) == 0x02000040)
#define v2_MMA444UHD_registerAq_registerB_registerD_simple_insn(registerAq,registerB,registerD) (unsigned int)(0x02040040 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerB) & 0x3f) << 12) | ((int)(registerD) & 0x3f))
#define is_v2_MMA444UHD_registerAq_registerB_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c00c0) == 0x02040040)
#define v2_MMA444SUHD_registerAq_registerB_registerD_simple_insn(registerAq,registerB,registerD) (unsigned int)(0x02080040 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerB) & 0x3f) << 12) | ((int)(registerD) & 0x3f))
#define is_v2_MMA444SUHD_registerAq_registerB_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c00c0) == 0x02080040)
#define v2_MMA444USHD_registerAq_registerB_registerD_simple_insn(registerAq,registerB,registerD) (unsigned int)(0x020c0040 | (((int)(registerAq) & 0xf) << 20) | (((int)(registerB) & 0x3f) << 12) | ((int)(registerD) & 0x3f))
#define is_v2_MMA444USHD_registerAq_registerB_registerD_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c00c0) == 0x020c0040)
#define v2_FMMA482HW0_rd__registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000280 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rd_s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000a80 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rna__registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000480 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rna_s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000c80 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rnz__registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000580 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rnz_s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000d80 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rn__registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000080 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rn_s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000880 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_ro__registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000680 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_ro_s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000e80 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_ru__registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000180 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_ru_s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000980 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rz__registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000380 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0_rz_s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000b80 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0___registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000780 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW0__s_registerAe_registerBp_registerDp_simple_insn(registerAe,registerBp,registerDp) (unsigned int)(0x02000f80 | (((int)(registerAe) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define is_v2_FMMA482HW0_rd__registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000280)
#define is_v2_FMMA482HW0_rd_s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000a80)
#define is_v2_FMMA482HW0_rna__registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000480)
#define is_v2_FMMA482HW0_rna_s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000c80)
#define is_v2_FMMA482HW0_rnz__registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000580)
#define is_v2_FMMA482HW0_rnz_s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000d80)
#define is_v2_FMMA482HW0_rn__registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000080)
#define is_v2_FMMA482HW0_rn_s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000880)
#define is_v2_FMMA482HW0_ro__registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000680)
#define is_v2_FMMA482HW0_ro_s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000e80)
#define is_v2_FMMA482HW0_ru__registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000180)
#define is_v2_FMMA482HW0_ru_s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000980)
#define is_v2_FMMA482HW0_rz__registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000380)
#define is_v2_FMMA482HW0_rz_s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000b80)
#define is_v2_FMMA482HW0___registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000780)
#define is_v2_FMMA482HW0__s_registerAe_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02000f80)
#define v2_FMMA482HW1_rd__registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040280 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rd_s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040a80 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rna__registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040480 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rna_s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040c80 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rnz__registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040580 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rnz_s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040d80 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rn__registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040080 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rn_s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040880 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_ro__registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040680 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_ro_s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040e80 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_ru__registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040180 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_ru_s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040980 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rz__registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040380 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1_rz_s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040b80 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1___registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040780 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define v2_FMMA482HW1__s_registerAo_registerBp_registerDp_simple_insn(registerAo,registerBp,registerDp) (unsigned int)(0x02040f80 | (((int)(registerAo) & 0x1f) << 19) | (((int)(registerBp) & 0x1f) << 13) | (((int)(registerDp) & 0x1f) << 1))
#define is_v2_FMMA482HW1_rd__registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040280)
#define is_v2_FMMA482HW1_rd_s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040a80)
#define is_v2_FMMA482HW1_rna__registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040480)
#define is_v2_FMMA482HW1_rna_s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040c80)
#define is_v2_FMMA482HW1_rnz__registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040580)
#define is_v2_FMMA482HW1_rnz_s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040d80)
#define is_v2_FMMA482HW1_rn__registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040080)
#define is_v2_FMMA482HW1_rn_s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040880)
#define is_v2_FMMA482HW1_ro__registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040680)
#define is_v2_FMMA482HW1_ro_s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040e80)
#define is_v2_FMMA482HW1_ru__registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040180)
#define is_v2_FMMA482HW1_ru_s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040980)
#define is_v2_FMMA482HW1_rz__registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040380)
#define is_v2_FMMA482HW1_rz_s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040b80)
#define is_v2_FMMA482HW1___registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040780)
#define is_v2_FMMA482HW1__s_registerAo_registerBp_registerDp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f041fc1) == 0x02040f80)
#define v2_FSCALEWV_rd_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000a00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rd_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000a80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rd___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000200 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rd__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000280 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rna_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000c00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rna_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000c80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rna___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000400 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rna__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000480 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rnz_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000d00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rnz_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000d80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rnz___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000500 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rnz__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000580 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rn_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000800 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rn_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000880 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rn___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000000 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rn__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000080 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ro_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000e00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ro_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000e80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ro___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000600 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ro__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000680 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ru_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000900 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ru_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000980 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ru___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000100 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_ru__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000180 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rz_s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000b00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rz_s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000b80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rz___registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000300 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV_rz__relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000380 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV__s__registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000f00 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV__s_relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000f80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV____registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000700 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define v2_FSCALEWV___relu_registerA_registerB_simple_insn(registerA,registerB) (unsigned int)(0x07000780 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerB) & 0x3f) << 12))
#define is_v2_FSCALEWV_rd_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000a00)
#define is_v2_FSCALEWV_rd_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000a80)
#define is_v2_FSCALEWV_rd___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000200)
#define is_v2_FSCALEWV_rd__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000280)
#define is_v2_FSCALEWV_rna_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000c00)
#define is_v2_FSCALEWV_rna_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000c80)
#define is_v2_FSCALEWV_rna___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000400)
#define is_v2_FSCALEWV_rna__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000480)
#define is_v2_FSCALEWV_rnz_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000d00)
#define is_v2_FSCALEWV_rnz_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000d80)
#define is_v2_FSCALEWV_rnz___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000500)
#define is_v2_FSCALEWV_rnz__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000580)
#define is_v2_FSCALEWV_rn_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000800)
#define is_v2_FSCALEWV_rn_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000880)
#define is_v2_FSCALEWV_rn___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000000)
#define is_v2_FSCALEWV_rn__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000080)
#define is_v2_FSCALEWV_ro_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000e00)
#define is_v2_FSCALEWV_ro_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000e80)
#define is_v2_FSCALEWV_ro___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000600)
#define is_v2_FSCALEWV_ro__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000680)
#define is_v2_FSCALEWV_ru_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000900)
#define is_v2_FSCALEWV_ru_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000980)
#define is_v2_FSCALEWV_ru___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000100)
#define is_v2_FSCALEWV_ru__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000180)
#define is_v2_FSCALEWV_rz_s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000b00)
#define is_v2_FSCALEWV_rz_s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000b80)
#define is_v2_FSCALEWV_rz___registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000300)
#define is_v2_FSCALEWV_rz__relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000380)
#define is_v2_FSCALEWV__s__registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000f00)
#define is_v2_FSCALEWV__s_relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000f80)
#define is_v2_FSCALEWV____registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000700)
#define is_v2_FSCALEWV___relu_registerA_registerB_simple_insn(x) (int)(((unsigned int)(x) & 0x7f000fc0) == 0x07000780)
#define v2_FNARROWWHV_rd__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000240 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rd_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000a40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rna__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000440 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rna_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000c40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rnz__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000540 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rnz_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000d40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rn__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000040 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rn_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000840 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_ro__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000640 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_ro_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000e40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_ru__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000140 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_ru_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000940 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rz__registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000340 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV_rz_s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000b40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV___registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000740 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define v2_FNARROWWHV__s_registerA_registerBp_simple_insn(registerA,registerBp) (unsigned int)(0x07000f40 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBp) & 0x1f) << 13))
#define is_v2_FNARROWWHV_rd__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000240)
#define is_v2_FNARROWWHV_rd_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000a40)
#define is_v2_FNARROWWHV_rna__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000440)
#define is_v2_FNARROWWHV_rna_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000c40)
#define is_v2_FNARROWWHV_rnz__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000540)
#define is_v2_FNARROWWHV_rnz_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000d40)
#define is_v2_FNARROWWHV_rn__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000040)
#define is_v2_FNARROWWHV_rn_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000840)
#define is_v2_FNARROWWHV_ro__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000640)
#define is_v2_FNARROWWHV_ro_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000e40)
#define is_v2_FNARROWWHV_ru__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000140)
#define is_v2_FNARROWWHV_ru_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000940)
#define is_v2_FNARROWWHV_rz__registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000340)
#define is_v2_FNARROWWHV_rz_s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000b40)
#define is_v2_FNARROWWHV___registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000740)
#define is_v2_FNARROWWHV__s_registerA_registerBp_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001f40) == 0x07000f40)
#define v2_CONVDHV0_rd_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x07001240 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_rd_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x070012c0 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_rhu_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x07001440 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_rhu_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x070014c0 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_rn_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x07001040 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_rn_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x070010c0 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_ru_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x07001140 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_ru_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x070011c0 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_rz_sat_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x07001340 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV0_rz_satu_registerAl_registerBq_simple_insn(registerAl,registerBq) (unsigned int)(0x070013c0 | (((int)(registerAl) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v2_CONVDHV0_rd_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07001240)
#define is_v2_CONVDHV0_rd_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070012c0)
#define is_v2_CONVDHV0_rhu_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07001440)
#define is_v2_CONVDHV0_rhu_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070014c0)
#define is_v2_CONVDHV0_rn_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07001040)
#define is_v2_CONVDHV0_rn_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070010c0)
#define is_v2_CONVDHV0_ru_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07001140)
#define is_v2_CONVDHV0_ru_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070011c0)
#define is_v2_CONVDHV0_rz_sat_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07001340)
#define is_v2_CONVDHV0_rz_satu_registerAl_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070013c0)
#define v2_CONVDHV1_rd_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x07003240 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_rd_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x070032c0 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_rhu_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x07003440 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_rhu_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x070034c0 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_rn_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x07003040 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_rn_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x070030c0 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_ru_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x07003140 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_ru_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x070031c0 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_rz_sat_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x07003340 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define v2_CONVDHV1_rz_satu_registerAh_registerBq_simple_insn(registerAh,registerBq) (unsigned int)(0x070033c0 | (((int)(registerAh) & 0x3f) << 18) | (((int)(registerBq) & 0xf) << 14))
#define is_v2_CONVDHV1_rd_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07003240)
#define is_v2_CONVDHV1_rd_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070032c0)
#define is_v2_CONVDHV1_rhu_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07003440)
#define is_v2_CONVDHV1_rhu_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070034c0)
#define is_v2_CONVDHV1_rn_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07003040)
#define is_v2_CONVDHV1_rn_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070030c0)
#define is_v2_CONVDHV1_ru_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07003140)
#define is_v2_CONVDHV1_ru_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070031c0)
#define is_v2_CONVDHV1_rz_sat_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x07003340)
#define is_v2_CONVDHV1_rz_satu_registerAh_registerBq_simple_insn(x) (int)(((unsigned int)(x) & 0x7f003fc0) == 0x070033c0)
#define v2_COPYD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_COPYD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6a000000)
#define v2_NOTD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6c00ffc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_NOTD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6c00ffc0)
#define v2_NEGD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x65000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_NEGD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x65000000)
#define v2_ABSD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x64000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ABSD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x64000000)
#define v2_ZXBD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x78003fc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ZXBD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x78003fc0)
#define v2_SXBD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x68037000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SXBD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x68037000)
#define v2_ZXHD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6403f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ZXHD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6403f000)
#define v2_SXHD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6803f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SXHD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6803f000)
#define v2_ZXWD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7800ffc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ZXWD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7800ffc0)
#define v2_SXWD_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x6903f000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SXWD_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x6903f000)
#define v2_SATDH_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7e022400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SATDH_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7e022400)
#define v2_SATUDH_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7f022400 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SATUDH_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7f022400)
#define v2_SATDW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7e022800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SATDW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7e022800)
#define v2_SATUDW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7f022800 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_SATUDW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7f022800)
#define v2_COPYW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7a000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_COPYW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7a000000)
#define v2_NOTW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x7c00ffc0 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_NOTW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x7c00ffc0)
#define v2_NEGW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x75000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_NEGW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x75000000)
#define v2_ABSW_registerW_registerZ_simple_insn(registerW,registerZ) (unsigned int)(0x74000000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ABSW_registerW_registerZ_simple_insn(x) (int)(((unsigned int)(x) & 0x7f03ffc0) == 0x74000000)
#define v2_NEGWP_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf5012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_NEGWP_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf5012000)
#define v2_ABSWP_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf4012000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ABSWP_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf4012000)
#define v2_NEGHQ_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf5013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_NEGHQ_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf5013000)
#define v2_ABSHQ_registerW_registerZ_double_insn(registerW,registerZ) (unsigned int)(0xf4013000 | (((int)(registerW) & 0x3f) << 18) | ((int)(registerZ) & 0x3f))
#define is_v2_ABSHQ_registerW_registerZ_double_insn(x) (int)(((unsigned int)(x) & 0xff03ffc0) == 0xf4013000)
#define v2_COPYV_registerA_registerBe_simple_insn(registerA,registerBe) (unsigned int)(0x01000f80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBe) & 0x1f) << 13))
#define is_v2_COPYV_registerA_registerBe_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001fff) == 0x01000f80)
#define v2_COPYV_registerA_registerBo_simple_insn(registerA,registerBo) (unsigned int)(0x01001f80 | (((int)(registerA) & 0x3f) << 18) | (((int)(registerBo) & 0x1f) << 13))
#define is_v2_COPYV_registerA_registerBo_simple_insn(x) (int)(((unsigned int)(x) & 0x7f001fff) == 0x01001f80)
#define v2_MOVEFO_registerN_registerBe_simple_insn(registerN,registerBe) (unsigned int)(0x00081f80 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBe) & 0x1f) << 13))
#define is_v2_MOVEFO_registerN_registerBe_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1fff) == 0x00081f80)
#define v2_MOVEFO_registerN_registerBo_simple_insn(registerN,registerBo) (unsigned int)(0x000c1f80 | (((int)(registerN) & 0xf) << 20) | (((int)(registerBo) & 0x1f) << 13))
#define is_v2_MOVEFO_registerN_registerBo_simple_insn(x) (int)(((unsigned int)(x) & 0x7f0c1fff) == 0x000c1f80)

#define KV3_DOUBLE_MASK 	0x80000000
#define KV3_DOUBLE_VAL  	0x80000000
#define KV3_SIMPLE_MASK 	0x00000000
#define KV3_SIMPLE_VAL  	0x00000000
#define KV3_TRIPLE_MASK 	0x80000000
#define KV3_TRIPLE_VAL  	0x80000000

/*
 * The following macros are provided for compatibility with old
 * code.  They should not be used in new code.
 */

enum Method_kv3_enum {
  Method_kv3_ERROR,
  Immediate_kv3_brknumber,
  Immediate_kv3_pcrel17,
  Immediate_kv3_pcrel27,
  Immediate_kv3_signed10,
  Immediate_kv3_signed12,
  Immediate_kv3_signed16,
  Immediate_kv3_signed27,
  Immediate_kv3_signed37,
  Immediate_kv3_signed43,
  Immediate_kv3_signed54,
  Immediate_kv3_signed6,
  Immediate_kv3_sysnumber,
  Immediate_kv3_unsigned6,
  Immediate_kv3_wrapped32,
  Immediate_kv3_wrapped64,
  Modifier_kv3_boolcas,
  Modifier_kv3_cachelev,
  Modifier_kv3_channel,
  Modifier_kv3_column,
  Modifier_kv3_comparison,
  Modifier_kv3_conjugate,
  Modifier_kv3_doscale,
  Modifier_kv3_exunum,
  Modifier_kv3_floatcomp,
  Modifier_kv3_rectify,
  Modifier_kv3_rounding,
  Modifier_kv3_roundint,
  Modifier_kv3_saturate,
  Modifier_kv3_scalarcond,
  Modifier_kv3_silent,
  Modifier_kv3_simplecond,
  Modifier_kv3_speculate,
  Modifier_kv3_splat32,
  Modifier_kv3_transpose,
  Modifier_kv3_variant,
  RegClass_kv3_aloneReg,
  RegClass_kv3_blockReg,
  RegClass_kv3_blockReg0M4,
  RegClass_kv3_blockReg1M4,
  RegClass_kv3_blockReg2M4,
  RegClass_kv3_blockReg3M4,
  RegClass_kv3_blockRegE,
  RegClass_kv3_blockRegO,
  RegClass_kv3_blockReg_0,
  RegClass_kv3_blockReg_1,
  RegClass_kv3_coproReg,
  RegClass_kv3_coproReg0M4,
  RegClass_kv3_coproReg1M4,
  RegClass_kv3_coproReg2M4,
  RegClass_kv3_coproReg3M4,
  RegClass_kv3_matrixReg,
  RegClass_kv3_matrixReg_0,
  RegClass_kv3_matrixReg_1,
  RegClass_kv3_matrixReg_2,
  RegClass_kv3_matrixReg_3,
  RegClass_kv3_onlyfxReg,
  RegClass_kv3_onlygetReg,
  RegClass_kv3_onlyraReg,
  RegClass_kv3_onlysetReg,
  RegClass_kv3_onlyswapReg,
  RegClass_kv3_pairedReg,
  RegClass_kv3_pairedReg_0,
  RegClass_kv3_pairedReg_1,
  RegClass_kv3_quadReg,
  RegClass_kv3_quadReg_0,
  RegClass_kv3_quadReg_1,
  RegClass_kv3_quadReg_2,
  RegClass_kv3_quadReg_3,
  RegClass_kv3_singleReg,
  RegClass_kv3_systemReg,
  RegClass_kv3_vectorReg,
  RegClass_kv3_vectorRegE,
  RegClass_kv3_vectorRegO,
  RegClass_kv3_vectorReg_0,
  RegClass_kv3_vectorReg_1,
  RegClass_kv3_vectorReg_2,
  RegClass_kv3_vectorReg_3,
  RegClass_kv3_wideReg,
  RegClass_kv3_wideReg_0,
  RegClass_kv3_wideReg_1,
  RegClass_v2_aloneReg,
  RegClass_v2_onlyfxReg,
  RegClass_v2_onlygetReg,
  RegClass_v2_onlysetReg,
  RegClass_v2_onlyswapReg,
  RegClass_v2_systemReg
};

enum Modifier_kv3_exunum_enum {
  Modifier_kv3_exunum_ALU0=0,
  Modifier_kv3_exunum_ALU1=1,
  Modifier_kv3_exunum_MAU=2,
  Modifier_kv3_exunum_LSU=3,
};

enum Modifier_kv3_scalarcond_enum {
  Modifier_kv3_scalarcond_DNEZ=0,
  Modifier_kv3_scalarcond_DEQZ=1,
  Modifier_kv3_scalarcond_DLTZ=2,
  Modifier_kv3_scalarcond_DGEZ=3,
  Modifier_kv3_scalarcond_DLEZ=4,
  Modifier_kv3_scalarcond_DGTZ=5,
  Modifier_kv3_scalarcond_ODD=6,
  Modifier_kv3_scalarcond_EVEN=7,
  Modifier_kv3_scalarcond_WNEZ=8,
  Modifier_kv3_scalarcond_WEQZ=9,
  Modifier_kv3_scalarcond_WLTZ=10,
  Modifier_kv3_scalarcond_WGEZ=11,
  Modifier_kv3_scalarcond_WLEZ=12,
  Modifier_kv3_scalarcond_WGTZ=13,
};

enum Modifier_kv3_simplecond_enum {
  Modifier_kv3_simplecond_NEZ=0,
  Modifier_kv3_simplecond_EQZ=1,
  Modifier_kv3_simplecond_LTZ=2,
  Modifier_kv3_simplecond_GEZ=3,
  Modifier_kv3_simplecond_LEZ=4,
  Modifier_kv3_simplecond_GTZ=5,
  Modifier_kv3_simplecond_ODD=6,
  Modifier_kv3_simplecond_EVEN=7,
};

enum Modifier_kv3_comparison_enum {
  Modifier_kv3_comparison_NE=0,
  Modifier_kv3_comparison_EQ=1,
  Modifier_kv3_comparison_LT=2,
  Modifier_kv3_comparison_GE=3,
  Modifier_kv3_comparison_LE=4,
  Modifier_kv3_comparison_GT=5,
  Modifier_kv3_comparison_LTU=6,
  Modifier_kv3_comparison_GEU=7,
  Modifier_kv3_comparison_LEU=8,
  Modifier_kv3_comparison_GTU=9,
  Modifier_kv3_comparison_ALL=10,
  Modifier_kv3_comparison_NALL=11,
  Modifier_kv3_comparison_ANY=12,
  Modifier_kv3_comparison_NONE=13,
};

enum Modifier_kv3_floatcomp_enum {
  Modifier_kv3_floatcomp_ONE=0,
  Modifier_kv3_floatcomp_UEQ=1,
  Modifier_kv3_floatcomp_OEQ=2,
  Modifier_kv3_floatcomp_UNE=3,
  Modifier_kv3_floatcomp_OLT=4,
  Modifier_kv3_floatcomp_UGE=5,
  Modifier_kv3_floatcomp_OGE=6,
  Modifier_kv3_floatcomp_ULT=7,
};

enum Modifier_kv3_rounding_enum {
  Modifier_kv3_rounding_RN=0,
  Modifier_kv3_rounding_RU=1,
  Modifier_kv3_rounding_RD=2,
  Modifier_kv3_rounding_RZ=3,
  Modifier_kv3_rounding_RNA=4,
  Modifier_kv3_rounding_RNZ=5,
  Modifier_kv3_rounding_RO=6,
  Modifier_kv3_rounding_=7,
};

enum Modifier_kv3_silent_enum {
  Modifier_kv3_silent_=0,
  Modifier_kv3_silent_S=1,
};

enum Modifier_kv3_roundint_enum {
  Modifier_kv3_roundint_RN=0,
  Modifier_kv3_roundint_RU=1,
  Modifier_kv3_roundint_RD=2,
  Modifier_kv3_roundint_RZ=3,
  Modifier_kv3_roundint_RHU=4,
};

enum Modifier_kv3_saturate_enum {
  Modifier_kv3_saturate_SAT=0,
  Modifier_kv3_saturate_SATU=1,
};

enum Modifier_kv3_rectify_enum {
  Modifier_kv3_rectify_=0,
  Modifier_kv3_rectify_RELU=1,
};

enum Modifier_kv3_variant_enum {
  Modifier_kv3_variant_=0,
  Modifier_kv3_variant_S=1,
  Modifier_kv3_variant_U=2,
  Modifier_kv3_variant_US=3,
};

enum Modifier_kv3_speculate_enum {
  Modifier_kv3_speculate_=0,
  Modifier_kv3_speculate_S=1,
};

enum Modifier_kv3_column_enum {
  Modifier_kv3_column_C0=0,
  Modifier_kv3_column_C1=1,
  Modifier_kv3_column_C2=2,
  Modifier_kv3_column_C3=3,
};

enum Modifier_kv3_doscale_enum {
  Modifier_kv3_doscale_=0,
  Modifier_kv3_doscale_XS=1,
};

enum Modifier_kv3_splat32_enum {
  Modifier_kv3_splat32_=0,
  Modifier_kv3_splat32__=1,
};

enum Modifier_kv3_cachelev_enum {
  Modifier_kv3_cachelev_=0,
  Modifier_kv3_cachelev_L2=1,
  Modifier_kv3_cachelev_L1L2=2,
};

enum Modifier_kv3_boolcas_enum {
  Modifier_kv3_boolcas_V=0,
  Modifier_kv3_boolcas_=1,
};

enum Modifier_kv3_channel_enum {
  Modifier_kv3_channel_F=0,
  Modifier_kv3_channel_B=1,
};

enum Modifier_kv3_conjugate_enum {
  Modifier_kv3_conjugate_=0,
  Modifier_kv3_conjugate_C=1,
};

enum Modifier_kv3_transpose_enum {
  Modifier_kv3_transpose_=0,
  Modifier_kv3_transpose_TN=1,
  Modifier_kv3_transpose_NT=2,
  Modifier_kv3_transpose_TT=3,
};

typedef enum {
  Bundling_kv3_ALL,
  Bundling_kv3_BCU,
  Bundling_kv3_TCA,
  Bundling_kv3_FULL,
  Bundling_kv3_FULL_X,
  Bundling_kv3_FULL_Y,
  Bundling_kv3_LITE,
  Bundling_kv3_LITE_X,
  Bundling_kv3_LITE_Y,
  Bundling_kv3_MAU,
  Bundling_kv3_MAU_X,
  Bundling_kv3_MAU_Y,
  Bundling_kv3_LSU,
  Bundling_kv3_LSU_X,
  Bundling_kv3_LSU_Y,
  Bundling_kv3_TINY,
  Bundling_kv3_TINY_X,
  Bundling_kv3_TINY_Y,
  Bundling_kv3_NOP,
} Bundling;


static const char *bundling_names(Bundling bundling) __attribute__((unused));
static const char *bundling_names(Bundling bundling) {
  switch(bundling) {
  case Bundling_kv3_ALL: return "Bundling_kv3_ALL";
  case Bundling_kv3_BCU: return "Bundling_kv3_BCU";
  case Bundling_kv3_TCA: return "Bundling_kv3_TCA";
  case Bundling_kv3_FULL: return "Bundling_kv3_FULL";
  case Bundling_kv3_FULL_X: return "Bundling_kv3_FULL_X";
  case Bundling_kv3_FULL_Y: return "Bundling_kv3_FULL_Y";
  case Bundling_kv3_LITE: return "Bundling_kv3_LITE";
  case Bundling_kv3_LITE_X: return "Bundling_kv3_LITE_X";
  case Bundling_kv3_LITE_Y: return "Bundling_kv3_LITE_Y";
  case Bundling_kv3_MAU: return "Bundling_kv3_MAU";
  case Bundling_kv3_MAU_X: return "Bundling_kv3_MAU_X";
  case Bundling_kv3_MAU_Y: return "Bundling_kv3_MAU_Y";
  case Bundling_kv3_LSU: return "Bundling_kv3_LSU";
  case Bundling_kv3_LSU_X: return "Bundling_kv3_LSU_X";
  case Bundling_kv3_LSU_Y: return "Bundling_kv3_LSU_Y";
  case Bundling_kv3_TINY: return "Bundling_kv3_TINY";
  case Bundling_kv3_TINY_X: return "Bundling_kv3_TINY_X";
  case Bundling_kv3_TINY_Y: return "Bundling_kv3_TINY_Y";
  case Bundling_kv3_NOP: return "Bundling_kv3_NOP";
  };
  return "unknown bundling";
};

/* Resources list */
#define Resource_kv3_ISSUE 0
#define Resource_kv3_TINY 1
#define Resource_kv3_LITE 2
#define Resource_kv3_FULL 3
#define Resource_kv3_LSU 4
#define Resource_kv3_MAU 5
#define Resource_kv3_BCU 6
#define Resource_kv3_TCA 7
#define Resource_kv3_AUXR 8
#define Resource_kv3_AUXW 9
#define Resource_kv3_CRRP 10
#define Resource_kv3_CRWL 11
#define Resource_kv3_CRWH 12
#define Resource_kv3_NOP 13
#define RESOURCE_MAX 14


/* Reservations list */
#define Reservation_kv3_ALL 0
#define Reservation_kv3_ALU_NOP 1
#define Reservation_kv3_ALU_TINY 2
#define Reservation_kv3_ALU_TINY_X 3
#define Reservation_kv3_ALU_TINY_Y 4
#define Reservation_kv3_ALU_LITE 5
#define Reservation_kv3_ALU_LITE_X 6
#define Reservation_kv3_ALU_LITE_Y 7
#define Reservation_kv3_ALU_LITE_CRWL 8
#define Reservation_kv3_ALU_LITE_CRWH 9
#define Reservation_kv3_ALU_FULL 10
#define Reservation_kv3_ALU_FULL_X 11
#define Reservation_kv3_ALU_FULL_Y 12
#define Reservation_kv3_BCU 13
#define Reservation_kv3_BCU_CRRP_CRWL_CRWH 14
#define Reservation_kv3_BCU_TINY_AUXW_CRRP 15
#define Reservation_kv3_BCU_TINY_TINY_MAU_XNOP 16
#define Reservation_kv3_TCA 17
#define Reservation_kv3_LSU 18
#define Reservation_kv3_LSU_X 19
#define Reservation_kv3_LSU_Y 20
#define Reservation_kv3_LSU_CRRP 21
#define Reservation_kv3_LSU_CRRP_X 22
#define Reservation_kv3_LSU_CRRP_Y 23
#define Reservation_kv3_LSU_AUXR 24
#define Reservation_kv3_LSU_AUXR_X 25
#define Reservation_kv3_LSU_AUXR_Y 26
#define Reservation_kv3_LSU_AUXW 27
#define Reservation_kv3_LSU_AUXW_X 28
#define Reservation_kv3_LSU_AUXW_Y 29
#define Reservation_kv3_LSU_AUXR_AUXW 30
#define Reservation_kv3_LSU_AUXR_AUXW_X 31
#define Reservation_kv3_LSU_AUXR_AUXW_Y 32
#define Reservation_kv3_MAU 33
#define Reservation_kv3_MAU_X 34
#define Reservation_kv3_MAU_Y 35
#define Reservation_kv3_MAU_AUXR 36
#define Reservation_kv3_MAU_AUXR_X 37
#define Reservation_kv3_MAU_AUXR_Y 38
#define Reservation_kv3_ALU_TINY_CRRP 39
#define Reservation_kv3_ALU_TINY_CRWL_CRWH 40



/***********************************************/
/*       DATA TYPES                            */
/***********************************************/

/*  Operand definition -- used in building     */
/*  format table                               */

typedef enum {
    KVX_REL_ABS,   /* Absolute relocation. */
    KVX_REL_PC,    /* PC relative relocation. */
    KVX_REL_GP,    /* GP relative relocation. */
    KVX_REL_TP,    /* TP relative relocation. */
    KVX_REL_GOT,   /* GOT relative relocation. */
    KVX_REL_BASE,  /* BASE load address relative relocation. */
} kvx_rel_t;

struct kvx_reloc
{
  int bitsize;                /* number of bits            */
  kvx_rel_t relative;          /* Type of relative relocation. */
  int reloc_nb;               /* number of BFD relocs      */
  unsigned int relocs[];      /* List of BFD relocations. */
};

typedef struct kvx_reloc kvx_reloc_t;

struct kvx_bitfield
{
  int size;           /* number of bits            */
  int from_offset;    /* Offset in abstract value. */
  int to_offset;      /* Offset in encoded value.  */
};

typedef struct kvx_bitfield kvx_bitfield_t;

struct kvx_operand
{
  const char *tname;                             /* operand type name     */
  int type;                                      /* type of operand       */
  int width;                                     /* width of the operand. */
  int shift;                                     /* encoded value shift   */
  int bias;                                      /* encoded value bias   */
  int flags;                                     /* Can be SIGNED|CANEXTEND|BITMASK|WRAPPED */
  int reg_nb;                                    /* Number of registers */
  int *regs;                                     /* valid registers for this operand (if no register get null pointer). */
  int reloc_nb;                                  /* Number of relocations */
  kvx_reloc_t **relocs;                           /* List of relocations that can be applied to this operand. */
  int bitfields;                                 /* Number of given bitfields */
  kvx_bitfield_t bfield[];  /* bitfields in most to least significant order. */
};

typedef struct kvx_operand kvxbfield;

/* some flags for kvx_operand                                 */
/* kvxSIGNED    : is this operand treated as signed ?         */
/* kvxCANEXTEND : can this operand have an extension          */
/* kvxBITMASK   : this operand is a bit mask */

/* kvxWRAPPED   : this operand can accept signed and unsigned integer ranges */

#define kvxSIGNED    1
#define kvxCANEXTEND 2
#define kvxBITMASK   4

#define kvxWRAPPED   8

#define kvxOPCODE_FLAG_UNDEF   	0
#define kvxOPCODE_FLAG_IMMX0   	1
#define kvxOPCODE_FLAG_IMMX1   	2
#define kvxOPCODE_FLAG_BCU   	4
#define kvxOPCODE_FLAG_ALU   	8
#define kvxOPCODE_FLAG_LSU   	16
#define kvxOPCODE_FLAG_MAU   	32
#define kvxOPCODE_FLAG_MODE64   	64
#define kvxOPCODE_FLAG_MODE32   	128

/* Opcode definition.                          */

struct kvx_codeword_s {
  unsigned           opcode;                      /* the opcode                     */
  unsigned           mask;                        /* disassembly mask               */
  unsigned           flags;                       /* Target dependent flags.        */
};

typedef struct kvx_codeword_s kvx_codeword_t;

struct kv3opc_s {
  const char         *as_op;                        /* asm name                       */
  kvx_codeword_t codewords[KVXMAXSYLLABLES];   /* 32 bits code words. */
  int                wordcount;                     /* Number of words in codewords[].  */
  unsigned           coding_size;                   /* coding size in case of variable length. */
  int                bundling;                      /* Bundling class */
  int                reservation;                   /* Reservation class */
  kvxbfield *format[KVXMAXOPERANDS + 1];  /* 0 terminated                   */
                                        /* list of operands               */
  const char *rclass;                   /* resource class                 */
  const char *fmtstring;                /* formating string               */
};
typedef struct kv3opc_s kv3opc_t;

struct kvx_core_info_s
  {
    kv3opc_t *optab;
    const char *names[2];
    const int *resources;
    int supported;
    int elf_cores[2];
  };

typedef struct kvx_core_info_s Kvx_Core_Info;
extern const int kv3_resource_max;

extern const int kv3_reservation_table_lines;
extern const int *kv3_reservation_table_table[];
extern const char *kv3_resource_names[];

extern const int kv3_v1_resources[];
extern kv3opc_t kv3_v1_optab[];
extern const Kvx_Core_Info kvx_kv3_v1_core_info;
extern const int kv3_v2_resources[];
extern kv3opc_t kv3_v2_optab[];
extern const Kvx_Core_Info kvx_kv3_v2_core_info;
extern const Kvx_Core_Info *kvx_core_info_table[];
extern const kvx_Register *kvx_registers_table[];
extern const int *kvx_regfiles_table[];
extern kvx_reloc_t kv3_rel16_reloc;
extern kvx_reloc_t kv3_rel32_reloc;
extern kvx_reloc_t kv3_rel64_reloc;
extern kvx_reloc_t kv3_pcrel_signed16_reloc;
extern kvx_reloc_t kv3_pcrel17_reloc;
extern kvx_reloc_t kv3_pcrel27_reloc;
extern kvx_reloc_t kv3_pcrel32_reloc;
extern kvx_reloc_t kv3_pcrel_signed37_reloc;
extern kvx_reloc_t kv3_pcrel_signed43_reloc;
extern kvx_reloc_t kv3_pcrel_signed64_reloc;
extern kvx_reloc_t kv3_pcrel64_reloc;
extern kvx_reloc_t kv3_signed16_reloc;
extern kvx_reloc_t kv3_signed32_reloc;
extern kvx_reloc_t kv3_signed37_reloc;
extern kvx_reloc_t kv3_gotoff_signed37_reloc;
extern kvx_reloc_t kv3_gotoff_signed43_reloc;
extern kvx_reloc_t kv3_gotoff_32_reloc;
extern kvx_reloc_t kv3_gotoff_64_reloc;
extern kvx_reloc_t kv3_got_32_reloc;
extern kvx_reloc_t kv3_got_signed37_reloc;
extern kvx_reloc_t kv3_got_signed43_reloc;
extern kvx_reloc_t kv3_got_64_reloc;
extern kvx_reloc_t kv3_glob_dat_reloc;
extern kvx_reloc_t kv3_copy_reloc;
extern kvx_reloc_t kv3_jump_slot_reloc;
extern kvx_reloc_t kv3_relative_reloc;
extern kvx_reloc_t kv3_signed43_reloc;
extern kvx_reloc_t kv3_signed64_reloc;
extern kvx_reloc_t kv3_gotaddr_signed37_reloc;
extern kvx_reloc_t kv3_gotaddr_signed43_reloc;
extern kvx_reloc_t kv3_gotaddr_signed64_reloc;
extern kvx_reloc_t kv3_dtpmod64_reloc;
extern kvx_reloc_t kv3_dtpoff64_reloc;
extern kvx_reloc_t kv3_dtpoff_signed37_reloc;
extern kvx_reloc_t kv3_dtpoff_signed43_reloc;
extern kvx_reloc_t kv3_tlsgd_signed37_reloc;
extern kvx_reloc_t kv3_tlsgd_signed43_reloc;
extern kvx_reloc_t kv3_tlsld_signed37_reloc;
extern kvx_reloc_t kv3_tlsld_signed43_reloc;
extern kvx_reloc_t kv3_tpoff64_reloc;
extern kvx_reloc_t kv3_tlsie_signed37_reloc;
extern kvx_reloc_t kv3_tlsie_signed43_reloc;
extern kvx_reloc_t kv3_tlsle_signed37_reloc;
extern kvx_reloc_t kv3_tlsle_signed43_reloc;
extern kvx_reloc_t kv3_rel8_reloc;


#endif /* OPCODE_KVX_H */
